@@ -7,7 +7,7 @@
 #ifndef _mozilla_a11y_aria_ARIAStateMap_h_
 #define _mozilla_a11y_aria_ARIAStateMap_h_
 
-#include "prtypes.h"
+#include "mozilla/StandardInteger.h"
 
 namespace mozilla {
 
@@ -39,6 +39,7 @@ enum EStateRule
   eARIAPressed,
   eARIAReadonly,
   eARIAReadonlyOrEditable,
+  eARIAReadonlyOrEditableIfDefined,
   eARIARequired,
   eARIASelectable,
   eReadonlyUntilEditable,
@@ -54,7 +55,7 @@ enum EStateRule
  * @param  aState    [in/out] accessible states
  * @return            true if state map rule ID is valid
  */
-bool MapToState(EStateRule aRule, dom::Element* aElement, PRUint64* aState);
+bool MapToState(EStateRule aRule, dom::Element* aElement, uint64_t* aState);
 
 } // namespace aria
 } // namespace a11y
