@@ -10,21 +10,41 @@ function Exclusion(pattern) {
 }
 
 function CookieRule(host, cookiename) {
-  this.host = host
+  this.host = host;
   this.host_c = new RegExp(host);
   this.name = cookiename;
   this.name_c = new RegExp(cookiename);
 }
 
+// Firefox 23+ blocks mixed content by default, so rulesets that create
+// mixed content situations should be disabled there
+
+try {
+  var appPrefs = CC["@mozilla.org/preferences-service;1"].getService(CI.nsIPrefBranch);
+  var blockMixedContent = appPrefs.getBoolPref("security.mixed_content.block_active_content");
+  if(blockMixedContent) {
+    localPlatformRegexp = new RegExp("firefox");
+  } else {
+    localPlatformRegexp = new RegExp("(firefox|mixedcontent)");
+  }
+} catch(e) {
+  localPlatformRegexp = new RegExp("(firefox|mixedcontent)");
+}
+
 ruleset_counter = 0;
-function RuleSet(name, match_rule, default_off) {
+function RuleSet(name, xmlName, match_rule, default_off, platform) {
+  if(xmlName == "WordPress.xml" || xmlName == "Github.xml") {
+    this.log(NOTE, "RuleSet( name="+name+", xmlName="+xmlName+", match_rule="+match_rule+", default_off="+default_off+", platform="+platform+" )");
+  }
+
   this.id="httpseR" + ruleset_counter;
   ruleset_counter += 1;
   this.on_by_default = true;
   this.name = name;
+  this.xmlName = xmlName;
   //this.ruleset_match = match_rule;
   this.notes = "";
-  if (match_rule)   this.ruleset_match_c = new RegExp(match_rule)
+  if (match_rule)   this.ruleset_match_c = new RegExp(match_rule);
   else              this.ruleset_match_c = null;
   if (default_off) {
     // Perhaps problematically, this currently ignores the actual content of
@@ -33,25 +53,31 @@ function RuleSet(name, match_rule, default_off) {
     this.notes = default_off;
     this.on_by_default = false;
   }
+  if (platform)
+    if (platform.search(localPlatformRegexp) == -1) {
+      this.on_by_default = false;
+      this.notes = "Only for " + platform;
+    }
+
   this.rules = [];
   this.exclusions = [];
   this.cookierules = [];
-  this.prefs = HTTPSEverywhere.instance.prefs;
+  
+  this.rule_toggle_prefs = HTTPSEverywhere.instance.rule_toggle_prefs;
+
   try {
     // if this pref exists, use it
-    this.active = this.prefs.getBoolPref(name);
-  } catch (e) {
-    // if not, create it
-    this.log(DBUG, "Creating new pref " + name);
+    this.active = this.rule_toggle_prefs.getBoolPref(name);
+  } catch(e) {
+    // if not, use the default
     this.active = this.on_by_default;
-    this.prefs.setBoolPref(name, this.on_by_default);
   }
 }
 
 var dom_parser = Cc["@mozilla.org/xmlextras/domparser;1"].createInstance(Ci.nsIDOMParser);
 
 RuleSet.prototype = {
-  _apply: function(urispec) {
+  apply: function(urispec) {
     // return null if it does not apply
     // and the new url if it does apply
     var i;
@@ -81,41 +107,40 @@ RuleSet.prototype = {
     https_everywhereLog(level, msg);
   },
  
- wouldMatch: function(hypothetical_uri, alist) {
-   // return true if this ruleset would match the uri, assuming it were http
-   // used for judging moot / inactive rulesets
+  wouldMatch: function(hypothetical_uri, alist) {
+    // return true if this ruleset would match the uri, assuming it were http
+    // used for judging moot / inactive rulesets
+    // alist is optional
+ 
+    // if the ruleset is already somewhere in this applicable list, we don't
+    // care about hypothetical wouldMatch questions
+    if (alist && (this.name in alist.all)) return false;
+ 
+    this.log(DBUG,"Would " +this.name + " match " +hypothetical_uri.spec +
+             "?  serial " + (alist && alist.serial));
+     
+    var uri = hypothetical_uri.clone();
+    if (uri.scheme == "https") uri.scheme = "http";
+    var urispec = uri.spec;
+ 
+    if (this.ruleset_match_c && !this.ruleset_match_c.test(urispec)) 
+      return false;
+ 
+    for (var i = 0; i < this.exclusions.length; ++i) 
+      if (this.exclusions[i].pattern_c.test(urispec)) return false;
+ 
+    for (var i = 0; i < this.rules.length; ++i) 
+      if (this.rules[i].from_c.test(urispec)) return true;
+    return false;
+  },
 
-   // if the ruleset is already somewhere in this applicable list, we don't
-   // care about hypothetical wouldMatch questios
-   if (this.name in alist.all) return false;
-
-   this.log(DBUG,"Would " +this.name + " match " +hypothetical_uri.spec +
-            "?  serial " + alist.serial);
-    
-   var uri = hypothetical_uri.clone();
-   if (uri.scheme == "https") uri.scheme = "http";
-   var urispec = uri.spec;
-
-   if (this.ruleset_match_c && !this.ruleset_match_c.test(urispec)) 
-     return false;
-
-   for (i = 0; i < this.exclusions.length; ++i) 
-     if (this.exclusions[i].pattern_c.test(urispec)) return false;
-
-   for (i = 0; i < this.rules.length; ++i) 
-     if (this.rules[i].from_c.test(urispec)) return true;
-   return false;
- },
-
- transformURI: function(uri) {
+  transformURI: function(uri) {
     // If no rule applies, return null; if a rule would have applied but was
     // inactive, return 0; otherwise, return a fresh uri instance
     // for the target
-    var newurl = this._apply(uri.spec);
+    var newurl = this.apply(uri.spec);
     if (null == newurl) 
       return null;
-    if (0 == newurl)
-      return 0;
     var newuri = Components.classes["@mozilla.org/network/standard-url;1"].
                  createInstance(CI.nsIStandardURL);
     newuri.init(CI.nsIStandardURL.URLTYPE_STANDARD, 80,
@@ -126,19 +151,28 @@ RuleSet.prototype = {
 
   enable: function() {
     // Enable us.
-    this.prefs.setBoolPref(this.name, true);
+    this.rule_toggle_prefs.setBoolPref(this.name, true);
     this.active = true;
   },
 
   disable: function() {
     // Disable us.
-    this.prefs.setBoolPref(this.name, false);
+    this.rule_toggle_prefs.setBoolPref(this.name, false);
     this.active = false;
   },
 
   toggle: function() {
     this.active = !this.active;
-    this.prefs.setBoolPref(this.name, this.active);
+    this.rule_toggle_prefs.setBoolPref(this.name, this.active);
+  },
+
+  clear: function() {
+    try {
+      this.rule_toggle_prefs.clearUserPref(this.name);
+    } catch(e) {
+      // this ruleset has never been toggled
+    }
+    this.active = this.on_by_default;
   }
 };
 
@@ -196,7 +230,7 @@ const RuleWriter = {
   },
 
   getRuleDir: function() {
-    loc = "chrome://https-everywhere/content/rules/";
+    var loc = "chrome://https-everywhere/content/rules/";
 
     var file =
       CC["@mozilla.org/file/local;1"]
@@ -243,7 +277,6 @@ const RuleWriter = {
     this.parseXmlRulesets(xmlrulesets, rule_store, file);
   },
 
-
   parseXmlRulesets: function(xmldom, rule_store, file) {
     // XML input files can either be a <ruleset> in a file, or a
     // <rulesetlibrary> with many <rulesets> inside it (the latter form exists
@@ -253,6 +286,16 @@ const RuleWriter = {
       this.parseOneRuleset(xmldom.documentElement, rule_store, file);
     } else {
       // The root of the XML tree is assumed to look like a <rulesetlibrary>
+      if (!xmldom.documentElement.getAttribute("gitcommitid")) {
+        // The gitcommitid is a tricky hack to let us display the true full
+        // source code of a ruleset, even though we strip out comments at build
+        // time, by having the UI fetch the ruleset from the public https git repo.
+        this.log(DBUG, "gitcommitid tag not found in <xmlruleset>");
+        rule_store.GITCommitID = "HEAD";
+      } else {
+        rule_store.GITCommitID = xmldom.documentElement.getAttribute("gitcommitid");
+      }
+
       var rulesets = xmldom.documentElement.getElementsByTagName("ruleset");
       if (rulesets.length == 0 && (file.path.search("00README") == -1))
         this.log(WARN, "Probable <rulesetlibrary> with no <rulesets> in "
@@ -273,7 +316,8 @@ const RuleWriter = {
 
     var match_rl = xmlruleset.getAttribute("match_rule");
     var dflt_off = xmlruleset.getAttribute("default_off");
-    var rs = new RuleSet(xmlruleset.getAttribute("name"), match_rl, dflt_off);
+    var platform = xmlruleset.getAttribute("platform");
+    var rs = new RuleSet(xmlruleset.getAttribute("name"), xmlruleset.getAttribute("f"), match_rl, dflt_off, platform);
 
     var targets = xmlruleset.getElementsByTagName("target");
     if (targets.length == 0) {
@@ -353,11 +397,11 @@ const HTTPSRules = {
                           // applicable rules
       this.rulesetsByID = {};
       this.rulesetsByName = {};
+      var t1 = new Date().getTime();
       var rulefiles = RuleWriter.enumerate(RuleWriter.getCustomRuleDir());
       this.scanRulefiles(rulefiles);
       rulefiles = RuleWriter.enumerate(RuleWriter.getRuleDir());
       this.scanRulefiles(rulefiles);
-      this.checkForBuggyDefaults();
       var t,i;
       for (t in this.targets) {
         for (i = 0 ; i < this.targets[t].length ; i++) {
@@ -379,7 +423,8 @@ const HTTPSRules = {
     } catch(e) {
       this.log(WARN,"Rules Failed: "+e);
     }
-    this.log(DBUG,"Rules loaded");
+    var t2 =  new Date().getTime();
+    this.log(NOTE,"Loading rulesets took " + (t2 - t1) / 1000.0 + " seconds");
     return;
   },
 
@@ -398,50 +443,11 @@ const HTTPSRules = {
     }
   },
 
-  checkForBuggyDefaults: function() {
-    // The 2.2 release had a ruleset parsing bug which caused the match_rule
-    // attribute to be misinterpreted as the default_off attribute.  That was
-    // a big problem iff 2.2 was the first version of HTTPS Everywhere
-    // installed in a particular browser profile.  If enough buggy rulesets
-    // are on, conclude that this is such a profile, and reset to the correct
-    // defaults.  More here:
-    //
-    // https://mail1.eff.org/pipermail/https-everywhere/2012-August/001511.html
-    //
-    this.log(DBUG, "Checking for buggy configurations from version 2.2");
-    // All of these were default_off in 2.2, and none of them had a
-    // match_rule:
-    var shouldBeOff = ["SchuelerVZ","Tcodevelopment.com","33Bits","BerliOS",
-                       "BuisnessInsider (broken)","Daft.ie","DVDFab",
-                       "YouMail (buggy)","StudiVZ (disabled)","Woot (broken)"];
-
-    var nonBuggy = 0;
-    for (rulename in shouldBeOff) {
-      // Some of these shouldBeOff rules may be removed in the future, so
-      // tolerate their absence
-      if (rulename in this.rulesetsByName) {
-        if (!this.rulesetsByName[rulename].active) 
-          nonBuggy += 1;
-      } else {
-        this.log("INFO", "Couldn't check state of ruleset " + rulename);
-      }
-    }
-
-    // Heuristic: a user starting with a buggy config might have disabled up
-    // to 2 of the 10 broken rulesets manually, but we will still conclude
-    // that their config is broken and reset it
-    if (nonBuggy <= 2) {
-      this.log(WARN, "Detected a buggy-looking configuration, probably from HTTPS Everywhere 2.2.  Resetting ruleset states to defaults");
-      this.resetRulesetsToDefaults();
-    }
-  },
-
   resetRulesetsToDefaults: function() {
     // Callable from within the prefs UI and also for cleaning up buggy
     // configurations...
     for (var i in this.rulesets) {
-      if (this.rulesets[i].on_by_default) this.rulesets[i].enable();
-      else                                this.rulesets[i].disable();
+      this.rulesets[i].clear();
     }
   },
 
@@ -452,20 +458,65 @@ const HTTPSRules = {
     // the new uri if there was a rewrite.  Now it returns a JS object with a
     // newuri attribute and an applied_ruleset attribute (or null if there's
     // no rewrite).
-    var i = 0, userpass_present = false;
-    var uri = input_uri;
-    var blob = {};
-    blob.newuri = null;
-    if (!alist) this.log(DBUG, "No applicable list rewriting " + uri.spec);
+    var i = 0; 
+    userpass_present = false; // Global so that sanitiseURI can tweak it.
+                              // Why does JS have no tuples, again?
+    var blob = {}; blob.newuri = null;
+    if (!alist) this.log(DBUG, "No applicable list rewriting " + input_uri.spec);
+    this.log(NOTE, "Processing " + input_uri.spec);
 
+    var uri = this.sanitiseURI(input_uri);
+
+    // Get the list of rulesets that target this host
+    try {
+      var rs = this.potentiallyApplicableRulesets(uri.host);
+    } catch(e) {
+      this.log(WARN, 'Could not check applicable rules for '+uri.spec);
+      return null;
+    }
+
+    // ponder each potentially applicable ruleset, working out if it applies
+    // and recording it as active/inactive/moot/breaking in the applicable list
+    for (i = 0; i < rs.length; ++i) {
+      if (!rs[i].active) {
+        if (alist && rs[i].wouldMatch(uri, alist))
+          alist.inactive_rule(rs[i]);
+        continue;
+      } 
+      blob.newuri = rs[i].transformURI(uri);
+      if (blob.newuri) {
+        // we rewrote the uri
+	this.log(DBUG, "Rewrote "+input_uri.spec);
+        if (alist) {
+          if (uri.spec in https_everywhere_blacklist) 
+            alist.breaking_rule(rs[i]);
+          else 
+            alist.active_rule(rs[i]);
+	}
+        if (userpass_present) blob.newuri.userPass = input_uri.userPass;
+        blob.applied_ruleset = rs[i];
+        return blob;
+      }
+      if (uri.scheme == "https" && alist) {
+        // we didn't rewrite but the rule applies to this domain and the
+        // requests are going over https
+        if (rs[i].wouldMatch(uri, alist)) alist.moot_rule(rs[i]);
+        continue;
+      } 
+    }
+    return null;
+  },
+
+  sanitiseURI: function(input_uri) {
     // Rulesets shouldn't try to parse usernames and passwords.  If we find
-    // those, apply the ruleset without them and then add them back.
+    // those, apply the ruleset without them (and then add them back later).
     // When .userPass is absent, sometimes it is false and sometimes trying
     // to read it raises an exception (probably depending on the URI type).
+    var uri = input_uri;
     try {
       if (input_uri.userPass) {
         uri = input_uri.clone();
-        userpass_present = true;
+        userpass_present = true; // tweaking a global in our caller :(
         uri.userPass = null;
       } 
     } catch(e) {}
@@ -492,58 +543,29 @@ const HTTPSRules = {
       try       { this.log(WARN, "(" + uri.spec + ")"); } 
       catch(e4) { this.log(WARN, "(and unprintable)"); }
     }
-
-    // Get the list of rulesets that target this host
-    try {
-      var rs = this.potentiallyApplicableRulesets(uri.host);
-    } catch(e) {
-      this.log(WARN, 'Could not check applicable rules for '+uri.spec);
-      return null;
-    }
-
-    // ponder each potentially applicable ruleset, working out if it applies
-    // and recording it as active/inactive/moot/breaking in the applicable list
-    for (i = 0; i < rs.length; ++i) {
-      if (!rs[i].active) {
-        if (alist && rs[i].wouldMatch(uri, alist))
-          alist.inactive_rule(rs[i]);
-        continue;
-      } 
-      blob.newuri = rs[i].transformURI(uri);
-      if (blob.newuri) {
-        // we rewrote the uri
-        if (alist)
-          if (uri.spec in https_everywhere_blacklist) 
-            alist.breaking_rule(rs[i])
-          else 
-            alist.active_rule(rs[i]);
-        if (userpass_present) blob.newuri.userPass = input_uri.userPass;
-        blob.applied_ruleset = rs[i];
-        return blob;
-      }
-      if (uri.scheme == "https" && alist) {
-        // we didn't rewrite but the rule applies to this domain and the
-        // requests are going over https
-        if (rs[i].wouldMatch(uri, alist)) alist.moot_rule(rs[i]);
-        continue;
-      } 
-    }
-    return null;
+    return uri;
   },
 
+  setInsert: function(intoList, fromList) {
+    // Insert any elements from fromList into intoList, if they are not
+    // already there.  fromList may be null.
+    if (!fromList) return;
+    for (var i = 0; i < fromList.length; i++)
+      if (intoList.indexOf(fromList[i]) == -1)
+        intoList.push(fromList[i]);
+  },
 
-  potentiallyApplicableRulesets: function(host) {  
+  potentiallyApplicableRulesets: function(host) {
     // Return a list of rulesets that declare targets matching this host
     var i, tmp, t;
-    var results = this.global_rulesets;
-	try{
-		if (this.targets[host])
-			results = results.concat(this.targets[host]);
-	}
-	catch(e){	
-		this.log(DBUG,"Couldn't check for ApplicableRulesets: " + e);
-		return [];
-	}
+    var results = this.global_rulesets.slice(0); // copy global_rulesets
+    try {
+      if (this.targets[host])
+        results = results.concat(this.targets[host]);
+    } catch(e) {   
+      this.log(DBUG,"Couldn't check for ApplicableRulesets: " + e);
+      return [];
+    }
     // replace each portion of the domain with a * in turn
     var segmented = host.split(".");
     for (i = 0; i < segmented.length; ++i) {
@@ -551,15 +573,13 @@ const HTTPSRules = {
       segmented[i] = "*";
       t = segmented.join(".");
       segmented[i] = tmp;
-      if (this.targets[t])
-        results = results.concat(this.targets[t]);
+      this.setInsert(results, this.targets[t]);
     }
     // now eat away from the left, with *, so that for x.y.z.google.com we
     // check *.z.google.com and *.google.com (we did *.y.z.google.com above)
-    for (i = 1; i < segmented.length - 2; ++i) {
+    for (i = 1; i <= segmented.length - 2; ++i) {
       t = "*." + segmented.slice(i,segmented.length).join(".");
-      if (this.targets[t])
-        results = results.concat(this.targets[t]);
+      this.setInsert(results, this.targets[t]);
     }
     this.log(DBUG,"Potentially applicable rules for " + host + ":");
     for (i = 0; i < results.length; ++i)
@@ -567,19 +587,22 @@ const HTTPSRules = {
     return results;
   },
 
-  shouldSecureCookie: function(applicable_list, c) {
+  shouldSecureCookie: function(applicable_list, c, known_https) {
     // Check to see if the Cookie object c meets any of our cookierule citeria
-    // for being marked as secure
-    //this.log(DBUG, "Testing cookie:");
-    //this.log(DBUG, "  name: " + c.name);
-    //this.log(DBUG, "  host: " + c.host);
-    //this.log(DBUG, "  domain: " + c.domain);
-    //this.log(DBUG, "  rawhost: " + c.rawHost);
+    // for being marked as secure.
+    // @applicable_list : an ApplicableList or record keeping
+    // @c : an nsICookie2
+    // @known_https : true if we know the page setting the cookie is https
+
+    this.log(DBUG,"  rawhost: " + c.rawHost + "\n  name: " + c.name + "\n  host" + c.host);
     var i,j;
     var rs = this.potentiallyApplicableRulesets(c.host);
     for (i = 0; i < rs.length; ++i) {
       var ruleset = rs[i];
       if (ruleset.active) {
+        // Never secure a cookie if this page might be HTTP
+        if (!known_https && !this.safeToSecureCookie(c.rawHost))
+          continue;
         for (j = 0; j < ruleset.cookierules.length; j++) {
           var cr = ruleset.cookierules[j];
           if (cr.host_c.test(c.host) && cr.name_c.test(c.name)) {
@@ -596,6 +619,50 @@ const HTTPSRules = {
       }
     }
     return false;
-  }
+  },
 
+  safeToSecureCookie: function(domain) {
+    // Check if the domain might be being served over HTTP.  If so, it isn't
+    // safe to secure a cookie!  We can't always know this for sure because
+    // observing cookie-changed doesn't give us enough context to know the
+    // full origin URI.
+
+    // First, if there are any redirect loops on this domain, don't secure
+    // cookies.  XXX This is not a very satisfactory heuristic.  Sometimes we
+    // would want to secure the cookie anyway, because the URLs that loop are
+    // not authenticated or not important.  Also by the time the loop has been
+    // observed and the domain blacklisted, a cookie might already have been
+    // flagged as secure.
+
+    if (domain in https_blacklist_domains) {
+      this.log(INFO, "cookies for " + domain + "blacklisted");
+      return false;
+    }
+
+    // If we passed that test, make up a random URL on the domain, and see if
+    // we would HTTPSify that.
+
+    try {
+      var nonce_path = "/" + Math.random().toString();
+      nonce_path = nonce_path + nonce_path;
+      var test_uri = "http://" + domain + nonce_path;
+    } catch (e) {
+      this.log(WARN, "explosion in safeToSecureCookie for " + domain + "\n" 
+                      + "(" + e + ")");
+      return false;
+    }
+
+    this.log(INFO, "Testing securecookie applicability with " + test_uri);
+    var rs = this.potentiallyApplicableRulesets(domain);
+    for (var i = 0; i < rs.length; ++i) {
+      if (!rs[i].active) continue;
+      var rewrite = rs[i].apply(test_uri);
+      if (rewrite) {
+        this.log(INFO, "Yes: " + rewrite);
+        return true;
+      }
+    }
+    this.log(INFO, "(NO)");
+    return false;
+  }
 };
