@@ -6,6 +6,10 @@ INFO=3;
 NOTE=4;
 WARN=5;
 
+// PREFERENCE BRANCHES
+PREFBRANCH_ROOT=0;
+PREFBRANCH_RULE_TOGGLE=1;
+
 //---------------
 
 https_domains = {};              // maps domain patterns (with at most one
@@ -14,6 +18,9 @@ https_domains = {};              // maps domain patterns (with at most one
 https_everywhere_blacklist = {}; // URLs we've given up on rewriting because
                                  // of redirection loops
 
+https_blacklist_domains = {};    // domains for which there is at least one
+                                 // blacklisted URL
+
 //
 const CI = Components.interfaces;
 const CC = Components.classes;
@@ -49,13 +56,11 @@ const INCLUDE = function(name) {
     for (var j = 0, len = arguments.length; j < len; j++)
       INCLUDE(arguments[j]);
   else if (!_INCLUDED[name]) {
-    try {
-      LOADER.loadSubScript("chrome://https-everywhere/content/code/"
-              + name + ".js");
-      _INCLUDED[name] = true;
-    } catch(e) {
-      dump("INCLUDE " + name + ": " + e + "\n");
-    }
+    // we used to try/catch here, but that was less useful because it didn't
+    // produce line numbers for syntax errors
+    LOADER.loadSubScript("chrome://https-everywhere/content/code/"
+            + name + ".js");
+    _INCLUDED[name] = true;
   }
 }
 
@@ -95,6 +100,8 @@ const WHERE_UNTRUSTED = 1;
 const WHERE_TRUSTED = 2;
 const ANYWHERE = 3;
 
+const N_COHORTS = 1000;  // For partial feature deployment
+
 const DUMMY_OBJ = {};
 DUMMY_OBJ.wrappedJSObject = DUMMY_OBJ;
 const DUMMY_FUNC = function() {}
@@ -176,6 +183,7 @@ function HTTPSEverywhere() {
   this.ApplicableList = ApplicableList;
   
   this.prefs = this.get_prefs();
+  this.rule_toggle_prefs = this.get_prefs(PREFBRANCH_RULE_TOGGLE);
   
   // We need to use observers instead of categories for FF3.0 for these:
   // https://developer.mozilla.org/en/Observer_Notifications
@@ -185,12 +193,13 @@ function HTTPSEverywhere() {
   // we rewrite.
   this.obsService = CC["@mozilla.org/observer-service;1"]
                     .getService(Components.interfaces.nsIObserverService);
-					
+                    
   if(this.prefs.getBoolPref("globalEnabled")){
     this.obsService.addObserver(this, "profile-before-change", false);
     this.obsService.addObserver(this, "profile-after-change", false);
     this.obsService.addObserver(this, "sessionstore-windows-restored", false);
   }
+  
   return;
 }
 
@@ -209,9 +218,9 @@ const TYPE_XBL = 9;
 const TYPE_PING = 10;
 const TYPE_XMLHTTPREQUEST = 11;
 const TYPE_OBJECT_SUBREQUEST = 12;
-const TYPE_DTD	= 13;
+const TYPE_DTD  = 13;
 const TYPE_FONT = 14;
-const TYPE_MEDIA = 15; 	
+const TYPE_MEDIA = 15;  
 // --------------
 // REJECT_SERVER = -3
 // ACCEPT = 1
@@ -371,11 +380,11 @@ HTTPSEverywhere.prototype = {
     try {
       var domWin = nc.getInterface(CI.nsIDOMWindow);
     } catch(e) {
-      this.log(INFO, "exploded getting DOMWin for " + channel.URI.spec);
+      this.log(INFO, "No window associated with request: " + channel.URI.spec);
       return null;
     }
     if (!domWin) {
-      this.log(WARN, "failed to get DOMWin for " + channel.URI.spec);
+      this.log(NOTE, "failed to get DOMWin for " + channel.URI.spec);
       return null;
     }
     domWin = domWin.top;
@@ -418,7 +427,6 @@ HTTPSEverywhere.prototype = {
     return alist;
   },
 
-
   observe: function(subject, topic, data) {
     // Top level glue for the nsIObserver API
     var channel = subject;
@@ -426,9 +434,9 @@ HTTPSEverywhere.prototype = {
 
     if (topic == "http-on-modify-request") {
       if (!(channel instanceof CI.nsIHttpChannel)) return;
-	  
+      
       this.log(DBUG,"Got http-on-modify-request: "+channel.URI.spec);
-      var lst = this.getApplicableListForChannel(channel);
+      var lst = this.getApplicableListForChannel(channel); // null if no window is associated (ex: xhr)
       if (channel.URI.spec in https_everywhere_blacklist) {
         this.log(DBUG, "Avoiding blacklisted " + channel.URI.spec);
         if (lst) lst.breaking_rule(https_everywhere_blacklist[channel.URI.spec])
@@ -440,11 +448,10 @@ HTTPSEverywhere.prototype = {
          this.log(DBUG, "Got http-on-examine-response @ "+ (channel.URI ? channel.URI.spec : '') );
          HTTPS.handleSecureCookies(channel);
     } else if (topic == "http-on-examine-merged-response") {
-		 this.log(DBUG, "Got http-on-examine-merged-response ");
+         this.log(DBUG, "Got http-on-examine-merged-response ");
          HTTPS.handleSecureCookies(channel);
     } else if (topic == "cookie-changed") {
       // Javascript can add cookies via document.cookie that are insecure.
-      // It might also be able to 
       if (data == "added" || data == "changed") {
         // subject can also be an nsIArray! bleh.
         try {
@@ -464,8 +471,6 @@ HTTPSEverywhere.prototype = {
           }
         }
       }
-    } else if (topic == "app-startup") {
-      this.log(DBUG,"Got app-startup");
     } else if (topic == "profile-before-change") {
       this.log(INFO, "Got profile-before-change");
       var catman = Components.classes["@mozilla.org/categorymanager;1"]
@@ -474,46 +479,72 @@ HTTPSEverywhere.prototype = {
       Thread.hostRunning = false;
     } else if (topic == "profile-after-change") {
       this.log(DBUG, "Got profile-after-change");
-	  
-	  if(this.prefs.getBoolPref("globalEnabled")){
-		OS.addObserver(this, "cookie-changed", false);
-		OS.addObserver(this, "http-on-modify-request", false);
-		OS.addObserver(this, "http-on-examine-merged-response", false);
-		OS.addObserver(this, "http-on-examine-response", false);
-		
-		var dls = CC['@mozilla.org/docloaderservice;1']
-			.getService(CI.nsIWebProgress);
-		dls.addProgressListener(this, CI.nsIWebProgress.NOTIFY_LOCATION);
-		this.log(INFO,"ChannelReplacement.supported = "+ChannelReplacement.supported);
+      
+      if(this.prefs.getBoolPref("globalEnabled")){
+        OS.addObserver(this, "cookie-changed", false);
+        OS.addObserver(this, "http-on-modify-request", false);
+        OS.addObserver(this, "http-on-examine-merged-response", false);
+        OS.addObserver(this, "http-on-examine-response", false);
+        
+        var dls = CC['@mozilla.org/docloaderservice;1']
+            .getService(CI.nsIWebProgress);
+        dls.addProgressListener(this, CI.nsIWebProgress.NOTIFY_LOCATION);
+        this.log(INFO,"ChannelReplacement.supported = "+ChannelReplacement.supported);
 
-		HTTPSRules.init();
+        HTTPSRules.init();
 
-		Thread.hostRunning = true;
-		var catman = Components.classes["@mozilla.org/categorymanager;1"]
+        Thread.hostRunning = true;
+        var catman = Components.classes["@mozilla.org/categorymanager;1"]
            .getService(Components.interfaces.nsICategoryManager);
-		// hook on redirections (non persistent, otherwise crashes on 1.8.x)
-		catman.addCategoryEntry("net-channel-event-sinks", SERVICE_CTRID,
-			SERVICE_CTRID, false, true);
-	  }
+        // hook on redirections (non persistent, otherwise crashes on 1.8.x)
+        catman.addCategoryEntry("net-channel-event-sinks", SERVICE_CTRID,
+            SERVICE_CTRID, false, true);
+      }
     } else if (topic == "sessionstore-windows-restored") {
-      var ssl_observatory = CC["@eff.org/ssl-observatory;1"]
-                        .getService(Components.interfaces.nsISupports)
-                        .wrappedJSObject;
-      // FIXME This prefs code is terrible spaghetti
-      var shown = ssl_observatory.myGetBoolPref("popup_shown");
-      // this is relevant if the user just installed torbutton bad had
-      // enabled the Observatory previously
-      var enabled = ssl_observatory.myGetBoolPref("enabled");
-      if (!shown && !enabled && ssl_observatory.torbutton_installed) 
-        this.chrome_opener("chrome://https-everywhere/content/observatory-popup.xul");
+      this.log(DBUG,"Got sessionstore-windows-restored");
+      this.maybeShowObservatoryPopup();
     }
     return;
   },
 
+  maybeShowObservatoryPopup: function() {
+    // Show the popup at most once.  Users who enabled the Observatory before
+    // a version that would have shown it to them, don't need to see it
+    // again.
+    var ssl_observatory = CC["@eff.org/ssl-observatory;1"]
+                      .getService(Components.interfaces.nsISupports)
+                      .wrappedJSObject;
+    var shown = ssl_observatory.myGetBoolPref("popup_shown");
+    var enabled = ssl_observatory.myGetBoolPref("enabled");
+    var that = this;
+    var obs_popup_callback = function(result) {
+      if (result) that.log(INFO, "Got positive proxy test.");
+      else        that.log(INFO, "Got negative proxy text.");
+      // We are now ready to show the popup in its most informative state
+      that.chrome_opener("chrome://https-everywhere/content/observatory-popup.xul");
+    };
+    if (!shown && !enabled)
+      ssl_observatory.registerProxyTestNotification(obs_popup_callback);
+  },
+
+  getExperimentalFeatureCohort: function() {
+    // This variable is used for gradually turning on features for testing and
+    // scalability purposes.  It is a random integer [0,N_COHORTS) generated
+    // once and stored thereafter.
+    var cohort;
+    try {
+      cohort = this.prefs.getIntPref("experimental_feature_cohort");
+    } catch(e) {
+      cohort = Math.round(Math.random() * N_COHORTS);
+      this.prefs.setIntPref("experimental_feature_cohort", cohort);
+    }
+    return cohort;
+  },
+
   // nsIChannelEventSink implementation
   onChannelRedirect: function(oldChannel, newChannel, flags) {  
     const uri = newChannel.URI;
-    this.log(DBUG,"Got onChannelRedirect.");
+    this.log(DBUG,"Got onChannelRedirect to "+uri.spec);
     if (!(newChannel instanceof CI.nsIHttpChannel)) {
       this.log(DBUG, newChannel + " is not an instance of nsIHttpChannel");
       return;
@@ -544,8 +575,8 @@ HTTPSEverywhere.prototype = {
   },
 
   asyncOnChannelRedirect: function(oldChannel, newChannel, flags, callback) {
-		this.onChannelRedirect(oldChannel, newChannel, flags);
-		callback.onRedirectVerifyCallback(0);
+        this.onChannelRedirect(oldChannel, newChannel, flags);
+        callback.onRedirectVerifyCallback(0);
   },
 
   // These implement the nsIContentPolicy API; they allow both yes/no answers
@@ -568,33 +599,40 @@ HTTPSEverywhere.prototype = {
     return this.shouldLoad(aContentType, aContentLocation, aRequestOrigin, aContext, aMimeType, CP_SHOULDPROCESS);
   },
 
-  get_prefs: function() {
-      // get our preferences branch object
-      // FIXME: Ugly hack stolen from https
-      var branch_name = "extensions.https_everywhere.";
-      var o_prefs = false;
-      var o_branch = false;
-      // this function needs to be called from inside https_everywhereLog, so
-      // it needs to do its own logging...
-      var econsole = Components.classes["@mozilla.org/consoleservice;1"]
-          .getService(Components.interfaces.nsIConsoleService);
+  get_prefs: function(prefBranch) {
+    if(!prefBranch) prefBranch = PREFBRANCH_ROOT;
 
-      o_prefs = Components.classes["@mozilla.org/preferences-service;1"]
-                          .getService(Components.interfaces.nsIPrefService);
+    // get our preferences branch object
+    // FIXME: Ugly hack stolen from https
+    var branch_name;
+    if(prefBranch == PREFBRANCH_RULE_TOGGLE)
+      branch_name = "extensions.https_everywhere.rule_toggle.";
+    else
+      branch_name = "extensions.https_everywhere.";
+    var o_prefs = false;
+    var o_branch = false;
+    // this function needs to be called from inside https_everywhereLog, so
+    // it needs to do its own logging...
+    var econsole = Components.classes["@mozilla.org/consoleservice;1"]
+      .getService(Components.interfaces.nsIConsoleService);
 
-      if (!o_prefs)
-      {
-          econsole.logStringMessage("HTTPS Everywhere: Failed to get preferences-service!");
-          return false;
-      }
+    o_prefs = Components.classes["@mozilla.org/preferences-service;1"]
+                        .getService(Components.interfaces.nsIPrefService);
 
-      o_branch = o_prefs.getBranch(branch_name);
-      if (!o_branch)
-      {
-          econsole.logStringMessage("HTTPS Everywhere: Failed to get prefs branch!");
-          return false;
-      }
+    if (!o_prefs)
+    {
+      econsole.logStringMessage("HTTPS Everywhere: Failed to get preferences-service!");
+      return false;
+    }
 
+    o_branch = o_prefs.getBranch(branch_name);
+    if (!o_branch)
+    {
+      econsole.logStringMessage("HTTPS Everywhere: Failed to get prefs branch!");
+      return false;
+    }
+
+    if(prefBranch == PREFBRANCH_ROOT) {
       // make sure there's an entry for our log level
       try {
         o_branch.getIntPref(LLVAR);
@@ -602,16 +640,13 @@ HTTPSEverywhere.prototype = {
         econsole.logStringMessage("Creating new about:config https_everywhere.LogLevel variable");
         o_branch.setIntPref(LLVAR, WARN);
       }
+    }
 
-      return o_branch;
+    return o_branch;
   },
 
   /**
-   * Notify observers of the topic OBSERVER_TOPIC_URI_REWRITE.  Intended for
-   * extension compatibility: 
-   *
-   * https://trac.torproject.org/projects/tor/ticket/1574
-   * https://trac.torproject.org/projects/tor/ticket/3190
+   * Notify observers of the topic OBSERVER_TOPIC_URI_REWRITE.
    *
    * @param nsIURI oldURI
    * @param string newSpec
@@ -638,64 +673,64 @@ HTTPSEverywhere.prototype = {
   },
 
   toggleEnabledState: function() {
-	if(this.prefs.getBoolPref("globalEnabled")){	
-		try{	
-			this.obsService.removeObserver(this, "profile-before-change");
-			this.obsService.removeObserver(this, "profile-after-change");
-			this.obsService.removeObserver(this, "sessionstore-windows-restored");		
-			OS.removeObserver(this, "cookie-changed");
-			OS.removeObserver(this, "http-on-modify-request");
-			OS.removeObserver(this, "http-on-examine-merged-response");
-			OS.removeObserver(this, "http-on-examine-response");  
-			
-			var catman = Components.classes["@mozilla.org/categorymanager;1"]
+    if(this.prefs.getBoolPref("globalEnabled")){    
+        try{    
+            this.obsService.removeObserver(this, "profile-before-change");
+            this.obsService.removeObserver(this, "profile-after-change");
+            this.obsService.removeObserver(this, "sessionstore-windows-restored");      
+            OS.removeObserver(this, "cookie-changed");
+            OS.removeObserver(this, "http-on-modify-request");
+            OS.removeObserver(this, "http-on-examine-merged-response");
+            OS.removeObserver(this, "http-on-examine-response");  
+            
+            var catman = Components.classes["@mozilla.org/categorymanager;1"]
            .getService(Components.interfaces.nsICategoryManager);
-			catman.deleteCategoryEntry("net-channel-event-sinks", SERVICE_CTRID, true);
-						
-			var dls = CC['@mozilla.org/docloaderservice;1']
-			.getService(CI.nsIWebProgress);
-			dls.removeProgressListener(this);
-			
-			this.prefs.setBoolPref("globalEnabled", false);
-		}
-		catch(e){
-			this.log(WARN, "Couldn't remove observers: " + e);			
-		}
-	}
-	else{	
-		try{	  
-			this.obsService.addObserver(this, "profile-before-change", false);
-			this.obsService.addObserver(this, "profile-after-change", false);
-			this.obsService.addObserver(this, "sessionstore-windows-restored", false);		
-			OS.addObserver(this, "cookie-changed", false);
-			OS.addObserver(this, "http-on-modify-request", false);
-			OS.addObserver(this, "http-on-examine-merged-response", false);
-			OS.addObserver(this, "http-on-examine-response", false);  
-			
-			var dls = CC['@mozilla.org/docloaderservice;1']
-			.getService(CI.nsIWebProgress);
-			dls.addProgressListener(this, CI.nsIWebProgress.NOTIFY_LOCATION);
-			
-			this.log(INFO,"ChannelReplacement.supported = "+ChannelReplacement.supported);
+            catman.deleteCategoryEntry("net-channel-event-sinks", SERVICE_CTRID, true);
+                        
+            var dls = CC['@mozilla.org/docloaderservice;1']
+            .getService(CI.nsIWebProgress);
+            dls.removeProgressListener(this);
+            
+            this.prefs.setBoolPref("globalEnabled", false);
+        }
+        catch(e){
+            this.log(WARN, "Couldn't remove observers: " + e);          
+        }
+    }
+    else{   
+        try{      
+            this.obsService.addObserver(this, "profile-before-change", false);
+            this.obsService.addObserver(this, "profile-after-change", false);
+            this.obsService.addObserver(this, "sessionstore-windows-restored", false);      
+            OS.addObserver(this, "cookie-changed", false);
+            OS.addObserver(this, "http-on-modify-request", false);
+            OS.addObserver(this, "http-on-examine-merged-response", false);
+            OS.addObserver(this, "http-on-examine-response", false);  
+            
+            var dls = CC['@mozilla.org/docloaderservice;1']
+            .getService(CI.nsIWebProgress);
+            dls.addProgressListener(this, CI.nsIWebProgress.NOTIFY_LOCATION);
+            
+            this.log(INFO,"ChannelReplacement.supported = "+ChannelReplacement.supported);
 
-			HTTPSRules.init();
+            HTTPSRules.init();
 
-			if(!Thread.hostRunning)
-				Thread.hostRunning = true;
-			
-			var catman = Components.classes["@mozilla.org/categorymanager;1"]
-			.getService(Components.interfaces.nsICategoryManager);
-			// hook on redirections (non persistent, otherwise crashes on 1.8.x)
-			catman.addCategoryEntry("net-channel-event-sinks", SERVICE_CTRID,
-				SERVICE_CTRID, false, true);			
-			
-			HTTPSRules.init();			
-			this.prefs.setBoolPref("globalEnabled", true);
-		}
-		catch(e){
-			this.log(WARN, "Couldn't add observers: " + e);			
-		}
-	}
+            if(!Thread.hostRunning)
+                Thread.hostRunning = true;
+            
+            var catman = Components.classes["@mozilla.org/categorymanager;1"]
+            .getService(Components.interfaces.nsICategoryManager);
+            // hook on redirections (non persistent, otherwise crashes on 1.8.x)
+            catman.addCategoryEntry("net-channel-event-sinks", SERVICE_CTRID,
+                SERVICE_CTRID, false, true);            
+            
+            HTTPSRules.init();          
+            this.prefs.setBoolPref("globalEnabled", true);
+        }
+        catch(e){
+            this.log(WARN, "Couldn't add observers: " + e);         
+        }
+    }
   }
 };
 
@@ -727,3 +762,5 @@ if (XPCOMUtils.generateNSGetFactory)
     var NSGetFactory = XPCOMUtils.generateNSGetFactory([HTTPSEverywhere]);
 else
     var NSGetModule = XPCOMUtils.generateNSGetModule([HTTPSEverywhere]);
+
+/* vim: set tabstop=4 expandtab: */
