@@ -20,6 +20,8 @@ PlacesViewBase.prototype = {
   _viewElt: null,
   get viewElt() this._viewElt,
 
+  get associatedElement() this._viewElt,
+
   get controllers() this._viewElt.controllers,
 
   // The xul element that represents the root container.
@@ -68,18 +70,37 @@ PlacesViewBase.prototype = {
     if (val) {
       this._resultNode = val.root;
       this._rootElt._placesNode = this._resultNode;
-      this._resultNode._DOMElement = this._rootElt;
+      this._domNodes = new Map();
+      this._domNodes.set(this._resultNode, this._rootElt);
 
       // This calls _rebuild through invalidateContainer.
       this._resultNode.containerOpen = true;
     }
     else {
       this._resultNode = null;
+      delete this._domNodes;
     }
 
     return val;
   },
 
+  /**
+   * Gets the DOM node used for the given places node.
+   *
+   * @param aPlacesNode
+   *        a places result node.
+   * @throws if there is no DOM node set for aPlacesNode.
+   */
+  _getDOMNodeForPlacesNode:
+  function PVB__getDOMNodeForPlacesNode(aPlacesNode) {
+    let node = this._domNodes.get(aPlacesNode, null);
+    if (!node) {
+      throw new Error("No DOM node set for aPlacesNode.\nnode.type: " +
+                      aPlacesNode.type + ". node.parent: " + aPlacesNode);
+    }
+    return node;
+  },
+
   get controller() this._controller,
 
   get selType() "single",
@@ -162,8 +183,6 @@ PlacesViewBase.prototype = {
 
   destroyContextMenu: function PVB_destroyContextMenu(aPopup) {
     this._contextMenuShown = false;
-    if (window.content)
-      window.content.focus();
   },
 
   _cleanPopup: function PVB_cleanPopup(aPopup, aDelay) {
@@ -194,7 +213,7 @@ PlacesViewBase.prototype = {
     if (!resultNode.containerOpen)
       return;
 
-    if (resultNode._feedURI) {
+    if (this.controller.hasCachedLivemarkInfo(resultNode)) {
       this._setEmptyPopupStatus(aPopup, false);
       aPopup._built = true;
       this._populateLivemarkPopup(aPopup);
@@ -254,7 +273,8 @@ PlacesViewBase.prototype = {
 
   _createMenuItemForPlacesNode:
   function PVB__createMenuItemForPlacesNode(aPlacesNode) {
-    delete aPlacesNode._DOMElement;
+    this._domNodes.delete(aPlacesNode);
+
     let element;
     let type = aPlacesNode.type;
     if (type == Ci.nsINavHistoryResultNode.RESULT_TYPE_SEPARATOR) {
@@ -262,7 +282,7 @@ PlacesViewBase.prototype = {
     }
     else {
       let itemId = aPlacesNode.itemId;
-      if (PlacesUtils.uriTypes.indexOf(type) != -1) {
+      if (type == Ci.nsINavHistoryResultNode.RESULT_TYPE_URI) {
         element = document.createElement("menuitem");
         element.className = "menuitem-iconic bookmark-item menuitem-with-favicon";
         element.setAttribute("scheme",
@@ -293,12 +313,9 @@ PlacesViewBase.prototype = {
                 element.setAttribute("image", "");
                 element.removeAttribute("image");
 #endif
-                // Set an expando on the node, controller will use it to build
-                // its metadata.
-                aPlacesNode._feedURI = aLivemark.feedURI;
-                aPlacesNode._siteURI = aLivemark.siteURI;
+                this.controller.cacheLivemarkInfo(aPlacesNode, aLivemark);
               }
-            }
+            }.bind(this)
           );
         }
 
@@ -316,7 +333,7 @@ PlacesViewBase.prototype = {
         element.appendChild(popup);
         element.className = "menu-iconic bookmark-item";
 
-        aPlacesNode._DOMElement = popup;
+        this._domNodes.set(aPlacesNode, popup);
       }
       else
         throw "Unexpected node";
@@ -329,8 +346,8 @@ PlacesViewBase.prototype = {
     }
 
     element._placesNode = aPlacesNode;
-    if (!aPlacesNode._DOMElement)
-      aPlacesNode._DOMElement = element;
+    if (!this._domNodes.has(aPlacesNode))
+      this._domNodes.set(aPlacesNode, element);
 
     return element;
   },
@@ -345,8 +362,9 @@ PlacesViewBase.prototype = {
 
   _setLivemarkSiteURIMenuItem:
   function PVB__setLivemarkSiteURIMenuItem(aPopup) {
-    let siteUrl = aPopup._placesNode._siteURI ? aPopup._placesNode._siteURI.spec
-                                              : null;
+    let livemarkInfo = this.controller.getCachedLivemarkInfo(aPopup._placesNode);
+    let siteUrl = livemarkInfo && livemarkInfo.siteURI ?
+                  livemarkInfo.siteURI.spec : null;
     if (!siteUrl && aPopup._siteURIMenuitem) {
       aPopup.removeChild(aPopup._siteURIMenuitem);
       aPopup._siteURIMenuitem = null;
@@ -412,23 +430,20 @@ PlacesViewBase.prototype = {
     }
   },
 
-  toggleCutNode: function PVB_toggleCutNode(aNode, aValue) {
-    let elt = aNode._DOMElement;
-    if (elt) {
-      // We may get the popup for menus, but we need the menu itself.
-      if (elt.localName == "menupopup")
-        elt = elt.parentNode;
-      if (aValue)
-        elt.setAttribute("cutting", "true");
-      else
-        elt.removeAttribute("cutting");
-    }
+  toggleCutNode: function PVB_toggleCutNode(aPlacesNode, aValue) {
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
+
+    // We may get the popup for menus, but we need the menu itself.
+    if (elt.localName == "menupopup")
+      elt = elt.parentNode;
+    if (aValue)
+      elt.setAttribute("cutting", "true");
+    else
+      elt.removeAttribute("cutting");
   },
 
   nodeURIChanged: function PVB_nodeURIChanged(aPlacesNode, aURIString) {
-    let elt = aPlacesNode._DOMElement;
-    if (!elt)
-      throw "aPlacesNode must have _DOMElement set";
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
 
     // Here we need the <menu>.
     if (elt.localName == "menupopup")
@@ -438,9 +453,7 @@ PlacesViewBase.prototype = {
   },
 
   nodeIconChanged: function PVB_nodeIconChanged(aPlacesNode) {
-    let elt = aPlacesNode._DOMElement;
-    if (!elt)
-      throw "aPlacesNode must have _DOMElement set";
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
 
     // There's no UI representation for the root node, thus there's nothing to
     // be done when the icon changes.
@@ -460,9 +473,7 @@ PlacesViewBase.prototype = {
 
   nodeAnnotationChanged:
   function PVB_nodeAnnotationChanged(aPlacesNode, aAnno) {
-    let elt = aPlacesNode._DOMElement;
-    if (!elt)
-      throw "aPlacesNode must have _DOMElement set";
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
 
     // All livemarks have a feedURI, so use it as our indicator of a livemark
     // being modified.
@@ -480,24 +491,20 @@ PlacesViewBase.prototype = {
 
       PlacesUtils.livemarks.getLivemark(
         { id: aPlacesNode.itemId },
-        (function (aStatus, aLivemark) {
+        function (aStatus, aLivemark) {
           if (Components.isSuccessCode(aStatus)) {
-            // Set an expando on the node, controller will use it to build
-            // its metadata.
-            aPlacesNode._feedURI = aLivemark.feedURI;
-            aPlacesNode._siteURI = aLivemark.siteURI;
+            // Controller will use this to build the meta data for the node.
+            this.controller.cacheLivemarkInfo(aPlacesNode, aLivemark);
             this.invalidateContainer(aPlacesNode);
           }
-        }).bind(this)
+        }.bind(this)
       );
     }
   },
 
   nodeTitleChanged:
   function PVB_nodeTitleChanged(aPlacesNode, aNewTitle) {
-    let elt = aPlacesNode._DOMElement;
-    if (!elt)
-      throw "aPlacesNode must have _DOMElement set";
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
 
     // There's no UI representation for the root node, thus there's
     // nothing to be done when the title changes.
@@ -521,13 +528,8 @@ PlacesViewBase.prototype = {
 
   nodeRemoved:
   function PVB_nodeRemoved(aParentPlacesNode, aPlacesNode, aIndex) {
-    let parentElt = aParentPlacesNode._DOMElement;
-    let elt = aPlacesNode._DOMElement;
-
-    if (!parentElt)
-      throw "aParentPlacesNode must have _DOMElement set";
-    if (!elt)
-      throw "aPlacesNode must have _DOMElement set";
+    let parentElt = this._getDOMNodeForPlacesNode(aParentPlacesNode);
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
 
     // Here we need the <menu>.
     if (elt.localName == "menupopup")
@@ -544,36 +546,12 @@ PlacesViewBase.prototype = {
     }
   },
 
-  nodeReplaced:
-  function PVB_nodeReplaced(aParentPlacesNode, aOldPlacesNode, aNewPlacesNode, aIndex) {
-    let parentElt = aParentPlacesNode._DOMElement;
-    if (!parentElt)
-      throw "aParentPlacesNode node must have _DOMElement set";
-
-    if (parentElt._built) {
-      let elt = aOldPlacesNode._DOMElement;
-      if (!elt)
-        throw "aOldPlacesNode must have _DOMElement set";
-
-      // Here we need the <menu>.
-      if (elt.localName == "menupopup")
-        elt = elt.parentNode;
-
-      parentElt.removeChild(elt);
-
-      // No worries: If elt is the last item (i.e. no nextSibling),
-      // _insertNewItem/_insertNewItemToPopup will insert the new element as
-      // the last item.
-      let nextElt = elt.nextSibling;
-      this._insertNewItemToPopup(aNewPlacesNode, parentElt, nextElt);
-    }
-  },
-
   nodeHistoryDetailsChanged:
   function PVB_nodeHistoryDetailsChanged(aPlacesNode, aTime, aCount) {
-    if (aPlacesNode.parent && aPlacesNode.parent._feedURI) {
+    if (aPlacesNode.parent &&
+        this.controller.hasCachedLivemarkInfo(aPlacesNode.parent)) {
       // Find the node in the parent.
-      let popup = aPlacesNode.parent._DOMElement;
+      let popup = this._getDOMNodeForPlacesNode(aPlacesNode.parent);
       for (let child = popup._startMarker.nextSibling;
            child != popup._endMarker;
            child = child.nextSibling) {
@@ -597,10 +575,7 @@ PlacesViewBase.prototype = {
 
   nodeInserted:
   function PVB_nodeInserted(aParentPlacesNode, aPlacesNode, aIndex) {
-    let parentElt = aParentPlacesNode._DOMElement;
-    if (!parentElt)
-      throw "aParentPlacesNode node must have _DOMElement set";
-
+    let parentElt = this._getDOMNodeForPlacesNode(aParentPlacesNode);
     if (!parentElt._built)
       return;
 
@@ -619,9 +594,7 @@ PlacesViewBase.prototype = {
     // use this notification when the item in question is moved from one
     // folder to another.  Instead, it calls nodeRemoved and nodeInserted
     // for the two folders.  Thus, we can assume old-parent == new-parent.
-    let elt = aPlacesNode._DOMElement;
-    if (!elt)
-      throw "aPlacesNode must have _DOMElement set";
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
 
     // Here we need the <menu>.
     if (elt.localName == "menupopup")
@@ -632,10 +605,7 @@ PlacesViewBase.prototype = {
     if (elt == this._rootElt)
       return;
 
-    let parentElt = aNewParentPlacesNode._DOMElement;
-    if (!parentElt)
-      throw "aNewParentPlacesNode node must have _DOMElement set";
-
+    let parentElt = this._getDOMNodeForPlacesNode(aNewParentPlacesNode);
     if (parentElt._built) {
       // Move the node.
       parentElt.removeChild(elt);
@@ -658,11 +628,11 @@ PlacesViewBase.prototype = {
         }
 
         PlacesUtils.livemarks.getLivemark({ id: aPlacesNode.itemId },
-          (function (aStatus, aLivemark) {
+          function (aStatus, aLivemark) {
             if (Components.isSuccessCode(aStatus)) {
-              let shouldInvalidate = !aPlacesNode._feedURI;
-              aPlacesNode._feedURI = aLivemark.feedURI;
-              aPlacesNode._siteURI = aLivemark.siteURI;
+              let shouldInvalidate =
+                !this.controller.hasCachedLivemarkInfo(aPlacesNode);
+              this.controller.cacheLivemarkInfo(aPlacesNode, aLivemark);
               if (aNewState == Ci.nsINavHistoryContainerResultNode.STATE_OPENED) {
                 aLivemark.registerForUpdates(aPlacesNode, this);
                 // Prioritize the current livemark.
@@ -675,7 +645,7 @@ PlacesViewBase.prototype = {
                 aLivemark.unregisterForUpdates(aPlacesNode);
               }
             }
-          }).bind(this)
+          }.bind(this)
         );
       }
     }
@@ -689,7 +659,7 @@ PlacesViewBase.prototype = {
       this._setLivemarkStatusMenuItem(aPopup, Ci.mozILivemark.STATUS_LOADING);
 
     PlacesUtils.livemarks.getLivemark({ id: aPopup._placesNode.itemId },
-      (function (aStatus, aLivemark) {
+      function (aStatus, aLivemark) {
         let placesNode = aPopup._placesNode;
         if (!Components.isSuccessCode(aStatus) || !placesNode.containerOpen)
           return;
@@ -704,19 +674,16 @@ PlacesViewBase.prototype = {
           let child = children[i];
           this.nodeInserted(placesNode, child, i);
           if (child.accessCount)
-            child._DOMElement.setAttribute("visited", true);
+            this._getDOMNodeForPlacesNode(child).setAttribute("visited", true);
           else
-            child._DOMElement.removeAttribute("visited");
+            this._getDOMNodeForPlacesNode(child).removeAttribute("visited");
         }
-      }).bind(this)
+      }.bind(this)
     );
   },
 
   invalidateContainer: function PVB_invalidateContainer(aPlacesNode) {
-    let elt = aPlacesNode._DOMElement;
-    if (!elt)
-      throw "aPlacesNode must have _DOMElement set";
-
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
     elt._built = false;
 
     // If the menupopup is open we should live-update it.
@@ -981,7 +948,8 @@ PlacesToolbar.prototype = {
 
   _insertNewItem:
   function PT__insertNewItem(aChild, aBefore) {
-    delete aChild._DOMElement;
+    this._domNodes.delete(aChild);
+
     let type = aChild.type;
     let button;
     if (type == Ci.nsINavHistoryResultNode.RESULT_TYPE_SEPARATOR) {
@@ -1010,12 +978,9 @@ PlacesToolbar.prototype = {
             function (aStatus, aLivemark) {
               if (Components.isSuccessCode(aStatus)) {
                 button.setAttribute("livemark", "true");
-                // Set an expando on the node, controller will use it to build
-                // its metadata.
-                aChild._feedURI = aLivemark.feedURI;
-                aChild._siteURI = aLivemark.siteURI;
+                this.controller.cacheLivemarkInfo(aChild, aLivemark);
               }
-            }
+            }.bind(this)
           );
         }
 
@@ -1027,7 +992,7 @@ PlacesToolbar.prototype = {
         popup.setAttribute("context", "placesContext");
 #endif
 
-        aChild._DOMElement = popup;
+        this._domNodes.set(aChild, popup);
       }
       else if (PlacesUtils.nodeIsURI(aChild)) {
         button.setAttribute("scheme",
@@ -1036,8 +1001,8 @@ PlacesToolbar.prototype = {
     }
 
     button._placesNode = aChild;
-    if (!aChild._DOMElement)
-      aChild._DOMElement = button;
+    if (!this._domNodes.has(aChild))
+      this._domNodes.set(aChild, button);
 
     if (aBefore)
       this._rootElt.insertBefore(button, aBefore);
@@ -1180,10 +1145,7 @@ PlacesToolbar.prototype = {
 
   nodeInserted:
   function PT_nodeInserted(aParentPlacesNode, aPlacesNode, aIndex) {
-    let parentElt = aParentPlacesNode._DOMElement;
-    if (!parentElt) 
-      throw "aParentPlacesNode node must have _DOMElement set";
-
+    let parentElt = this._getDOMNodeForPlacesNode(aParentPlacesNode);
     if (parentElt == this._rootElt) {
       let children = this._rootElt.childNodes;
       this._insertNewItem(aPlacesNode,
@@ -1197,13 +1159,8 @@ PlacesToolbar.prototype = {
 
   nodeRemoved:
   function PT_nodeRemoved(aParentPlacesNode, aPlacesNode, aIndex) {
-    let parentElt = aParentPlacesNode._DOMElement;
-    let elt = aPlacesNode._DOMElement;
-
-    if (!parentElt)
-      throw "aParentPlacesNode node must have _DOMElement set";
-    if (!elt)
-      throw "aPlacesNode must have _DOMElement set";
+    let parentElt = this._getDOMNodeForPlacesNode(aParentPlacesNode);
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
 
     // Here we need the <menu>.
     if (elt.localName == "menupopup")
@@ -1222,17 +1179,12 @@ PlacesToolbar.prototype = {
   function PT_nodeMoved(aPlacesNode,
                         aOldParentPlacesNode, aOldIndex,
                         aNewParentPlacesNode, aNewIndex) {
-    let parentElt = aNewParentPlacesNode._DOMElement;
-    if (!parentElt) 
-      throw "aNewParentPlacesNode node must have _DOMElement set";
-
+    let parentElt = this._getDOMNodeForPlacesNode(aNewParentPlacesNode);
     if (parentElt == this._rootElt) {
       // Container is on the toolbar.
 
       // Move the element.
-      let elt = aPlacesNode._DOMElement;
-      if (!elt)
-        throw "aPlacesNode must have _DOMElement set";
+      let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
 
       // Here we need the <menu>.
       if (elt.localName == "menupopup")
@@ -1241,14 +1193,13 @@ PlacesToolbar.prototype = {
       this._removeChild(elt);
       this._rootElt.insertBefore(elt, this._rootElt.childNodes[aNewIndex]);
 
-      // If the chevron popup is open, keep it in sync.
-      if (this._chevron.open) {
-        let chevronPopup = this._chevronPopup;
-        let menuitem = chevronPopup.childNodes[aOldIndex];
-        chevronPopup.removeChild(menuitem);
-        chevronPopup.insertBefore(menuitem,
-                                  chevronPopup.childNodes[aNewIndex]);
-      }
+      // The chevron view may get nodeMoved after the toolbar.  In such a case,
+      // we should ensure (by manually swapping menuitems) that the actual nodes
+      // are in the final position before updateChevron tries to updates their
+      // visibility, or the chevron may go out of sync.
+      // Luckily updateChevron runs on a timer, so, by the time it updates
+      // nodes, the menu has already handled the notification.
+
       this.updateChevron();
       return;
     }
@@ -1258,10 +1209,7 @@ PlacesToolbar.prototype = {
 
   nodeAnnotationChanged:
   function PT_nodeAnnotationChanged(aPlacesNode, aAnno) {
-    let elt = aPlacesNode._DOMElement;
-    if (!elt)
-      throw "aPlacesNode must have _DOMElement set";
-
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
     if (elt == this._rootElt)
       return;
 
@@ -1278,15 +1226,12 @@ PlacesToolbar.prototype = {
 
         PlacesUtils.livemarks.getLivemark(
           { id: aPlacesNode.itemId },
-          (function (aStatus, aLivemark) {
+          function (aStatus, aLivemark) {
             if (Components.isSuccessCode(aStatus)) {
-              // Set an expando on the node, controller will use it to build
-              // its metadata.
-              aPlacesNode._feedURI = aLivemark.feedURI;
-              aPlacesNode._siteURI = aLivemark.siteURI;
+              this.controller.cacheLivemarkInfo(aPlacesNode, aLivemark);
               this.invalidateContainer(aPlacesNode);
             }
-          }).bind(this)
+          }.bind(this)
         );
       }
     }
@@ -1297,9 +1242,7 @@ PlacesToolbar.prototype = {
   },
 
   nodeTitleChanged: function PT_nodeTitleChanged(aPlacesNode, aNewTitle) {
-    let elt = aPlacesNode._DOMElement;
-    if (!elt)
-      throw "aPlacesNode must have _DOMElement set";
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
 
     // There's no UI representation for the root node, thus there's
     // nothing to be done when the title changes.
@@ -1318,41 +1261,8 @@ PlacesToolbar.prototype = {
     }
   },
 
-  nodeReplaced:
-  function PT_nodeReplaced(aParentPlacesNode,
-                           aOldPlacesNode, aNewPlacesNode, aIndex) {
-    let parentElt = aParentPlacesNode._DOMElement;
-    if (!parentElt) 
-      throw "aParentPlacesNode node must have _DOMElement set";
-
-    if (parentElt == this._rootElt) {
-      let elt = aOldPlacesNode._DOMElement;
-      if (!elt)
-        throw "aOldPlacesNode must have _DOMElement set";
-
-      // Here we need the <menu>.
-      if (elt.localName == "menupopup")
-        elt = elt.parentNode;
-
-      this._removeChild(elt);
-
-      // No worries: If elt is the last item (i.e. no nextSibling),
-      // _insertNewItem/_insertNewItemToPopup will insert the new element as
-      // the last item.
-      let next = elt.nextSibling;
-      this._insertNewItem(aNewPlacesNode, next);
-      this.updateChevron();
-      return;
-    }
-
-    PlacesViewBase.prototype.nodeReplaced.apply(this, arguments);
-  },
-
   invalidateContainer: function PT_invalidateContainer(aPlacesNode) {
-    let elt = aPlacesNode._DOMElement;
-    if (!elt)
-      throw "aPlacesNode must have _DOMElement set";
-
+    let elt = this._getDOMNodeForPlacesNode(aPlacesNode);
     if (elt == this._rootElt) {
       // Container is the toolbar itself.
       this._rebuild();
@@ -1649,7 +1559,7 @@ PlacesToolbar.prototype = {
         }
       }
 
-      ind.style.MozTransform = "translate(" + Math.round(translateX) + "px)";
+      ind.style.transform = "translate(" + Math.round(translateX) + "px)";
       ind.style.MozMarginStart = (-ind.clientWidth) + "px";
       ind.collapsed = false;
 
@@ -1716,7 +1626,8 @@ PlacesToolbar.prototype = {
       // so we don't rebuild its contents whenever the popup is reopened.
       // Though, we want to always close feed containers so their expiration
       // status will be checked at next opening.
-      if (!PlacesUtils.nodeIsFolder(placesNode) || placesNode._feedURI) {
+      if (!PlacesUtils.nodeIsFolder(placesNode) ||
+          this.controller.hasCachedLivemarkInfo(placesNode)) {
         placesNode.containerOpen = false;
       }
     }
@@ -1822,7 +1733,8 @@ PlacesMenu.prototype = {
     // so we don't rebuild its contents whenever the popup is reopened.
     // Though, we want to always close feed containers so their expiration
     // status will be checked at next opening.
-    if (!PlacesUtils.nodeIsFolder(placesNode) || placesNode._feedURI)
+    if (!PlacesUtils.nodeIsFolder(placesNode) ||
+        this.controller.hasCachedLivemarkInfo(placesNode))
       placesNode.containerOpen = false;
 
     // The autoopened attribute is set for folders which have been
