@@ -1,40 +1,8 @@
 /* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim:set ts=2 sw=2 sts=2 et: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Places Unit Test code.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *  Marco Bonardo <mak77@bonardo.net>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /**
  * Tests that requesting clear history at shutdown will really clear history.
@@ -62,6 +30,12 @@ const UNEXPECTED_NOTIFICATIONS = [
 
 const URL = "ftp://localhost/clearHistoryOnShutdown/";
 
+// Send the profile-after-change notification to the form history component to ensure
+// that it has been initialized.
+var formHistoryStartup = Cc["@mozilla.org/satchel/form-history-startup;1"].
+                         getService(Ci.nsIObserver);
+formHistoryStartup.observe(null, "profile-after-change", null);
+
 let notificationIndex = 0;
 
 let notificationsObserver = {
@@ -98,15 +72,17 @@ let notificationsObserver = {
     }
 
     // Check cache.
-    do_check_false(cacheExists(URL));
-
-    do_test_finished();
+    checkCache(URL);
   }
 }
 
 let timeInMicroseconds = Date.now() * 1000;
 
 function run_test() {
+  run_next_test();
+}
+
+add_task(function test_execute() {
   do_test_pending();
 
   print("Initialize browserglue before Places");
@@ -130,14 +106,16 @@ function run_test() {
   Services.prefs.setBoolPref("privacy.sanitize.sanitizeOnShutdown", true);
 
   print("Add visits.");
-  URIS.forEach(function(aUrl) {
-    PlacesUtils.history.addVisit(uri(aUrl), timeInMicroseconds++, null,
-                                 PlacesUtils.history.TRANSITION_TYPED,
-                                 false, 0);
-  });
+  for (let aUrl of URIS) {
+    yield promiseAddVisits({uri: uri(aUrl), visitDate: timeInMicroseconds++,
+                            transition: PlacesUtils.history.TRANSITION_TYPED})
+  }
   print("Add cache.");
   storeCache(URL, "testData");
+});
 
+function run_test_continue()
+{
   print("Simulate and wait shutdown.");
   getDistinctNotifications().forEach(
     function (topic)
@@ -160,38 +138,46 @@ function storeCache(aURL, aContent) {
               getService(Ci.nsICacheService);
   let session = cache.createSession("FTP", Ci.nsICache.STORE_ANYWHERE,
                                     Ci.nsICache.STREAM_BASED);
-  let cacheEntry =
-    session.openCacheEntry(aURL, Ci.nsICache.ACCESS_READ_WRITE, false);
 
-  cacheEntry.setMetaDataElement("servertype", "0");
-  var oStream = cacheEntry.openOutputStream(0);
+  var storeCacheListener = {
+    onCacheEntryAvailable: function (entry, access, status) {
+      do_check_eq(status, Cr.NS_OK);
 
-  var written = oStream.write(aContent, aContent.length);
-  if (written != aContent.length) {
-    do_throw("oStream.write has not written all data!\n" +
-             "  Expected: " + written  + "\n" +
-             "  Actual: " + aContent.length + "\n");
-  }
-  oStream.close();
-  cacheEntry.close();
+      entry.setMetaDataElement("servertype", "0");
+      var os = entry.openOutputStream(0);
+
+      var written = os.write(aContent, aContent.length);
+      if (written != aContent.length) {
+        do_throw("os.write has not written all data!\n" +
+                 "  Expected: " + written  + "\n" +
+                 "  Actual: " + aContent.length + "\n");
+      }
+      os.close();
+      entry.close();
+      do_execute_soon(run_test_continue);
+    }
+  };
+
+  session.asyncOpenCacheEntry(aURL,
+                              Ci.nsICache.ACCESS_READ_WRITE,
+                              storeCacheListener);
 }
 
-function cacheExists(aURL) {
+
+function checkCache(aURL) {
   let cache = Cc["@mozilla.org/network/cache-service;1"].
               getService(Ci.nsICacheService);
   let session = cache.createSession("FTP", Ci.nsICache.STORE_ANYWHERE,
                                     Ci.nsICache.STREAM_BASED);
-  try {
-    let cacheEntry =
-      session.openCacheEntry(aURL, Ci.nsICache.ACCESS_READ, true);
-  } catch (e) {
-    if (e.result == Cr.NS_ERROR_CACHE_KEY_NOT_FOUND ||
-        e.result == Cr.NS_ERROR_FAILURE)
-      return false;
- 
-    // Throw the textual error description.
-    do_throw(e);
-  }
-  cacheEntry.close();
-  return true;
+
+  var checkCacheListener = {
+    onCacheEntryAvailable: function (entry, access, status) {
+      do_check_eq(status, Cr.NS_ERROR_CACHE_KEY_NOT_FOUND);
+      do_test_finished();
+    }
+  };
+
+  session.asyncOpenCacheEntry(aURL,
+                              Ci.nsICache.ACCESS_READ,
+                              checkCacheListener);
 }
