@@ -13,12 +13,8 @@ let HUD;
 
 let outputItem;
 
-function tabLoad1(aEvent) {
-  browser.removeEventListener(aEvent.type, arguments.callee, true);
-
-  openConsole();
-
-  HUD = HUDService.getHudByWindow(content);
+function consoleOpened(aHud) {
+  HUD = aHud;
 
   outputNode = HUD.outputNode;
 
@@ -26,23 +22,34 @@ function tabLoad1(aEvent) {
 
   // Reload so we get some output in the console.
   browser.contentWindow.location.reload();
-  log(document);
 }
 
 function tabLoad2(aEvent) {
-  browser.removeEventListener(aEvent.type, arguments.callee, true);
+  browser.removeEventListener(aEvent.type, tabLoad2, true);
 
-  outputItem = outputNode.querySelector(".hud-networkinfo .hud-clickable");
-  ok(outputItem, "found a network message");
-  document.addEventListener("popupshown", networkPanelShown, false);
+  waitForMessages({
+    webconsole: HUD,
+    messages: [{
+      text: "test-console.html",
+      category: CATEGORY_NETWORK,
+      severity: SEVERITY_LOG,
+    }],
+  }).then(([result]) => {
+    let msg = [...result.matched][0];
+    outputItem = msg.querySelector(".hud-clickable");
+    ok(outputItem, "found a network message");
+    document.addEventListener("popupshown", networkPanelShown, false);
 
-  // Send the mousedown and click events such that the network panel opens.
-  EventUtils.sendMouseEvent({type: "mousedown"}, outputItem);
-  EventUtils.sendMouseEvent({type: "click"}, outputItem);
+    // Send the mousedown and click events such that the network panel opens.
+    EventUtils.sendMouseEvent({type: "mousedown"}, outputItem);
+    EventUtils.sendMouseEvent({type: "click"}, outputItem);
+  });
 }
 
 function networkPanelShown(aEvent) {
-  document.removeEventListener(aEvent.type, arguments.callee, false);
+  document.removeEventListener(aEvent.type, networkPanelShown, false);
+
+  info("networkPanelShown");
 
   document.addEventListener("popupshown", networkPanelShowFailure, false);
 
@@ -57,13 +64,15 @@ function networkPanelShown(aEvent) {
 }
 
 function networkPanelShowFailure(aEvent) {
-  document.removeEventListener(aEvent.type, arguments.callee, false);
+  document.removeEventListener(aEvent.type, networkPanelShowFailure, false);
 
   ok(false, "the network panel should not show");
 }
 
 function networkPanelHidden(aEvent) {
-  this.removeEventListener(aEvent.type, arguments.callee, false);
+  this.removeEventListener(aEvent.type, networkPanelHidden, false);
+
+  info("networkPanelHidden");
 
   // The network panel should not show because this is a mouse event that starts
   // in a position and ends in another.
@@ -89,78 +98,35 @@ function networkPanelHidden(aEvent) {
 
     // Done with the network output. Now test the jsterm output and the property
     // panel.
-    HUD.jsterm.setInputValue("document");
-    HUD.jsterm.execute();
+    HUD.jsterm.execute("document", () => {
+      info("jsterm execute 'document' callback");
 
-    outputItem = outputNode.querySelector(".webconsole-msg-output " +
-                                          ".hud-clickable");
-    ok(outputItem, "found a jsterm output message");
+      HUD.jsterm.once("variablesview-open", onVariablesViewOpen);
+      let outputItem = outputNode
+                       .querySelector(".webconsole-msg-output .hud-clickable");
+      ok(outputItem, "jsterm output message found");
 
-    document.addEventListener("popupshown", properyPanelShown, false);
-
-    // Send the mousedown and click events such that the property panel opens.
-    EventUtils.sendMouseEvent({type: "mousedown"}, outputItem);
-    EventUtils.sendMouseEvent({type: "click"}, outputItem);
+      // Send the mousedown and click events such that the property panel opens.
+      EventUtils.sendMouseEvent({type: "mousedown"}, outputItem);
+      EventUtils.sendMouseEvent({type: "click"}, outputItem);
+    });
   });
 }
 
-function properyPanelShown(aEvent) {
-  document.removeEventListener(aEvent.type, arguments.callee, false);
-
-  document.addEventListener("popupshown", propertyPanelShowFailure, false);
-
-  // The property panel should not open for the second time.
-  EventUtils.sendMouseEvent({type: "mousedown"}, outputItem);
-  EventUtils.sendMouseEvent({type: "click"}, outputItem);
+function onVariablesViewOpen() {
+  info("onVariablesViewOpen");
 
   executeSoon(function() {
-    aEvent.target.addEventListener("popuphidden", propertyPanelHidden, false);
-    aEvent.target.hidePopup();
+    HUD = outputItem = null;
+    executeSoon(finishTest);
   });
 }
 
-function propertyPanelShowFailure(aEvent) {
-  document.removeEventListener(aEvent.type, arguments.callee, false);
-
-  ok(false, "the property panel should not show");
-}
-
-function propertyPanelHidden(aEvent) {
-  this.removeEventListener(aEvent.type, arguments.callee, false);
-
-  // The property panel should not show because this is a mouse event that
-  // starts in a position and ends in another.
-  EventUtils.sendMouseEvent({type: "mousedown", clientX: 3, clientY: 4},
-    outputItem);
-  EventUtils.sendMouseEvent({type: "click", clientX: 5, clientY: 6},
-    outputItem);
-
-  // The property panel should not show because this is a middle-click.
-  EventUtils.sendMouseEvent({type: "mousedown", button: 1},
-    outputItem);
-  EventUtils.sendMouseEvent({type: "click", button: 1},
-    outputItem);
-
-  // The property panel should not show because this is a right-click.
-  EventUtils.sendMouseEvent({type: "mousedown", button: 2},
-    outputItem);
-  EventUtils.sendMouseEvent({type: "click", button: 2},
-    outputItem);
-
-  executeSoon(function() {
-    document.removeEventListener("popupshown", propertyPanelShowFailure, false);
-    outputItem = null;
-    finishTest();
-  });
-}
-
-registerCleanupFunction(function() {
-  Services.prefs.clearUserPref("devtools.gcli.enable");
-});
-
 function test() {
-  Services.prefs.setBoolPref("devtools.gcli.enable", false);
   addTab(TEST_URI);
-  browser.addEventListener("load", tabLoad1, true);
+  browser.addEventListener("load", function onLoad() {
+    browser.removeEventListener("load", onLoad, true);
+    openConsole(null, consoleOpened);
+  }, true);
 }
 
