@@ -21,7 +21,7 @@
 namespace mozilla {
 namespace dom {
 
-class ImageData MOZ_FINAL : public nsIDOMImageData
+class ImageData MOZ_FINAL : public nsISupports
 {
 public:
   ImageData(uint32_t aWidth, uint32_t aHeight, JSObject& aData)
@@ -40,23 +40,28 @@ public:
   }
 
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_NSIDOMIMAGEDATA
   NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(ImageData)
 
-  uint32_t GetWidth()
+  uint32_t Width() const
   {
     return mWidth;
   }
-  uint32_t GetHeight()
+  uint32_t Height() const
   {
     return mHeight;
   }
-  JSObject* GetDataObject()
+  JSObject* Data(JSContext* cx) const
+  {
+    return GetDataObject();
+  }
+  JSObject* GetDataObject() const
   {
     xpc_UnmarkGrayObject(mData);
     return mData;
   }
 
+  JSObject* WrapObject(JSContext* cx, JS::Handle<JSObject*> scope);
+
 private:
   void HoldData();
   void DropData();
@@ -64,7 +69,7 @@ private:
   ImageData() MOZ_DELETE;
 
   uint32_t mWidth, mHeight;
-  JSObject* mData;
+  JS::Heap<JSObject*> mData;
 };
 
 } // namespace dom
