@@ -1,44 +1,8 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim: set sw=2 ts=2 et tw=78: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   Peter Annema <disttsc@bart.nl>
- *   Daniel Glazman <glazman@netscape.com>
- *   Henri Sivonen <hsivonen@iki.fi>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /**
  * This file is near-OBSOLETE. It is used for about:blank only and for the
@@ -102,7 +66,7 @@
 
 #include "nsIStyleSheetLinkingElement.h"
 #include "nsITimer.h"
-#include "nsDOMError.h"
+#include "nsError.h"
 #include "nsContentPolicyUtils.h"
 #include "nsIScriptContext.h"
 #include "nsStyleLinkElement.h"
@@ -117,11 +81,12 @@
 #include "nsNodeInfoManager.h"
 #include "nsContentCreatorFunctions.h"
 #include "mozAutoDocUpdate.h"
+#include "nsTextNode.h"
 
 using namespace mozilla;
 using namespace mozilla::dom;
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
 static PRLogModuleInfo* gSinkLogModuleInfo;
 
 #define SINK_TRACE_NODE(_bit, _msg, _tag, _sp, _obj) \
@@ -142,7 +107,7 @@ NS_NewHTMLNOTUSEDElement(already_AddRefed<nsINodeInfo> aNodeInfo,
                          FromParser aFromParser)
 {
   NS_NOTREACHED("The element ctor should never be called");
-  return nsnull;
+  return nullptr;
 }
 
 #define HTML_TAG(_tag, _classname) NS_NewHTML##_classname##Element,
@@ -205,10 +170,10 @@ public:
   NS_IMETHOD DidProcessTokens(void);
   NS_IMETHOD WillProcessAToken(void);
   NS_IMETHOD DidProcessAToken(void);
-  NS_IMETHOD BeginContext(PRInt32 aID);
-  NS_IMETHOD EndContext(PRInt32 aID);
+  NS_IMETHOD BeginContext(int32_t aID);
+  NS_IMETHOD EndContext(int32_t aID);
   NS_IMETHOD OpenHead();
-  NS_IMETHOD IsEnabled(PRInt32 aTag, bool* aReturn);
+  NS_IMETHOD IsEnabled(int32_t aTag, bool* aReturn);
 
 #ifdef DEBUG
   // nsIDebugDumpContent
@@ -224,18 +189,18 @@ protected:
   already_AddRefed<nsGenericHTMLElement>
   CreateContentObject(const nsIParserNode& aNode, nsHTMLTag aNodeType);
 
-#ifdef NS_DEBUG
-  void SinkTraceNode(PRUint32 aBit,
+#ifdef DEBUG
+  void SinkTraceNode(uint32_t aBit,
                      const char* aMsg,
                      const nsHTMLTag aTag,
-                     PRInt32 aStackPos,
+                     int32_t aStackPos,
                      void* aThis);
 #endif
 
   nsCOMPtr<nsIHTMLDocument> mHTMLDocument;
 
   // The maximum length of a text run
-  PRInt32 mMaxTextRun;
+  int32_t mMaxTextRun;
 
   nsRefPtr<nsGenericHTMLElement> mRoot;
   nsRefPtr<nsGenericHTMLElement> mBody;
@@ -253,9 +218,9 @@ protected:
   // yet.  We want to make sure to only do this once.
   bool mNotifiedRootInsertion;
 
-  PRUint8 mScriptEnabled : 1;
-  PRUint8 mFramesEnabled : 1;
-  PRUint8 unused : 6;  // bits available if someone needs one
+  uint8_t mScriptEnabled : 1;
+  uint8_t mFramesEnabled : 1;
+  uint8_t unused : 6;  // bits available if someone needs one
 
   nsINodeInfo* mNodeInfoCache[NS_HTML_TAG_MAX + 1];
 
@@ -274,12 +239,12 @@ protected:
 
   void NotifyInsert(nsIContent* aContent,
                     nsIContent* aChildContent,
-                    PRInt32 aIndexInContainer);
+                    int32_t aIndexInContainer);
   void NotifyRootInsertion();
   
   bool IsMonolithicContainer(nsHTMLTag aTag);
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
   void ForceReflow();
 #endif
 };
@@ -291,7 +256,7 @@ public:
   ~SinkContext();
 
   nsresult Begin(nsHTMLTag aNodeType, nsGenericHTMLElement* aRoot,
-                 PRUint32 aNumFlushed, PRInt32 aInsertionPoint);
+                 uint32_t aNumFlushed, int32_t aInsertionPoint);
   nsresult OpenContainer(const nsIParserNode& aNode);
   nsresult CloseContainer(const nsHTMLTag aTag);
   nsresult AddLeaf(const nsIParserNode& aNode);
@@ -300,9 +265,9 @@ public:
 
   nsresult GrowStack();
   nsresult AddText(const nsAString& aText);
-  nsresult FlushText(bool* aDidFlush = nsnull,
+  nsresult FlushText(bool* aDidFlush = nullptr,
                      bool aReleaseLast = false);
-  nsresult FlushTextAndRelease(bool* aDidFlush = nsnull)
+  nsresult FlushTextAndRelease(bool* aDidFlush = nullptr)
   {
     return FlushText(aDidFlush, true);
   }
@@ -322,26 +287,26 @@ private:
   
 public:
   HTMLContentSink* mSink;
-  PRInt32 mNotifyLevel;
+  int32_t mNotifyLevel;
   nsCOMPtr<nsIContent> mLastTextNode;
-  PRInt32 mLastTextNodeSize;
+  int32_t mLastTextNodeSize;
 
   struct Node {
     nsHTMLTag mType;
     nsGenericHTMLElement* mContent;
-    PRUint32 mNumFlushed;
-    PRInt32 mInsertionPoint;
+    uint32_t mNumFlushed;
+    int32_t mInsertionPoint;
 
     nsIContent *Add(nsIContent *child);
   };
 
   Node* mStack;
-  PRInt32 mStackSize;
-  PRInt32 mStackPos;
+  int32_t mStackSize;
+  int32_t mStackPos;
 
   PRUnichar* mText;
-  PRInt32 mTextLength;
-  PRInt32 mTextSize;
+  int32_t mTextLength;
+  int32_t mTextSize;
 
 private:
   bool mLastTextCharWasCR;
@@ -349,12 +314,12 @@ private:
 
 //----------------------------------------------------------------------
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
 void
-HTMLContentSink::SinkTraceNode(PRUint32 aBit,
+HTMLContentSink::SinkTraceNode(uint32_t aBit,
                                const char* aMsg,
                                const nsHTMLTag aTag,
-                               PRInt32 aStackPos,
+                               int32_t aStackPos,
                                void* aThis)
 {
   if (SINK_LOG_TEST(gSinkLogModuleInfo, aBit)) {
@@ -376,7 +341,7 @@ HTMLContentSink::AddAttributes(const nsIParserNode& aNode,
 {
   // Add tag attributes to the content attributes
 
-  PRInt32 ac = aNode.GetAttributeCount();
+  int32_t ac = aNode.GetAttributeCount();
 
   if (ac == 0) {
     // No attributes, nothing to do. Do an early return to avoid
@@ -402,7 +367,7 @@ HTMLContentSink::AddAttributes(const nsIParserNode& aNode,
   // multiple <html>, <head>, or <body> tags, so we're doing fixup anyway at
   // that point.
 
-  PRInt32 i, limit, step;
+  int32_t i, limit, step;
   if (aCheckIfPresent) {
     i = 0;
     limit = ac;
@@ -469,7 +434,7 @@ HTMLContentSink::CreateContentObject(const nsIParserNode& aNode,
     nsAutoString lower;
     nsContentUtils::ASCIIToLower(aNode.GetText(), lower);
     nsCOMPtr<nsIAtom> name = do_GetAtom(lower);
-    nodeInfo = mNodeInfoManager->GetNodeInfo(name, nsnull, kNameSpaceID_XHTML,
+    nodeInfo = mNodeInfoManager->GetNodeInfo(name, nullptr, kNameSpaceID_XHTML,
                                              nsIDOMNode::ELEMENT_NODE);
   }
   else if (mNodeInfoCache[aNodeType]) {
@@ -478,17 +443,17 @@ HTMLContentSink::CreateContentObject(const nsIParserNode& aNode,
   else {
     nsIParserService *parserService = nsContentUtils::GetParserService();
     if (!parserService)
-      return nsnull;
+      return nullptr;
 
     nsIAtom *name = parserService->HTMLIdToAtomTag(aNodeType);
     NS_ASSERTION(name, "What? Reverse mapping of id to string broken!!!");
 
-    nodeInfo = mNodeInfoManager->GetNodeInfo(name, nsnull, kNameSpaceID_XHTML,
+    nodeInfo = mNodeInfoManager->GetNodeInfo(name, nullptr, kNameSpaceID_XHTML,
                                              nsIDOMNode::ELEMENT_NODE);
     NS_IF_ADDREF(mNodeInfoCache[aNodeType] = nodeInfo);
   }
 
-  NS_ENSURE_TRUE(nodeInfo, nsnull);
+  NS_ENSURE_TRUE(nodeInfo, nullptr);
 
   // Make the content object
   return CreateHTMLElement(aNodeType, nodeInfo.forget(), FROM_PARSER_NETWORK);
@@ -498,7 +463,7 @@ nsresult
 NS_NewHTMLElement(nsIContent** aResult, already_AddRefed<nsINodeInfo> aNodeInfo,
                   FromParser aFromParser)
 {
-  *aResult = nsnull;
+  *aResult = nullptr;
 
   nsCOMPtr<nsINodeInfo> nodeInfo = aNodeInfo;
 
@@ -518,7 +483,7 @@ NS_NewHTMLElement(nsIContent** aResult, already_AddRefed<nsINodeInfo> aNodeInfo,
 }
 
 already_AddRefed<nsGenericHTMLElement>
-CreateHTMLElement(PRUint32 aNodeType, already_AddRefed<nsINodeInfo> aNodeInfo,
+CreateHTMLElement(uint32_t aNodeType, already_AddRefed<nsINodeInfo> aNodeInfo,
                   FromParser aFromParser)
 {
   NS_ASSERTION(aNodeType <= NS_HTML_TAG_MAX ||
@@ -530,10 +495,9 @@ CreateHTMLElement(PRUint32 aNodeType, already_AddRefed<nsINodeInfo> aNodeInfo,
   NS_ASSERTION(cb != NS_NewHTMLNOTUSEDElement,
                "Don't know how to construct tag element!");
 
-  nsGenericHTMLElement* result = cb(aNodeInfo, aFromParser);
-  NS_IF_ADDREF(result);
+  nsRefPtr<nsGenericHTMLElement> result = cb(aNodeInfo, aFromParser);
 
-  return result;
+  return result.forget();
 }
 
 //----------------------------------------------------------------------
@@ -542,10 +506,10 @@ SinkContext::SinkContext(HTMLContentSink* aSink)
   : mSink(aSink),
     mNotifyLevel(0),
     mLastTextNodeSize(0),
-    mStack(nsnull),
+    mStack(nullptr),
     mStackSize(0),
     mStackPos(0),
-    mText(nsnull),
+    mText(nullptr),
     mTextLength(0),
     mTextSize(0),
     mLastTextCharWasCR(false)
@@ -558,7 +522,7 @@ SinkContext::~SinkContext()
   MOZ_COUNT_DTOR(SinkContext);
 
   if (mStack) {
-    for (PRInt32 i = 0; i < mStackPos; i++) {
+    for (int32_t i = 0; i < mStackPos; i++) {
       NS_RELEASE(mStack[i].mContent);
     }
     delete [] mStack;
@@ -570,8 +534,8 @@ SinkContext::~SinkContext()
 nsresult
 SinkContext::Begin(nsHTMLTag aNodeType,
                    nsGenericHTMLElement* aRoot,
-                   PRUint32 aNumFlushed,
-                   PRInt32 aInsertionPoint)
+                   uint32_t aNumFlushed,
+                   int32_t aInsertionPoint)
 {
   if (mStackSize < 1) {
     nsresult rv = GrowStack();
@@ -618,7 +582,7 @@ SinkContext::DidAddContent(nsIContent* aContent)
       mStack[mStackPos - 1].mContent->GetChildCount()) {
     nsIContent* parent = mStack[mStackPos - 1].mContent;
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
     // Tracing code
     nsIParserService *parserService = nsContentUtils::GetParserService();
     if (parserService) {
@@ -633,7 +597,7 @@ SinkContext::DidAddContent(nsIContent* aContent)
     }
 #endif
 
-    PRInt32 childIndex = mStack[mStackPos - 1].mInsertionPoint - 1;
+    int32_t childIndex = mStack[mStackPos - 1].mInsertionPoint - 1;
     NS_ASSERTION(parent->GetChildAt(childIndex) == aContent,
                  "Flushing the wrong child.");
     mSink->NotifyInsert(parent, aContent, childIndex);
@@ -716,10 +680,8 @@ SinkContext::OpenContainer(const nsIParserNode& aNode)
       break;
 
     case eHTMLTag_button:
-#ifdef MOZ_MEDIA
     case eHTMLTag_audio:
     case eHTMLTag_video:
-#endif
       content->DoneCreatingElement();
       break;
 
@@ -793,7 +755,7 @@ SinkContext::CloseContainer(const nsHTMLTag aTag)
     // notification
 
     if (mStack[mStackPos].mNumFlushed < content->GetChildCount()) {
-#ifdef NS_DEBUG
+#ifdef DEBUG
       {
         // Tracing code
         SINK_TRACE(gSinkLogModuleInfo, SINK_TRACE_REFLOW,
@@ -828,10 +790,8 @@ SinkContext::CloseContainer(const nsHTMLTag aTag)
     MOZ_NOT_REACHED("Must not use HTMLContentSink for forms.");
     break;
 
-#ifdef MOZ_MEDIA
   case eHTMLTag_video:
   case eHTMLTag_audio:
-#endif
   case eHTMLTag_select:
   case eHTMLTag_textarea:
   case eHTMLTag_object:
@@ -922,7 +882,7 @@ SinkContext::AddLeaf(const nsIParserNode& aNode)
   case eToken_entity:
     {
       nsAutoString tmp;
-      PRInt32 unicode = aNode.TranslateToUnicodeStr(tmp);
+      int32_t unicode = aNode.TranslateToUnicodeStr(tmp);
       if (unicode < 0) {
         rv = AddText(aNode.GetText());
       } else {
@@ -966,7 +926,7 @@ SinkContext::AddLeaf(nsIContent* aContent)
 nsresult
 SinkContext::End()
 {
-  for (PRInt32 i = 0; i < mStackPos; i++) {
+  for (int32_t i = 0; i < mStackPos; i++) {
     NS_RELEASE(mStack[i].mContent);
   }
 
@@ -979,7 +939,7 @@ SinkContext::End()
 nsresult
 SinkContext::GrowStack()
 {
-  PRInt32 newSize = mStackSize * 2;
+  int32_t newSize = mStackSize * 2;
   if (newSize == 0) {
     newSize = 32;
   }
@@ -1011,7 +971,7 @@ SinkContext::GrowStack()
 nsresult
 SinkContext::AddText(const nsAString& aText)
 {
-  PRInt32 addLen = aText.Length();
+  int32_t addLen = aText.Length();
   if (addLen == 0) {
     return NS_OK;
   }
@@ -1026,10 +986,10 @@ SinkContext::AddText(const nsAString& aText)
   }
 
   // Copy data from string into our buffer; flush buffer when it fills up
-  PRInt32 offset = 0;
+  int32_t offset = 0;
 
   while (addLen != 0) {
-    PRInt32 amount = mTextSize - mTextLength;
+    int32_t amount = mTextSize - mTextLength;
 
     if (amount > addLen) {
       amount = addLen;
@@ -1074,7 +1034,7 @@ SinkContext::FlushTags()
 {
   mSink->mDeferredFlushTags = false;
   bool oldBeganUpdate = mSink->mBeganUpdate;
-  PRUint32 oldUpdates = mSink->mUpdatesInNotification;
+  uint32_t oldUpdates = mSink->mUpdatesInNotification;
 
   ++(mSink->mInNotification);
   mSink->mUpdatesInNotification = 0;
@@ -1094,9 +1054,9 @@ SinkContext::FlushTags()
     // Note that we can start at stackPos == 0 here, because it's the caller's
     // responsibility to handle flushing interactions between contexts (see
     // HTMLContentSink::BeginContext).
-    PRInt32 stackPos = 0;
+    int32_t stackPos = 0;
     bool flushed = false;
-    PRUint32 childCount;
+    uint32_t childCount;
     nsGenericHTMLElement* content;
 
     while (stackPos < mStackPos) {
@@ -1104,7 +1064,7 @@ SinkContext::FlushTags()
       childCount = content->GetChildCount();
 
       if (!flushed && (mStack[stackPos].mNumFlushed < childCount)) {
-#ifdef NS_DEBUG
+#ifdef DEBUG
         {
           // Tracing code
           SINK_TRACE(gSinkLogModuleInfo, SINK_TRACE_REFLOW,
@@ -1119,7 +1079,7 @@ SinkContext::FlushTags()
           // but not notified on it yet, which is why we have to get it
           // directly from its parent node.
 
-          PRInt32 childIndex = mStack[stackPos].mInsertionPoint - 1;
+          int32_t childIndex = mStack[stackPos].mInsertionPoint - 1;
           nsIContent* child = content->GetChildAt(childIndex);
           // Child not on stack anymore; can't assert it's correct
           NS_ASSERTION(!(mStackPos > (stackPos + 1)) ||
@@ -1161,7 +1121,7 @@ SinkContext::UpdateChildCounts()
   // have been generated for it and we should make sure that no
   // further reflows occur.  Note that we have to include stackPos == 0
   // to properly notify on kids of <html>.
-  PRInt32 stackPos = mStackPos - 1;
+  int32_t stackPos = mStackPos - 1;
   while (stackPos >= 0) {
     Node & node = mStack[stackPos];
     node.mNumFlushed = node.mContent->GetChildCount();
@@ -1186,7 +1146,7 @@ SinkContext::FlushText(bool* aDidFlush, bool aReleaseLast)
     if (mLastTextNode) {
       if ((mLastTextNodeSize + mTextLength) > mSink->mMaxTextRun) {
         mLastTextNodeSize = 0;
-        mLastTextNode = nsnull;
+        mLastTextNode = nullptr;
         FlushText(aDidFlush, aReleaseLast);
       } else {
         bool notify = HaveNotifiedForCurrentContent();
@@ -1206,10 +1166,8 @@ SinkContext::FlushText(bool* aDidFlush, bool aReleaseLast)
         didFlush = true;
       }
     } else {
-      nsCOMPtr<nsIContent> textContent;
-      rv = NS_NewTextNode(getter_AddRefs(textContent),
-                          mSink->mNodeInfoManager);
-      NS_ENSURE_SUCCESS(rv, rv);
+      nsRefPtr<nsTextNode> textContent =
+        new nsTextNode(mSink->mNodeInfoManager);
 
       mLastTextNode = textContent;
 
@@ -1233,7 +1191,7 @@ SinkContext::FlushText(bool* aDidFlush, bool aReleaseLast)
 
   if (aReleaseLast) {
     mLastTextNodeSize = 0;
-    mLastTextNode = nsnull;
+    mLastTextNode = nullptr;
     mLastTextCharWasCR = false;
   }
 
@@ -1278,7 +1236,7 @@ HTMLContentSink::HTMLContentSink()
   // Note: operator new zeros our memory
 
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
   if (!gSinkLogModuleInfo) {
     gSinkLogModuleInfo = PR_NewLogModule("htmlcontentsink");
   }
@@ -1291,20 +1249,20 @@ HTMLContentSink::~HTMLContentSink()
     mNotificationTimer->Cancel();
   }
 
-  PRInt32 numContexts = mContextStack.Length();
+  int32_t numContexts = mContextStack.Length();
 
   if (mCurrentContext == mHeadContext && numContexts > 0) {
     // Pop off the second html context if it's not done earlier
     mContextStack.RemoveElementAt(--numContexts);
   }
 
-  PRInt32 i;
+  int32_t i;
   for (i = 0; i < numContexts; i++) {
     SinkContext* sc = mContextStack.ElementAt(i);
     if (sc) {
       sc->End();
       if (sc == mCurrentContext) {
-        mCurrentContext = nsnull;
+        mCurrentContext = nullptr;
       }
 
       delete sc;
@@ -1312,36 +1270,34 @@ HTMLContentSink::~HTMLContentSink()
   }
 
   if (mCurrentContext == mHeadContext) {
-    mCurrentContext = nsnull;
+    mCurrentContext = nullptr;
   }
 
   delete mCurrentContext;
 
   delete mHeadContext;
 
-  for (i = 0; PRUint32(i) < ArrayLength(mNodeInfoCache); ++i) {
+  for (i = 0; uint32_t(i) < ArrayLength(mNodeInfoCache); ++i) {
     NS_IF_RELEASE(mNodeInfoCache[i]);
   }
 }
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(HTMLContentSink)
-
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(HTMLContentSink, nsContentSink)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mHTMLDocument)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mRoot)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mBody)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mHead)
-  for (PRUint32 i = 0; i < ArrayLength(tmp->mNodeInfoCache); ++i) {
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mHTMLDocument)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mRoot)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mBody)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mHead)
+  for (uint32_t i = 0; i < ArrayLength(tmp->mNodeInfoCache); ++i) {
     NS_IF_RELEASE(tmp->mNodeInfoCache[i]);
   }
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(HTMLContentSink,
                                                   nsContentSink)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mHTMLDocument)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mRoot)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mBody)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mHead)
-  for (PRUint32 i = 0; i < ArrayLength(tmp->mNodeInfoCache); ++i) {
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mHTMLDocument)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mRoot)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mBody)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mHead)
+  for (uint32_t i = 0; i < ArrayLength(tmp->mNodeInfoCache); ++i) {
     NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "mNodeInfoCache[i]");
     cb.NoteXPCOMChild(tmp->mNodeInfoCache[i]);
   }
@@ -1365,7 +1321,8 @@ IsScriptEnabled(nsIDocument *aDoc, nsIDocShell *aContainer)
 {
   NS_ENSURE_TRUE(aDoc && aContainer, true);
 
-  nsCOMPtr<nsIScriptGlobalObject> globalObject = aDoc->GetScriptGlobalObject();
+  nsCOMPtr<nsIScriptGlobalObject> globalObject =
+    do_QueryInterface(aDoc->GetWindow());
 
   // Getting context is tricky if the document hasn't had its
   // GlobalObject set yet
@@ -1379,8 +1336,7 @@ IsScriptEnabled(nsIDocument *aDoc, nsIDocShell *aContainer)
 
   nsIScriptContext *scriptContext = globalObject->GetContext();
   NS_ENSURE_TRUE(scriptContext, true);
-
-  JSContext* cx = scriptContext->GetNativeContext();
+  JSContext *cx = scriptContext->GetNativeContext();
   NS_ENSURE_TRUE(cx, true);
 
   bool enabled = true;
@@ -1428,10 +1384,9 @@ HTMLContentSink::Init(nsIDocument* aDoc,
   mMaxTextRun = Preferences::GetInt("content.maxtextrun", 8191);
 
   nsCOMPtr<nsINodeInfo> nodeInfo;
-  nodeInfo = mNodeInfoManager->GetNodeInfo(nsGkAtoms::html, nsnull,
+  nodeInfo = mNodeInfoManager->GetNodeInfo(nsGkAtoms::html, nullptr,
                                            kNameSpaceID_XHTML,
                                            nsIDOMNode::ELEMENT_NODE);
-  NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
 
   // Make root part
   mRoot = NS_NewHTMLHtmlElement(nodeInfo.forget());
@@ -1446,9 +1401,8 @@ HTMLContentSink::Init(nsIDocument* aDoc,
 
   // Make head part
   nodeInfo = mNodeInfoManager->GetNodeInfo(nsGkAtoms::head,
-                                           nsnull, kNameSpaceID_XHTML,
+                                           nullptr, kNameSpaceID_XHTML,
                                            nsIDOMNode::ELEMENT_NODE);
-  NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
 
   mHead = NS_NewHTMLHeadElement(nodeInfo.forget());
   if (NS_FAILED(rv)) {
@@ -1462,8 +1416,8 @@ HTMLContentSink::Init(nsIDocument* aDoc,
   mCurrentContext->Begin(eHTMLTag_html, mRoot, 0, -1);
   mContextStack.AppendElement(mCurrentContext);
 
-#ifdef NS_DEBUG
-  nsCAutoString spec;
+#ifdef DEBUG
+  nsAutoCString spec;
   (void)aURI->GetSpec(spec);
   SINK_TRACE(gSinkLogModuleInfo, SINK_TRACE_CALLS,
              ("HTMLContentSink::Init: this=%p url='%s'",
@@ -1562,7 +1516,7 @@ HTMLContentSink::SetParser(nsParserBase* aParser)
 }
 
 NS_IMETHODIMP
-HTMLContentSink::BeginContext(PRInt32 aPosition)
+HTMLContentSink::BeginContext(int32_t aPosition)
 {
   NS_PRECONDITION(aPosition > -1, "out of bounds");
 
@@ -1582,7 +1536,7 @@ HTMLContentSink::BeginContext(PRInt32 aPosition)
     return NS_ERROR_FAILURE;
   }
 
-  PRInt32 insertionPoint = -1;
+  int32_t insertionPoint = -1;
   nsHTMLTag nodeType      = mCurrentContext->mStack[aPosition].mType;
   nsGenericHTMLElement* content = mCurrentContext->mStack[aPosition].mContent;
 
@@ -1606,11 +1560,11 @@ HTMLContentSink::BeginContext(PRInt32 aPosition)
 }
 
 NS_IMETHODIMP
-HTMLContentSink::EndContext(PRInt32 aPosition)
+HTMLContentSink::EndContext(int32_t aPosition)
 {
   NS_PRECONDITION(mCurrentContext && aPosition > -1, "nonexistent context");
 
-  PRUint32 n = mContextStack.Length() - 1;
+  uint32_t n = mContextStack.Length() - 1;
   SinkContext* sc = mContextStack.ElementAt(n);
 
   const SinkContext::Node &bottom = mCurrentContext->mStack[0];
@@ -1632,19 +1586,19 @@ HTMLContentSink::EndContext(PRInt32 aPosition)
 
   sc->mStack[aPosition].mNumFlushed = bottom.mNumFlushed;
 
-  for (PRInt32 i = 0; i<mCurrentContext->mStackPos; i++) {
+  for (int32_t i = 0; i<mCurrentContext->mStackPos; i++) {
     NS_IF_RELEASE(mCurrentContext->mStack[i].mContent);
   }
 
   delete [] mCurrentContext->mStack;
 
-  mCurrentContext->mStack      = nsnull;
+  mCurrentContext->mStack      = nullptr;
   mCurrentContext->mStackPos   = 0;
   mCurrentContext->mStackSize  = 0;
 
   delete [] mCurrentContext->mText;
 
-  mCurrentContext->mText       = nsnull;
+  mCurrentContext->mText       = nullptr;
   mCurrentContext->mTextLength = 0;
   mCurrentContext->mTextSize   = 0;
 
@@ -1666,7 +1620,7 @@ HTMLContentSink::CloseHTML()
 
   if (mHeadContext) {
     if (mCurrentContext == mHeadContext) {
-      PRUint32 numContexts = mContextStack.Length();
+      uint32_t numContexts = mContextStack.Length();
 
       // Pop off the second html context if it's not done earlier
       mCurrentContext = mContextStack.ElementAt(--numContexts);
@@ -1678,7 +1632,7 @@ HTMLContentSink::CloseHTML()
     mHeadContext->End();
 
     delete mHeadContext;
-    mHeadContext = nsnull;
+    mHeadContext = nullptr;
   }
 
   return NS_OK;
@@ -1717,20 +1671,20 @@ HTMLContentSink::OpenBody(const nsIParserNode& aNode)
   mBody = mCurrentContext->mStack[mCurrentContext->mStackPos - 1].mContent;
 
   if (mCurrentContext->mStackPos > 1) {
-    PRInt32 parentIndex    = mCurrentContext->mStackPos - 2;
+    int32_t parentIndex    = mCurrentContext->mStackPos - 2;
     nsGenericHTMLElement *parent = mCurrentContext->mStack[parentIndex].mContent;
-    PRInt32 numFlushed     = mCurrentContext->mStack[parentIndex].mNumFlushed;
-    PRInt32 childCount = parent->GetChildCount();
+    int32_t numFlushed     = mCurrentContext->mStack[parentIndex].mNumFlushed;
+    int32_t childCount = parent->GetChildCount();
     NS_ASSERTION(numFlushed < childCount, "Already notified on the body?");
     
-    PRInt32 insertionPoint =
+    int32_t insertionPoint =
       mCurrentContext->mStack[parentIndex].mInsertionPoint;
 
     // XXX: I have yet to see a case where numFlushed is non-zero and
     // insertionPoint is not -1, but this code will try to handle
     // those cases too.
 
-    PRUint32 oldUpdates = mUpdatesInNotification;
+    uint32_t oldUpdates = mUpdatesInNotification;
     mUpdatesInNotification = 0;
     if (insertionPoint != -1) {
       NotifyInsert(parent, mBody, insertionPoint - 1);
@@ -1775,7 +1729,7 @@ HTMLContentSink::CloseBody()
 }
 
 NS_IMETHODIMP
-HTMLContentSink::IsEnabled(PRInt32 aTag, bool* aReturn)
+HTMLContentSink::IsEnabled(int32_t aTag, bool* aReturn)
 {
   nsHTMLTag theHTMLTag = nsHTMLTag(aTag);
 
@@ -1966,7 +1920,7 @@ HTMLContentSink::CloseHeadContext()
 
   if (!mContextStack.IsEmpty())
   {
-    PRUint32 n = mContextStack.Length() - 1;
+    uint32_t n = mContextStack.Length() - 1;
     mCurrentContext = mContextStack.ElementAt(n);
     mContextStack.RemoveElementAt(n);
   }
@@ -1983,7 +1937,7 @@ HTMLContentSink::ForceReflow()
 void
 HTMLContentSink::NotifyInsert(nsIContent* aContent,
                               nsIContent* aChildContent,
-                              PRInt32 aIndexInContainer)
+                              int32_t aIndexInContainer)
 {
   if (aContent && aContent->GetCurrentDoc() != mDocument) {
     // aContent is not actually in our document anymore.... Just bail out of
@@ -2018,9 +1972,9 @@ HTMLContentSink::NotifyRootInsertion()
   // document; in those cases we just want to put all the attrs on one
   // tag.
   mNotifiedRootInsertion = true;
-  PRInt32 index = mDocument->IndexOf(mRoot);
+  int32_t index = mDocument->IndexOf(mRoot);
   NS_ASSERTION(index != -1, "mRoot not child of document?");
-  NotifyInsert(nsnull, mRoot, index);
+  NotifyInsert(nullptr, mRoot, index);
 
   // Now update the notification information in all our
   // contexts, since we just inserted the root and notified on
@@ -2044,8 +1998,8 @@ HTMLContentSink::IsMonolithicContainer(nsHTMLTag aTag)
 void
 HTMLContentSink::UpdateChildCounts()
 {
-  PRUint32 numContexts = mContextStack.Length();
-  for (PRUint32 i = 0; i < numContexts; i++) {
+  uint32_t numContexts = mContextStack.Length();
+  for (uint32_t i = 0; i < numContexts; i++) {
     SinkContext* sc = mContextStack.ElementAt(i);
 
     sc->UpdateChildCounts();
@@ -2107,12 +2061,10 @@ HTMLContentSink::SetDocumentCharset(nsACString& aCharset)
       // it but if we get a null pointer, that's perfectly legal for
       // parent and parentContentViewer
 
-      nsCOMPtr<nsIDocShellTreeItem> docShellAsItem =
-        do_QueryInterface(mDocShell);
-      NS_ENSURE_TRUE(docShellAsItem, NS_ERROR_FAILURE);
+      NS_ENSURE_TRUE(mDocShell, NS_ERROR_FAILURE);
 
       nsCOMPtr<nsIDocShellTreeItem> parentAsItem;
-      docShellAsItem->GetSameTypeParent(getter_AddRefs(parentAsItem));
+      mDocShell->GetSameTypeParent(getter_AddRefs(parentAsItem));
 
       nsCOMPtr<nsIDocShell> parent(do_QueryInterface(parentAsItem));
       if (parent) {
@@ -2167,7 +2119,7 @@ HTMLContentSink::DumpContentModel()
       Element* root = mDocument->GetRootElement();
       if (root) {
         if (mDocumentURI) {
-          nsCAutoString buf;
+          nsAutoCString buf;
           mDocumentURI->GetSpec(buf);
           fputs(buf.get(), out);
         }
