@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Mozilla SMIL module.
- *
- * The Initial Developer of the Original Code is Brian Birtles.
- * Portions created by the Initial Developer are Copyright (C) 2005
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Brian Birtles <birtles@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef NS_SMILTIMEDELEMENT_H_
 #define NS_SMILTIMEDELEMENT_H_
@@ -50,12 +18,17 @@
 #include "nsAutoPtr.h"
 #include "nsAttrValue.h"
 
-class nsISMILAnimationElement;
 class nsSMILAnimationFunction;
 class nsSMILTimeContainer;
 class nsSMILTimeValue;
 class nsIAtom;
 
+namespace mozilla {
+namespace dom {
+class SVGAnimationElement;
+}
+}
+
 //----------------------------------------------------------------------
 // nsSMILTimedElement
 
@@ -71,11 +44,11 @@ public:
    * Sets the owning animation element which this class uses to convert between
    * container times and to register timebase elements.
    */
-  void SetAnimationElement(nsISMILAnimationElement* aElement);
+  void SetAnimationElement(mozilla::dom::SVGAnimationElement* aElement);
 
   /*
    * Returns the time container with which this timed element is associated or
-   * nsnull if it is not associated with a time container.
+   * nullptr if it is not associated with a time container.
    */
   nsSMILTimeContainer* GetTimeContainer();
 
@@ -83,12 +56,7 @@ public:
    * Returns the element targeted by the animation element. Needed for
    * registering event listeners against the appropriate element.
    */
-  mozilla::dom::Element* GetTargetElement()
-  {
-    return mAnimationElement ?
-        mAnimationElement->GetTargetElementContent() :
-        nsnull;
-  }
+  mozilla::dom::Element* GetTargetElement();
 
   /**
    * Methods for supporting the nsIDOMElementTimeControl interface.
@@ -139,10 +107,29 @@ public:
     return mSimpleDur;
   }
 
+  /**
+   * Methods for supporting hyperlinking
+   */
+
   /**
    * Internal SMIL methods
    */
 
+  /**
+   * Returns the time to seek the document to when this element is targetted by
+   * a hyperlink.
+   *
+   * The behavior is defined here:
+   *   http://www.w3.org/TR/smil-animation/#HyperlinkSemantics
+   *
+   * It is very similar to GetStartTime() with the exception that when the
+   * element is not active, the begin time of the *first* interval is returned.
+   *
+   * @return the time to seek the documen to in milliseconds or an unresolved
+   * time if there is no resolved interval.
+   */
+  nsSMILTimeValue GetHyperlinkTime() const;
+
   /**
    * Adds an instance time object this element's list of instance times.
    * These instance times are used when creating intervals.
@@ -205,7 +192,7 @@ public:
    *
    * @param aClient   The time client to associate. Any previous time client
    *                  will be disassociated and no longer sampled. Setting this
-   *                  to nsnull will simply disassociate the previous client, if
+   *                  to nullptr will simply disassociate the previous client, if
    *                  any.
    */
   void SetTimeClient(nsSMILAnimationFunction* aClient);
@@ -272,7 +259,7 @@ public:
    */
   bool SetAttr(nsIAtom* aAttribute, const nsAString& aValue,
                  nsAttrValue& aResult, Element* aContextNode,
-                 nsresult* aParseResult = nsnull);
+                 nsresult* aParseResult = nullptr);
 
   /**
    * Attempts to unset an attribute on this timed element.
@@ -471,14 +458,14 @@ protected:
    *
    * @param aPrevInterval   The previous interval used. If supplied, the first
    *                        interval that begins after aPrevInterval will be
-   *                        returned. May be nsnull.
+   *                        returned. May be nullptr.
    * @param aReplacedInterval The interval that is being updated (if any). This
    *                        used to ensure we don't return interval endpoints
-   *                        that are dependent on themselves. May be nsnull.
+   *                        that are dependent on themselves. May be nullptr.
    * @param aFixedBeginTime The time to use for the start of the interval. This
    *                        is used when only the endpoint of the interval
    *                        should be updated such as when the animation is in
-   *                        the ACTIVE state. May be nsnull.
+   *                        the ACTIVE state. May be nullptr.
    * @param[out] aResult    The next interval. Will be unchanged if no suitable
    *                        interval was found (in which case false will be
    *                        returned).
@@ -490,16 +477,16 @@ protected:
                                     nsSMILInterval& aResult) const;
   nsSMILInstanceTime* GetNextGreater(const InstanceTimeList& aList,
                                      const nsSMILTimeValue& aBase,
-                                     PRInt32& aPosition) const;
+                                     int32_t& aPosition) const;
   nsSMILInstanceTime* GetNextGreaterOrEqual(const InstanceTimeList& aList,
                                             const nsSMILTimeValue& aBase,
-                                            PRInt32& aPosition) const;
+                                            int32_t& aPosition) const;
   nsSMILTimeValue   CalcActiveEnd(const nsSMILTimeValue& aBegin,
                                   const nsSMILTimeValue& aEnd) const;
   nsSMILTimeValue   GetRepeatDuration() const;
   nsSMILTimeValue   ApplyMinAndMax(const nsSMILTimeValue& aDuration) const;
   nsSMILTime        ActiveTimeToSimpleTime(nsSMILTime aActiveTime,
-                                           PRUint32& aRepeatIteration);
+                                           uint32_t& aRepeatIteration);
   nsSMILInstanceTime* CheckForEarlyEnd(
                         const nsSMILTimeValue& aContainerTime) const;
   void              UpdateCurrentInterval(bool aForceChangeNotice = false);
@@ -521,7 +508,7 @@ protected:
                                           bool aBeginObjectChanged,
                                           bool aEndObjectChanged);
 
-  void              FireTimeEventAsync(PRUint32 aMsg, PRInt32 aDetail);
+  void              FireTimeEventAsync(uint32_t aMsg, int32_t aDetail);
   const nsSMILInstanceTime* GetEffectiveBeginInstance() const;
   const nsSMILInterval* GetPreviousInterval() const;
   bool              HasPlayed() const { return !mOldIntervals.IsEmpty(); }
@@ -531,7 +518,7 @@ protected:
 
   // Reset the current interval by first passing ownership to a temporary
   // variable so that if Unlink() results in us receiving a callback,
-  // mCurrentInterval will be nsnull and we will be in a consistent state.
+  // mCurrentInterval will be nullptr and we will be in a consistent state.
   void ResetCurrentInterval()
   {
     if (mCurrentInterval) {
@@ -542,14 +529,14 @@ protected:
   }
 
   // Hashtable callback methods
-  PR_STATIC_CALLBACK(PLDHashOperator) NotifyNewIntervalCallback(
+  static PLDHashOperator NotifyNewIntervalCallback(
       TimeValueSpecPtrKey* aKey, void* aData);
 
   //
   // Members
   //
-  nsISMILAnimationElement*        mAnimationElement; // [weak] won't outlive
-                                                     // owner
+  mozilla::dom::SVGAnimationElement* mAnimationElement; // [weak] won't outlive
+                                                        // owner
   TimeValueSpecList               mBeginSpecs; // [strong]
   TimeValueSpecList               mEndSpecs; // [strong]
 
@@ -580,16 +567,16 @@ protected:
 
   InstanceTimeList                mBeginInstances;
   InstanceTimeList                mEndInstances;
-  PRUint32                        mInstanceSerialIndex;
+  uint32_t                        mInstanceSerialIndex;
 
   nsSMILAnimationFunction*        mClient;
   nsAutoPtr<nsSMILInterval>       mCurrentInterval;
   IntervalList                    mOldIntervals;
-  PRUint32                        mCurrentRepeatIteration;
+  uint32_t                        mCurrentRepeatIteration;
   nsSMILMilestone                 mPrevRegisteredMilestone;
   static const nsSMILMilestone    sMaxMilestone;
-  static const PRUint8            sMaxNumIntervals;
-  static const PRUint8            sMaxNumInstanceTimes;
+  static const uint8_t            sMaxNumIntervals;
+  static const uint8_t            sMaxNumInstanceTimes;
 
   // Set of dependent time value specs to be notified when establishing a new
   // current interval. Change notifications and delete notifications are handled
@@ -629,9 +616,9 @@ protected:
                           // requested while mDeferIntervalUpdates was set
 
   // Recursion depth checking
-  PRUint8              mDeleteCount;
-  PRUint8              mUpdateIntervalRecursionDepth;
-  static const PRUint8 sMaxUpdateIntervalRecursionDepth;
+  uint8_t              mDeleteCount;
+  uint8_t              mUpdateIntervalRecursionDepth;
+  static const uint8_t sMaxUpdateIntervalRecursionDepth;
 };
 
 #endif // NS_SMILTIMEDELEMENT_H_
