@@ -1,44 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Robert Churchill <rjc@netscape.com>
- *   David Hyatt <hyatt@netscape.com>
- *   Chris Waterson <waterson@netscape.com>
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   Neil Deakin <enndeakin@sympatico.ca>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/Util.h"
 
@@ -68,6 +31,7 @@
 #include "nsAttrName.h"
 #include "nsNodeUtils.h"
 #include "mozAutoDocUpdate.h"
+#include "nsTextNode.h"
 
 #include "jsapi.h"
 #include "pldhash.h"
@@ -156,7 +120,7 @@ protected:
                              bool aNotify,
                              nsTemplateMatch* aMatch,
                              nsIContent** aContainer,
-                             PRInt32* aNewIndexInContainer);
+                             int32_t* aNewIndexInContainer);
 
     /**
      * Copy the attributes from the template node to the node generated
@@ -252,7 +216,7 @@ protected:
                                        bool aNotify,
                                        nsTemplateQuerySet* aQuerySet,
                                        nsIContent** aContainer,
-                                       PRInt32* aNewIndexInContainer);
+                                       int32_t* aNewIndexInContainer);
 
     /**
      * Check if an element with a particular tag exists with a container.
@@ -267,7 +231,7 @@ protected:
      */
     nsresult
     EnsureElementHasGenericChild(nsIContent* aParent,
-                                 PRInt32 aNameSpaceID,
+                                 int32_t aNameSpaceID,
                                  nsIAtom* aTag,
                                  bool aNotify,
                                  nsIContent** aResult);
@@ -283,7 +247,7 @@ protected:
                          nsCOMArray<nsIContent>& aElements);
 
     nsresult
-    CreateElement(PRInt32 aNameSpaceID,
+    CreateElement(int32_t aNameSpaceID,
                   nsIAtom* aTag,
                   nsIContent** aResult);
 
@@ -344,7 +308,7 @@ protected:
      */
     nsresult
     CompareResultToNode(nsIXULTemplateResult* aResult, nsIContent* aContent,
-                        PRInt32* aSortOrder);
+                        int32_t* aSortOrder);
 
     /**
      * Insert a generated node into the container where it should go according
@@ -378,7 +342,7 @@ protected:
 nsresult
 NS_NewXULContentBuilder(nsISupports* aOuter, REFNSIID aIID, void** aResult)
 {
-    NS_PRECONDITION(aOuter == nsnull, "no aggregation");
+    NS_PRECONDITION(aOuter == nullptr, "no aggregation");
     if (aOuter)
         return NS_ERROR_NO_AGGREGATION;
 
@@ -431,7 +395,7 @@ nsXULContentBuilder::BuildContentFromTemplate(nsIContent *aTemplateNode,
                                               bool aNotify,
                                               nsTemplateMatch* aMatch,
                                               nsIContent** aContainer,
-                                              PRInt32* aNewIndexInContainer)
+                                              int32_t* aNewIndexInContainer)
 {
     // This is the mother lode. Here is where we grovel through an
     // element in the template, copying children from the template
@@ -505,7 +469,7 @@ nsXULContentBuilder::BuildContentFromTemplate(nsIContent *aTemplateNode,
          tmplKid;
          tmplKid = tmplKid->GetNextSibling()) {
 
-        PRInt32 nameSpaceID = tmplKid->GetNameSpaceID();
+        int32_t nameSpaceID = tmplKid->GetNameSpaceID();
 
         // Check whether this element is the generation element. The generation
         // element is the element that is cookie-cutter copied once for each
@@ -597,7 +561,7 @@ nsXULContentBuilder::BuildContentFromTemplate(nsIContent *aTemplateNode,
                     *aContainer = aRealNode;
                     NS_ADDREF(*aContainer);
 
-                    PRUint32 indx = aRealNode->GetChildCount();
+                    uint32_t indx = aRealNode->GetChildCount();
 
                     // Since EnsureElementHasGenericChild() added us, make
                     // sure to subtract one for our real index.
@@ -656,10 +620,8 @@ nsXULContentBuilder::BuildContentFromTemplate(nsIContent *aTemplateNode,
                 rv = SubstituteText(aChild, attrValue, value);
                 if (NS_FAILED(rv)) return rv;
 
-                nsCOMPtr<nsIContent> content;
-                rv = NS_NewTextNode(getter_AddRefs(content),
-                                    mRoot->NodeInfo()->NodeInfoManager());
-                if (NS_FAILED(rv)) return rv;
+                nsRefPtr<nsTextNode> content =
+                  new nsTextNode(mRoot->NodeInfo()->NodeInfoManager());
 
                 content->SetText(value, false);
 
@@ -699,7 +661,7 @@ nsXULContentBuilder::BuildContentFromTemplate(nsIContent *aTemplateNode,
                 *aContainer = aRealNode;
                 NS_ADDREF(*aContainer);
 
-                PRUint32 indx = aRealNode->GetChildCount();
+                uint32_t indx = aRealNode->GetChildCount();
 
                 // Since we haven't inserted any content yet, our new
                 // index in the container will be the current count of
@@ -732,8 +694,8 @@ nsXULContentBuilder::BuildContentFromTemplate(nsIContent *aTemplateNode,
                 //  </vbox>
                 rv = BuildContentFromTemplate(tmplKid, aResourceNode, realKid, false,
                                               false, aChild, false, aMatch,
-                                              nsnull /* don't care */,
-                                              nsnull /* don't care */);
+                                              nullptr /* don't care */,
+                                              nullptr /* don't care */);
                 if (NS_FAILED(rv)) return rv;
 
                 if (isGenerationElement) {
@@ -773,11 +735,11 @@ nsXULContentBuilder::CopyAttributesToElement(nsIContent* aTemplateNode,
     nsresult rv;
 
     // Copy all attributes from the template to the new element
-    PRUint32 numAttribs = aTemplateNode->GetAttrCount();
+    uint32_t numAttribs = aTemplateNode->GetAttrCount();
 
-    for (PRUint32 attr = 0; attr < numAttribs; attr++) {
+    for (uint32_t attr = 0; attr < numAttribs; attr++) {
         const nsAttrName* name = aTemplateNode->GetAttrNameAt(attr);
-        PRInt32 attribNameSpaceID = name->NamespaceID();
+        int32_t attribNameSpaceID = name->NamespaceID();
         // Hold a strong reference here so that the atom doesn't go away
         // during UnsetAttr.
         nsCOMPtr<nsIAtom> attribName = name->LocalName();
@@ -838,7 +800,7 @@ nsXULContentBuilder::AddPersistentAttributes(nsIContent* aTemplateNode,
     while (!persist.IsEmpty()) {
         attribute.Truncate();
 
-        PRInt32 offset = persist.FindCharInSet(" ,");
+        int32_t offset = persist.FindCharInSet(" ,");
         if (offset > 0) {
             persist.Left(attribute, offset);
             persist.Cut(0, offset + 1);
@@ -854,7 +816,7 @@ nsXULContentBuilder::AddPersistentAttributes(nsIContent* aTemplateNode,
             break;
 
         nsCOMPtr<nsIAtom> tag;
-        PRInt32 nameSpaceID;
+        int32_t nameSpaceID;
 
         nsCOMPtr<nsINodeInfo> ni =
             aTemplateNode->GetExistingAttrNameFromQName(attribute);
@@ -881,7 +843,7 @@ nsXULContentBuilder::AddPersistentAttributes(nsIContent* aTemplateNode,
             continue;
 
         nsCOMPtr<nsIRDFLiteral> value = do_QueryInterface(target);
-        NS_ASSERTION(value != nsnull, "unable to stomach that sort of node");
+        NS_ASSERTION(value != nullptr, "unable to stomach that sort of node");
         if (! value)
             continue;
 
@@ -909,9 +871,9 @@ nsXULContentBuilder::SynchronizeUsingTemplate(nsIContent* aTemplateNode,
     if (NS_FAILED(rv))
         return rv;
 
-    PRUint32 count = aTemplateNode->GetChildCount();
+    uint32_t count = aTemplateNode->GetChildCount();
 
-    for (PRUint32 loop = 0; loop < count; ++loop) {
+    for (uint32_t loop = 0; loop < count; ++loop) {
         nsIContent *tmplKid = aTemplateNode->GetChildAt(loop);
 
         if (! tmplKid)
@@ -948,7 +910,7 @@ nsXULContentBuilder::RemoveMember(nsIContent* aContent)
 {
     nsCOMPtr<nsIContent> parent = aContent->GetParent();
     if (parent) {
-        PRInt32 pos = parent->IndexOf(aContent);
+        int32_t pos = parent->IndexOf(aContent);
 
         NS_ASSERTION(pos >= 0, "parent doesn't think this child has an index");
         if (pos < 0) return NS_OK;
@@ -1006,7 +968,7 @@ nsXULContentBuilder::CreateTemplateAndContainerContents(nsIContent* aElement,
         // The content map will contain the generation elements (the ones that
         // are given ids) and only those elements, so get the reference point
         // from the corresponding match.
-        nsTemplateMatch *match = nsnull;
+        nsTemplateMatch *match = nullptr;
         if (mContentSupportMap.Get(aElement, &match))
             CreateContainerContents(aElement, match->mResult, aForceCreation,
                                     false, true);
@@ -1073,12 +1035,12 @@ nsXULContentBuilder::CreateContainerContents(nsIContent* aElement,
         xulcontent->SetTemplateGenerated();
     }
 
-    PRInt32 newIndexInContainer = -1;
-    nsIContent* container = nsnull;
+    int32_t newIndexInContainer = -1;
+    nsIContent* container = nullptr;
 
-    PRInt32 querySetCount = mQuerySets.Length();
+    int32_t querySetCount = mQuerySets.Length();
 
-    for (PRInt32 r = 0; r < querySetCount; r++) {
+    for (int32_t r = 0; r < querySetCount; r++) {
         nsTemplateQuerySet* queryset = mQuerySets[r];
 
         nsIAtom* tag = queryset->GetTag();
@@ -1108,7 +1070,7 @@ nsXULContentBuilder::CreateContainerContentsForQuerySet(nsIContent* aElement,
                                                         bool aNotify,
                                                         nsTemplateQuerySet* aQuerySet,
                                                         nsIContent** aContainer,
-                                                        PRInt32* aNewIndexInContainer)
+                                                        int32_t* aNewIndexInContainer)
 {
 #ifdef PR_LOGGING
     if (PR_LOG_TEST(gXULTemplateLog, PR_LOG_DEBUG)) {
@@ -1153,7 +1115,7 @@ nsXULContentBuilder::CreateContainerContentsForQuerySet(nsIContent* aElement,
             continue;
 
         nsTemplateMatch *newmatch =
-            nsTemplateMatch::Create(mPool, aQuerySet->Priority(),
+            nsTemplateMatch::Create(aQuerySet->Priority(),
                                     nextresult, aElement);
         if (!newmatch)
             return NS_ERROR_OUT_OF_MEMORY;
@@ -1164,16 +1126,16 @@ nsXULContentBuilder::CreateContainerContentsForQuerySet(nsIContent* aElement,
 
         bool generateContent = true;
 
-        nsTemplateMatch* prevmatch = nsnull;
-        nsTemplateMatch* existingmatch = nsnull;
-        nsTemplateMatch* removematch = nsnull;
+        nsTemplateMatch* prevmatch = nullptr;
+        nsTemplateMatch* existingmatch = nullptr;
+        nsTemplateMatch* removematch = nullptr;
         if (mMatchMap.Get(resultid, &existingmatch)){
             // check if there is an existing match that matched a rule
             while (existingmatch) {
                 // break out once we've reached a query in the list with a
                 // higher priority, as the new match list is sorted by
                 // priority, and the new match should be inserted here
-                PRInt32 priority = existingmatch->QuerySetPriority();
+                int32_t priority = existingmatch->QuerySetPriority();
                 if (priority > aQuerySet->Priority())
                     break;
 
@@ -1197,7 +1159,7 @@ nsXULContentBuilder::CreateContainerContentsForQuerySet(nsIContent* aElement,
 
         if (removematch) {
             // remove the generated content for the existing match
-            rv = ReplaceMatch(removematch->mResult, nsnull, nsnull, aElement);
+            rv = ReplaceMatch(removematch->mResult, nullptr, nullptr, aElement);
             if (NS_FAILED(rv))
                 return rv;
 
@@ -1209,12 +1171,12 @@ nsXULContentBuilder::CreateContainerContentsForQuerySet(nsIContent* aElement,
             // find the rule that matches. If none match, the content does not
             // need to be generated
 
-            PRInt16 ruleindex;
-            nsTemplateRule* matchedrule = nsnull;
+            int16_t ruleindex;
+            nsTemplateRule* matchedrule = nullptr;
             rv = DetermineMatchedRule(aElement, nextresult, aQuerySet,
                                       &matchedrule, &ruleindex);
             if (NS_FAILED(rv)) {
-                nsTemplateMatch::Destroy(mPool, newmatch, false);
+                nsTemplateMatch::Destroy(newmatch, false);
                 return rv;
             }
 
@@ -1222,7 +1184,7 @@ nsXULContentBuilder::CreateContainerContentsForQuerySet(nsIContent* aElement,
                 rv = newmatch->RuleMatched(aQuerySet, matchedrule,
                                            ruleindex, nextresult);
                 if (NS_FAILED(rv)) {
-                    nsTemplateMatch::Destroy(mPool, newmatch, false);
+                    nsTemplateMatch::Destroy(newmatch, false);
                     return rv;
                 }
 
@@ -1241,14 +1203,13 @@ nsXULContentBuilder::CreateContainerContentsForQuerySet(nsIContent* aElement,
         if (prevmatch) {
             prevmatch->mNext = newmatch;
         }
-        else if (!mMatchMap.Put(resultid, newmatch)) {
-            nsTemplateMatch::Destroy(mPool, newmatch, true);
-            return NS_ERROR_OUT_OF_MEMORY;
+        else {
+            mMatchMap.Put(resultid, newmatch);
         }
 
         if (removematch) {
             newmatch->mNext = removematch->mNext;
-            nsTemplateMatch::Destroy(mPool, removematch, true);
+            nsTemplateMatch::Destroy(removematch, true);
         }
         else {
             newmatch->mNext = existingmatch;
@@ -1260,7 +1221,7 @@ nsXULContentBuilder::CreateContainerContentsForQuerySet(nsIContent* aElement,
 
 nsresult
 nsXULContentBuilder::EnsureElementHasGenericChild(nsIContent* parent,
-                                                  PRInt32 nameSpaceID,
+                                                  int32_t nameSpaceID,
                                                   nsIAtom* tag,
                                                   bool aNotify,
                                                   nsIContent** result)
@@ -1318,17 +1279,17 @@ nsXULContentBuilder::RemoveGeneratedContent(nsIContent* aElement)
     // Keep a queue of "ungenerated" elements that we have to probe
     // for generated content.
     nsAutoTArray<nsIContent*, 8> ungenerated;
-    if (ungenerated.AppendElement(aElement) == nsnull)
+    if (ungenerated.AppendElement(aElement) == nullptr)
         return NS_ERROR_OUT_OF_MEMORY;
 
-    PRUint32 count;
+    uint32_t count;
     while (0 != (count = ungenerated.Length())) {
         // Pull the next "ungenerated" element off the queue.
-        PRUint32 last = count - 1;
+        uint32_t last = count - 1;
         nsCOMPtr<nsIContent> element = ungenerated[last];
         ungenerated.RemoveElementAt(last);
 
-        PRUint32 i = element->GetChildCount();
+        uint32_t i = element->GetChildCount();
 
         while (i-- > 0) {
             nsCOMPtr<nsIContent> child = element->GetChildAt(i);
@@ -1351,7 +1312,7 @@ nsXULContentBuilder::RemoveGeneratedContent(nsIContent* aElement)
             if (! tmpl) {
                 // No 'template' attribute, so this must not have been
                 // generated. We'll need to examine its kids.
-                if (ungenerated.AppendElement(child) == nsnull)
+                if (ungenerated.AppendElement(child) == nullptr)
                     return NS_ERROR_OUT_OF_MEMORY;
                 continue;
             }
@@ -1389,18 +1350,18 @@ nsXULContentBuilder::GetElementsForResult(nsIXULTemplateResult* aResult,
 }
 
 nsresult
-nsXULContentBuilder::CreateElement(PRInt32 aNameSpaceID,
+nsXULContentBuilder::CreateElement(int32_t aNameSpaceID,
                                    nsIAtom* aTag,
                                    nsIContent** aResult)
 {
     nsCOMPtr<nsIDocument> doc = mRoot->GetDocument();
-    NS_ASSERTION(doc != nsnull, "not initialized");
+    NS_ASSERTION(doc != nullptr, "not initialized");
     if (! doc)
         return NS_ERROR_NOT_INITIALIZED;
 
     nsCOMPtr<nsIContent> result;
     nsCOMPtr<nsINodeInfo> nodeInfo =
-        doc->NodeInfoManager()->GetNodeInfo(aTag, nsnull, aNameSpaceID,
+        doc->NodeInfoManager()->GetNodeInfo(aTag, nullptr, aNameSpaceID,
                                             nsIDOMNode::ELEMENT_NODE);
 
     nsresult rv = NS_NewElement(getter_AddRefs(result), nodeInfo.forget(),
@@ -1418,7 +1379,7 @@ nsXULContentBuilder::SetContainerAttrs(nsIContent *aElement,
                                        bool aIgnoreNonContainers,
                                        bool aNotify)
 {
-    NS_PRECONDITION(aResult != nsnull, "null ptr");
+    NS_PRECONDITION(aResult != nullptr, "null ptr");
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
@@ -1460,7 +1421,7 @@ nsXULContentBuilder::SetContainerAttrs(nsIContent *aElement,
 NS_IMETHODIMP
 nsXULContentBuilder::CreateContents(nsIContent* aElement, bool aForceCreation)
 {
-    NS_PRECONDITION(aElement != nsnull, "null ptr");
+    NS_PRECONDITION(aElement != nullptr, "null ptr");
     if (! aElement)
         return NS_ERROR_NULL_POINTER;
 
@@ -1505,9 +1466,9 @@ nsXULContentBuilder::HasGeneratedContent(nsIRDFResource* aResource,
         nsCOMArray<nsIContent> elements;
         xuldoc->GetElementsForID(refID, elements);
 
-        PRUint32 cnt = elements.Count();
+        uint32_t cnt = elements.Count();
 
-        for (PRInt32 i = PRInt32(cnt) - 1; i >= 0; --i) {
+        for (int32_t i = int32_t(cnt) - 1; i >= 0; --i) {
             nsCOMPtr<nsIContent> content = elements.SafeObjectAt(i);
 
             do {
@@ -1540,11 +1501,11 @@ nsXULContentBuilder::GetResultForContent(nsIDOMElement* aElement,
         *aResult = mRootResult;
     }
     else {
-        nsTemplateMatch *match = nsnull;
+        nsTemplateMatch *match = nullptr;
         if (mContentSupportMap.Get(content, &match))
             *aResult = match->mResult;
         else
-            *aResult = nsnull;
+            *aResult = nullptr;
     }
 
     NS_IF_ADDREF(*aResult);
@@ -1559,9 +1520,9 @@ nsXULContentBuilder::GetResultForContent(nsIDOMElement* aElement,
 void
 nsXULContentBuilder::AttributeChanged(nsIDocument* aDocument,
                                       Element*     aElement,
-                                      PRInt32      aNameSpaceID,
+                                      int32_t      aNameSpaceID,
                                       nsIAtom*     aAttribute,
-                                      PRInt32      aModType)
+                                      int32_t      aModType)
 {
     nsCOMPtr<nsIMutationObserver> kungFuDeathGrip(this);
 
@@ -1610,7 +1571,7 @@ bool
 nsXULContentBuilder::GetInsertionLocations(nsIXULTemplateResult* aResult,
                                            nsCOMArray<nsIContent>** aLocations)
 {
-    *aLocations = nsnull;
+    *aLocations = nullptr;
 
     nsAutoString ref;
     nsresult rv = aResult->GetBindingFor(mRefVariable, ref);
@@ -1625,11 +1586,11 @@ nsXULContentBuilder::GetInsertionLocations(nsIXULTemplateResult* aResult,
     NS_ENSURE_TRUE(*aLocations, false);
 
     xuldoc->GetElementsForID(ref, **aLocations);
-    PRUint32 count = (*aLocations)->Count();
+    uint32_t count = (*aLocations)->Count();
 
     bool found = false;
 
-    for (PRUint32 t = 0; t < count; t++) {
+    for (uint32_t t = 0; t < count; t++) {
         nsCOMPtr<nsIContent> content = (*aLocations)->SafeObjectAt(t);
 
         nsTemplateMatch* refmatch;
@@ -1647,7 +1608,7 @@ nsXULContentBuilder::GetInsertionLocations(nsIXULTemplateResult* aResult,
         }
 
         // clear the item in the list since we don't want to insert there
-        (*aLocations)->ReplaceObjectAt(nsnull, t);
+        (*aLocations)->ReplaceObjectAt(nullptr, t);
     }
 
     return found;
@@ -1690,9 +1651,9 @@ nsXULContentBuilder::ReplaceMatch(nsIXULTemplateResult* aOldResult,
         if (NS_FAILED(rv))
             return rv;
 
-        PRUint32 count = elements.Count();
+        uint32_t count = elements.Count();
 
-        for (PRInt32 e = PRInt32(count) - 1; e >= 0; --e) {
+        for (int32_t e = int32_t(count) - 1; e >= 0; --e) {
             nsCOMPtr<nsIContent> child = elements.SafeObjectAt(e);
 
             nsTemplateMatch* match;
@@ -1708,7 +1669,7 @@ nsXULContentBuilder::ReplaceMatch(nsIXULTemplateResult* aOldResult,
         return BuildContentFromTemplate(action, content, content, true,
                                         mRefVariable == aNewMatchRule->GetMemberVariable(),
                                         aNewMatch->mResult, true, aNewMatch,
-                                        nsnull, nsnull);
+                                        nullptr, nullptr);
     }
 
     return NS_OK;
@@ -1721,9 +1682,9 @@ nsXULContentBuilder::SynchronizeResult(nsIXULTemplateResult* aResult)
     nsCOMArray<nsIContent> elements;
     GetElementsForResult(aResult, elements);
 
-    PRUint32 cnt = elements.Count();
+    uint32_t cnt = elements.Count();
 
-    for (PRInt32 i = PRInt32(cnt) - 1; i >= 0; --i) {
+    for (int32_t i = int32_t(cnt) - 1; i >= 0; --i) {
         nsCOMPtr<nsIContent> element = elements.SafeObjectAt(i);
 
         nsTemplateMatch* match;
@@ -1827,13 +1788,13 @@ nsXULContentBuilder::RebuildAll()
 nsresult
 nsXULContentBuilder::CompareResultToNode(nsIXULTemplateResult* aResult,
                                          nsIContent* aContent,
-                                         PRInt32* aSortOrder)
+                                         int32_t* aSortOrder)
 {
     NS_ASSERTION(aSortOrder, "CompareResultToNode: null out param aSortOrder");
   
     *aSortOrder = 0;
 
-    nsTemplateMatch *match = nsnull;
+    nsTemplateMatch *match = nullptr;
     if (!mContentSupportMap.Get(aContent, &match)) {
         *aSortOrder = mSortState.sortStaticsLast ? -1 : 1;
         return NS_OK;
@@ -1845,15 +1806,15 @@ nsXULContentBuilder::CompareResultToNode(nsIXULTemplateResult* aResult,
     if (mSortState.direction == nsSortState_natural) {
         // sort in natural order
         nsresult rv = mQueryProcessor->CompareResults(aResult, match->mResult,
-                                                      nsnull, mSortState.sortHints,
+                                                      nullptr, mSortState.sortHints,
                                                       aSortOrder);
         NS_ENSURE_SUCCESS(rv, rv);
     }
     else {
         // iterate over each sort key and compare. If the nodes are equal,
         // continue to compare using the next sort key. If not equal, stop.
-        PRInt32 length = mSortState.sortKeys.Count();
-        for (PRInt32 t = 0; t < length; t++) {
+        int32_t length = mSortState.sortKeys.Count();
+        for (int32_t t = 0; t < length; t++) {
             nsresult rv = mQueryProcessor->CompareResults(aResult, match->mResult,
                                                           mSortState.sortKeys[t],
                                                           mSortState.sortHints, aSortOrder);
@@ -1910,26 +1871,26 @@ nsXULContentBuilder::InsertSortedNode(nsIContent* aContainer,
     }
 
     bool childAdded = false;
-    PRUint32 numChildren = aContainer->GetChildCount();
+    uint32_t numChildren = aContainer->GetChildCount();
 
     if (mSortState.direction != nsSortState_natural ||
         (mSortState.direction == nsSortState_natural && mSortState.isContainerRDFSeq))
     {
         // because numChildren gets modified
-        PRInt32 realNumChildren = numChildren;
-        nsIContent *child = nsnull;
+        int32_t realNumChildren = numChildren;
+        nsIContent *child = nullptr;
 
         // rjc says: determine where static XUL ends and generated XUL/RDF begins
-        PRInt32 staticCount = 0;
+        int32_t staticCount = 0;
 
         nsAutoString staticValue;
         aContainer->GetAttr(kNameSpaceID_None, nsGkAtoms::staticHint, staticValue);
         if (!staticValue.IsEmpty())
         {
             // found "static" XUL element count hint
-            PRInt32 strErr = 0;
+            nsresult strErr = NS_OK;
             staticCount = staticValue.ToInteger(&strErr);
-            if (strErr)
+            if (NS_FAILED(strErr))
                 staticCount = 0;
         } else {
             // compute the "static" XUL element count
@@ -1959,7 +1920,7 @@ nsXULContentBuilder::InsertSortedNode(nsIContent* aContainer,
         if (staticCount <= 0) {
             numChildren += staticCount;
             staticCount = 0;
-        } else if (staticCount > (PRInt32)numChildren) {
+        } else if (staticCount > (int32_t)numChildren) {
             staticCount = numChildren;
             numChildren -= staticCount;
         }
@@ -1967,7 +1928,7 @@ nsXULContentBuilder::InsertSortedNode(nsIContent* aContainer,
         // figure out where to insert the node when a sort order is being imposed
         if (numChildren > 0) {
             nsIContent *temp;
-            PRInt32 direction;
+            int32_t direction;
 
             // rjc says: The following is an implementation of a fairly optimal
             // binary search insertion sort... with interpolation at either end-point.
@@ -1992,7 +1953,7 @@ nsXULContentBuilder::InsertSortedNode(nsIContent* aContainer,
                     mSortState.lastWasLast = false;
             }
 
-            PRInt32 left = staticCount + 1, right = realNumChildren, x;
+            int32_t left = staticCount + 1, right = realNumChildren, x;
             while (!childAdded && right >= left) {
                 x = (left + right) / 2;
                 child = aContainer->GetChildAt(x - 1);
@@ -2003,7 +1964,7 @@ nsXULContentBuilder::InsertSortedNode(nsIContent* aContainer,
                     (x == right && direction >= 0) ||
                     left == right)
                 {
-                    PRInt32 thePos = (direction > 0 ? x : x - 1);
+                    int32_t thePos = (direction > 0 ? x : x - 1);
                     aContainer->InsertChildAt(aNode, thePos, aNotify);
                     childAdded = true;
 
