@@ -1,51 +1,13 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Robert Churchill <rjc@netscape.com>
- *   David Hyatt <hyatt@netscape.com>
- *   Chris Waterson <waterson@netscape.com>
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   Neil Deakin <enndeakin@sympatico.ca>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef nsXULTemplateBuilder_h__
 #define nsXULTemplateBuilder_h__
 
 #include "nsStubDocumentObserver.h"
 #include "nsIScriptSecurityManager.h"
-#include "nsIContent.h"
 #include "nsIObserver.h"
 #include "nsIRDFCompositeDataSource.h"
 #include "nsIRDFContainer.h"
@@ -55,7 +17,6 @@
 #include "nsIRDFService.h"
 #include "nsIXULTemplateBuilder.h"
 
-#include "nsFixedSizeAllocator.h"
 #include "nsCOMArray.h"
 #include "nsTArray.h"
 #include "nsDataHashtable.h"
@@ -69,9 +30,10 @@
 extern PRLogModuleInfo* gXULTemplateLog;
 #endif
 
-class nsIXULDocument;
-class nsIRDFCompositeDataSource;
+class nsIContent;
 class nsIObserverService;
+class nsIRDFCompositeDataSource;
+class nsIXULDocument;
 
 /**
  * An object that translates an RDF graph into a presentation using a
@@ -157,7 +119,7 @@ public:
     void RunnableLoadAndRebuild() {
       Uninit(false);  // Reset results
 
-      nsCOMPtr<nsIDocument> doc = mRoot ? mRoot->GetDocument() : nsnull;
+      nsCOMPtr<nsIDocument> doc = mRoot ? mRoot->GetDocument() : nullptr;
       if (doc) {
         bool shouldDelay;
         LoadDataSources(doc, &shouldDelay);
@@ -169,8 +131,8 @@ public:
 
     // mRoot should not be cleared until after Uninit is finished so that
     // generated content can be removed during uninitialization.
-    void UninitFalse() { Uninit(false); mRoot = nsnull; }
-    void UninitTrue() { Uninit(true); mRoot = nsnull; }
+    void UninitFalse() { Uninit(false); mRoot = nullptr; }
+    void UninitTrue() { Uninit(true); mRoot = nullptr; }
 
     /**
      * Find the <template> tag that applies for this builder
@@ -204,7 +166,7 @@ public:
     CompileTemplate(nsIContent* aTemplate,
                     nsTemplateQuerySet* aQuerySet,
                     bool aIsQuerySet,
-                    PRInt32* aPriority,
+                    int32_t* aPriority,
                     bool* aCanUseTemplate);
 
     /**
@@ -339,7 +301,7 @@ public:
                          nsIXULTemplateResult* aResult,
                          nsTemplateQuerySet* aQuerySet,
                          nsTemplateRule** aMatchedRule,
-                         PRInt16 *aRuleIndex);
+                         int16_t *aRuleIndex);
 
     // XXX sigh, the string template foo doesn't mix with
     // operator->*() on egcs-1.1.2, so we'll need to explicitly pass
@@ -419,16 +381,6 @@ protected:
      */
     nsDataHashtable<nsISupportsHashKey, nsTemplateMatch*> mMatchMap;
 
-    /**
-     * Fixed size allocator used to allocate matches
-     */
-    nsFixedSizeAllocator mPool;
-
-public:
-
-    nsFixedSizeAllocator& GetPool() { return mPool; }
-
-protected:
     // pseudo-constants
     static nsrefcnt gRefCnt;
     static nsIRDFService*            gRDFService;
@@ -443,7 +395,7 @@ protected:
         eLoggingEnabled = (1 << 2)
     };
 
-    PRInt32 mFlags;
+    int32_t mFlags;
 
     /**
      * Stack-based helper class to maintain a list of ``activated''
