@@ -7,7 +7,6 @@
 #ifndef nsXULTemplateQueryProcessorRDF_h__
 #define nsXULTemplateQueryProcessorRDF_h__
 
-#include "nsIContent.h"
 #include "nsIRDFContainer.h"
 #include "nsIRDFContainerUtils.h"
 #include "nsIRDFDataSource.h"
@@ -15,17 +14,14 @@
 #include "nsIRDFService.h"
 #include "nsIXULTemplateBuilder.h"
 #include "nsIXULTemplateQueryProcessor.h"
-#include "nsICollation.h"
 #include "nsCollationCID.h"
 
-#include "nsFixedSizeAllocator.h"
 #include "nsResourceSet.h"
 #include "nsRuleNetwork.h"
 #include "nsRDFQuery.h"
 #include "nsRDFBinding.h"
 #include "nsXULTemplateResultSetRDF.h"
 #include "nsCOMArray.h"
-#include "nsIArray.h"
 #include "nsString.h"
 #include "nsClassHashtable.h"
 #include "nsRefPtrHashtable.h"
@@ -37,6 +33,7 @@
 extern PRLogModuleInfo* gXULTemplateLog;
 #endif
 
+class nsIContent;
 class nsIRDFCompositeDataSource;
 class nsXULTemplateResultRDF;
 
