@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Mozilla browser.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications, Inc.
- * Portions created by the Initial Developer are Copyright (C) 1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Radha Kulkarni <radha@netscape.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 // Local Includes
 #include "nsSHEntry.h"
@@ -42,8 +9,6 @@
 #include "nsReadableUtils.h"
 #include "nsIDocShellLoadInfo.h"
 #include "nsIDocShellTreeItem.h"
-#include "nsIDocument.h"
-#include "nsIDOMDocument.h"
 #include "nsISHistory.h"
 #include "nsISHistoryInternal.h"
 #include "nsDocShellEditorData.h"
@@ -51,10 +16,12 @@
 #include "nsILayoutHistoryState.h"
 #include "nsIContentViewer.h"
 #include "nsISupportsArray.h"
+#include "nsIStructuredCloneContainer.h"
+#include <algorithm>
 
 namespace dom = mozilla::dom;
 
-static PRUint32 gEntryID = 0;
+static uint32_t gEntryID = 0;
 
 //*****************************************************************************
 //***    nsSHEntry: Object Management
@@ -66,7 +33,7 @@ nsSHEntry::nsSHEntry()
   , mID(gEntryID++)
   , mScrollPositionX(0)
   , mScrollPositionY(0)
-  , mParent(nsnull)
+  , mParent(nullptr)
   , mURIWasModified(false)
 {
   mShared = new nsSHEntryShared();
@@ -92,7 +59,7 @@ static bool
 ClearParentPtr(nsISHEntry* aEntry, void* /* aData */)
 {
   if (aEntry) {
-    aEntry->SetParent(nsnull);
+    aEntry->SetParent(nullptr);
   }
   return true;
 }
@@ -100,7 +67,7 @@ ClearParentPtr(nsISHEntry* aEntry, void* /* aData */)
 nsSHEntry::~nsSHEntry()
 {
   // Null out the mParent pointers on all our kids.
-  mChildren.EnumerateForwards(ClearParentPtr, nsnull);
+  mChildren.EnumerateForwards(ClearParentPtr, nullptr);
 }
 
 //*****************************************************************************
@@ -114,14 +81,14 @@ NS_IMPL_ISUPPORTS4(nsSHEntry, nsISHContainer, nsISHEntry, nsIHistoryEntry,
 //    nsSHEntry: nsISHEntry
 //*****************************************************************************
 
-NS_IMETHODIMP nsSHEntry::SetScrollPosition(PRInt32 x, PRInt32 y)
+NS_IMETHODIMP nsSHEntry::SetScrollPosition(int32_t x, int32_t y)
 {
   mScrollPositionX = x;
   mScrollPositionY = y;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsSHEntry::GetScrollPosition(PRInt32 *x, PRInt32 *y)
+NS_IMETHODIMP nsSHEntry::GetScrollPosition(int32_t *x, int32_t *y)
 {
   *x = mScrollPositionX;
   *y = mScrollPositionY;
@@ -197,7 +164,7 @@ nsSHEntry::GetAnyContentViewer(nsISHEntry **aOwnerEntry,
     return NS_OK;
   }
   // The root SHEntry doesn't have a ContentViewer, so check child nodes
-  for (PRInt32 i = 0; i < mChildren.Count(); i++) {
+  for (int32_t i = 0; i < mChildren.Count(); i++) {
     nsISHEntry* child = mChildren[i];
     if (child) {
 #ifdef DEBUG_PAGE_CACHE
@@ -231,7 +198,7 @@ NS_IMETHODIMP nsSHEntry::GetTitle(PRUnichar** aTitle)
   // Check for empty title...
   if (mTitle.IsEmpty() && mURI) {
     // Default title is the URL.
-    nsCAutoString spec;
+    nsAutoCString spec;
     if (NS_SUCCEEDED(mURI->GetSpec(spec)))
       AppendUTF8toUTF16(spec, mTitle);
   }
@@ -277,25 +244,25 @@ NS_IMETHODIMP nsSHEntry::SetLayoutHistoryState(nsILayoutHistoryState* aState)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsSHEntry::GetLoadType(PRUint32 * aResult)
+NS_IMETHODIMP nsSHEntry::GetLoadType(uint32_t * aResult)
 {
   *aResult = mLoadType;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsSHEntry::SetLoadType(PRUint32  aLoadType)
+NS_IMETHODIMP nsSHEntry::SetLoadType(uint32_t  aLoadType)
 {
   mLoadType = aLoadType;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsSHEntry::GetID(PRUint32 * aResult)
+NS_IMETHODIMP nsSHEntry::GetID(uint32_t * aResult)
 {
   *aResult = mID;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsSHEntry::SetID(PRUint32  aID)
+NS_IMETHODIMP nsSHEntry::SetID(uint32_t  aID)
 {
   mID = aID;
   return NS_OK;
@@ -377,14 +344,14 @@ nsSHEntry::Create(nsIURI * aURI, const nsAString &aTitle,
                   nsILayoutHistoryState * aLayoutHistoryState,
                   nsISupports * aCacheKey, const nsACString& aContentType,
                   nsISupports* aOwner,
-                  PRUint64 aDocShellID, bool aDynamicCreation)
+                  uint64_t aDocShellID, bool aDynamicCreation)
 {
   mURI = aURI;
   mTitle = aTitle;
   mPostData = aInputStream;
 
   // Set the LoadType by default to loadHistory during creation
-  mLoadType = (PRUint32) nsIDocShellLoadInfo::loadHistory;
+  mLoadType = (uint32_t) nsIDocShellLoadInfo::loadHistory;
 
   mShared->mCacheKey = aCacheKey;
   mShared->mContentType = aContentType;
@@ -529,14 +496,14 @@ nsSHEntry::AbandonBFCacheEntry()
 //*****************************************************************************
 
 NS_IMETHODIMP 
-nsSHEntry::GetChildCount(PRInt32 * aCount)
+nsSHEntry::GetChildCount(int32_t * aCount)
 {
   *aCount = mChildren.Count();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSHEntry::AddChild(nsISHEntry * aChild, PRInt32 aOffset)
+nsSHEntry::AddChild(nsISHEntry * aChild, int32_t aOffset)
 {
   if (aChild) {
     NS_ENSURE_SUCCESS(aChild->SetParent(this), NS_ERROR_FAILURE);
@@ -566,8 +533,8 @@ nsSHEntry::AddChild(nsISHEntry * aChild, PRInt32 aOffset)
   // If the new child is dynamically added, try to add it to aOffset, but if
   // there are non-dynamically added children, the child must be after those.
   if (newChildIsDyn) {
-    PRInt32 lastNonDyn = aOffset - 1;
-    for (PRInt32 i = aOffset; i < mChildren.Count(); ++i) {
+    int32_t lastNonDyn = aOffset - 1;
+    for (int32_t i = aOffset; i < mChildren.Count(); ++i) {
       nsISHEntry* entry = mChildren[i];
       if (entry) {
         bool dyn = false;
@@ -587,7 +554,7 @@ nsSHEntry::AddChild(nsISHEntry * aChild, PRInt32 aOffset)
     }
     if (!mChildren.InsertObjectAt(aChild, lastNonDyn + 1)) {
       NS_WARNING("Adding a child failed!");
-      aChild->SetParent(nsnull);
+      aChild->SetParent(nullptr);
       return NS_ERROR_FAILURE;
     }
   } else {
@@ -595,10 +562,10 @@ nsSHEntry::AddChild(nsISHEntry * aChild, PRInt32 aOffset)
     // If there are dynamically added children before that, those must be
     // moved to be after aOffset.
     if (mChildren.Count() > 0) {
-      PRInt32 start = NS_MIN(mChildren.Count() - 1, aOffset);
-      PRInt32 dynEntryIndex = -1;
-      nsISHEntry* dynEntry = nsnull;
-      for (PRInt32 i = start; i >= 0; --i) {
+      int32_t start = std::min(mChildren.Count() - 1, aOffset);
+      int32_t dynEntryIndex = -1;
+      nsISHEntry* dynEntry = nullptr;
+      for (int32_t i = start; i >= 0; --i) {
         nsISHEntry* entry = mChildren[i];
         if (entry) {
           bool dyn = false;
@@ -626,13 +593,13 @@ nsSHEntry::AddChild(nsISHEntry * aChild, PRInt32 aOffset)
       nsISHEntry* oldChild = mChildren[aOffset];
       if (oldChild && oldChild != aChild) {
         NS_ERROR("Adding a child where we already have a child? This may misbehave");
-        oldChild->SetParent(nsnull);
+        oldChild->SetParent(nullptr);
       }
     }
 
     if (!mChildren.ReplaceObjectAt(aChild, aOffset)) {
       NS_WARNING("Adding a child failed!");
-      aChild->SetParent(nsnull);
+      aChild->SetParent(nullptr);
       return NS_ERROR_FAILURE;
     }
   }
@@ -650,16 +617,16 @@ nsSHEntry::RemoveChild(nsISHEntry * aChild)
   if (dynamic) {
     childRemoved = mChildren.RemoveObject(aChild);
   } else {
-    PRInt32 index = mChildren.IndexOfObject(aChild);
+    int32_t index = mChildren.IndexOfObject(aChild);
     if (index >= 0) {
-      childRemoved = mChildren.ReplaceObjectAt(nsnull, index);
+      childRemoved = mChildren.ReplaceObjectAt(nullptr, index);
     }
   }
   if (childRemoved) {
-    aChild->SetParent(nsnull);
+    aChild->SetParent(nullptr);
 
     // reduce the child count, i.e. remove empty children at the end
-    for (PRInt32 i = mChildren.Count() - 1; i >= 0 && !mChildren[i]; --i) {
+    for (int32_t i = mChildren.Count() - 1; i >= 0 && !mChildren[i]; --i) {
       if (!mChildren.RemoveObjectAt(i)) {
         break;
       }
@@ -669,7 +636,7 @@ nsSHEntry::RemoveChild(nsISHEntry * aChild)
 }
 
 NS_IMETHODIMP
-nsSHEntry::GetChildAt(PRInt32 aIndex, nsISHEntry ** aResult)
+nsSHEntry::GetChildAt(int32_t aIndex, nsISHEntry ** aResult)
 {
   if (aIndex >= 0 && aIndex < mChildren.Count()) {
     *aResult = mChildren[aIndex];
@@ -677,7 +644,7 @@ nsSHEntry::GetChildAt(PRInt32 aIndex, nsISHEntry ** aResult)
     // that possible.
     NS_IF_ADDREF(*aResult);
   } else {
-    *aResult = nsnull;
+    *aResult = nullptr;
   }
   return NS_OK;
 }
@@ -691,7 +658,7 @@ nsSHEntry::AddChildShell(nsIDocShellTreeItem *aShell)
 }
 
 NS_IMETHODIMP
-nsSHEntry::ChildShellAt(PRInt32 aIndex, nsIDocShellTreeItem **aShell)
+nsSHEntry::ChildShellAt(int32_t aIndex, nsIDocShellTreeItem **aShell)
 {
   NS_IF_ADDREF(*aShell = mShared->mChildShells.SafeObjectAt(aIndex));
   return NS_OK;
@@ -756,7 +723,7 @@ nsSHEntry::SetEditorData(nsDocShellEditorData* aData)
 bool
 nsSHEntry::HasDetachedEditor()
 {
-  return mShared->mEditorData != nsnull;
+  return mShared->mEditorData != nullptr;
 }
 
 NS_IMETHODIMP
@@ -785,7 +752,7 @@ NS_IMETHODIMP
 nsSHEntry::HasDynamicallyAddedChild(bool* aAdded)
 {
   *aAdded = false;
-  for (PRInt32 i = 0; i < mChildren.Count(); ++i) {
+  for (int32_t i = 0; i < mChildren.Count(); ++i) {
     nsISHEntry* entry = mChildren[i];
     if (entry) {
       entry->IsDynamicallyAdded(aAdded);
@@ -798,14 +765,14 @@ nsSHEntry::HasDynamicallyAddedChild(bool* aAdded)
 }
 
 NS_IMETHODIMP
-nsSHEntry::GetDocshellID(PRUint64* aID)
+nsSHEntry::GetDocshellID(uint64_t* aID)
 {
   *aID = mShared->mDocShellID;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSHEntry::SetDocshellID(PRUint64 aID)
+nsSHEntry::SetDocshellID(uint64_t aID)
 {
   mShared->mDocShellID = aID;
   return NS_OK;
@@ -813,14 +780,14 @@ nsSHEntry::SetDocshellID(PRUint64 aID)
 
 
 NS_IMETHODIMP
-nsSHEntry::GetLastTouched(PRUint32 *aLastTouched)
+nsSHEntry::GetLastTouched(uint32_t *aLastTouched)
 {
   *aLastTouched = mShared->mLastTouched;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSHEntry::SetLastTouched(PRUint32 aLastTouched)
+nsSHEntry::SetLastTouched(uint32_t aLastTouched)
 {
   mShared->mLastTouched = aLastTouched;
   return NS_OK;
