@@ -14,7 +14,6 @@
 #include "nsIPresShell.h"
 #include "nsPresContext.h"
 #include "nsIDocShell.h"
-#include "nsIDocShellTreeItem.h"
 #include "nsIWebNavigation.h"
 #include "nsIHistoryEntry.h"
 #include "nsIURI.h"
@@ -65,6 +64,10 @@ NS_IMPL_RELEASE(nsHistory)
 NS_IMETHODIMP
 nsHistory::GetLength(int32_t* aLength)
 {
+  nsCOMPtr<nsPIDOMWindow> win(do_QueryReferent(mInnerWindow));
+  if (!win || !nsContentUtils::CanCallerAccess(win->GetOuterWindow()))
+    return NS_ERROR_DOM_SECURITY_ERR;
+
   nsCOMPtr<nsISHistory>   sHistory;
 
   // Get session History from docshell
@@ -76,11 +79,11 @@ nsHistory::GetLength(int32_t* aLength)
 NS_IMETHODIMP
 nsHistory::GetCurrent(nsAString& aCurrent)
 {
-  if (!nsContentUtils::IsCallerTrustedForRead())
+  if (!nsContentUtils::IsCallerChrome())
     return NS_ERROR_DOM_SECURITY_ERR;
 
   int32_t curIndex=0;
-  nsCAutoString curURL;
+  nsAutoCString curURL;
   nsCOMPtr<nsISHistory> sHistory;
 
   // Get SessionHistory from docshell
@@ -108,11 +111,11 @@ nsHistory::GetCurrent(nsAString& aCurrent)
 NS_IMETHODIMP
 nsHistory::GetPrevious(nsAString& aPrevious)
 {
-  if (!nsContentUtils::IsCallerTrustedForRead())
+  if (!nsContentUtils::IsCallerChrome())
     return NS_ERROR_DOM_SECURITY_ERR;
 
   int32_t curIndex;
-  nsCAutoString prevURL;
+  nsAutoCString prevURL;
   nsCOMPtr<nsISHistory>  sHistory;
 
   // Get session History from docshell
@@ -140,11 +143,11 @@ nsHistory::GetPrevious(nsAString& aPrevious)
 NS_IMETHODIMP
 nsHistory::GetNext(nsAString& aNext)
 {
-  if (!nsContentUtils::IsCallerTrustedForRead())
+  if (!nsContentUtils::IsCallerChrome())
     return NS_ERROR_DOM_SECURITY_ERR;
 
   int32_t curIndex;
-  nsCAutoString nextURL;
+  nsAutoCString nextURL;
   nsCOMPtr<nsISHistory>  sHistory;
 
   // Get session History from docshell
@@ -172,6 +175,10 @@ nsHistory::GetNext(nsAString& aNext)
 NS_IMETHODIMP
 nsHistory::Back()
 {
+  nsCOMPtr<nsPIDOMWindow> win(do_QueryReferent(mInnerWindow));
+  if (!win || !nsContentUtils::CanCallerAccess(win->GetOuterWindow()))
+    return NS_ERROR_DOM_SECURITY_ERR;
+
   nsCOMPtr<nsISHistory>  sHistory;
 
   GetSessionHistoryFromDocShell(GetDocShell(), getter_AddRefs(sHistory));
@@ -188,6 +195,10 @@ nsHistory::Back()
 NS_IMETHODIMP
 nsHistory::Forward()
 {
+  nsCOMPtr<nsPIDOMWindow> win(do_QueryReferent(mInnerWindow));
+  if (!win || !nsContentUtils::CanCallerAccess(win->GetOuterWindow()))
+    return NS_ERROR_DOM_SECURITY_ERR;
+
   nsCOMPtr<nsISHistory>  sHistory;
 
   GetSessionHistoryFromDocShell(GetDocShell(), getter_AddRefs(sHistory));
@@ -204,6 +215,10 @@ nsHistory::Forward()
 NS_IMETHODIMP
 nsHistory::Go(int32_t aDelta)
 {
+  nsCOMPtr<nsPIDOMWindow> win(do_QueryReferent(mInnerWindow));
+  if (!win || !nsContentUtils::CanCallerAccess(win->GetOuterWindow()))
+    return NS_ERROR_DOM_SECURITY_ERR;
+
   if (aDelta == 0) {
     nsCOMPtr<nsPIDOMWindow> window(do_GetInterface(GetDocShell()));
 
@@ -216,8 +231,7 @@ nsHistory::Go(int32_t aDelta)
       // trick to work around gecko reflow bugs, and this should have
       // the same effect.
 
-      nsCOMPtr<nsIDocument> doc =
-        do_QueryInterface(window->GetExtantDocument());
+      nsCOMPtr<nsIDocument> doc = window->GetExtantDoc();
 
       nsIPresShell *shell;
       nsPresContext *pcx;
@@ -323,8 +337,7 @@ nsHistory::GetState(nsIVariant **aState)
   if (!nsContentUtils::CanCallerAccess(win->GetOuterWindow()))
     return NS_ERROR_DOM_SECURITY_ERR;
 
-  nsCOMPtr<nsIDocument> doc =
-    do_QueryInterface(win->GetExtantDocument());
+  nsCOMPtr<nsIDocument> doc = win->GetExtantDoc();
   if (!doc)
     return NS_ERROR_NOT_AVAILABLE;
 
@@ -335,7 +348,7 @@ NS_IMETHODIMP
 nsHistory::Item(uint32_t aIndex, nsAString& aReturn)
 {
   aReturn.Truncate();
-  if (!nsContentUtils::IsCallerTrustedForRead()) {
+  if (!nsContentUtils::IsCallerChrome()) {
     return NS_ERROR_DOM_SECURITY_ERR;
   }
 
@@ -356,7 +369,7 @@ nsHistory::Item(uint32_t aIndex, nsAString& aReturn)
   }
 
   if (uri) {
-    nsCAutoString urlCString;
+    nsAutoCString urlCString;
     rv = uri->GetSpec(urlCString);
 
     CopyUTF8toUTF16(urlCString, aReturn);
@@ -377,12 +390,11 @@ nsHistory::GetSessionHistoryFromDocShell(nsIDocShell * aDocShell,
    */
   
   // QI mDocShell to nsIDocShellTreeItem
-  nsCOMPtr<nsIDocShellTreeItem> dsTreeItem(do_QueryInterface(aDocShell));
-  NS_ENSURE_TRUE(dsTreeItem, NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(aDocShell, NS_ERROR_FAILURE);
 
   // Get the root DocShell from it
   nsCOMPtr<nsIDocShellTreeItem> root;
-  dsTreeItem->GetSameTypeRootTreeItem(getter_AddRefs(root));
+  aDocShell->GetSameTypeRootTreeItem(getter_AddRefs(root));
   NS_ENSURE_TRUE(root, NS_ERROR_FAILURE);
   
   //QI root to nsIWebNavigation
