@@ -1,51 +1,18 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim: set ts=2 et sw=2 tw=80: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Indexed Database.
- *
- * The Initial Developer of the Original Code is The Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Jan Varga <Jan.Varga@gmail.com>
- *   Jonas Sicking <jonas@sicking.cc>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/FloatingPoint.h"
 
 #include "Key.h"
-#include "nsIStreamBufferAccess.h"
 #include "jsfriendapi.h"
 #include "nsAlgorithm.h"
-#include "nsContentUtils.h"
 #include "nsJSUtils.h"
 #include "xpcpublic.h"
+#include "mozilla/Endian.h"
+#include <algorithm>
 
 USING_INDEXEDDB_NAMESPACE
 
@@ -131,15 +98,21 @@ USING_INDEXEDDB_NAMESPACE
 
 const int MaxArrayCollapse = 3;
 
+const int MaxRecursionDepth = 256;
+
 nsresult
-Key::EncodeJSVal(JSContext* aCx, const jsval aVal, PRUint8 aTypeOffset)
+Key::EncodeJSValInternal(JSContext* aCx, const jsval aVal,
+                         uint8_t aTypeOffset, uint16_t aRecursionDepth)
 {
-  PR_STATIC_ASSERT(eMaxType * MaxArrayCollapse < 256);
+  NS_ENSURE_TRUE(aRecursionDepth < MaxRecursionDepth, NS_ERROR_DOM_INDEXEDDB_DATA_ERR);
+
+  MOZ_STATIC_ASSERT(eMaxType * MaxArrayCollapse < 256,
+                    "Unable to encode jsvals.");
 
   if (JSVAL_IS_STRING(aVal)) {
     nsDependentJSString str;
     if (!str.init(aCx, aVal)) {
-      return NS_ERROR_OUT_OF_MEMORY;
+      return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
     }
     EncodeString(str, aTypeOffset);
     return NS_OK;
@@ -152,7 +125,7 @@ Key::EncodeJSVal(JSContext* aCx, const jsval aVal, PRUint8 aTypeOffset)
 
   if (JSVAL_IS_DOUBLE(aVal)) {
     double d = JSVAL_TO_DOUBLE(aVal);
-    if (MOZ_DOUBLE_IS_NaN(d)) {
+    if (mozilla::IsNaN(d)) {
       return NS_ERROR_DOM_INDEXEDDB_DATA_ERR;
     }
     EncodeNumber(d, eFloat + aTypeOffset);
@@ -160,7 +133,7 @@ Key::EncodeJSVal(JSContext* aCx, const jsval aVal, PRUint8 aTypeOffset)
   }
 
   if (!JSVAL_IS_PRIMITIVE(aVal)) {
-    JSObject* obj = JSVAL_TO_OBJECT(aVal);
+    JS::Rooted<JSObject*> obj(aCx, JSVAL_TO_OBJECT(aVal));
     if (JS_IsArrayObject(aCx, obj)) {
       aTypeOffset += eMaxType;
 
@@ -178,13 +151,16 @@ Key::EncodeJSVal(JSContext* aCx, const jsval aVal, PRUint8 aTypeOffset)
       }
 
       for (uint32_t index = 0; index < length; index++) {
-        jsval val;
-        if (!JS_GetElement(aCx, obj, index, &val)) {
+        JS::Rooted<JS::Value> val(aCx);
+        if (!JS_GetElement(aCx, obj, index, val.address())) {
           return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
         }
 
-        nsresult rv = EncodeJSVal(aCx, val, aTypeOffset);
-        NS_ENSURE_SUCCESS(rv, rv);
+        nsresult rv = EncodeJSValInternal(aCx, val, aTypeOffset,
+                                          aRecursionDepth + 1);
+        if (NS_FAILED(rv)) {
+          return rv;
+        }
 
         aTypeOffset = 0;
       }
@@ -195,7 +171,10 @@ Key::EncodeJSVal(JSContext* aCx, const jsval aVal, PRUint8 aTypeOffset)
     }
 
     if (JS_ObjectIsDate(aCx, obj)) {
-      EncodeNumber(js_DateGetMsecSinceEpoch(aCx, obj), eDate + aTypeOffset);
+      if (!js_DateIsValid(obj))  {
+        return NS_ERROR_DOM_INDEXEDDB_DATA_ERR;
+      }
+      EncodeNumber(js_DateGetMsecSinceEpoch(obj), eDate + aTypeOffset);
       return NS_OK;
     }
   }
@@ -205,11 +184,14 @@ Key::EncodeJSVal(JSContext* aCx, const jsval aVal, PRUint8 aTypeOffset)
 
 // static
 nsresult
-Key::DecodeJSVal(const unsigned char*& aPos, const unsigned char* aEnd,
-                 JSContext* aCx, PRUint8 aTypeOffset, jsval* aVal)
+Key::DecodeJSValInternal(const unsigned char*& aPos, const unsigned char* aEnd,
+                         JSContext* aCx, uint8_t aTypeOffset, JS::MutableHandle<JS::Value> aVal,
+                         uint16_t aRecursionDepth)
 {
+  NS_ENSURE_TRUE(aRecursionDepth < MaxRecursionDepth, NS_ERROR_DOM_INDEXEDDB_DATA_ERR);
+
   if (*aPos - aTypeOffset >= eArray) {
-    JSObject* array = JS_NewArrayObject(aCx, 0, nsnull);
+    JS::Rooted<JSObject*> array(aCx, JS_NewArrayObject(aCx, 0, nullptr));
     if (!array) {
       NS_WARNING("Failed to make array!");
       return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
@@ -224,13 +206,14 @@ Key::DecodeJSVal(const unsigned char*& aPos, const unsigned char* aEnd,
 
     uint32_t index = 0;
     while (aPos < aEnd && *aPos - aTypeOffset != eTerminator) {
-      jsval val;
-      nsresult rv = DecodeJSVal(aPos, aEnd, aCx, aTypeOffset, &val);
+      JS::Rooted<JS::Value> val(aCx);
+      nsresult rv = DecodeJSValInternal(aPos, aEnd, aCx, aTypeOffset,
+                                        &val, aRecursionDepth + 1);
       NS_ENSURE_SUCCESS(rv, rv);
 
       aTypeOffset = 0;
 
-      if (!JS_SetElement(aCx, array, index++, &val)) {
+      if (!JS_SetElement(aCx, array, index++, val.address())) {
         NS_WARNING("Failed to set array element!");
         return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
       }
@@ -240,12 +223,12 @@ Key::DecodeJSVal(const unsigned char*& aPos, const unsigned char* aEnd,
                  "Should have found end-of-array marker");
     ++aPos;
 
-    *aVal = OBJECT_TO_JSVAL(array);
+    aVal.setObject(*array);
   }
   else if (*aPos - aTypeOffset == eString) {
     nsString key;
     DecodeString(aPos, aEnd, key);
-    if (!xpc::StringToJsval(aCx, key, aVal)) {
+    if (!xpc::StringToJsval(aCx, key, aVal.address())) {
       return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
     }
   }
@@ -257,10 +240,10 @@ Key::DecodeJSVal(const unsigned char*& aPos, const unsigned char* aEnd,
       return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
     }
 
-    *aVal = OBJECT_TO_JSVAL(date);
+    aVal.setObject(*date);
   }
   else if (*aPos - aTypeOffset == eFloat) {
-    *aVal = DOUBLE_TO_JSVAL(DecodeNumber(aPos, aEnd));
+    aVal.setDouble(DecodeNumber(aPos, aEnd));
   }
   else {
     NS_NOTREACHED("Unknown key type!");
@@ -269,7 +252,6 @@ Key::DecodeJSVal(const unsigned char*& aPos, const unsigned char* aEnd,
   return NS_OK;
 }
 
-
 #define ONE_BYTE_LIMIT 0x7E
 #define TWO_BYTE_LIMIT (0x3FFF+0x7F)
 
@@ -278,13 +260,13 @@ Key::DecodeJSVal(const unsigned char*& aPos, const unsigned char* aEnd,
 #define THREE_BYTE_SHIFT 6
 
 void
-Key::EncodeString(const nsAString& aString, PRUint8 aTypeOffset)
+Key::EncodeString(const nsAString& aString, uint8_t aTypeOffset)
 {
   // First measure how long the encoded string will be.
 
   // The +2 is for initial 3 and trailing 0. We'll compensate for multi-byte
   // chars below.
-  PRUint32 size = aString.Length() + 2;
+  uint32_t size = aString.Length() + 2;
   
   const PRUnichar* start = aString.BeginReading();
   const PRUnichar* end = aString.EndReading();
@@ -295,7 +277,7 @@ Key::EncodeString(const nsAString& aString, PRUint8 aTypeOffset)
   }
 
   // Allocate memory for the new size
-  PRUint32 oldLen = mBuffer.Length();
+  uint32_t oldLen = mBuffer.Length();
   char* buffer;
   if (!mBuffer.GetMutableData(&buffer, oldLen + size)) {
     return;
@@ -316,7 +298,7 @@ Key::EncodeString(const nsAString& aString, PRUint8 aTypeOffset)
       *(buffer++) = (char)(c & 0xFF);
     }
     else {
-      PRUint32 c = (PRUint32(*iter) << THREE_BYTE_SHIFT) | 0x00C00000;
+      uint32_t c = (uint32_t(*iter) << THREE_BYTE_SHIFT) | 0x00C00000;
       *(buffer++) = (char)(c >> 16);
       *(buffer++) = (char)(c >> 8);
       *(buffer++) = (char)c;
@@ -339,7 +321,7 @@ Key::DecodeString(const unsigned char*& aPos, const unsigned char* aEnd,
   const unsigned char* buffer = aPos + 1;
 
   // First measure how big the decoded string will be.
-  PRUint32 size = 0;
+  uint32_t size = 0;
   const unsigned char* iter; 
   for (iter = buffer; iter < aEnd && *iter != eTerminator; ++iter) {
     if (*iter & 0x80) {
@@ -371,9 +353,9 @@ Key::DecodeString(const unsigned char*& aPos, const unsigned char* aEnd,
       *out = c - TWO_BYTE_ADJUST - 0x8000;
     }
     else {
-      PRUint32 c = PRUint32(*(iter++)) << (16 - THREE_BYTE_SHIFT);
+      uint32_t c = uint32_t(*(iter++)) << (16 - THREE_BYTE_SHIFT);
       if (iter < aEnd) {
-        c |= PRUint32(*(iter++)) << (8 - THREE_BYTE_SHIFT);
+        c |= uint32_t(*(iter++)) << (8 - THREE_BYTE_SHIFT);
       }
       if (iter < aEnd) {
         c |= *(iter++) >> THREE_BYTE_SHIFT;
@@ -392,14 +374,14 @@ Key::DecodeString(const unsigned char*& aPos, const unsigned char* aEnd,
 
 union Float64Union {
   double d;
-  PRUint64 u;
+  uint64_t u;
 }; 
 
 void
-Key::EncodeNumber(double aFloat, PRUint8 aType)
+Key::EncodeNumber(double aFloat, uint8_t aType)
 {
   // Allocate memory for the new size
-  PRUint32 oldLen = mBuffer.Length();
+  uint32_t oldLen = mBuffer.Length();
   char* buffer;
   if (!mBuffer.GetMutableData(&buffer, oldLen + 1 + sizeof(double))) {
     return;
@@ -410,12 +392,13 @@ Key::EncodeNumber(double aFloat, PRUint8 aType)
 
   Float64Union pun;
   pun.d = aFloat;
-  PRUint64 number = pun.u & PR_UINT64(0x8000000000000000) ?
-                    -pun.u :
+  // Note: The subtraction from 0 below is necessary to fix
+  // MSVC build warning C4146 (negating an unsigned value).
+  uint64_t number = pun.u & PR_UINT64(0x8000000000000000) ?
+                    (0 - pun.u) :
                     (pun.u | PR_UINT64(0x8000000000000000));
 
-  number = NS_SWAP64(number);
-  memcpy(buffer, &number, sizeof(number));
+  mozilla::BigEndian::writeUint64(buffer, number);
 }
 
 // static
@@ -427,16 +410,18 @@ Key::DecodeNumber(const unsigned char*& aPos, const unsigned char* aEnd)
 
   ++aPos;
 
-  PRUint64 number = 0;
-  memcpy(&number, aPos, NS_MIN<size_t>(sizeof(number), aEnd - aPos));
-  number = NS_SWAP64(number);
+  uint64_t number = 0;
+  memcpy(&number, aPos, std::min<size_t>(sizeof(number), aEnd - aPos));
+  number = mozilla::NativeEndian::swapFromBigEndian(number);
 
   aPos += sizeof(number);
 
   Float64Union pun;
+  // Note: The subtraction from 0 below is necessary to fix
+  // MSVC build warning C4146 (negating an unsigned value).
   pun.u = number & PR_UINT64(0x8000000000000000) ?
           (number & ~PR_UINT64(0x8000000000000000)) :
-          -number;
+          (0 - number);
 
   return pun.d;
 }
