@@ -14,6 +14,8 @@
 #include "nsServiceManagerUtils.h"
 #include "nsXPCOMCIDInternal.h"
 
+#include "ProfilerHelpers.h"
+
 using mozilla::MonitorAutoLock;
 
 USING_INDEXEDDB_NAMESPACE
@@ -27,31 +29,20 @@ const uint32_t kIdleThreadTimeoutMs = 30000;
 TransactionThreadPool* gInstance = nullptr;
 bool gShutdown = false;
 
-inline
-nsresult
-CheckOverlapAndMergeObjectStores(nsTArray<nsString>& aLockedStores,
-                                 const nsTArray<nsString>& aObjectStores,
-                                 bool aShouldMerge,
-                                 bool* aStoresOverlap)
+#ifdef MOZ_ENABLE_PROFILER_SPS
+
+class TransactionThreadPoolListener : public nsIThreadPoolListener
 {
-  uint32_t length = aObjectStores.Length();
+public:
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSITHREADPOOLLISTENER
 
-  bool overlap = false;
+private:
+  virtual ~TransactionThreadPoolListener()
+  { }
+};
 
-  for (uint32_t index = 0; index < length; index++) {
-    const nsString& storeName = aObjectStores[index];
-    if (aLockedStores.Contains(storeName)) {
-      overlap = true;
-    }
-    else if (aShouldMerge && !aLockedStores.AppendElement(storeName)) {
-      NS_WARNING("Out of memory!");
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
-  }
-
-  *aStoresOverlap = overlap;
-  return NS_OK;
-}
+#endif // MOZ_ENABLE_PROFILER_SPS
 
 } // anonymous namespace
 
@@ -149,6 +140,14 @@ TransactionThreadPool::Init()
   rv = mThreadPool->SetIdleThreadTimeout(kIdleThreadTimeoutMs);
   NS_ENSURE_SUCCESS(rv, rv);
 
+#ifdef MOZ_ENABLE_PROFILER_SPS
+  nsCOMPtr<nsIThreadPoolListener> listener =
+    new TransactionThreadPoolListener();
+
+  rv = mThreadPool->SetListener(listener);
+  NS_ENSURE_SUCCESS(rv, rv);
+#endif
+
   return NS_OK;
 }
 
@@ -157,6 +156,8 @@ TransactionThreadPool::Cleanup()
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 
+  PROFILER_MAIN_THREAD_LABEL("IndexedDB", "TransactionThreadPool::Cleanup");
+
   nsresult rv = mThreadPool->Shutdown();
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -180,12 +181,36 @@ TransactionThreadPool::Cleanup()
   return NS_OK;
 }
 
+// static
+PLDHashOperator
+TransactionThreadPool::MaybeUnblockTransaction(nsPtrHashKey<TransactionInfo>* aKey,
+                                               void* aUserArg)
+{
+  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+
+  TransactionInfo* maybeUnblockedInfo = aKey->GetKey();
+  TransactionInfo* finishedInfo = static_cast<TransactionInfo*>(aUserArg);
+
+  NS_ASSERTION(maybeUnblockedInfo->blockedOn.Contains(finishedInfo),
+               "Huh?");
+  maybeUnblockedInfo->blockedOn.RemoveEntry(finishedInfo);
+  if (!maybeUnblockedInfo->blockedOn.Count()) {
+    // Let this transaction run.
+    maybeUnblockedInfo->queue->Unblock();
+  }
+
+  return PL_DHASH_NEXT;
+}
+
 void
 TransactionThreadPool::FinishTransaction(IDBTransaction* aTransaction)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(aTransaction, "Null pointer!");
 
+  PROFILER_MAIN_THREAD_LABEL("IndexedDB",
+                             "TransactionThreadPool::FinishTransaction");
+
   // AddRef here because removing from the hash will call Release.
   nsRefPtr<IDBTransaction> transaction(aTransaction);
 
@@ -197,10 +222,10 @@ TransactionThreadPool::FinishTransaction(IDBTransaction* aTransaction)
     return;
   }
 
-  nsTArray<TransactionInfo>& transactionsInProgress =
+  DatabaseTransactionInfo::TransactionHashtable& transactionsInProgress =
     dbTransactionInfo->transactions;
 
-  uint32_t transactionCount = transactionsInProgress.Length();
+  uint32_t transactionCount = transactionsInProgress.Count();
 
 #ifdef DEBUG
   if (aTransaction->mMode == IDBTransaction::VERSION_CHANGE) {
@@ -212,8 +237,8 @@ TransactionThreadPool::FinishTransaction(IDBTransaction* aTransaction)
   if (transactionCount == 1) {
 #ifdef DEBUG
     {
-      TransactionInfo& info = transactionsInProgress[0];
-      NS_ASSERTION(info.transaction == aTransaction, "Transaction mismatch!");
+      const TransactionInfo* info = transactionsInProgress.Get(aTransaction);
+      NS_ASSERTION(info->transaction == aTransaction, "Transaction mismatch!");
     }
 #endif
     mTransactionsInProgress.Remove(databaseId);
@@ -228,73 +253,40 @@ TransactionThreadPool::FinishTransaction(IDBTransaction* aTransaction)
         index++;
       }
     }
+
+    return;
   }
-  else {
-    // We need to rebuild the locked object store list.
-    nsTArray<nsString> storesWriting, storesReading;
+  TransactionInfo* info = transactionsInProgress.Get(aTransaction);
+  NS_ASSERTION(info, "We've never heard of this transaction?!?");
 
-    for (uint32_t index = 0, count = transactionCount; index < count; index++) {
-      IDBTransaction* transaction = transactionsInProgress[index].transaction;
-      if (transaction == aTransaction) {
-        NS_ASSERTION(count == transactionCount, "More than one match?!");
+  const nsTArray<nsString>& objectStoreNames = aTransaction->mObjectStoreNames;
+  for (uint32_t index = 0, count = objectStoreNames.Length(); index < count;
+       index++) {
+    TransactionInfoPair* blockInfo =
+      dbTransactionInfo->blockingTransactions.Get(objectStoreNames[index]);
+    NS_ASSERTION(blockInfo, "Huh?");
 
-        transactionsInProgress.RemoveElementAt(index);
-        index--;
-        count--;
-
-        continue;
-      }
-
-      const nsTArray<nsString>& objectStores = transaction->mObjectStoreNames;
-
-      bool dummy;
-      if (transaction->mMode == IDBTransaction::READ_WRITE) {
-        if (NS_FAILED(CheckOverlapAndMergeObjectStores(storesWriting,
-                                                       objectStores,
-                                                       true, &dummy))) {
-          NS_WARNING("Out of memory!");
-        }
-      }
-      else if (transaction->mMode == IDBTransaction::READ_ONLY) {
-        if (NS_FAILED(CheckOverlapAndMergeObjectStores(storesReading,
-                                                       objectStores,
-                                                       true, &dummy))) {
-          NS_WARNING("Out of memory!");
-        }
-      }
-      else {
-        NS_NOTREACHED("Unknown mode!");
-      }
+    if (aTransaction->mMode == IDBTransaction::READ_WRITE &&
+        blockInfo->lastBlockingReads == info) {
+      blockInfo->lastBlockingReads = nullptr;
     }
 
-    NS_ASSERTION(transactionsInProgress.Length() == transactionCount - 1,
-                 "Didn't find the transaction we were looking for!");
-
-    dbTransactionInfo->storesWriting.SwapElements(storesWriting);
-    dbTransactionInfo->storesReading.SwapElements(storesReading);
-  }
-
-  // Try to dispatch all the queued transactions again.
-  nsTArray<QueuedDispatchInfo> queuedDispatch;
-  queuedDispatch.SwapElements(mDelayedDispatchQueue);
-
-  transactionCount = queuedDispatch.Length();
-  for (uint32_t index = 0; index < transactionCount; index++) {
-    if (NS_FAILED(Dispatch(queuedDispatch[index]))) {
-      NS_WARNING("Dispatch failed!");
+    uint32_t i = blockInfo->lastBlockingWrites.IndexOf(info);
+    if (i != blockInfo->lastBlockingWrites.NoIndex) {
+      blockInfo->lastBlockingWrites.RemoveElementAt(i);
     }
   }
+
+  info->blocking.EnumerateEntries(MaybeUnblockTransaction, info);
+
+  transactionsInProgress.Remove(aTransaction);
 }
 
-nsresult
-TransactionThreadPool::TransactionCanRun(IDBTransaction* aTransaction,
-                                         bool* aCanRun,
-                                         TransactionQueue** aExistingQueue)
+TransactionThreadPool::TransactionQueue&
+TransactionThreadPool::GetQueueForTransaction(IDBTransaction* aTransaction)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(aTransaction, "Null pointer!");
-  NS_ASSERTION(aCanRun, "Null pointer!");
-  NS_ASSERTION(aExistingQueue, "Null pointer!");
 
   nsIAtom* databaseId = aTransaction->mDatabase->Id();
   const nsTArray<nsString>& objectStoreNames = aTransaction->mObjectStoreNames;
@@ -303,55 +295,66 @@ TransactionThreadPool::TransactionCanRun(IDBTransaction* aTransaction,
   // See if we can run this transaction now.
   DatabaseTransactionInfo* dbTransactionInfo;
   if (!mTransactionsInProgress.Get(databaseId, &dbTransactionInfo)) {
-    // First transaction for this database, fine to run.
-    *aCanRun = true;
-    *aExistingQueue = nullptr;
-    return NS_OK;
+    // First transaction for this database.
+    dbTransactionInfo = new DatabaseTransactionInfo();
+    mTransactionsInProgress.Put(databaseId, dbTransactionInfo);
   }
 
-  nsTArray<TransactionInfo>& transactionsInProgress =
+  DatabaseTransactionInfo::TransactionHashtable& transactionsInProgress =
     dbTransactionInfo->transactions;
+  TransactionInfo* info = transactionsInProgress.Get(aTransaction);
+  if (info) {
+    // We recognize this one.
+    return *info->queue;
+  }
 
-  uint32_t transactionCount = transactionsInProgress.Length();
-  NS_ASSERTION(transactionCount, "Should never be 0!");
+  TransactionInfo* transactionInfo = new TransactionInfo(aTransaction);
 
-  for (uint32_t index = 0; index < transactionCount; index++) {
-    // See if this transaction is in out list of current transactions.
-    const TransactionInfo& info = transactionsInProgress[index];
-    if (info.transaction == aTransaction) {
-      *aCanRun = true;
-      *aExistingQueue = info.queue;
-      return NS_OK;
+  dbTransactionInfo->transactions.Put(aTransaction, transactionInfo);;
+
+  for (uint32_t index = 0, count = objectStoreNames.Length(); index < count;
+       index++) {
+    TransactionInfoPair* blockInfo =
+      dbTransactionInfo->blockingTransactions.Get(objectStoreNames[index]);
+    if (!blockInfo) {
+      blockInfo = new TransactionInfoPair();
+      blockInfo->lastBlockingReads = nullptr;
+      dbTransactionInfo->blockingTransactions.Put(objectStoreNames[index],
+                                                  blockInfo);
+    }
+
+    // Mark what we are blocking on.
+    if (blockInfo->lastBlockingReads) {
+      TransactionInfo* blockingInfo = blockInfo->lastBlockingReads;
+      transactionInfo->blockedOn.PutEntry(blockingInfo);
+      blockingInfo->blocking.PutEntry(transactionInfo);
+    }
+
+    if (mode == IDBTransaction::READ_WRITE &&
+        blockInfo->lastBlockingWrites.Length()) {
+      for (uint32_t index = 0,
+           count = blockInfo->lastBlockingWrites.Length(); index < count;
+           index++) {
+        TransactionInfo* blockingInfo = blockInfo->lastBlockingWrites[index];
+        transactionInfo->blockedOn.PutEntry(blockingInfo);
+        blockingInfo->blocking.PutEntry(transactionInfo);
+      }
+    }
+
+    if (mode == IDBTransaction::READ_WRITE) {
+      blockInfo->lastBlockingReads = transactionInfo;
+      blockInfo->lastBlockingWrites.Clear();
+    }
+    else {
+      blockInfo->lastBlockingWrites.AppendElement(transactionInfo);
     }
   }
 
-  NS_ASSERTION(mode != IDBTransaction::VERSION_CHANGE, "How did we get here?");
-
-  bool writeOverlap;
-  nsresult rv =
-    CheckOverlapAndMergeObjectStores(dbTransactionInfo->storesWriting,
-                                     objectStoreNames,
-                                     mode == IDBTransaction::READ_WRITE,
-                                     &writeOverlap);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  bool readOverlap;
-  rv = CheckOverlapAndMergeObjectStores(dbTransactionInfo->storesReading,
-                                        objectStoreNames,
-                                        mode == IDBTransaction::READ_ONLY,
-                                        &readOverlap);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (writeOverlap ||
-      (readOverlap && mode == IDBTransaction::READ_WRITE)) {
-    *aCanRun = false;
-    *aExistingQueue = nullptr;
-    return NS_OK;
+  if (!transactionInfo->blockedOn.Count()) {
+    transactionInfo->queue->Unblock();
   }
 
-  *aCanRun = true;
-  *aExistingQueue = nullptr;
-  return NS_OK;
+  return *transactionInfo->queue;
 }
 
 nsresult
@@ -368,111 +371,45 @@ TransactionThreadPool::Dispatch(IDBTransaction* aTransaction,
     return NS_ERROR_NOT_AVAILABLE;
   }
 
-  bool canRun;
-  TransactionQueue* existingQueue;
-  nsresult rv = TransactionCanRun(aTransaction, &canRun, &existingQueue);
-  NS_ENSURE_SUCCESS(rv, rv);
+  TransactionQueue& queue = GetQueueForTransaction(aTransaction);
 
-  if (!canRun) {
-    QueuedDispatchInfo* info = mDelayedDispatchQueue.AppendElement();
-    NS_ENSURE_TRUE(info, NS_ERROR_OUT_OF_MEMORY);
-
-    info->transaction = aTransaction;
-    info->runnable = aRunnable;
-    info->finish = aFinish;
-    info->finishRunnable = aFinishRunnable;
-
-    return NS_OK;
-  }
-
-  if (existingQueue) {
-    existingQueue->Dispatch(aRunnable);
-    if (aFinish) {
-      existingQueue->Finish(aFinishRunnable);
-    }
-    return NS_OK;
-  }
-
-  nsIAtom* databaseId = aTransaction->mDatabase->Id();
-
-#ifdef DEBUG
-  if (aTransaction->mMode == IDBTransaction::VERSION_CHANGE) {
-    NS_ASSERTION(!mTransactionsInProgress.Get(databaseId, nullptr),
-                 "Shouldn't have anything in progress!");
-  }
-#endif
-
-  DatabaseTransactionInfo* dbTransactionInfo;
-  nsAutoPtr<DatabaseTransactionInfo> autoDBTransactionInfo;
-
-  if (!mTransactionsInProgress.Get(databaseId, &dbTransactionInfo)) {
-    // Make a new struct for this transaction.
-    autoDBTransactionInfo = new DatabaseTransactionInfo();
-    dbTransactionInfo = autoDBTransactionInfo;
-  }
-
-  const nsTArray<nsString>& objectStoreNames = aTransaction->mObjectStoreNames;
-
-  nsTArray<nsString>& storesInUse =
-    aTransaction->mMode == IDBTransaction::READ_WRITE ?
-    dbTransactionInfo->storesWriting :
-    dbTransactionInfo->storesReading;
-
-  if (!storesInUse.AppendElements(objectStoreNames)) {
-    NS_WARNING("Out of memory!");
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  nsTArray<TransactionInfo>& transactionInfoArray =
-    dbTransactionInfo->transactions;
-
-  TransactionInfo* transactionInfo = transactionInfoArray.AppendElement();
-  NS_ENSURE_TRUE(transactionInfo, NS_ERROR_OUT_OF_MEMORY);
-
-  transactionInfo->transaction = aTransaction;
-  transactionInfo->queue = new TransactionQueue(aTransaction, aRunnable);
+  queue.Dispatch(aRunnable);
   if (aFinish) {
-    transactionInfo->queue->Finish(aFinishRunnable);
+    queue.Finish(aFinishRunnable);
   }
-
-  if (!transactionInfo->objectStoreNames.AppendElements(objectStoreNames)) {
-    NS_WARNING("Out of memory!");
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  if (autoDBTransactionInfo) {
-    mTransactionsInProgress.Put(databaseId, autoDBTransactionInfo);
-    autoDBTransactionInfo.forget();
-  }
-
-  return mThreadPool->Dispatch(transactionInfo->queue, NS_DISPATCH_NORMAL);
+  return NS_OK;
 }
 
-bool
-TransactionThreadPool::WaitForAllDatabasesToComplete(
-                                            nsTArray<IDBDatabase*>& aDatabases,
-                                            nsIRunnable* aCallback)
+void
+TransactionThreadPool::WaitForDatabasesToComplete(
+                                   nsTArray<nsRefPtr<IDBDatabase> >& aDatabases,
+                                   nsIRunnable* aCallback)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(!aDatabases.IsEmpty(), "No databases to wait on!");
   NS_ASSERTION(aCallback, "Null pointer!");
 
   DatabasesCompleteCallback* callback = mCompleteCallbacks.AppendElement();
-  if (!callback) {
-    NS_WARNING("Out of memory!");
-    return false;
-  }
 
   callback->mCallback = aCallback;
-  if (!callback->mDatabases.SwapElements(aDatabases)) {
-    NS_ERROR("This should never fail!");
-  }
+  callback->mDatabases.SwapElements(aDatabases);
 
   if (MaybeFireCallback(*callback)) {
     mCompleteCallbacks.RemoveElementAt(mCompleteCallbacks.Length() - 1);
   }
+}
 
-  return true;
+// static
+PLDHashOperator
+TransactionThreadPool::CollectTransactions(IDBTransaction* aKey,
+                                           TransactionInfo* aValue,
+                                           void* aUserArg)
+{
+  nsAutoTArray<nsRefPtr<IDBTransaction>, 50>* transactionArray =
+    static_cast<nsAutoTArray<nsRefPtr<IDBTransaction>, 50>*>(aUserArg);
+  transactionArray->AppendElement(aKey);
+
+  return PL_DHASH_NEXT;
 }
 
 void
@@ -481,42 +418,33 @@ TransactionThreadPool::AbortTransactionsForDatabase(IDBDatabase* aDatabase)
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(aDatabase, "Null pointer!");
 
+  PROFILER_MAIN_THREAD_LABEL("IndexedDB",
+                             "TransactionThreadPool::"
+                             "AbortTransactionsForDatabase");
+
   // Get list of transactions for this database id
   DatabaseTransactionInfo* dbTransactionInfo;
   if (!mTransactionsInProgress.Get(aDatabase->Id(), &dbTransactionInfo)) {
-    // If there are no running transactions, there can't be any pending ones
+    // If there are no transactions, we're done.
     return;
   }
 
-  nsAutoTArray<nsRefPtr<IDBTransaction>, 50> transactions;
-
   // Collect any running transactions
-  nsTArray<TransactionInfo>& transactionsInProgress =
+  DatabaseTransactionInfo::TransactionHashtable& transactionsInProgress =
     dbTransactionInfo->transactions;
 
-  uint32_t transactionCount = transactionsInProgress.Length();
-  NS_ASSERTION(transactionCount, "Should never be 0!");
+  NS_ASSERTION(transactionsInProgress.Count(), "Should never be 0!");
 
-  for (uint32_t index = 0; index < transactionCount; index++) {
-    // See if any transaction belongs to this IDBDatabase instance
-    IDBTransaction* transaction = transactionsInProgress[index].transaction;
-    if (transaction->Database() == aDatabase) {
-      transactions.AppendElement(transaction);
-    }
-  }
-
-  // Collect any pending transactions.
-  for (uint32_t index = 0; index < mDelayedDispatchQueue.Length(); index++) {
-    // See if any transaction belongs to this IDBDatabase instance
-    IDBTransaction* transaction = mDelayedDispatchQueue[index].transaction;
-    if (transaction->Database() == aDatabase) {
-      transactions.AppendElement(transaction);
-    }
-  }
+  nsAutoTArray<nsRefPtr<IDBTransaction>, 50> transactions;
+  transactionsInProgress.EnumerateRead(CollectTransactions, &transactions);
 
   // Abort transactions. Do this after collecting the transactions in case
   // calling Abort() modifies the data structures we're iterating above.
   for (uint32_t index = 0; index < transactions.Length(); index++) {
+    if (transactions[index]->Database() != aDatabase) {
+      continue;
+    }
+
     // This can fail, for example if the transaction is in the process of
     // being comitted. That is expected and fine, so we ignore any returned
     // errors.
@@ -524,41 +452,65 @@ TransactionThreadPool::AbortTransactionsForDatabase(IDBDatabase* aDatabase)
   }
 }
 
+struct MOZ_STACK_CLASS TransactionSearchInfo
+{
+  TransactionSearchInfo(nsIOfflineStorage* aDatabase)
+    : db(aDatabase), found(false)
+  {
+  }
+
+  nsIOfflineStorage* db;
+  bool found;
+};
+
+// static
+PLDHashOperator
+TransactionThreadPool::FindTransaction(IDBTransaction* aKey,
+                                       TransactionInfo* aValue,
+                                       void* aUserArg)
+{
+  TransactionSearchInfo* info = static_cast<TransactionSearchInfo*>(aUserArg);
+
+  if (aKey->Database() == info->db) {
+    info->found = true;
+    return PL_DHASH_STOP;
+  }
+
+  return PL_DHASH_NEXT;
+}
 bool
 TransactionThreadPool::HasTransactionsForDatabase(IDBDatabase* aDatabase)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(aDatabase, "Null pointer!");
 
-  // Get list of transactions for this database id
-  DatabaseTransactionInfo* dbTransactionInfo;
-  if (!mTransactionsInProgress.Get(aDatabase->Id(), &dbTransactionInfo)) {
+  DatabaseTransactionInfo* dbTransactionInfo = nullptr;
+  dbTransactionInfo = mTransactionsInProgress.Get(aDatabase->Id());
+  if (!dbTransactionInfo) {
     return false;
   }
 
-  nsTArray<TransactionInfo>& transactionsInProgress =
-    dbTransactionInfo->transactions;
+  TransactionSearchInfo info(aDatabase);
+  dbTransactionInfo->transactions.EnumerateRead(FindTransaction, &info);
 
-  uint32_t transactionCount = transactionsInProgress.Length();
-  NS_ASSERTION(transactionCount, "Should never be 0!");
-
-  for (uint32_t index = 0; index < transactionCount; index++) {
-    // See if any transaction belongs to this IDBDatabase instance
-    if (transactionsInProgress[index].transaction->Database() == aDatabase) {
-      return true;
-    }
-  }
-
-  return false;
+  return info.found;
 }
 
 bool
-TransactionThreadPool::MaybeFireCallback(DatabasesCompleteCallback& aCallback)
+TransactionThreadPool::MaybeFireCallback(DatabasesCompleteCallback aCallback)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 
+  PROFILER_MAIN_THREAD_LABEL("IndexedDB",
+                             "TransactionThreadPool::MaybeFireCallback");
+
   for (uint32_t index = 0; index < aCallback.mDatabases.Length(); index++) {
-    if (mTransactionsInProgress.Get(aCallback.mDatabases[index]->Id(),
+    IDBDatabase* database = aCallback.mDatabases[index];
+    if (!database) {
+      MOZ_CRASH();
+    }
+
+    if (mTransactionsInProgress.Get(database->Id(),
                                     nullptr)) {
       return false;
     }
@@ -569,15 +521,24 @@ TransactionThreadPool::MaybeFireCallback(DatabasesCompleteCallback& aCallback)
 }
 
 TransactionThreadPool::
-TransactionQueue::TransactionQueue(IDBTransaction* aTransaction,
-                                   nsIRunnable* aRunnable)
+TransactionQueue::TransactionQueue(IDBTransaction* aTransaction)
 : mMonitor("TransactionQueue::mMonitor"),
   mTransaction(aTransaction),
   mShouldFinish(false)
 {
+  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(aTransaction, "Null pointer!");
-  NS_ASSERTION(aRunnable, "Null pointer!");
-  mQueue.AppendElement(aRunnable);
+}
+
+void
+TransactionThreadPool::TransactionQueue::Unblock()
+{
+  MonitorAutoLock lock(mMonitor);
+
+  // NB: Finish may be called before Unblock.
+
+  TransactionThreadPool::Get()->mThreadPool->
+    Dispatch(this, NS_DISPATCH_NORMAL);
 }
 
 void
@@ -611,11 +572,20 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(TransactionThreadPool::TransactionQueue,
 NS_IMETHODIMP
 TransactionThreadPool::TransactionQueue::Run()
 {
+  NS_ASSERTION(!NS_IsMainThread(), "Wrong thread!");
+  NS_ASSERTION(IndexedDatabaseManager::IsMainProcess(), "Wrong process!");
+
+  PROFILER_LABEL("IndexedDB", "TransactionQueue::Run");
+
+  IDB_PROFILER_MARK("IndexedDB Transaction %llu: Beginning database work",
+                    "IDBTransaction[%llu] DT Start",
+                    mTransaction->GetSerialNumber());
+
   nsAutoTArray<nsCOMPtr<nsIRunnable>, 10> queue;
   nsCOMPtr<nsIRunnable> finishRunnable;
   bool shouldFinish = false;
 
-  while(!shouldFinish) {
+  do {
     NS_ASSERTION(queue.IsEmpty(), "Should have cleared this!");
 
     {
@@ -643,7 +613,11 @@ TransactionThreadPool::TransactionQueue::Run()
     if (count) {
       queue.Clear();
     }
-  }
+  } while (!shouldFinish);
+
+  IDB_PROFILER_MARK("IndexedDB Transaction %llu: Finished database work",
+                    "IDBTransaction[%llu] DT Done",
+                    mTransaction->GetSerialNumber());
 
   nsCOMPtr<nsIRunnable> finishTransactionRunnable =
     new FinishTransactionRunnable(mTransaction, finishRunnable);
@@ -671,6 +645,9 @@ NS_IMETHODIMP
 FinishTransactionRunnable::Run()
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+
+  PROFILER_MAIN_THREAD_LABEL("IndexedDB", "FinishTransactionRunnable::Run");
+
   if (!gInstance) {
     NS_ERROR("Running after shutdown!");
     return NS_ERROR_FAILURE;
@@ -685,3 +662,27 @@ FinishTransactionRunnable::Run()
 
   return NS_OK;
 }
+
+#ifdef MOZ_ENABLE_PROFILER_SPS
+
+NS_IMPL_THREADSAFE_ISUPPORTS1(TransactionThreadPoolListener,
+                              nsIThreadPoolListener)
+
+NS_IMETHODIMP
+TransactionThreadPoolListener::OnThreadCreated()
+{
+  MOZ_ASSERT(!NS_IsMainThread());
+  char aLocal;
+  profiler_register_thread("IndexedDB Transaction", &aLocal);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+TransactionThreadPoolListener::OnThreadShuttingDown()
+{
+  MOZ_ASSERT(!NS_IsMainThread());
+  profiler_unregister_thread();
+  return NS_OK;
+}
+
+#endif // MOZ_ENABLE_PROFILER_SPS
