@@ -26,10 +26,10 @@ class Connection : public nsDOMEventTargetHelper
                  , public NetworkObserver
 {
 public:
-  NS_DECL_ISUPPORTS
+  NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_NSIDOMMOZCONNECTION
 
-  NS_FORWARD_NSIDOMEVENTTARGET(nsDOMEventTargetHelper::)
+  NS_REALLY_FORWARD_NSIDOMEVENTTARGET(nsDOMEventTargetHelper)
 
   Connection();
 
@@ -39,15 +39,7 @@ public:
   // For IObserver
   void Notify(const hal::NetworkInformation& aNetworkInfo);
 
-  NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(Connection,
-                                           nsDOMEventTargetHelper)
-
 private:
-  /**
-   * Dispatch a trusted non-cancellable and non-bubbling event to itself.
-   */
-  nsresult DispatchTrustedEventToSelf(const nsAString& aEventName);
-
   /**
    * Update the connection information stored in the object using a
    * NetworkInformation object.
@@ -64,8 +56,6 @@ private:
    */
   double mBandwidth;
 
-  NS_DECL_EVENT_HANDLER(change)
-
   static const char* sMeteredPrefName;
   static const bool  sMeteredDefaultValue;
 };
