@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef nsNPAPIPlugin_h_
 #define nsNPAPIPlugin_h_
@@ -43,6 +11,7 @@
 #include "nsPluginHost.h"
 
 #include "jsapi.h"
+#include "nsCxPusher.h"
 
 #include "mozilla/PluginLibrary.h"
 
@@ -52,11 +21,7 @@
  * itself), to ensure that the function has the
  * right calling conventions on OS/2.
  */
-#ifdef XP_OS2
-#define NP_CALLBACK _System
-#else
-#define NP_CALLBACK
-#endif
+#define NP_CALLBACK NP_LOADDS
 
 #if defined(XP_WIN)
 #define NS_NPAPIPLUGIN_CALLBACK(_type, _name) _type (__stdcall * _name)
@@ -173,11 +138,7 @@ JSContext* GetJSContext(NPP npp);
 inline bool
 NPStringIdentifierIsPermanent(NPP npp, NPIdentifier id)
 {
-  JSContext* cx = GetJSContext(npp);
-  if (!cx) // OOM?
-    return false;
-
-  JSAutoRequest ar(cx);
+  AutoSafeJSContext cx;
   return JS_StringHasBeenInterned(cx, NPIdentifierToString(id));
 }
 
@@ -398,7 +359,7 @@ OnShutdown();
  * within a lexical scope, locks and unlocks the mutex used to
  * serialize modifications to plugin async callback state.
  */
-struct NS_STACK_CLASS AsyncCallbackAutoLock
+struct MOZ_STACK_CLASS AsyncCallbackAutoLock
 {
   AsyncCallbackAutoLock();
   ~AsyncCallbackAutoLock();
@@ -425,8 +386,8 @@ protected:
 // separate objects on the stack since we always want a
 // PluginDestructionGuard where we use an NPPAutoPusher.
 
-class NPPAutoPusher : public NPPStack,
-                      protected PluginDestructionGuard
+class MOZ_STACK_CLASS NPPAutoPusher : public NPPStack,
+                                      protected PluginDestructionGuard
 {
 public:
   NPPAutoPusher(NPP npp)
