@@ -4,12 +4,14 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "mozilla/DebugOnly.h"
+
 #include "PluginScriptableObjectParent.h"
 #include "PluginScriptableObjectUtils.h"
 
 #include "nsNPAPIPlugin.h"
 #include "mozilla/unused.h"
-#include "mozilla/Util.h"
+#include "nsCxPusher.h"
 
 using namespace mozilla::plugins;
 using namespace mozilla::plugins::parent;
@@ -1037,15 +1039,9 @@ PluginScriptableObjectParent::AnswerEnumerate(InfallibleTArray<PPluginIdentifier
     return true;
   }
 
-  if (!aProperties->SetCapacity(idCount)) {
-    npn->memfree(ids);
-    *aSuccess = false;
-    return true;
-  }
-
-  JSContext* cx = GetJSContext(instance->GetNPP());
-  JSAutoRequest ar(cx);
+  aProperties->SetCapacity(idCount);
 
+  mozilla::AutoSafeJSContext cx;
   for (uint32_t index = 0; index < idCount; index++) {
     // Because of GC hazards, all identifiers returned from enumerate
     // must be made permanent.
