@@ -11,9 +11,9 @@
 // I hate having to export this...
 #include "mozilla/dom/workers/bindings/EventListenerManager.h"
 
-#include "mozilla/dom/bindings/Nullable.h"
+#include "mozilla/dom/Nullable.h"
+#include "mozilla/ErrorResult.h"
 
-using namespace mozilla::dom::bindings;
 
 BEGIN_WORKERS_NAMESPACE
 
@@ -31,37 +31,55 @@ protected:
 
 public:
   virtual void
-  _Trace(JSTracer* aTrc) MOZ_OVERRIDE;
+  _trace(JSTracer* aTrc) MOZ_OVERRIDE;
 
   virtual void
-  _Finalize(JSFreeOp* aFop) MOZ_OVERRIDE;
+  _finalize(JSFreeOp* aFop) MOZ_OVERRIDE;
 
   void
-  AddEventListener(const nsAString& aType, JSObject* aListener,
-                   bool aCapture, Nullable<bool> aWantsUntrusted, nsresult& aRv);
+  AddEventListener(const nsAString& aType, JS::Handle<JSObject*> aListener,
+                   bool aCapture, Nullable<bool> aWantsUntrusted,
+                   ErrorResult& aRv);
 
   void
-  RemoveEventListener(const nsAString& aType, JSObject* aListener,
-                      bool aCapture, nsresult& aRv);
+  RemoveEventListener(const nsAString& aType, JS::Handle<JSObject*> aListener,
+                      bool aCapture, ErrorResult& aRv);
 
   bool
-  DispatchEvent(JSObject* aEvent, nsresult& aRv) const
+  DispatchEvent(JS::Handle<JSObject*> aEvent, ErrorResult& aRv) const
   {
     return mListenerManager.DispatchEvent(GetJSContext(), *this, aEvent, aRv);
   }
 
   JSObject*
-  GetEventListener(const nsAString& aType, nsresult& aRv) const;
+  GetEventListener(const nsAString& aType, ErrorResult& aRv) const;
 
   void
-  SetEventListener(const nsAString& aType, JSObject* aListener,
-                   nsresult& aRv);
+  SetEventListener(const nsAString& aType, JS::Handle<JSObject*> aListener,
+                   ErrorResult& aRv);
 
   bool
   HasListeners() const
   {
     return mListenerManager.HasListeners();
   }
+
+  void SetEventHandler(JSContext*, const nsAString& aType, JSObject* aHandler,
+                       ErrorResult& rv)
+  {
+    rv.Throw(NS_ERROR_NOT_IMPLEMENTED);
+  }
+
+  JSObject* GetEventHandler(JSContext*, const nsAString& aType)
+  {
+    return nullptr;
+  }
+
+  JSObject* GetOwnerGlobal() const
+  {
+    // We have no windows
+    return nullptr;
+  }
 };
 
 END_WORKERS_NAMESPACE
