@@ -1,67 +1,37 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "mozilla/Assertions.h"         // for MOZ_ASSERT, etc
+#include "mozilla/Util.h"               // for ArrayLength
+#include "mozilla/dom/Element.h"        // for Element, nsINode
+#include "nsAString.h"                  // for nsAString_internal::IsEmpty
+#include "nsCOMPtr.h"                   // for nsCOMPtr, operator==, etc
+#include "nsCaseTreatment.h"
+#include "nsDebug.h"                    // for NS_PRECONDITION, etc
+#include "nsEditProperty.h"             // for nsEditProperty, etc
+#include "nsEditor.h"                   // for nsEditor
+#include "nsError.h"                    // for NS_SUCCEEDED
+#include "nsGkAtoms.h"                  // for nsGkAtoms, nsGkAtoms::a, etc
 #include "nsHTMLEditUtils.h"
-
-#include "mozilla/Assertions.h"
-#include "mozilla/Util.h"
-#include "mozilla/dom/Element.h"
-
-#include "nsTextEditUtils.h"
-
-#include "nsString.h"
-#include "nsUnicharUtils.h"
-#include "nsEditor.h"
-#include "nsEditProperty.h"
-#include "nsIAtom.h"
-#include "nsIDOMNode.h"
-#include "nsIContent.h"
-#include "nsIDOMNodeList.h"
-#include "nsIDOMHTMLAnchorElement.h"
 #include "nsHTMLTags.h"
+#include "nsIAtom.h"                    // for nsIAtom
+#include "nsIDOMHTMLAnchorElement.h"    // for nsIDOMHTMLAnchorElement
+#include "nsIDOMNode.h"                 // for nsIDOMNode
+#include "nsINameSpaceManager.h"        // for kNameSpaceID_None
+#include "nsLiteralString.h"            // for NS_LITERAL_STRING
+#include "nsString.h"                   // for nsAutoString
+#include "nsTextEditUtils.h"            // for nsTextEditUtils
 
 using namespace mozilla;
 
 ///////////////////////////////////////////////////////////////////////////
 //                  
 bool 
-nsHTMLEditUtils::IsBig(nsIDOMNode *node)
+nsHTMLEditUtils::IsBig(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::big);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::big);
 }
 
 
@@ -69,10 +39,18 @@ nsHTMLEditUtils::IsBig(nsIDOMNode *node)
 // IsInlineStyle true if node is an inline style
 //                  
 bool 
-nsHTMLEditUtils::IsInlineStyle(nsIDOMNode *node)
+nsHTMLEditUtils::IsInlineStyle(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null parent passed to nsHTMLEditUtils::IsInlineStyle");
-  nsCOMPtr<nsIAtom> nodeAtom = nsEditor::GetTag(node);
+  NS_PRECONDITION(aNode, "null parent passed to nsHTMLEditUtils::IsInlineStyle");
+  nsCOMPtr<nsINode> node = do_QueryInterface(aNode);
+  return node && IsInlineStyle(node);
+}
+
+bool
+nsHTMLEditUtils::IsInlineStyle(nsINode* aNode)
+{
+  MOZ_ASSERT(aNode);
+  nsIAtom* nodeAtom = aNode->Tag();
   return (nodeAtom == nsEditProperty::b)
       || (nodeAtom == nsEditProperty::i)
       || (nodeAtom == nsEditProperty::u)
@@ -81,7 +59,6 @@ nsHTMLEditUtils::IsInlineStyle(nsIDOMNode *node)
       || (nodeAtom == nsEditProperty::strike)
       || (nodeAtom == nsEditProperty::big)
       || (nodeAtom == nsEditProperty::small)
-      || (nodeAtom == nsEditProperty::blink)
       || (nodeAtom == nsEditProperty::sub)
       || (nodeAtom == nsEditProperty::sup)
       || (nodeAtom == nsEditProperty::font);
@@ -91,10 +68,18 @@ nsHTMLEditUtils::IsInlineStyle(nsIDOMNode *node)
 // IsFormatNode true if node is a format node
 // 
 bool
-nsHTMLEditUtils::IsFormatNode(nsIDOMNode *node)
+nsHTMLEditUtils::IsFormatNode(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null parent passed to nsHTMLEditUtils::IsFormatNode");
-  nsCOMPtr<nsIAtom> nodeAtom = nsEditor::GetTag(node);
+  NS_PRECONDITION(aNode, "null parent passed to nsHTMLEditUtils::IsFormatNode");
+  nsCOMPtr<nsINode> node = do_QueryInterface(aNode);
+  return node && IsFormatNode(node);
+}
+
+bool
+nsHTMLEditUtils::IsFormatNode(nsINode* aNode)
+{
+  MOZ_ASSERT(aNode);
+  nsIAtom* nodeAtom = aNode->Tag();
   return (nodeAtom == nsEditProperty::p)
       || (nodeAtom == nsEditProperty::pre)
       || (nodeAtom == nsEditProperty::h1)
@@ -110,10 +95,10 @@ nsHTMLEditUtils::IsFormatNode(nsIDOMNode *node)
 // IsNodeThatCanOutdent true if node is a list, list item, or blockquote      
 //
 bool
-nsHTMLEditUtils::IsNodeThatCanOutdent(nsIDOMNode *node)
+nsHTMLEditUtils::IsNodeThatCanOutdent(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null parent passed to nsHTMLEditUtils::IsNodeThatCanOutdent");
-  nsCOMPtr<nsIAtom> nodeAtom = nsEditor::GetTag(node);
+  NS_PRECONDITION(aNode, "null parent passed to nsHTMLEditUtils::IsNodeThatCanOutdent");
+  nsCOMPtr<nsIAtom> nodeAtom = nsEditor::GetTag(aNode);
   return (nodeAtom == nsEditProperty::ul)
       || (nodeAtom == nsEditProperty::ol)
       || (nodeAtom == nsEditProperty::dl)
@@ -126,9 +111,9 @@ nsHTMLEditUtils::IsNodeThatCanOutdent(nsIDOMNode *node)
 ///////////////////////////////////////////////////////////////////////////
 //                  
 bool 
-nsHTMLEditUtils::IsSmall(nsIDOMNode *node)
+nsHTMLEditUtils::IsSmall(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::small);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::small);
 }
 
 
@@ -140,10 +125,10 @@ nsHTMLEditUtils::IsSmall(nsIDOMNode *node)
 // IsHeader: true if node an html header
 //                  
 bool 
-nsHTMLEditUtils::IsHeader(nsIDOMNode *node)
+nsHTMLEditUtils::IsHeader(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null parent passed to nsHTMLEditUtils::IsHeader");
-  nsCOMPtr<nsIAtom> nodeAtom = nsEditor::GetTag(node);
+  NS_PRECONDITION(aNode, "null parent passed to nsHTMLEditUtils::IsHeader");
+  nsCOMPtr<nsIAtom> nodeAtom = nsEditor::GetTag(aNode);
   return (nodeAtom == nsEditProperty::h1)
       || (nodeAtom == nsEditProperty::h2)
       || (nodeAtom == nsEditProperty::h3)
@@ -157,9 +142,9 @@ nsHTMLEditUtils::IsHeader(nsIDOMNode *node)
 // IsParagraph: true if node an html paragraph
 //                  
 bool 
-nsHTMLEditUtils::IsParagraph(nsIDOMNode *node)
+nsHTMLEditUtils::IsParagraph(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::p);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::p);
 }
 
 
@@ -167,9 +152,9 @@ nsHTMLEditUtils::IsParagraph(nsIDOMNode *node)
 // IsHR: true if node an horizontal rule
 //                  
 bool 
-nsHTMLEditUtils::IsHR(nsIDOMNode *node)
+nsHTMLEditUtils::IsHR(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::hr);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::hr);
 }
 
 
@@ -177,15 +162,15 @@ nsHTMLEditUtils::IsHR(nsIDOMNode *node)
 // IsListItem: true if node an html list item
 //                  
 bool 
-nsHTMLEditUtils::IsListItem(nsIDOMNode *node)
+nsHTMLEditUtils::IsListItem(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null parent passed to nsHTMLEditUtils::IsListItem");
-  nsCOMPtr<dom::Element> element = do_QueryInterface(node);
-  return element && IsListItem(element);
+  NS_PRECONDITION(aNode, "null parent passed to nsHTMLEditUtils::IsListItem");
+  nsCOMPtr<nsINode> node = do_QueryInterface(aNode);
+  return node && IsListItem(node);
 }
 
 bool
-nsHTMLEditUtils::IsListItem(dom::Element* node)
+nsHTMLEditUtils::IsListItem(nsINode* node)
 {
   MOZ_ASSERT(node);
   nsCOMPtr<nsIAtom> nodeAtom = node->Tag();
@@ -199,15 +184,15 @@ nsHTMLEditUtils::IsListItem(dom::Element* node)
 // IsTableElement: true if node an html table, td, tr, ...
 //                  
 bool
-nsHTMLEditUtils::IsTableElement(nsIDOMNode *node)
+nsHTMLEditUtils::IsTableElement(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null node passed to nsHTMLEditor::IsTableElement");
-  nsCOMPtr<dom::Element> element = do_QueryInterface(node);
-  return element && IsTableElement(element);
+  NS_PRECONDITION(aNode, "null node passed to nsHTMLEditor::IsTableElement");
+  nsCOMPtr<nsINode> node = do_QueryInterface(aNode);
+  return node && IsTableElement(node);
 }
 
 bool
-nsHTMLEditUtils::IsTableElement(dom::Element* node)
+nsHTMLEditUtils::IsTableElement(nsINode* node)
 {
   MOZ_ASSERT(node);
   nsCOMPtr<nsIAtom> nodeAtom = node->Tag();
@@ -225,10 +210,18 @@ nsHTMLEditUtils::IsTableElement(dom::Element* node)
 // IsTableElementButNotTable: true if node an html td, tr, ... (doesn't include table)
 //                  
 bool 
-nsHTMLEditUtils::IsTableElementButNotTable(nsIDOMNode *node)
+nsHTMLEditUtils::IsTableElementButNotTable(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null node passed to nsHTMLEditor::IsTableElementButNotTable");
-  nsCOMPtr<nsIAtom> nodeAtom = nsEditor::GetTag(node);
+  NS_PRECONDITION(aNode, "null node passed to nsHTMLEditor::IsTableElementButNotTable");
+  nsCOMPtr<nsINode> node = do_QueryInterface(aNode);
+  return node && IsTableElementButNotTable(node);
+}
+
+bool
+nsHTMLEditUtils::IsTableElementButNotTable(nsINode* aNode)
+{
+  MOZ_ASSERT(aNode);
+  nsCOMPtr<nsIAtom> nodeAtom = aNode->Tag();
   return (nodeAtom == nsEditProperty::tr)
       || (nodeAtom == nsEditProperty::td)
       || (nodeAtom == nsEditProperty::th)
@@ -242,18 +235,18 @@ nsHTMLEditUtils::IsTableElementButNotTable(nsIDOMNode *node)
 // IsTable: true if node an html table
 //                  
 bool 
-nsHTMLEditUtils::IsTable(nsIDOMNode *node)
+nsHTMLEditUtils::IsTable(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::table);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::table);
 }
 
 ///////////////////////////////////////////////////////////////////////////
 // IsTableRow: true if node an html tr
 //                  
 bool 
-nsHTMLEditUtils::IsTableRow(nsIDOMNode *node)
+nsHTMLEditUtils::IsTableRow(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::tr);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::tr);
 }
 
 
@@ -261,15 +254,15 @@ nsHTMLEditUtils::IsTableRow(nsIDOMNode *node)
 // IsTableCell: true if node an html td or th
 //                  
 bool 
-nsHTMLEditUtils::IsTableCell(nsIDOMNode *node)
+nsHTMLEditUtils::IsTableCell(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null parent passed to nsHTMLEditUtils::IsTableCell");
-  nsCOMPtr<dom::Element> element = do_QueryInterface(node);
-  return element && IsTableCell(element);
+  NS_PRECONDITION(aNode, "null parent passed to nsHTMLEditUtils::IsTableCell");
+  nsCOMPtr<nsINode> node = do_QueryInterface(aNode);
+  return node && IsTableCell(node);
 }
 
 bool
-nsHTMLEditUtils::IsTableCell(dom::Element* node)
+nsHTMLEditUtils::IsTableCell(nsINode* node)
 {
   MOZ_ASSERT(node);
   nsCOMPtr<nsIAtom> nodeAtom = node->Tag();
@@ -282,10 +275,10 @@ nsHTMLEditUtils::IsTableCell(dom::Element* node)
 // IsTableCell: true if node an html td or th
 //                  
 bool 
-nsHTMLEditUtils::IsTableCellOrCaption(nsIDOMNode *node)
+nsHTMLEditUtils::IsTableCellOrCaption(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null parent passed to nsHTMLEditUtils::IsTableCell");
-  nsCOMPtr<nsIAtom> nodeAtom = nsEditor::GetTag(node);
+  NS_PRECONDITION(aNode, "null parent passed to nsHTMLEditUtils::IsTableCell");
+  nsCOMPtr<nsIAtom> nodeAtom = nsEditor::GetTag(aNode);
   return (nodeAtom == nsEditProperty::td)
       || (nodeAtom == nsEditProperty::th)
       || (nodeAtom == nsEditProperty::caption);
@@ -296,15 +289,15 @@ nsHTMLEditUtils::IsTableCellOrCaption(nsIDOMNode *node)
 // IsList: true if node an html list
 //                  
 bool
-nsHTMLEditUtils::IsList(nsIDOMNode *node)
+nsHTMLEditUtils::IsList(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null parent passed to nsHTMLEditUtils::IsList");
-  nsCOMPtr<dom::Element> element = do_QueryInterface(node);
-  return element && IsList(element);
+  NS_PRECONDITION(aNode, "null parent passed to nsHTMLEditUtils::IsList");
+  nsCOMPtr<nsINode> node = do_QueryInterface(aNode);
+  return node && IsList(node);
 }
 
 bool
-nsHTMLEditUtils::IsList(dom::Element* node)
+nsHTMLEditUtils::IsList(nsINode* node)
 {
   MOZ_ASSERT(node);
   nsCOMPtr<nsIAtom> nodeAtom = node->Tag();
@@ -318,9 +311,9 @@ nsHTMLEditUtils::IsList(dom::Element* node)
 // IsOrderedList: true if node an html ordered list
 //                  
 bool 
-nsHTMLEditUtils::IsOrderedList(nsIDOMNode *node)
+nsHTMLEditUtils::IsOrderedList(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::ol);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::ol);
 }
 
 
@@ -328,9 +321,9 @@ nsHTMLEditUtils::IsOrderedList(nsIDOMNode *node)
 // IsUnorderedList: true if node an html unordered list
 //                  
 bool 
-nsHTMLEditUtils::IsUnorderedList(nsIDOMNode *node)
+nsHTMLEditUtils::IsUnorderedList(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::ul);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::ul);
 }
 
 
@@ -338,9 +331,9 @@ nsHTMLEditUtils::IsUnorderedList(nsIDOMNode *node)
 // IsBlockquote: true if node an html blockquote node
 //                  
 bool 
-nsHTMLEditUtils::IsBlockquote(nsIDOMNode *node)
+nsHTMLEditUtils::IsBlockquote(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::blockquote);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::blockquote);
 }
 
 
@@ -348,9 +341,9 @@ nsHTMLEditUtils::IsBlockquote(nsIDOMNode *node)
 // IsPre: true if node an html pre node
 //                  
 bool 
-nsHTMLEditUtils::IsPre(nsIDOMNode *node)
+nsHTMLEditUtils::IsPre(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::pre);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::pre);
 }
 
 
@@ -358,9 +351,9 @@ nsHTMLEditUtils::IsPre(nsIDOMNode *node)
 // IsImage: true if node an html image node
 //                  
 bool 
-nsHTMLEditUtils::IsImage(nsIDOMNode *node)
+nsHTMLEditUtils::IsImage(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::img);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::img);
 }
 
 bool 
@@ -380,21 +373,21 @@ nsHTMLEditUtils::IsLink(nsIDOMNode *aNode)
 bool 
 nsHTMLEditUtils::IsNamedAnchor(nsIDOMNode *aNode)
 {
-  nsCOMPtr<dom::Element> element = do_QueryInterface(aNode);
-  return element && IsNamedAnchor(element);
+  nsCOMPtr<nsINode> node = do_QueryInterface(aNode);
+  return node && IsNamedAnchor(node);
 }
 
 bool
-nsHTMLEditUtils::IsNamedAnchor(dom::Element* aNode)
+nsHTMLEditUtils::IsNamedAnchor(nsINode* aNode)
 {
   MOZ_ASSERT(aNode);
-  if (!aNode->IsHTML(nsGkAtoms::a)) {
+  if (!aNode->IsElement() || !aNode->AsElement()->IsHTML(nsGkAtoms::a)) {
     return false;
   }
 
   nsAutoString text;
-  return aNode->GetAttr(kNameSpaceID_None, nsGkAtoms::name, text) &&
-         !text.IsEmpty();
+  return aNode->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::name,
+                                     text) && !text.IsEmpty();
 }
 
 
@@ -402,9 +395,9 @@ nsHTMLEditUtils::IsNamedAnchor(dom::Element* aNode)
 // IsDiv: true if node an html div node
 //                  
 bool 
-nsHTMLEditUtils::IsDiv(nsIDOMNode *node)
+nsHTMLEditUtils::IsDiv(nsIDOMNode* aNode)
 {
-  return nsEditor::NodeIsType(node, nsEditProperty::div);
+  return nsEditor::NodeIsType(aNode, nsEditProperty::div);
 }
 
 
@@ -412,9 +405,9 @@ nsHTMLEditUtils::IsDiv(nsIDOMNode *node)
 // IsMozDiv: true if node an html div node with type = _moz
 //                  
 bool 
-nsHTMLEditUtils::IsMozDiv(nsIDOMNode *node)
+nsHTMLEditUtils::IsMozDiv(nsIDOMNode* aNode)
 {
-  if (IsDiv(node) && nsTextEditUtils::HasMozAttr(node)) return true;
+  if (IsDiv(aNode) && nsTextEditUtils::HasMozAttr(aNode)) return true;
   return false;
 }
 
@@ -423,34 +416,33 @@ nsHTMLEditUtils::IsMozDiv(nsIDOMNode *node)
 ///////////////////////////////////////////////////////////////////////////
 // IsMailCite: true if node an html blockquote with type=cite
 //                  
-bool 
-nsHTMLEditUtils::IsMailCite(nsIDOMNode *node)
+bool
+nsHTMLEditUtils::IsMailCite(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null parent passed to nsHTMLEditUtils::IsMailCite");
-  nsCOMPtr<nsIDOMElement> elem = do_QueryInterface(node);
-  if (!elem) {
-    return false;
-  }
-  nsAutoString attrName (NS_LITERAL_STRING("type")); 
-  
+  NS_PRECONDITION(aNode, "null parent passed to nsHTMLEditUtils::IsMailCite");
+  nsCOMPtr<nsINode> node = do_QueryInterface(aNode);
+  return node && IsMailCite(node);
+}
+
+bool
+nsHTMLEditUtils::IsMailCite(nsINode* aNode)
+{
+  MOZ_ASSERT(aNode);
+
   // don't ask me why, but our html mailcites are id'd by "type=cite"...
-  nsAutoString attrVal;
-  nsresult res = elem->GetAttribute(attrName, attrVal);
-  ToLowerCase(attrVal);
-  if (NS_SUCCEEDED(res))
-  {
-    if (attrVal.EqualsLiteral("cite"))
-      return true;
+  if (aNode->IsElement() &&
+      aNode->AsElement()->AttrValueIs(kNameSpaceID_None, nsGkAtoms::type,
+                                      NS_LITERAL_STRING("cite"),
+                                      eIgnoreCase)) {
+    return true;
   }
 
   // ... but our plaintext mailcites by "_moz_quote=true".  go figure.
-  attrName.AssignLiteral("_moz_quote");
-  res = elem->GetAttribute(attrName, attrVal);
-  if (NS_SUCCEEDED(res))
-  {
-    ToLowerCase(attrVal);
-    if (attrVal.EqualsLiteral("true"))
-      return true;
+  if (aNode->IsElement() &&
+      aNode->AsElement()->AttrValueIs(kNameSpaceID_None, nsGkAtoms::mozquote,
+                                      NS_LITERAL_STRING("true"),
+                                      eIgnoreCase)) {
+    return true;
   }
 
   return false;
@@ -461,29 +453,30 @@ nsHTMLEditUtils::IsMailCite(nsIDOMNode *node)
 // IsFormWidget: true if node is a form widget of some kind
 //                  
 bool
-nsHTMLEditUtils::IsFormWidget(nsIDOMNode *node)
+nsHTMLEditUtils::IsFormWidget(nsIDOMNode* aNode)
 {
-  NS_PRECONDITION(node, "null node passed to nsHTMLEditUtils::IsFormWidget");
-  nsCOMPtr<dom::Element> element = do_QueryInterface(node);
-  return element && IsFormWidget(element);
+  NS_PRECONDITION(aNode, "null node passed to nsHTMLEditUtils::IsFormWidget");
+  nsCOMPtr<nsINode> node = do_QueryInterface(aNode);
+  return node && IsFormWidget(node);
 }
 
 bool
-nsHTMLEditUtils::IsFormWidget(dom::Element* node)
+nsHTMLEditUtils::IsFormWidget(nsINode* aNode)
 {
-  MOZ_ASSERT(node);
-  nsCOMPtr<nsIAtom> nodeAtom = node->Tag();
+  MOZ_ASSERT(aNode);
+  nsCOMPtr<nsIAtom> nodeAtom = aNode->Tag();
   return (nodeAtom == nsEditProperty::textarea)
       || (nodeAtom == nsEditProperty::select)
       || (nodeAtom == nsEditProperty::button)
       || (nodeAtom == nsEditProperty::output)
       || (nodeAtom == nsEditProperty::keygen)
       || (nodeAtom == nsEditProperty::progress)
+      || (nodeAtom == nsEditProperty::meter)
       || (nodeAtom == nsEditProperty::input);
 }
 
 bool
-nsHTMLEditUtils::SupportsAlignAttr(nsIDOMNode * aNode)
+nsHTMLEditUtils::SupportsAlignAttr(nsIDOMNode* aNode)
 {
   NS_PRECONDITION(aNode, "null node passed to nsHTMLEditUtils::SupportsAlignAttr");
   nsCOMPtr<nsIAtom> nodeAtom = nsEditor::GetTag(aNode);
@@ -530,7 +523,7 @@ nsHTMLEditUtils::SupportsAlignAttr(nsIDOMNode * aNode)
 // strong, var
 #define GROUP_PHRASE           (1 << 4)
 
-// a, applet, basefont, bdo, br, font, iframe, img, map, object, output,
+// a, applet, basefont, bdo, br, font, iframe, img, map, meter, object, output,
 // progress, q, script, span, sub, sup
 #define GROUP_SPECIAL          (1 << 5)
 
@@ -539,7 +532,7 @@ nsHTMLEditUtils::SupportsAlignAttr(nsIDOMNode * aNode)
 
 // address, applet, article, aside, blockquote, button, center, del, dir, div,
 // dl, fieldset, figure, footer, form, h1, h2, h3, h4, h5, h6, header, hgroup,
-// hr, iframe, ins, map, menu, nav, noframes, noscript, object, ol, p,
+// hr, iframe, ins, main, map, menu, nav, noframes, noscript, object, ol, p,
 // pre, table, section, ul
 #define GROUP_BLOCK            (1 << 7)
 
@@ -603,8 +596,8 @@ struct nsElementInfo
 #ifdef DEBUG
   eHTMLTags mTag;
 #endif
-  PRUint32 mGroup;
-  PRUint32 mCanContainGroups;
+  uint32_t mGroup;
+  uint32_t mCanContainGroups;
   bool mIsContainer;
   bool mCanContainSelf;
 };
@@ -628,16 +621,13 @@ static const nsElementInfo kElements[eHTMLTag_userdefined] = {
   ELEM(area, false, false, GROUP_MAP_CONTENT, GROUP_NONE),
   ELEM(article, true, true, GROUP_BLOCK, GROUP_FLOW_ELEMENT),
   ELEM(aside, true, true, GROUP_BLOCK, GROUP_FLOW_ELEMENT),
-#if defined(MOZ_MEDIA)
   ELEM(audio, false, false, GROUP_NONE, GROUP_NONE),
-#endif
   ELEM(b, true, true, GROUP_FONTSTYLE, GROUP_INLINE_ELEMENT),
   ELEM(base, false, false, GROUP_HEAD_CONTENT, GROUP_NONE),
   ELEM(basefont, false, false, GROUP_SPECIAL, GROUP_NONE),
   ELEM(bdo, true, true, GROUP_SPECIAL, GROUP_INLINE_ELEMENT),
   ELEM(bgsound, false, false, GROUP_NONE, GROUP_NONE),
   ELEM(big, true, true, GROUP_FONTSTYLE, GROUP_INLINE_ELEMENT),
-  ELEM(blink, false, false, GROUP_NONE, GROUP_NONE),
   ELEM(blockquote, true, true, GROUP_BLOCK, GROUP_FLOW_ELEMENT),
   ELEM(body, true, true, GROUP_TOPLEVEL, GROUP_FLOW_ELEMENT),
   ELEM(br, false, false, GROUP_SPECIAL, GROUP_NONE),
@@ -651,6 +641,7 @@ static const nsElementInfo kElements[eHTMLTag_userdefined] = {
   ELEM(col, false, false, GROUP_TABLE_CONTENT | GROUP_COLGROUP_CONTENT,
        GROUP_NONE),
   ELEM(colgroup, true, false, GROUP_NONE, GROUP_COLGROUP_CONTENT),
+  ELEM(data, true, false, GROUP_PHRASE, GROUP_INLINE_ELEMENT),
   ELEM(datalist, true, false, GROUP_PHRASE,
        GROUP_OPTIONS | GROUP_INLINE_ELEMENT),
   ELEM(dd, true, false, GROUP_DL_CONTENT, GROUP_FLOW_ELEMENT),
@@ -702,12 +693,14 @@ static const nsElementInfo kElements[eHTMLTag_userdefined] = {
   ELEM(li, true, false, GROUP_LI, GROUP_FLOW_ELEMENT),
   ELEM(link, false, false, GROUP_HEAD_CONTENT, GROUP_NONE),
   ELEM(listing, false, false, GROUP_NONE, GROUP_NONE),
+  ELEM(main, true, true, GROUP_BLOCK, GROUP_FLOW_ELEMENT),
   ELEM(map, true, true, GROUP_SPECIAL, GROUP_BLOCK | GROUP_MAP_CONTENT),
   ELEM(mark, true, true, GROUP_PHRASE, GROUP_INLINE_ELEMENT),
   ELEM(marquee, false, false, GROUP_NONE, GROUP_NONE),
   ELEM(menu, true, true, GROUP_BLOCK, GROUP_LI | GROUP_FLOW_ELEMENT),
   ELEM(menuitem, false, false, GROUP_NONE, GROUP_NONE),
   ELEM(meta, false, false, GROUP_HEAD_CONTENT, GROUP_NONE),
+  ELEM(meter, true, false, GROUP_SPECIAL, GROUP_FLOW_ELEMENT),
   ELEM(multicol, false, false, GROUP_NONE, GROUP_NONE),
   ELEM(nav, true, true, GROUP_BLOCK, GROUP_FLOW_ELEMENT),
   ELEM(nobr, false, false, GROUP_NONE, GROUP_NONE),
@@ -737,9 +730,7 @@ static const nsElementInfo kElements[eHTMLTag_userdefined] = {
   ELEM(section, true, true, GROUP_BLOCK, GROUP_FLOW_ELEMENT),
   ELEM(select, true, false, GROUP_FORMCONTROL, GROUP_SELECT_CONTENT),
   ELEM(small, true, true, GROUP_FONTSTYLE, GROUP_INLINE_ELEMENT),
-#if defined(MOZ_MEDIA)
   ELEM(source, false, false, GROUP_NONE, GROUP_NONE),
-#endif
   ELEM(span, true, true, GROUP_SPECIAL, GROUP_INLINE_ELEMENT),
   ELEM(strike, true, true, GROUP_FONTSTYLE, GROUP_INLINE_ELEMENT),
   ELEM(strong, true, true, GROUP_PHRASE, GROUP_INLINE_ELEMENT),
@@ -753,17 +744,18 @@ static const nsElementInfo kElements[eHTMLTag_userdefined] = {
   ELEM(tfoot, true, false, GROUP_NONE, GROUP_TBODY_CONTENT),
   ELEM(th, true, false, GROUP_TR_CONTENT, GROUP_FLOW_ELEMENT),
   ELEM(thead, true, false, GROUP_NONE, GROUP_TBODY_CONTENT),
+  ELEM(template, false, false, GROUP_NONE, GROUP_NONE),
+  ELEM(time, true, false, GROUP_PHRASE, GROUP_INLINE_ELEMENT),
   ELEM(title, true, false, GROUP_HEAD_CONTENT, GROUP_LEAF),
   ELEM(tr, true, false, GROUP_TBODY_CONTENT, GROUP_TR_CONTENT),
+  ELEM(track, false, false, GROUP_NONE, GROUP_NONE),
   ELEM(tt, true, true, GROUP_FONTSTYLE, GROUP_INLINE_ELEMENT),
   ELEM(u, true, true, GROUP_FONTSTYLE, GROUP_INLINE_ELEMENT),
   // XXX Can contain self and ol because editor does sublists illegally.
   ELEM(ul, true, true, GROUP_BLOCK | GROUP_OL_UL,
        GROUP_LI | GROUP_OL_UL),
   ELEM(var, true, true, GROUP_PHRASE, GROUP_INLINE_ELEMENT),
-#if defined(MOZ_MEDIA)
   ELEM(video, false, false, GROUP_NONE, GROUP_NONE),
-#endif
   ELEM(wbr, false, false, GROUP_NONE, GROUP_NONE),
   ELEM(xmp, false, false, GROUP_NONE, GROUP_NONE),
 
@@ -781,7 +773,7 @@ static const nsElementInfo kElements[eHTMLTag_userdefined] = {
 };
 
 bool
-nsHTMLEditUtils::CanContain(PRInt32 aParent, PRInt32 aChild)
+nsHTMLEditUtils::CanContain(int32_t aParent, int32_t aChild)
 {
   NS_ASSERTION(aParent > eHTMLTag_unknown && aParent <= eHTMLTag_userdefined,
                "aParent out of range!");
@@ -792,7 +784,7 @@ nsHTMLEditUtils::CanContain(PRInt32 aParent, PRInt32 aChild)
   static bool checked = false;
   if (!checked) {
     checked = true;
-    PRInt32 i;
+    int32_t i;
     for (i = 1; i <= eHTMLTag_userdefined; ++i) {
       NS_ASSERTION(kElements[i - 1].mTag == i,
                    "You need to update kElements (missing tags).");
@@ -812,7 +804,7 @@ nsHTMLEditUtils::CanContain(PRInt32 aParent, PRInt32 aChild)
       eHTMLTag_textarea
     };
 
-    PRUint32 j;
+    uint32_t j;
     for (j = 0; j < ArrayLength(kButtonExcludeKids); ++j) {
       if (kButtonExcludeKids[j] == aChild) {
         return false;
@@ -840,7 +832,7 @@ nsHTMLEditUtils::CanContain(PRInt32 aParent, PRInt32 aChild)
 } 
 
 bool
-nsHTMLEditUtils::IsContainer(PRInt32 aTag)
+nsHTMLEditUtils::IsContainer(int32_t aTag)
 {
   NS_ASSERTION(aTag > eHTMLTag_unknown && aTag <= eHTMLTag_userdefined,
                "aTag out of range!");
