@@ -1,52 +1,26 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "nsITransaction.h"
-#include "nsITransactionListener.h"
-
-#include "nsTransactionItem.h"
-#include "nsTransactionStack.h"
-#include "nsVoidArray.h"
-#include "nsTransactionManager.h"
-#include "nsTransactionList.h"
+#include "mozilla/Assertions.h"
+#include "mozilla/mozalloc.h"
 #include "nsAutoPtr.h"
 #include "nsCOMPtr.h"
+#include "nsDebug.h"
+#include "nsError.h"
+#include "nsISupportsBase.h"
+#include "nsISupportsUtils.h"
+#include "nsITransaction.h"
+#include "nsITransactionList.h"
+#include "nsITransactionListener.h"
+#include "nsIWeakReference.h"
+#include "nsTransactionItem.h"
+#include "nsTransactionList.h"
+#include "nsTransactionManager.h"
+#include "nsTransactionStack.h"
 
-nsTransactionManager::nsTransactionManager(PRInt32 aMaxTransactionCount)
+nsTransactionManager::nsTransactionManager(int32_t aMaxTransactionCount)
   : mMaxTransactionCount(aMaxTransactionCount)
   , mDoStack(nsTransactionStack::FOR_UNDO)
   , mUndoStack(nsTransactionStack::FOR_UNDO)
@@ -58,17 +32,15 @@ nsTransactionManager::~nsTransactionManager()
 {
 }
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(nsTransactionManager)
-
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsTransactionManager)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMARRAY(mListeners)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mListeners)
   tmp->mDoStack.DoUnlink();
   tmp->mUndoStack.DoUnlink();
   tmp->mRedoStack.DoUnlink();
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsTransactionManager)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMARRAY(mListeners)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mListeners)
   tmp->mDoStack.DoTraverse(cb);
   tmp->mUndoStack.DoTraverse(cb);
   tmp->mRedoStack.DoTraverse(cb);
@@ -102,14 +74,14 @@ nsTransactionManager::DoTransaction(nsITransaction *aTransaction)
     return NS_OK;
   }
 
-  result = BeginTransaction(aTransaction);
+  result = BeginTransaction(aTransaction, nullptr);
 
   if (NS_FAILED(result)) {
     DidDoNotify(aTransaction, result);
     return result;
   }
 
-  result = EndTransaction();
+  result = EndTransaction(false);
 
   nsresult result2 = DidDoNotify(aTransaction, result);
 
@@ -143,13 +115,7 @@ nsTransactionManager::UndoTransaction()
     return NS_OK;
   }
 
-  nsCOMPtr<nsITransaction> t;
-
-  result = tx->GetTransaction(getter_AddRefs(t));
-
-  if (NS_FAILED(result)) {
-    return result;
-  }
+  nsCOMPtr<nsITransaction> t = tx->GetTransaction();
 
   bool doInterrupt = false;
 
@@ -202,13 +168,7 @@ nsTransactionManager::RedoTransaction()
     return NS_OK;
   }
 
-  nsCOMPtr<nsITransaction> t;
-
-  result = tx->GetTransaction(getter_AddRefs(t));
-
-  if (NS_FAILED(result)) {
-    return result;
-  }
+  nsCOMPtr<nsITransaction> t = tx->GetTransaction();
 
   bool doInterrupt = false;
 
@@ -254,7 +214,7 @@ nsTransactionManager::Clear()
 }
 
 NS_IMETHODIMP
-nsTransactionManager::BeginBatch()
+nsTransactionManager::BeginBatch(nsISupports* aData)
 {
   nsresult result;
 
@@ -275,7 +235,7 @@ nsTransactionManager::BeginBatch()
     return NS_OK;
   }
 
-  result = BeginTransaction(0);
+  result = BeginTransaction(0, aData);
   
   nsresult result2 = DidBeginBatchNotify(result);
 
@@ -286,7 +246,7 @@ nsTransactionManager::BeginBatch()
 }
 
 NS_IMETHODIMP
-nsTransactionManager::EndBatch()
+nsTransactionManager::EndBatch(bool aAllowEmpty)
 {
   nsCOMPtr<nsITransaction> ti;
   nsresult result;
@@ -298,14 +258,15 @@ nsTransactionManager::EndBatch()
   //
   //      For now, we can detect this case by checking the value of the
   //      dummy transaction's mTransaction field. If it is our dummy
-  //      transaction, it should be NULL. This may not be true in the
+  //      transaction, it should be nullptr. This may not be true in the
   //      future when we allow users to execute a transaction when beginning
   //      a batch!!!!
 
   nsRefPtr<nsTransactionItem> tx = mDoStack.Peek();
 
-  if (tx)
-    tx->GetTransaction(getter_AddRefs(ti));
+  if (tx) {
+    ti = tx->GetTransaction();
+  }
 
   if (!tx || ti) {
     return NS_ERROR_FAILURE;
@@ -323,7 +284,7 @@ nsTransactionManager::EndBatch()
     return NS_OK;
   }
 
-  result = EndTransaction();
+  result = EndTransaction(aAllowEmpty);
 
   nsresult result2 = DidEndBatchNotify(result);
 
@@ -334,21 +295,21 @@ nsTransactionManager::EndBatch()
 }
 
 NS_IMETHODIMP
-nsTransactionManager::GetNumberOfUndoItems(PRInt32 *aNumItems)
+nsTransactionManager::GetNumberOfUndoItems(int32_t *aNumItems)
 {
   *aNumItems = mUndoStack.GetSize();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsTransactionManager::GetNumberOfRedoItems(PRInt32 *aNumItems)
+nsTransactionManager::GetNumberOfRedoItems(int32_t *aNumItems)
 {
   *aNumItems = mRedoStack.GetSize();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsTransactionManager::GetMaxTransactionCount(PRInt32 *aMaxCount)
+nsTransactionManager::GetMaxTransactionCount(int32_t *aMaxCount)
 {
   NS_ENSURE_TRUE(aMaxCount, NS_ERROR_NULL_POINTER);
 
@@ -358,9 +319,9 @@ nsTransactionManager::GetMaxTransactionCount(PRInt32 *aMaxCount)
 }
 
 NS_IMETHODIMP
-nsTransactionManager::SetMaxTransactionCount(PRInt32 aMaxCount)
+nsTransactionManager::SetMaxTransactionCount(int32_t aMaxCount)
 {
-  PRInt32 numUndoItems  = 0, numRedoItems = 0, total = 0;
+  int32_t numUndoItems  = 0, numRedoItems = 0, total = 0;
 
   // It is illegal to call SetMaxTransactionCount() while the transaction
   // manager is executing a  transaction's DoTransaction() method because
@@ -431,41 +392,41 @@ nsTransactionManager::SetMaxTransactionCount(PRInt32 aMaxCount)
 NS_IMETHODIMP
 nsTransactionManager::PeekUndoStack(nsITransaction **aTransaction)
 {
-  nsresult result;
-
-  NS_ENSURE_TRUE(aTransaction, NS_ERROR_NULL_POINTER);
-
-  *aTransaction = 0;
+  MOZ_ASSERT(aTransaction);
+  *aTransaction = PeekUndoStack().get();
+  return NS_OK;
+}
 
+already_AddRefed<nsITransaction>
+nsTransactionManager::PeekUndoStack()
+{
   nsRefPtr<nsTransactionItem> tx = mUndoStack.Peek();
 
   if (!tx) {
-    return NS_OK;
+    return nullptr;
   }
 
-  result = tx->GetTransaction(aTransaction);
-
-  return result;
+  return tx->GetTransaction();
 }
 
 NS_IMETHODIMP
-nsTransactionManager::PeekRedoStack(nsITransaction **aTransaction)
+nsTransactionManager::PeekRedoStack(nsITransaction** aTransaction)
 {
-  nsresult result;
-
-  NS_ENSURE_TRUE(aTransaction, NS_ERROR_NULL_POINTER);
-
-  *aTransaction = 0;
+  MOZ_ASSERT(aTransaction);
+  *aTransaction = PeekRedoStack().get();
+  return NS_OK;
+}
 
+already_AddRefed<nsITransaction>
+nsTransactionManager::PeekRedoStack()
+{
   nsRefPtr<nsTransactionItem> tx = mRedoStack.Peek();
 
   if (!tx) {
-    return NS_OK;
+    return nullptr;
   }
 
-  result = tx->GetTransaction(aTransaction);
-
-  return result;
+  return tx->GetTransaction();
 }
 
 NS_IMETHODIMP
@@ -492,6 +453,50 @@ nsTransactionManager::GetRedoList(nsITransactionList **aTransactionList)
   return (! *aTransactionList) ? NS_ERROR_OUT_OF_MEMORY : NS_OK;
 }
 
+nsresult
+nsTransactionManager::BatchTopUndo()
+{
+  if (mUndoStack.GetSize() < 2) {
+    // Not enough transactions to merge into one batch.
+    return NS_OK;
+  }
+
+  nsRefPtr<nsTransactionItem> lastUndo;
+  nsRefPtr<nsTransactionItem> previousUndo;
+
+  lastUndo = mUndoStack.Pop();
+  MOZ_ASSERT(lastUndo, "There should be at least two transactions.");
+
+  previousUndo = mUndoStack.Peek();
+  MOZ_ASSERT(previousUndo, "There should be at least two transactions.");
+
+  nsresult result = previousUndo->AddChild(lastUndo);
+
+  // Transfer data from the transactions that is going to be
+  // merged to the transaction that it is being merged with.
+  nsCOMArray<nsISupports>& lastData = lastUndo->GetData();
+  nsCOMArray<nsISupports>& previousData = previousUndo->GetData();
+  NS_ENSURE_TRUE(previousData.AppendObjects(lastData), NS_ERROR_UNEXPECTED);
+  lastData.Clear();
+
+  return result;
+}
+
+nsresult
+nsTransactionManager::RemoveTopUndo()
+{
+  nsRefPtr<nsTransactionItem> lastUndo;
+
+  lastUndo = mUndoStack.Peek();
+  if (!lastUndo) {
+    return NS_OK;
+  }
+
+  lastUndo = mUndoStack.Pop();
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsTransactionManager::AddListener(nsITransactionListener *aListener)
 {
@@ -508,14 +513,14 @@ nsTransactionManager::RemoveListener(nsITransactionListener *aListener)
   return mListeners.RemoveObject(aListener) ? NS_OK : NS_ERROR_FAILURE;
 }
 
-nsresult
+NS_IMETHODIMP
 nsTransactionManager::ClearUndoStack()
 {
   mUndoStack.Clear();
   return NS_OK;
 }
 
-nsresult
+NS_IMETHODIMP
 nsTransactionManager::ClearRedoStack()
 {
   mRedoStack.Clear();
@@ -526,7 +531,7 @@ nsresult
 nsTransactionManager::WillDoNotify(nsITransaction *aTransaction, bool *aInterrupt)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -545,7 +550,7 @@ nsresult
 nsTransactionManager::DidDoNotify(nsITransaction *aTransaction, nsresult aDoResult)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -564,7 +569,7 @@ nsresult
 nsTransactionManager::WillUndoNotify(nsITransaction *aTransaction, bool *aInterrupt)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -583,7 +588,7 @@ nsresult
 nsTransactionManager::DidUndoNotify(nsITransaction *aTransaction, nsresult aUndoResult)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -602,7 +607,7 @@ nsresult
 nsTransactionManager::WillRedoNotify(nsITransaction *aTransaction, bool *aInterrupt)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -621,7 +626,7 @@ nsresult
 nsTransactionManager::DidRedoNotify(nsITransaction *aTransaction, nsresult aRedoResult)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -640,7 +645,7 @@ nsresult
 nsTransactionManager::WillBeginBatchNotify(bool *aInterrupt)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -659,7 +664,7 @@ nsresult
 nsTransactionManager::DidBeginBatchNotify(nsresult aResult)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -678,7 +683,7 @@ nsresult
 nsTransactionManager::WillEndBatchNotify(bool *aInterrupt)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -697,7 +702,7 @@ nsresult
 nsTransactionManager::DidEndBatchNotify(nsresult aResult)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -716,7 +721,7 @@ nsresult
 nsTransactionManager::WillMergeNotify(nsITransaction *aTop, nsITransaction *aTransaction, bool *aInterrupt)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -738,7 +743,7 @@ nsTransactionManager::DidMergeNotify(nsITransaction *aTop,
                                      nsresult aMergeResult)
 {
   nsresult result = NS_OK;
-  for (PRInt32 i = 0, lcount = mListeners.Count(); i < lcount; i++)
+  for (int32_t i = 0, lcount = mListeners.Count(); i < lcount; i++)
   {
     nsITransactionListener *listener = mListeners[i];
 
@@ -754,7 +759,8 @@ nsTransactionManager::DidMergeNotify(nsITransaction *aTop,
 }
 
 nsresult
-nsTransactionManager::BeginTransaction(nsITransaction *aTransaction)
+nsTransactionManager::BeginTransaction(nsITransaction *aTransaction,
+                                       nsISupports *aData)
 {
   nsresult result = NS_OK;
 
@@ -762,6 +768,11 @@ nsTransactionManager::BeginTransaction(nsITransaction *aTransaction)
   //      We could use a factory that pre-allocates/recycles transaction items.
   nsRefPtr<nsTransactionItem> tx = new nsTransactionItem(aTransaction);
 
+  if (aData) {
+    nsCOMArray<nsISupports>& data = tx->GetData();
+    data.AppendObject(aData);
+  }
+
   if (!tx) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -779,9 +790,8 @@ nsTransactionManager::BeginTransaction(nsITransaction *aTransaction)
 }
 
 nsresult
-nsTransactionManager::EndTransaction()
+nsTransactionManager::EndTransaction(bool aAllowEmpty)
 {
-  nsCOMPtr<nsITransaction> tint;
   nsresult result              = NS_OK;
 
   nsRefPtr<nsTransactionItem> tx = mDoStack.Pop();
@@ -789,15 +799,10 @@ nsTransactionManager::EndTransaction()
   if (!tx)
     return NS_ERROR_FAILURE;
 
-  result = tx->GetTransaction(getter_AddRefs(tint));
+  nsCOMPtr<nsITransaction> tint = tx->GetTransaction();
 
-  if (NS_FAILED(result)) {
-    // XXX: What do we do with the transaction item at this point?
-    return result;
-  }
-
-  if (!tint) {
-    PRInt32 nc = 0;
+  if (!tint && !aAllowEmpty) {
+    int32_t nc = 0;
 
     // If we get here, the transaction must be a dummy batch transaction
     // created by BeginBatch(). If it contains no children, get rid of it!
@@ -851,9 +856,7 @@ nsTransactionManager::EndTransaction()
 
   if (tint && top) {
     bool didMerge = false;
-    nsCOMPtr<nsITransaction> topTransaction;
-
-    result = top->GetTransaction(getter_AddRefs(topTransaction));
+    nsCOMPtr<nsITransaction> topTransaction = top->GetTransaction();
 
     if (topTransaction) {
 
@@ -885,7 +888,7 @@ nsTransactionManager::EndTransaction()
   // Check to see if we've hit the max level of undo. If so,
   // pop the bottom transaction off the undo stack and release it!
 
-  PRInt32 sz = mUndoStack.GetSize();
+  int32_t sz = mUndoStack.GetSize();
 
   if (mMaxTransactionCount > 0 && sz >= mMaxTransactionCount) {
     nsRefPtr<nsTransactionItem> overflow = mUndoStack.PopBottom();
