@@ -1,38 +1,7 @@
 /* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Bas Schouten <bschouten@mozilla.com>
- *   Vladimir Vukicevic <vladimir@pobox.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "GLContextProvider.h"
 #include "GLContext.h"
@@ -51,20 +20,25 @@
 
 #include "mozilla/Preferences.h"
 
+using namespace mozilla::gfx;
+
 namespace mozilla {
 namespace gl {
 
-WGLLibrary sWGLLibrary;
+typedef WGLLibrary::LibraryType LibType;
 
-static HWND gSharedWindow = 0;
-static HDC gSharedWindowDC = 0;
-static HGLRC gSharedWindowGLContext = 0;
-static int gSharedWindowPixelFormat = 0;
+WGLLibrary sWGLLib[WGLLibrary::LIBS_MAX];
 
-static bool gUseDoubleBufferedWindows = false;
+LibType
+WGLLibrary::SelectLibrary(const GLContext::ContextFlags& aFlags)
+{
+  return (aFlags & GLContext::ContextFlagsMesaLLVMPipe) 
+          ? WGLLibrary::MESA_LLVMPIPE_LIB
+          : WGLLibrary::OPENGL_LIB;
+}
 
-static HWND
-CreateDummyWindow(HDC *aWindowDC = nsnull)
+HWND
+WGLLibrary::CreateDummyWindow(HDC *aWindowDC)
 {
     WNDCLASSW wc;
     if (!GetClassInfoW(GetModuleHandle(NULL), L"GLContextWGLClass", &wc)) {
@@ -88,13 +62,13 @@ CreateDummyWindow(HDC *aWindowDC = nsnull)
     HDC dc = GetDC(win);
     NS_ENSURE_TRUE(dc, NULL);
 
-    if (gSharedWindowPixelFormat == 0) {
+    if (mWindowPixelFormat == 0) {
         PIXELFORMATDESCRIPTOR pfd;
         ZeroMemory(&pfd, sizeof(PIXELFORMATDESCRIPTOR));
         pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);
         pfd.nVersion = 1;
         pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL;
-        if (gUseDoubleBufferedWindows)
+        if (mUseDoubleBufferedWindows)
             pfd.dwFlags |= PFD_DOUBLEBUFFER;
         pfd.iPixelType = PFD_TYPE_RGBA;
         pfd.cColorBits = 24;
@@ -105,11 +79,11 @@ CreateDummyWindow(HDC *aWindowDC = nsnull)
         pfd.cDepthBits = 0;
         pfd.iLayerType = PFD_MAIN_PLANE;
 
-        gSharedWindowPixelFormat = ChoosePixelFormat(dc, &pfd);
+        mWindowPixelFormat = ChoosePixelFormat(dc, &pfd);
     }
 
-    if (!gSharedWindowPixelFormat ||
-        !SetPixelFormat(dc, gSharedWindowPixelFormat, NULL))
+    if (!mWindowPixelFormat ||
+        !SetPixelFormat(dc, mWindowPixelFormat, NULL))
     {
         NS_WARNING("SetPixelFormat failed!");
         DestroyWindow(win);
@@ -131,22 +105,25 @@ HasExtension(const char* aExtensions, const char* aRequiredExtension)
 }
 
 bool
-WGLLibrary::EnsureInitialized()
+WGLLibrary::EnsureInitialized(bool aUseMesaLlvmPipe)
 {
     if (mInitialized)
         return true;
+    
+    mozilla::ScopedGfxFeatureReporter reporter("WGL", aUseMesaLlvmPipe);
 
-    mozilla::ScopedGfxFeatureReporter reporter("WGL");
-
+    const char* libGLFilename = aUseMesaLlvmPipe 
+                                ? "mesallvmpipe.dll" 
+                                : "Opengl32.dll";
     if (!mOGLLibrary) {
-        mOGLLibrary = PR_LoadLibrary("Opengl32.dll");
+        mOGLLibrary = PR_LoadLibrary(libGLFilename);
         if (!mOGLLibrary) {
-            NS_WARNING("Couldn't load OpenGL DLL.");
+            NS_WARNING("Couldn't load OpenGL library.");
             return false;
         }
     }
 
-    gUseDoubleBufferedWindows = PR_GetEnv("MOZ_WGL_DB") != nsnull;
+    mUseDoubleBufferedWindows = PR_GetEnv("MOZ_WGL_DB") != nullptr;
 
     GLLibraryLoader::SymLoadStruct earlySymbols[] = {
         { (PRFuncPtr*) &fCreateContext, { "wglCreateContext", NULL } },
@@ -166,17 +143,17 @@ WGLLibrary::EnsureInitialized()
 
     // This is ridiculous -- we have to actually create a context to
     // get the OpenGL ICD to load.
-    gSharedWindow = CreateDummyWindow(&gSharedWindowDC);
-    NS_ENSURE_TRUE(gSharedWindow, false);
+    mWindow = CreateDummyWindow(&mWindowDC);
+    NS_ENSURE_TRUE(mWindow, false);
 
     // create rendering context
-    gSharedWindowGLContext = fCreateContext(gSharedWindowDC);
-    NS_ENSURE_TRUE(gSharedWindowGLContext, false);
+    mWindowGLContext = fCreateContext(mWindowDC);
+    NS_ENSURE_TRUE(mWindowGLContext, false);
 
     HGLRC curCtx = fGetCurrentContext();
     HDC curDC = fGetCurrentDC();
 
-    if (!fMakeCurrent((HDC)gSharedWindowDC, (HGLRC)gSharedWindowGLContext)) {
+    if (!fMakeCurrent((HDC)mWindowDC, (HGLRC)mWindowGLContext)) {
         NS_WARNING("wglMakeCurrent failed");
         return false;
     }
@@ -203,14 +180,14 @@ WGLLibrary::EnsureInitialized()
          (GLLibraryLoader::PlatformLookupFunction)fGetProcAddress))
     {
         // this isn't an error, just means that pbuffers aren't supported
-        fCreatePbuffer = nsnull;
+        fCreatePbuffer = nullptr;
     }
 
     if (!GLLibraryLoader::LoadSymbols(mOGLLibrary, &pixFmtSymbols[0],
          (GLLibraryLoader::PlatformLookupFunction)fGetProcAddress))
     {
         // this isn't an error, just means that we don't have the pixel format extension
-        fChoosePixelFormat = nsnull;
+        fChoosePixelFormat = nullptr;
     }
 
     GLLibraryLoader::SymLoadStruct extensionsSymbols[] = {
@@ -225,7 +202,7 @@ WGLLibrary::EnsureInitialized()
 
     if (GLLibraryLoader::LoadSymbols(mOGLLibrary, &extensionsSymbols[0],
         (GLLibraryLoader::PlatformLookupFunction)fGetProcAddress)) {
-        const char *wglExts = fGetExtensionsString(gSharedWindowDC);
+        const char *wglExts = fGetExtensionsString(mWindowDC);
         if (wglExts && HasExtension(wglExts, "WGL_ARB_create_context")) {
             GLLibraryLoader::LoadSymbols(mOGLLibrary, &robustnessSymbols[0],
             (GLLibraryLoader::PlatformLookupFunction)fGetProcAddress);
@@ -239,27 +216,33 @@ WGLLibrary::EnsureInitialized()
     fMakeCurrent(curDC, curCtx);
 
     if (mHasRobustness) {
-        fDeleteContext(gSharedWindowGLContext);
+        fDeleteContext(mWindowGLContext);
 
         int attribs[] = {
             LOCAL_WGL_CONTEXT_FLAGS_ARB, LOCAL_WGL_CONTEXT_ROBUST_ACCESS_BIT_ARB,
             LOCAL_WGL_CONTEXT_RESET_NOTIFICATION_STRATEGY_ARB, LOCAL_WGL_LOSE_CONTEXT_ON_RESET_ARB,
-            NULL
+            0
         };
 
-        gSharedWindowGLContext = fCreateContextAttribs(gSharedWindowDC, NULL, attribs);
-        if (!gSharedWindowGLContext) {
+        mWindowGLContext = fCreateContextAttribs(mWindowDC, NULL, attribs);
+        if (!mWindowGLContext) {
             mHasRobustness = false;
-            gSharedWindowGLContext = fCreateContext(gSharedWindowDC);
+            mWindowGLContext = fCreateContext(mWindowDC);
         }
     }
 
     mInitialized = true;
 
+    GLContext::ContextFlags flag = GLContext::ContextFlagsNone;
+    if (aUseMesaLlvmPipe) {
+      mLibType = WGLLibrary::MESA_LLVMPIPE_LIB;
+      flag = GLContext::ContextFlagsMesaLLVMPipe;
+    }
+
     // Call this to create the global GLContext instance,
     // and to check for errors.  Note that this must happen /after/
     // setting mInitialized to TRUE, or an infinite loop results.
-    if (GLContextProviderWGL::GetGlobalContext() == nsnull) {
+    if (GLContextProviderWGL::GetGlobalContext(flag) == nullptr) {
         mInitialized = false;
         return false;
     }
@@ -271,34 +254,41 @@ WGLLibrary::EnsureInitialized()
 class GLContextWGL : public GLContext
 {
 public:
-    GLContextWGL(const ContextFormat& aFormat,
-                 GLContext *aSharedContext,
+    // From Window: (possibly for offscreen!)
+    GLContextWGL(const SurfaceCaps& caps,
+                 GLContext* sharedContext,
+                 bool isOffscreen,
                  HDC aDC,
                  HGLRC aContext,
-                 HWND aWindow = nsnull,
-                 bool aIsOffscreen = false)
-        : GLContext(aFormat, aIsOffscreen, aSharedContext),
+                 LibType aLibUsed,
+                 HWND aWindow = nullptr)
+        : GLContext(caps, sharedContext, isOffscreen),
           mDC(aDC),
           mContext(aContext),
           mWnd(aWindow),
           mPBuffer(NULL),
           mPixelFormat(0),
+          mLibType(aLibUsed),
           mIsDoubleBuffered(false)
     {
     }
 
-    GLContextWGL(const ContextFormat& aFormat,
-                 GLContext *aSharedContext,
+    // From PBuffer
+    GLContextWGL(const SurfaceCaps& caps,
+                 GLContext* sharedContext,
+                 bool isOffscreen,
                  HANDLE aPbuffer,
                  HDC aDC,
                  HGLRC aContext,
-                 int aPixelFormat)
-        : GLContext(aFormat, true, aSharedContext),
+                 int aPixelFormat,
+                 LibType aLibUsed)
+        : GLContext(caps, sharedContext, isOffscreen),
           mDC(aDC),
           mContext(aContext),
           mWnd(NULL),
           mPBuffer(aPbuffer),
           mPixelFormat(aPixelFormat),
+          mLibType(aLibUsed),
           mIsDoubleBuffered(false)
     {
     }
@@ -307,10 +297,10 @@ public:
     {
         MarkDestroyed();
 
-        sWGLLibrary.fDeleteContext(mContext);
+        sWGLLib[mLibType].fDeleteContext(mContext);
 
         if (mPBuffer)
-            sWGLLibrary.fDestroyPbuffer(mPBuffer);
+            sWGLLib[mLibType].fDestroyPbuffer(mPBuffer);
         if (mWnd)
             DestroyWindow(mWnd);
     }
@@ -329,7 +319,6 @@ public:
         if (!InitWithPrefix("gl", true))
             return false;
 
-        InitFramebuffers();
         return true;
     }
 
@@ -341,14 +330,18 @@ public:
         // of its TLS slot, so no need to do our own tls slot.
         // You would think that wglMakeCurrent would avoid doing
         // work if mContext was already current, but not so much..
-        if (aForce || sWGLLibrary.fGetCurrentContext() != mContext) {
-            succeeded = sWGLLibrary.fMakeCurrent(mDC, mContext);
+        if (aForce || sWGLLib[mLibType].fGetCurrentContext() != mContext) {
+            succeeded = sWGLLib[mLibType].fMakeCurrent(mDC, mContext);
             NS_ASSERTION(succeeded, "Failed to make GL context current!");
         }
 
         return succeeded;
     }
 
+    virtual bool IsCurrent() {
+        return sWGLLib[mLibType].fGetCurrentContext() == mContext;
+    }
+
     void SetIsDoubleBuffered(bool aIsDB) {
         mIsDoubleBuffered = aIsDB;
     }
@@ -359,7 +352,7 @@ public:
 
     bool SupportsRobustness()
     {
-        return sWGLLibrary.HasRobustness();
+        return sWGLLib[mLibType].HasRobustness();
     }
 
     virtual bool SwapBuffers() {
@@ -370,7 +363,7 @@ public:
 
     bool SetupLookupFunction()
     {
-        mLookupFunc = (PlatformLookupFunction)sWGLLibrary.fGetProcAddress;
+        mLookupFunc = (PlatformLookupFunction)sWGLLib[mLibType].fGetProcAddress;
         return true;
     }
 
@@ -381,12 +374,10 @@ public:
             return mContext;
 
         default:
-            return nsnull;
+            return nullptr;
         }
     }
 
-    bool BindTex2DOffscreen(GLContext *aOffscreen);
-    void UnbindTex2DOffscreen(GLContext *aOffscreen);
     bool ResizeOffscreen(const gfxIntSize& aNewSize);
 
     HGLRC Context() { return mContext; }
@@ -399,71 +390,19 @@ protected:
     HWND mWnd;
     HANDLE mPBuffer;
     int mPixelFormat;
-
+    LibType mLibType;
     bool mIsDoubleBuffered;
 };
 
-bool
-GLContextWGL::BindTex2DOffscreen(GLContext *aOffscreen)
-{
-    if (aOffscreen->GetContextType() != ContextTypeWGL) {
-        NS_WARNING("non-WGL context");
-        return false;
-    }
-
-    if (!aOffscreen->IsOffscreen()) {
-        NS_WARNING("non-offscreen context");
-        return false;
-    }
-
-    GLContextWGL *offs = static_cast<GLContextWGL*>(aOffscreen);
-
-    if (offs->mPBuffer) {
-        BOOL ok = sWGLLibrary.fBindTexImage(offs->mPBuffer,
-                                            LOCAL_WGL_FRONT_LEFT_ARB);
-        if (!ok) {
-            NS_WARNING("CanvasLayerOGL::Updated wglBindTexImageARB failed");
-            return false;
-        }
-    } else if (offs->mOffscreenTexture) {
-        if (offs->GetSharedContext() != GLContextProviderWGL::GetGlobalContext())
-        {
-            NS_WARNING("offscreen FBO context can only be bound with context sharing!");
-            return false;
-        }
-
-        fBindTexture(LOCAL_GL_TEXTURE_2D, offs->mOffscreenTexture);
-    } else {
-        NS_WARNING("don't know how to bind this!");
-        return false;
-    }
-
-    return true;
-}
-
-void
-GLContextWGL::UnbindTex2DOffscreen(GLContext *aOffscreen)
-{
-    NS_ASSERTION(aOffscreen->GetContextType() == ContextTypeWGL, "wrong type");
-
-    GLContextWGL *offs = static_cast<GLContextWGL*>(aOffscreen);
-    if (offs->mPBuffer) {
-        // XXX so, according to the extension, ReleaseTexImage is not required to
-        // preserve color buffer contents.  This sucks, but everywhere that I've
-        // tried it the color buffer is preserved.  So let's cross our fingers..
-        sWGLLibrary.fReleaseTexImage(offs->mPBuffer, LOCAL_WGL_FRONT_LEFT_ARB);
-    }
-}
-
 
 static bool
-GetMaxSize(HDC hDC, int format, gfxIntSize& size)
+GetMaxSize(HDC hDC, int format, gfxIntSize& size, LibType aLibToUse)
 {
     int query[] = {LOCAL_WGL_MAX_PBUFFER_WIDTH_ARB, LOCAL_WGL_MAX_PBUFFER_HEIGHT_ARB};
     int result[2];
 
     // (HDC hdc, int iPixelFormat, int iLayerPlane, UINT nAttributes, int* piAttributes, int *piValues)
-    if (!sWGLLibrary.fGetPixelFormatAttribiv(hDC, format, 0, 2, query, result))
+    if (!sWGLLib[aLibToUse].fGetPixelFormatAttribiv(hDC, format, 0, 2, query, result))
         return false;
 
     size.width = result[0];
@@ -472,10 +411,12 @@ GetMaxSize(HDC hDC, int format, gfxIntSize& size)
 }
 
 static bool
-IsValidSizeForFormat(HDC hDC, int format, const gfxIntSize& requested)
+IsValidSizeForFormat(HDC hDC, int format, 
+                     const gfxIntSize& requested,
+                     LibType aLibUsed)
 {
     gfxIntSize max;
-    if (!GetMaxSize(hDC, format, max))
+    if (!GetMaxSize(hDC, format, max, aLibUsed))
         return true;
 
     if (requested.width > max.width)
@@ -489,58 +430,22 @@ IsValidSizeForFormat(HDC hDC, int format, const gfxIntSize& requested)
 bool
 GLContextWGL::ResizeOffscreen(const gfxIntSize& aNewSize)
 {
-    if (mPBuffer) {
-        if (!IsValidSizeForFormat(gSharedWindowDC, mPixelFormat, aNewSize))
-            return false;
-
-        int pbattrs[] = {
-            LOCAL_WGL_TEXTURE_FORMAT_ARB,
-              mCreationFormat.alpha > 0 ? LOCAL_WGL_TEXTURE_RGBA_ARB
-                                        : LOCAL_WGL_TEXTURE_RGB_ARB,
-            LOCAL_WGL_TEXTURE_TARGET_ARB, LOCAL_WGL_TEXTURE_2D_ARB,
-            0
-        };
-
-        HANDLE newbuf = sWGLLibrary.fCreatePbuffer(gSharedWindowDC, mPixelFormat,
-                                                   aNewSize.width, aNewSize.height,
-                                                   pbattrs);
-        if (!newbuf)
-            return false;
-
-        bool isCurrent = false;
-        if (sWGLLibrary.fGetCurrentContext() == mContext) {
-            sWGLLibrary.fMakeCurrent(NULL, NULL);
-            isCurrent = true;
-        }
-
-        sWGLLibrary.fDestroyPbuffer(mPBuffer);
-
-        mPBuffer = newbuf;
-        mDC = sWGLLibrary.fGetPbufferDC(mPBuffer);
-
-        mOffscreenSize = aNewSize;
-        mOffscreenActualSize = aNewSize;
-
-        MakeCurrent();
-        ClearSafely();
-
-        return ResizeOffscreenFBO(aNewSize, false);
-    }
-
-    return ResizeOffscreenFBO(aNewSize, true);
+    return ResizeScreenBuffer(aNewSize);
 }
 
 static GLContextWGL *
-GetGlobalContextWGL()
+GetGlobalContextWGL(const GLContext::ContextFlags aFlags = GLContext::ContextFlagsNone)
 {
-    return static_cast<GLContextWGL*>(GLContextProviderWGL::GetGlobalContext());
+    return static_cast<GLContextWGL*>(GLContextProviderWGL::GetGlobalContext(aFlags));
 }
 
 already_AddRefed<GLContext>
 GLContextProviderWGL::CreateForWindow(nsIWidget *aWidget)
 {
-    if (!sWGLLibrary.EnsureInitialized()) {
-        return nsnull;
+    LibType libToUse = WGLLibrary::OPENGL_LIB;
+    
+    if (!sWGLLib[libToUse].EnsureInitialized(false)) {
+        return nullptr;
     }
 
     /**
@@ -551,61 +456,68 @@ GLContextProviderWGL::CreateForWindow(nsIWidget *aWidget)
 
     HDC dc = (HDC)aWidget->GetNativeData(NS_NATIVE_GRAPHIC);
 
-    SetPixelFormat(dc, gSharedWindowPixelFormat, NULL);
+    SetPixelFormat(dc, sWGLLib[libToUse].GetWindowPixelFormat(), NULL);
     HGLRC context;
 
     GLContextWGL *shareContext = GetGlobalContextWGL();
 
-    if (sWGLLibrary.HasRobustness()) {
+    if (sWGLLib[libToUse].HasRobustness()) {
         int attribs[] = {
             LOCAL_WGL_CONTEXT_FLAGS_ARB, LOCAL_WGL_CONTEXT_ROBUST_ACCESS_BIT_ARB,
             LOCAL_WGL_CONTEXT_RESET_NOTIFICATION_STRATEGY_ARB, LOCAL_WGL_LOSE_CONTEXT_ON_RESET_ARB,
-            NULL
+            0
         };
 
-        context = sWGLLibrary.fCreateContextAttribs(dc,
-                                                    shareContext ? shareContext->Context() : nsnull,
+        context = sWGLLib[libToUse].fCreateContextAttribs(dc,
+                                                    shareContext ? shareContext->Context() : nullptr,
                                                     attribs);
         if (!context && shareContext) {
-            context = sWGLLibrary.fCreateContextAttribs(dc, nsnull, attribs);
+            context = sWGLLib[libToUse].fCreateContextAttribs(dc, nullptr, attribs);
             if (context) {
-                shareContext = nsnull;
+                shareContext = nullptr;
             }
         } else {
-            context = sWGLLibrary.fCreateContext(dc);
-            if (context && shareContext && !sWGLLibrary.fShareLists(shareContext->Context(), context)) {
-                shareContext = nsnull;
+            context = sWGLLib[libToUse].fCreateContext(dc);
+            if (context && shareContext && !sWGLLib[libToUse].fShareLists(shareContext->Context(), context)) {
+                shareContext = nullptr;
             }
         }
     } else {
-        context = sWGLLibrary.fCreateContext(dc);
+        context = sWGLLib[libToUse].fCreateContext(dc);
         if (context &&
             shareContext &&
-            !sWGLLibrary.fShareLists(shareContext->Context(), context))
+            !sWGLLib[libToUse].fShareLists(shareContext->Context(), context))
         {
-            shareContext = nsnull;
+            shareContext = nullptr;
         }
     }
 
     if (!context) {
-        return nsnull;
+        return nullptr;
     }
 
-    nsRefPtr<GLContextWGL> glContext = new GLContextWGL(ContextFormat(ContextFormat::BasicRGB24),
-                                                        shareContext, dc, context);
+    SurfaceCaps caps = SurfaceCaps::ForRGBA();
+    nsRefPtr<GLContextWGL> glContext = new GLContextWGL(caps,
+                                                        shareContext,
+                                                        false,
+                                                        dc,
+                                                        context,
+                                                        libToUse);
     if (!glContext->Init()) {
-        return nsnull;
+        return nullptr;
     }
 
-    glContext->SetIsDoubleBuffered(gUseDoubleBufferedWindows);
+    glContext->SetIsDoubleBuffered(sWGLLib[libToUse].UseDoubleBufferedWindows());
 
     return glContext.forget();
 }
 
 static already_AddRefed<GLContextWGL>
 CreatePBufferOffscreenContext(const gfxIntSize& aSize,
-                              const ContextFormat& aFormat)
+                              LibType aLibToUse)
 {
+    WGLLibrary& wgl = sWGLLib[aLibToUse];
+
 #define A1(_a,_x)  do { _a.AppendElement(_x); } while(0)
 #define A2(_a,_x,_y)  do { _a.AppendElement(_x); _a.AppendElement(_y); } while(0)
 
@@ -617,219 +529,219 @@ CreatePBufferOffscreenContext(const gfxIntSize& aSize,
 
     A2(attrs, LOCAL_WGL_ACCELERATION_ARB, LOCAL_WGL_FULL_ACCELERATION_ARB);
 
-    A2(attrs, LOCAL_WGL_COLOR_BITS_ARB, aFormat.colorBits());
-    A2(attrs, LOCAL_WGL_RED_BITS_ARB, aFormat.red);
-    A2(attrs, LOCAL_WGL_GREEN_BITS_ARB, aFormat.green);
-    A2(attrs, LOCAL_WGL_BLUE_BITS_ARB, aFormat.blue);
-    A2(attrs, LOCAL_WGL_ALPHA_BITS_ARB, aFormat.alpha);
-
-    A2(attrs, LOCAL_WGL_DEPTH_BITS_ARB, aFormat.depth);
-    A2(attrs, LOCAL_WGL_STENCIL_BITS_ARB, aFormat.stencil);
-
-    if (aFormat.alpha > 0) {
-        A2(attrs, LOCAL_WGL_BIND_TO_TEXTURE_RGBA_ARB, LOCAL_GL_TRUE);
-    } else {
-        A2(attrs, LOCAL_WGL_BIND_TO_TEXTURE_RGB_ARB, LOCAL_GL_TRUE);
-    }
-
     A2(attrs, LOCAL_WGL_DOUBLE_BUFFER_ARB, LOCAL_GL_FALSE);
     A2(attrs, LOCAL_WGL_STEREO_ARB, LOCAL_GL_FALSE);
 
     A1(attrs, 0);
 
     nsTArray<int> pbattrs;
-    A2(pbattrs, LOCAL_WGL_TEXTURE_TARGET_ARB, LOCAL_WGL_TEXTURE_2D_ARB);
-
-    if (aFormat.alpha > 0) {
-        A2(pbattrs, LOCAL_WGL_TEXTURE_FORMAT_ARB, LOCAL_WGL_TEXTURE_RGBA_ARB);
-    } else {
-        A2(pbattrs, LOCAL_WGL_TEXTURE_FORMAT_ARB, LOCAL_WGL_TEXTURE_RGB_ARB);
-    }
     A1(pbattrs, 0);
 
-    UINT numFormats = 256;
-    int formats[256];
+#undef A1
+#undef A2
 
-    if (!sWGLLibrary.fChoosePixelFormat(gSharedWindowDC,
-                                        attrs.Elements(), NULL,
-                                        numFormats, formats, &numFormats)
+    // We only need one!
+    UINT numFormats = 1;
+    int formats[1];
+    HDC windowDC = wgl.GetWindowDC();
+    if (!wgl.fChoosePixelFormat(windowDC,
+                                attrs.Elements(), NULL,
+                                numFormats, formats, &numFormats)
         || numFormats == 0)
     {
-        return nsnull;
+        return nullptr;
     }
 
-    // XXX add back the priority choosing code here
+    // We don't care; just pick the first one.
     int chosenFormat = formats[0];
+    if (!IsValidSizeForFormat(windowDC, chosenFormat, aSize, aLibToUse))
+        return nullptr;
 
-    if (!IsValidSizeForFormat(gSharedWindowDC, chosenFormat, aSize))
-        return nsnull;
-
-    HANDLE pbuffer = sWGLLibrary.fCreatePbuffer(gSharedWindowDC, chosenFormat,
-                                                aSize.width, aSize.height,
-                                                pbattrs.Elements());
+    HANDLE pbuffer = wgl.fCreatePbuffer(windowDC, chosenFormat,
+                                        aSize.width, aSize.height,
+                                        pbattrs.Elements());
     if (!pbuffer) {
-        return nsnull;
+        return nullptr;
     }
 
-    HDC pbdc = sWGLLibrary.fGetPbufferDC(pbuffer);
+    HDC pbdc = wgl.fGetPbufferDC(pbuffer);
     NS_ASSERTION(pbdc, "expected a dc");
 
     HGLRC context;
-    if (sWGLLibrary.HasRobustness()) {
+    if (wgl.HasRobustness()) {
         int attribs[] = {
             LOCAL_WGL_CONTEXT_FLAGS_ARB, LOCAL_WGL_CONTEXT_ROBUST_ACCESS_BIT_ARB,
             LOCAL_WGL_CONTEXT_RESET_NOTIFICATION_STRATEGY_ARB, LOCAL_WGL_LOSE_CONTEXT_ON_RESET_ARB,
-            NULL
+            0
         };
 
-        context = sWGLLibrary.fCreateContextAttribs(pbdc, nsnull, attribs);
+        context = wgl.fCreateContextAttribs(pbdc, nullptr, attribs);
     } else {
-        context = sWGLLibrary.fCreateContext(pbdc);
+        context = wgl.fCreateContext(pbdc);
     }
 
     if (!context) {
-        sWGLLibrary.fDestroyPbuffer(pbuffer);
-        return false;
+        wgl.fDestroyPbuffer(pbuffer);
+        return nullptr;
     }
 
-    nsRefPtr<GLContextWGL> glContext = new GLContextWGL(aFormat,
-                                                        nsnull,
+	SurfaceCaps dummyCaps = SurfaceCaps::Any();
+    nsRefPtr<GLContextWGL> glContext = new GLContextWGL(dummyCaps,
+                                                        nullptr, true,
                                                         pbuffer,
                                                         pbdc,
                                                         context,
-                                                        chosenFormat);
+                                                        chosenFormat,
+                                                        aLibToUse);
 
     return glContext.forget();
 }
 
 static already_AddRefed<GLContextWGL>
-CreateWindowOffscreenContext(const ContextFormat& aFormat)
+CreateWindowOffscreenContext(GLContext::ContextFlags aFlags)
 {
     // CreateWindowOffscreenContext must return a global-shared context
-    GLContextWGL *shareContext = GetGlobalContextWGL();
+    GLContextWGL *shareContext = GetGlobalContextWGL(aFlags);
     if (!shareContext) {
-        return nsnull;
+        return nullptr;
     }
     
+    LibType libToUse = WGLLibrary::SelectLibrary(aFlags);
     HDC dc;
-    HWND win = CreateDummyWindow(&dc);
+    HWND win = sWGLLib[libToUse].CreateDummyWindow(&dc);
     if (!win) {
-        return nsnull;
+        return nullptr;
     }
     
-    HGLRC context = sWGLLibrary.fCreateContext(dc);
-    if (sWGLLibrary.HasRobustness()) {
+    HGLRC context = sWGLLib[libToUse].fCreateContext(dc);
+    if (sWGLLib[libToUse].HasRobustness()) {
         int attribs[] = {
             LOCAL_WGL_CONTEXT_FLAGS_ARB, LOCAL_WGL_CONTEXT_ROBUST_ACCESS_BIT_ARB,
             LOCAL_WGL_CONTEXT_RESET_NOTIFICATION_STRATEGY_ARB, LOCAL_WGL_LOSE_CONTEXT_ON_RESET_ARB,
-            NULL
+            0
         };
 
-        context = sWGLLibrary.fCreateContextAttribs(dc, shareContext->Context(), attribs);
+        context = sWGLLib[libToUse].fCreateContextAttribs(dc, shareContext->Context(), attribs);
     } else {
-        context = sWGLLibrary.fCreateContext(dc);
+        context = sWGLLib[libToUse].fCreateContext(dc);
         if (context && shareContext &&
-            !sWGLLibrary.fShareLists(shareContext->Context(), context))
+            !sWGLLib[libToUse].fShareLists(shareContext->Context(), context))
         {
             NS_WARNING("wglShareLists failed!");
 
-            sWGLLibrary.fDeleteContext(context);
+            sWGLLib[libToUse].fDeleteContext(context);
             DestroyWindow(win);
-            return nsnull;
+            return nullptr;
         }
     }
 
     if (!context) {
-        return nsnull;
+        return nullptr;
     }
 
-    nsRefPtr<GLContextWGL> glContext = new GLContextWGL(aFormat, shareContext,
-                                                        dc, context, win, true);
+    SurfaceCaps caps = SurfaceCaps::ForRGBA();
+    nsRefPtr<GLContextWGL> glContext = new GLContextWGL(caps,
+                                                        shareContext, true,
+                                                        dc, context,
+                                                        libToUse, win);
 
     return glContext.forget();
 }
 
 already_AddRefed<GLContext>
-GLContextProviderWGL::CreateOffscreen(const gfxIntSize& aSize,
-                                      const ContextFormat& aFormat,
-                                      const ContextFlags)
+GLContextProviderWGL::CreateOffscreen(const gfxIntSize& size,
+                                      const SurfaceCaps& caps,
+                                      ContextFlags flags)
 {
-    if (!sWGLLibrary.EnsureInitialized()) {
-        return nsnull;
+    LibType libToUse = WGLLibrary::SelectLibrary(flags);
+    
+    if (!sWGLLib[libToUse].EnsureInitialized(libToUse == WGLLibrary::MESA_LLVMPIPE_LIB)) {
+        return nullptr;
     }
 
     nsRefPtr<GLContextWGL> glContext;
 
     // Always try to create a pbuffer context first, because we
     // want the context isolation.
-    NS_ENSURE_TRUE(Preferences::GetRootBranch(), nsnull);
-    const bool preferFBOs = Preferences::GetBool("wgl.prefer-fbo", false);
-    if (!preferFBOs &&
-        sWGLLibrary.fCreatePbuffer &&
-        sWGLLibrary.fChoosePixelFormat)
+    if (sWGLLib[libToUse].fCreatePbuffer &&
+        sWGLLib[libToUse].fChoosePixelFormat)
     {
-        glContext = CreatePBufferOffscreenContext(aSize, aFormat);
+        gfxIntSize dummySize = gfxIntSize(16, 16);
+        glContext = CreatePBufferOffscreenContext(dummySize, libToUse);
     }
 
     // If it failed, then create a window context and use a FBO.
     if (!glContext) {
-        glContext = CreateWindowOffscreenContext(aFormat);
+        glContext = CreateWindowOffscreenContext(flags);
     }
 
     if (!glContext ||
         !glContext->Init())
     {
-        return nsnull;
+        return nullptr;
     }
 
-    if (!glContext->ResizeOffscreenFBO(aSize, !glContext->mPBuffer))
-        return nsnull;
-
-    glContext->mOffscreenSize = aSize;
-    glContext->mOffscreenActualSize = aSize;
+    if (!glContext->InitOffscreen(size, caps))
+        return nullptr;
 
     return glContext.forget();
 }
 
-already_AddRefed<GLContext>
-GLContextProviderWGL::CreateForNativePixmapSurface(gfxASurface *aSurface)
+SharedTextureHandle
+GLContextProviderWGL::CreateSharedHandle(GLContext::SharedTextureShareType shareType,
+                                         void* buffer,
+                                         GLContext::SharedTextureBufferType bufferType)
 {
-    return nsnull;
+  return 0;
 }
 
-static nsRefPtr<GLContextWGL> gGlobalContext;
+already_AddRefed<gfxASurface>
+GLContextProviderWGL::GetSharedHandleAsSurface(GLContext::SharedTextureShareType shareType,
+                                               SharedTextureHandle sharedHandle)
+{
+  return nullptr;
+}
+
+static nsRefPtr<GLContextWGL> gGlobalContext[WGLLibrary::LIBS_MAX];
 
 GLContext *
-GLContextProviderWGL::GetGlobalContext()
+GLContextProviderWGL::GetGlobalContext(const ContextFlags flags)
 {
-    if (!sWGLLibrary.EnsureInitialized()) {
-        return nsnull;
+    LibType libToUse = WGLLibrary::SelectLibrary(flags);
+    
+    if (!sWGLLib[libToUse].EnsureInitialized(libToUse == WGLLibrary::MESA_LLVMPIPE_LIB)) {
+        return nullptr;
     }
 
-    static bool triedToCreateContext = false;
+    static bool triedToCreateContext[WGLLibrary::LIBS_MAX] = {false, false};
 
-    if (!triedToCreateContext && !gGlobalContext) {
-        triedToCreateContext = true;
+    if (!triedToCreateContext[libToUse] && !gGlobalContext[libToUse]) {
+        triedToCreateContext[libToUse] = true;
 
         // conveniently, we already have what we need...
-        gGlobalContext = new GLContextWGL(ContextFormat(ContextFormat::BasicRGB24), nsnull,
-                                          gSharedWindowDC, gSharedWindowGLContext);
-        if (!gGlobalContext->Init()) {
+        SurfaceCaps dummyCaps = SurfaceCaps::Any();
+        gGlobalContext[libToUse] = new GLContextWGL(dummyCaps,
+                                                    nullptr, true,
+                                                    sWGLLib[libToUse].GetWindowDC(),
+                                                    sWGLLib[libToUse].GetWindowGLContext(),
+                                                    libToUse);
+        if (!gGlobalContext[libToUse]->Init()) {
             NS_WARNING("Global context GLContext initialization failed?");
-            gGlobalContext = nsnull;
-            return false;
+            gGlobalContext[libToUse] = nullptr;
+            return nullptr;
         }
 
-        gGlobalContext->SetIsGlobalSharedContext(true);
+        gGlobalContext[libToUse]->SetIsGlobalSharedContext(true);
     }
 
-    return static_cast<GLContext*>(gGlobalContext);
+    return static_cast<GLContext*>(gGlobalContext[libToUse]);
 }
 
 void
 GLContextProviderWGL::Shutdown()
 {
-    gGlobalContext = nsnull;
+    for (int i = 0; i < WGLLibrary::LIBS_MAX; ++i)
+      gGlobalContext[i] = nullptr;
 }
 
 } /* namespace gl */
