@@ -27,22 +27,16 @@
 #ifndef HB_UNISCRIBE_H
 #define HB_UNISCRIBE_H
 
-#include "hb-common.h"
-#include "hb-shape.h"
+#include "hb.h"
 
-#define _WIN32_WINNT 0x0500
+#ifndef _WIN32_WINNT
+#define _WIN32_WINNT 0x0600
+#endif
 #include <windows.h>
 
 HB_BEGIN_DECLS
 
 
-hb_bool_t
-hb_uniscribe_shape (hb_font_t          *font,
-		    hb_buffer_t        *buffer,
-		    const hb_feature_t *features,
-		    unsigned int        num_features,
-		    const char * const *shaper_options);
-
 LOGFONTW *
 hb_uniscribe_font_get_logfontw (hb_font_t *font);
 
