@@ -1,43 +1,10 @@
 /* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Corporation code.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Bas Schouten <bschouten@mozilla.org>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/layers/PLayers.h"
-#include "mozilla/layers/ShadowLayers.h"
-#include "ShadowBufferD3D9.h"
+#include "ipc/AutoOpenSurface.h"
+#include "mozilla/layers/PLayerTransaction.h"
 #include "gfxSharedImageSurface.h"
 
 #include "ImageLayerD3D9.h"
@@ -45,86 +12,121 @@
 #include "gfxPlatform.h"
 #include "gfxImageSurface.h"
 #include "yuv_convert.h"
-#include "nsIServiceManager.h" 
-#include "nsIConsoleService.h" 
+#include "nsIServiceManager.h"
+#include "nsIConsoleService.h"
 #include "Nv3DVUtils.h"
+#include "D3D9SurfaceImage.h"
 
 namespace mozilla {
 namespace layers {
 
+static inline _D3DFORMAT
+D3dFormatForGfxFormat(gfxImageFormat aFormat)
+{
+  if (aFormat == gfxASurface::ImageFormatA8) {
+    return D3DFMT_A8;
+  }
+
+  return D3DFMT_A8R8G8B8;
+}
+
 static already_AddRefed<IDirect3DTexture9>
 DataToTexture(IDirect3DDevice9 *aDevice,
               unsigned char *aData,
               int aStride,
-              const gfxIntSize &aSize)
+              const gfxIntSize &aSize,
+              _D3DFORMAT aFormat)
 {
   nsRefPtr<IDirect3DTexture9> texture;
   nsRefPtr<IDirect3DDevice9Ex> deviceEx;
   aDevice->QueryInterface(IID_IDirect3DDevice9Ex,
                           (void**)getter_AddRefs(deviceEx));
 
+  nsRefPtr<IDirect3DSurface9> surface;
+  D3DLOCKED_RECT lockedRect;
   if (deviceEx) {
     // D3D9Ex doesn't support managed textures. We could use dynamic textures
     // here but since Images are immutable that probably isn't such a great
     // idea.
     if (FAILED(aDevice->
                CreateTexture(aSize.width, aSize.height,
-                             1, 0, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT,
+                             1, 0, aFormat, D3DPOOL_DEFAULT,
                              getter_AddRefs(texture), NULL)))
     {
       return NULL;
     }
 
-    nsRefPtr<IDirect3DSurface9> surface;
+    nsRefPtr<IDirect3DTexture9> tmpTexture;
     if (FAILED(aDevice->
-               CreateOffscreenPlainSurface(aSize.width,
-                                           aSize.height,
-                                           D3DFMT_A8R8G8B8,
-                                           D3DPOOL_SYSTEMMEM,
-                                           getter_AddRefs(surface),
-                                           NULL)))
+               CreateTexture(aSize.width, aSize.height,
+                             1, 0, aFormat, D3DPOOL_SYSTEMMEM,
+                             getter_AddRefs(tmpTexture), NULL)))
     {
       return NULL;
     }
 
-    D3DLOCKED_RECT lockedRect;
+    tmpTexture->GetSurfaceLevel(0, getter_AddRefs(surface));
     surface->LockRect(&lockedRect, NULL, 0);
-    for (int y = 0; y < aSize.height; y++) {
-      memcpy((char*)lockedRect.pBits + lockedRect.Pitch * y,
-             aData + aStride * y,
-             aSize.width * 4);
+    NS_ASSERTION(lockedRect.pBits, "Could not lock surface");
+  } else {
+    if (FAILED(aDevice->
+               CreateTexture(aSize.width, aSize.height,
+                             1, 0, aFormat, D3DPOOL_MANAGED,
+                             getter_AddRefs(texture), NULL)))
+    {
+      return NULL;
     }
+
+    /* lock the entire texture */
+    texture->LockRect(0, &lockedRect, NULL, 0);
+  }
+
+  uint32_t width = aSize.width;
+  if (aFormat == D3DFMT_A8R8G8B8) {
+    width *= 4;
+  }
+  for (int y = 0; y < aSize.height; y++) {
+    memcpy((char*)lockedRect.pBits + lockedRect.Pitch * y,
+            aData + aStride * y,
+            width);
+  }
+
+  if (deviceEx) {
     surface->UnlockRect();
     nsRefPtr<IDirect3DSurface9> dstSurface;
     texture->GetSurfaceLevel(0, getter_AddRefs(dstSurface));
     aDevice->UpdateSurface(surface, NULL, dstSurface, NULL);
   } else {
-    if (FAILED(aDevice->
-               CreateTexture(aSize.width, aSize.height,
-                             1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED,
-                             getter_AddRefs(texture), NULL)))
-    {
-      return NULL;
-    }
-
-    D3DLOCKED_RECT lockrect;
-    /* lock the entire texture */
-    texture->LockRect(0, &lockrect, NULL, 0);
-
-    // copy over data. If we don't need to do any swaping we can
-    // use memcpy
-    for (int y = 0; y < aSize.height; y++) {
-      memcpy((char*)lockrect.pBits + lockrect.Pitch * y,
-             aData + aStride * y,
-             aSize.width * 4);
-    }
-
     texture->UnlockRect(0);
   }
 
   return texture.forget();
 }
 
+static already_AddRefed<IDirect3DTexture9>
+OpenSharedTexture(const D3DSURFACE_DESC& aDesc,
+                  HANDLE aShareHandle,
+                  IDirect3DDevice9 *aDevice)
+{
+  MOZ_ASSERT(aDesc.Format == D3DFMT_X8R8G8B8);
+
+  // Open the frame from DXVA's device in our device using the resource
+  // sharing handle.
+  nsRefPtr<IDirect3DTexture9> sharedTexture;
+  HRESULT hr = aDevice->CreateTexture(aDesc.Width,
+                                      aDesc.Height,
+                                      1,
+                                      D3DUSAGE_RENDERTARGET,
+                                      D3DFMT_X8R8G8B8,
+                                      D3DPOOL_DEFAULT,
+                                      getter_AddRefs(sharedTexture),
+                                      &aShareHandle);
+  if (FAILED(hr)) {
+    NS_WARNING("Failed to open shared texture on our device");
+  }
+  return sharedTexture.forget();
+}
+
 static already_AddRefed<IDirect3DTexture9>
 SurfaceToTexture(IDirect3DDevice9 *aDevice,
                  gfxASurface *aSurface,
@@ -136,14 +138,15 @@ SurfaceToTexture(IDirect3DDevice9 *aDevice,
   if (!imageSurface) {
     imageSurface = new gfxImageSurface(aSize,
                                        gfxASurface::ImageFormatARGB32);
-    
+
     nsRefPtr<gfxContext> context = new gfxContext(imageSurface);
     context->SetSource(aSurface);
     context->SetOperator(gfxContext::OPERATOR_SOURCE);
     context->Paint();
   }
 
-  return DataToTexture(aDevice, imageSurface->Data(), imageSurface->Stride(), aSize);
+  return DataToTexture(aDevice, imageSurface->Data(), imageSurface->Stride(),
+                       aSize, D3dFormatForGfxFormat(imageSurface->Format()));
 }
 
 static void AllocateTexturesYCbCr(PlanarYCbCrImage *aImage,
@@ -153,13 +156,13 @@ static void AllocateTexturesYCbCr(PlanarYCbCrImage *aImage,
   nsAutoPtr<PlanarYCbCrD3D9BackendData> backendData(
     new PlanarYCbCrD3D9BackendData);
 
-  PlanarYCbCrImage::Data &data = aImage->mData;
+  const PlanarYCbCrImage::Data *data = aImage->GetData();
 
   D3DLOCKED_RECT lockrectY;
   D3DLOCKED_RECT lockrectCb;
   D3DLOCKED_RECT lockrectCr;
-  PRUint8* src;
-  PRUint8* dest;
+  uint8_t* src;
+  uint8_t* dest;
 
   nsRefPtr<IDirect3DSurface9> tmpSurfaceY;
   nsRefPtr<IDirect3DSurface9> tmpSurfaceCb;
@@ -180,31 +183,31 @@ static void AllocateTexturesYCbCr(PlanarYCbCrImage *aImage,
     // better idea.
 
     HRESULT hr;
-    hr = aDevice->CreateTexture(data.mYSize.width, data.mYSize.height,
+    hr = aDevice->CreateTexture(data->mYSize.width, data->mYSize.height,
                                 1, 0, D3DFMT_L8, D3DPOOL_DEFAULT,
                                 getter_AddRefs(backendData->mYTexture), NULL);
     if (!FAILED(hr)) {
-      hr = aDevice->CreateTexture(data.mCbCrSize.width, data.mCbCrSize.height,
+      hr = aDevice->CreateTexture(data->mCbCrSize.width, data->mCbCrSize.height,
                                   1, 0, D3DFMT_L8, D3DPOOL_DEFAULT,
                                   getter_AddRefs(backendData->mCbTexture), NULL);
     }
     if (!FAILED(hr)) {
-      hr = aDevice->CreateTexture(data.mCbCrSize.width, data.mCbCrSize.height,
+      hr = aDevice->CreateTexture(data->mCbCrSize.width, data->mCbCrSize.height,
                                   1, 0, D3DFMT_L8, D3DPOOL_DEFAULT,
                                   getter_AddRefs(backendData->mCrTexture), NULL);
     }
     if (!FAILED(hr)) {
-      hr = aDevice->CreateTexture(data.mYSize.width, data.mYSize.height,
+      hr = aDevice->CreateTexture(data->mYSize.width, data->mYSize.height,
                                   1, 0, D3DFMT_L8, D3DPOOL_SYSTEMMEM,
                                   getter_AddRefs(tmpYTexture), NULL);
     }
     if (!FAILED(hr)) {
-      hr = aDevice->CreateTexture(data.mCbCrSize.width, data.mCbCrSize.height,
+      hr = aDevice->CreateTexture(data->mCbCrSize.width, data->mCbCrSize.height,
                                   1, 0, D3DFMT_L8, D3DPOOL_SYSTEMMEM,
                                   getter_AddRefs(tmpCbTexture), NULL);
     }
     if (!FAILED(hr)) {
-      hr = aDevice->CreateTexture(data.mCbCrSize.width, data.mCbCrSize.height,
+      hr = aDevice->CreateTexture(data->mCbCrSize.width, data->mCbCrSize.height,
                                   1, 0, D3DFMT_L8, D3DPOOL_SYSTEMMEM,
                                   getter_AddRefs(tmpCrTexture), NULL);
     }
@@ -223,16 +226,16 @@ static void AllocateTexturesYCbCr(PlanarYCbCrImage *aImage,
     tmpSurfaceCr->LockRect(&lockrectCr, NULL, 0);
   } else {
     HRESULT hr;
-    hr = aDevice->CreateTexture(data.mYSize.width, data.mYSize.height,
+    hr = aDevice->CreateTexture(data->mYSize.width, data->mYSize.height,
                                 1, 0, D3DFMT_L8, D3DPOOL_MANAGED,
                                 getter_AddRefs(backendData->mYTexture), NULL);
     if (!FAILED(hr)) {
-      aDevice->CreateTexture(data.mCbCrSize.width, data.mCbCrSize.height,
+      aDevice->CreateTexture(data->mCbCrSize.width, data->mCbCrSize.height,
                              1, 0, D3DFMT_L8, D3DPOOL_MANAGED,
                              getter_AddRefs(backendData->mCbTexture), NULL);
     }
     if (!FAILED(hr)) {
-      aDevice->CreateTexture(data.mCbCrSize.width, data.mCbCrSize.height,
+      aDevice->CreateTexture(data->mCbCrSize.width, data->mCbCrSize.height,
                              1, 0, D3DFMT_L8, D3DPOOL_MANAGED,
                              getter_AddRefs(backendData->mCrTexture), NULL);
     }
@@ -249,37 +252,37 @@ static void AllocateTexturesYCbCr(PlanarYCbCrImage *aImage,
     backendData->mCrTexture->LockRect(0, &lockrectCr, NULL, 0);
   }
 
-  src  = data.mYChannel;
+  src  = data->mYChannel;
   //FIX cast
-  dest = (PRUint8*)lockrectY.pBits;
+  dest = (uint8_t*)lockrectY.pBits;
 
   // copy over data
-  for (int h=0; h<data.mYSize.height; h++) {
-    memcpy(dest, src, data.mYSize.width);
+  for (int h=0; h<data->mYSize.height; h++) {
+    memcpy(dest, src, data->mYSize.width);
     dest += lockrectY.Pitch;
-    src += data.mYStride;
+    src += data->mYStride;
   }
 
-  src  = data.mCbChannel;
+  src  = data->mCbChannel;
   //FIX cast
-  dest = (PRUint8*)lockrectCb.pBits;
+  dest = (uint8_t*)lockrectCb.pBits;
 
   // copy over data
-  for (int h=0; h<data.mCbCrSize.height; h++) {
-    memcpy(dest, src, data.mCbCrSize.width);
+  for (int h=0; h<data->mCbCrSize.height; h++) {
+    memcpy(dest, src, data->mCbCrSize.width);
     dest += lockrectCb.Pitch;
-    src += data.mCbCrStride;
+    src += data->mCbCrStride;
   }
 
-  src  = data.mCrChannel;
+  src  = data->mCrChannel;
   //FIX cast
-  dest = (PRUint8*)lockrectCr.pBits;
+  dest = (uint8_t*)lockrectCr.pBits;
 
   // copy over data
-  for (int h=0; h<data.mCbCrSize.height; h++) {
-    memcpy(dest, src, data.mCbCrSize.width);
+  for (int h=0; h<data->mCbCrSize.height; h++) {
+    memcpy(dest, src, data->mCbCrSize.width);
     dest += lockrectCr.Pitch;
-    src += data.mCbCrStride;
+    src += data->mCbCrStride;
   }
 
   if (isD3D9Ex) {
@@ -299,7 +302,7 @@ static void AllocateTexturesYCbCr(PlanarYCbCrImage *aImage,
     backendData->mCrTexture->UnlockRect(0);
   }
 
-  aImage->SetBackendData(LayerManager::LAYERS_D3D9, backendData.forget());
+  aImage->SetBackendData(mozilla::layers::LAYERS_D3D9, backendData.forget());
 }
 
 Layer*
@@ -308,11 +311,86 @@ ImageLayerD3D9::GetLayer()
   return this;
 }
 
+/*
+  * Returns a texture which backs aImage
+  * Will only work if aImage is a cairo or remote image.
+  * Returns nullptr if unsuccessful.
+  * If successful, aHasAlpha will be set to true if the texture has an
+  * alpha component, false otherwise.
+  */
+IDirect3DTexture9*
+ImageLayerD3D9::GetTexture(Image *aImage, bool& aHasAlpha)
+{
+  NS_ASSERTION(aImage, "Null image.");
+
+  if (aImage->GetFormat() == REMOTE_IMAGE_BITMAP) {
+    RemoteBitmapImage *remoteImage =
+      static_cast<RemoteBitmapImage*>(aImage);
+
+    if (!aImage->GetBackendData(mozilla::layers::LAYERS_D3D9)) {
+      nsAutoPtr<TextureD3D9BackendData> dat(new TextureD3D9BackendData());
+      dat->mTexture = DataToTexture(device(), remoteImage->mData, remoteImage->mStride, remoteImage->mSize, D3DFMT_A8R8G8B8);
+      if (dat->mTexture) {
+        aImage->SetBackendData(mozilla::layers::LAYERS_D3D9, dat.forget());
+      }
+    }
+
+    aHasAlpha = remoteImage->mFormat == RemoteImageData::BGRA32;
+  } else if (aImage->GetFormat() == CAIRO_SURFACE) {
+    CairoImage *cairoImage =
+      static_cast<CairoImage*>(aImage);
+
+    if (!cairoImage->mSurface) {
+      return nullptr;
+    }
+
+    if (!aImage->GetBackendData(mozilla::layers::LAYERS_D3D9)) {
+      nsAutoPtr<TextureD3D9BackendData> dat(new TextureD3D9BackendData());
+      dat->mTexture = SurfaceToTexture(device(), cairoImage->mSurface, cairoImage->mSize);
+      if (dat->mTexture) {
+        aImage->SetBackendData(mozilla::layers::LAYERS_D3D9, dat.forget());
+      }
+    }
+
+    aHasAlpha = cairoImage->mSurface->GetContentType() == gfxASurface::CONTENT_COLOR_ALPHA;
+  } else if (aImage->GetFormat() == D3D9_RGB32_TEXTURE) {
+    if (!aImage->GetBackendData(mozilla::layers::LAYERS_D3D9)) {
+      // The texture in which the frame is stored belongs to DXVA's D3D9 device.
+      // We need to open it on our device before we can use it.
+      nsAutoPtr<TextureD3D9BackendData> backendData(new TextureD3D9BackendData());
+      D3D9SurfaceImage* image = static_cast<D3D9SurfaceImage*>(aImage);
+      backendData->mTexture = OpenSharedTexture(image->GetDesc(), image->GetShareHandle(), device());
+      if (backendData->mTexture) {
+        aImage->SetBackendData(mozilla::layers::LAYERS_D3D9, backendData.forget());
+      }
+    }
+    aHasAlpha = false;
+  } else {
+    NS_WARNING("Inappropriate image type.");
+    return nullptr;
+  }
+
+  TextureD3D9BackendData *data =
+    static_cast<TextureD3D9BackendData*>(aImage->GetBackendData(mozilla::layers::LAYERS_D3D9));
+
+  if (!data) {
+    return nullptr;
+  }
+
+  nsRefPtr<IDirect3DDevice9> dev;
+  data->mTexture->GetDevice(getter_AddRefs(dev));
+  if (dev != device()) {
+    return nullptr;
+  }
+
+  return data->mTexture;
+}
+
 void
 ImageLayerD3D9::RenderLayer()
 {
   ImageContainer *container = GetContainer();
-  if (!container) {
+  if (!container || mD3DManager->CompositingDisabled()) {
     return;
   }
 
@@ -325,56 +403,19 @@ ImageLayerD3D9::RenderLayer()
 
   SetShaderTransformAndOpacity();
 
-  gfxIntSize size = mScaleMode == SCALE_NONE ? image->GetSize() : mScaleToSize;
+  gfxIntSize size = image->GetSize();
 
-  if (image->GetFormat() == Image::CAIRO_SURFACE || image->GetFormat() == Image::REMOTE_IMAGE_BITMAP)
+  if (image->GetFormat() == CAIRO_SURFACE ||
+      image->GetFormat() == REMOTE_IMAGE_BITMAP ||
+      image->GetFormat() == D3D9_RGB32_TEXTURE)
   {
+    NS_ASSERTION(image->GetFormat() != CAIRO_SURFACE ||
+                 !static_cast<CairoImage*>(image)->mSurface ||
+                 static_cast<CairoImage*>(image)->mSurface->GetContentType() != gfxASurface::CONTENT_ALPHA,
+                 "Image layer has alpha image");
+
     bool hasAlpha = false;
-
-    if (image->GetFormat() == Image::REMOTE_IMAGE_BITMAP) {
-      RemoteBitmapImage *remoteImage =
-        static_cast<RemoteBitmapImage*>(image);
-      
-      if (!image->GetBackendData(LayerManager::LAYERS_D3D9)) {
-        nsAutoPtr<TextureD3D9BackendData> dat = new TextureD3D9BackendData();
-        dat->mTexture = DataToTexture(device(), remoteImage->mData, remoteImage->mStride, remoteImage->mSize);
-        if (dat->mTexture) {
-          image->SetBackendData(LayerManager::LAYERS_D3D9, dat.forget());
-        }
-      }
-
-      hasAlpha = remoteImage->mFormat == RemoteImageData::BGRA32;
-    } else {
-      CairoImage *cairoImage =
-        static_cast<CairoImage*>(image);
-
-      if (!cairoImage->mSurface) {
-        return;
-      }
-
-      if (!image->GetBackendData(LayerManager::LAYERS_D3D9)) {
-        nsAutoPtr<TextureD3D9BackendData> dat = new TextureD3D9BackendData();
-        dat->mTexture = SurfaceToTexture(device(), cairoImage->mSurface, cairoImage->mSize);
-        if (dat->mTexture) {
-          image->SetBackendData(LayerManager::LAYERS_D3D9, dat.forget());
-        }
-      }
-
-      hasAlpha = cairoImage->mSurface->GetContentType() == gfxASurface::CONTENT_COLOR_ALPHA;
-    }
-
-    TextureD3D9BackendData *data =
-      static_cast<TextureD3D9BackendData*>(image->GetBackendData(LayerManager::LAYERS_D3D9));
-
-    if (!data) {
-      return;
-    }
-
-    nsRefPtr<IDirect3DDevice9> dev;
-    data->mTexture->GetDevice(getter_AddRefs(dev));
-    if (dev != device()) {
-      return;
-    }
+    nsRefPtr<IDirect3DTexture9> texture = GetTexture(image, hasAlpha);
 
     device()->SetVertexShaderConstantF(CBvLayerQuad,
                                        ShaderConstantRect(0,
@@ -384,19 +425,18 @@ ImageLayerD3D9::RenderLayer()
                                        1);
 
     if (hasAlpha) {
-      mD3DManager->SetShaderMode(DeviceManagerD3D9::RGBALAYER);
+      mD3DManager->SetShaderMode(DeviceManagerD3D9::RGBALAYER, GetMaskLayer());
     } else {
-      mD3DManager->SetShaderMode(DeviceManagerD3D9::RGBLAYER);
+      mD3DManager->SetShaderMode(DeviceManagerD3D9::RGBLAYER, GetMaskLayer());
     }
 
     if (mFilter == gfxPattern::FILTER_NEAREST) {
       device()->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_POINT);
       device()->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_POINT);
     }
-    device()->SetTexture(0, data->mTexture);
+    device()->SetTexture(0, texture);
 
-    image = nsnull;
-    data = nsnull;
+    image = nullptr;
     autoLock.Unlock();
 
     device()->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
@@ -408,16 +448,16 @@ ImageLayerD3D9::RenderLayer()
     PlanarYCbCrImage *yuvImage =
       static_cast<PlanarYCbCrImage*>(image);
 
-    if (!yuvImage->mBufferSize) {
+    if (!yuvImage->IsValid()) {
       return;
     }
 
-    if (!yuvImage->GetBackendData(LayerManager::LAYERS_D3D9)) {
+    if (!yuvImage->GetBackendData(mozilla::layers::LAYERS_D3D9)) {
       AllocateTexturesYCbCr(yuvImage, device(), mD3DManager);
     }
 
     PlanarYCbCrD3D9BackendData *data =
-      static_cast<PlanarYCbCrD3D9BackendData*>(yuvImage->GetBackendData(LayerManager::LAYERS_D3D9));
+      static_cast<PlanarYCbCrD3D9BackendData*>(yuvImage->GetBackendData(mozilla::layers::LAYERS_D3D9));
 
     if (!data) {
       return;
@@ -438,21 +478,21 @@ ImageLayerD3D9::RenderLayer()
 
     device()->SetVertexShaderConstantF(CBvTextureCoords,
       ShaderConstantRect(
-        (float)yuvImage->mData.mPicX / yuvImage->mData.mYSize.width,
-        (float)yuvImage->mData.mPicY / yuvImage->mData.mYSize.height,
-        (float)yuvImage->mData.mPicSize.width / yuvImage->mData.mYSize.width,
-        (float)yuvImage->mData.mPicSize.height / yuvImage->mData.mYSize.height
+        (float)yuvImage->GetData()->mPicX / yuvImage->GetData()->mYSize.width,
+        (float)yuvImage->GetData()->mPicY / yuvImage->GetData()->mYSize.height,
+        (float)yuvImage->GetData()->mPicSize.width / yuvImage->GetData()->mYSize.width,
+        (float)yuvImage->GetData()->mPicSize.height / yuvImage->GetData()->mYSize.height
       ),
       1);
 
-    mD3DManager->SetShaderMode(DeviceManagerD3D9::YCBCRLAYER);
+    mD3DManager->SetShaderMode(DeviceManagerD3D9::YCBCRLAYER, GetMaskLayer());
 
     /*
      * Send 3d control data and metadata
      */
     if (mD3DManager->GetNv3DVUtils()) {
       Nv_Stereo_Mode mode;
-      switch (yuvImage->mData.mStereoMode) {
+      switch (yuvImage->GetData()->mStereoMode) {
       case STEREO_MODE_LEFT_RIGHT:
         mode = NV_STEREO_MODE_LEFT_RIGHT;
         break;
@@ -473,13 +513,13 @@ ImageLayerD3D9::RenderLayer()
       // Send control data even in mono case so driver knows to leave stereo mode.
       mD3DManager->GetNv3DVUtils()->SendNv3DVControl(mode, true, FIREFOX_3DV_APP_HANDLE);
 
-      if (yuvImage->mData.mStereoMode != STEREO_MODE_MONO) {
+      if (yuvImage->GetData()->mStereoMode != STEREO_MODE_MONO) {
         mD3DManager->GetNv3DVUtils()->SendNv3DVControl(mode, true, FIREFOX_3DV_APP_HANDLE);
 
         nsRefPtr<IDirect3DSurface9> renderTarget;
         device()->GetRenderTarget(0, getter_AddRefs(renderTarget));
-        mD3DManager->GetNv3DVUtils()->SendNv3DVMetaData((unsigned int)yuvImage->mSize.width,
-                                                        (unsigned int)yuvImage->mSize.height, (HANDLE)(data->mYTexture), (HANDLE)(renderTarget));
+        mD3DManager->GetNv3DVUtils()->SendNv3DVMetaData((unsigned int)yuvImage->GetSize().width,
+                                                        (unsigned int)yuvImage->GetSize().height, (HANDLE)(data->mYTexture), (HANDLE)(renderTarget));
       }
     }
 
@@ -490,8 +530,8 @@ ImageLayerD3D9::RenderLayer()
     device()->SetTexture(1, data->mCbTexture);
     device()->SetTexture(2, data->mCrTexture);
 
-    image = nsnull;
-    data = nsnull;
+    image = nullptr;
+    data = nullptr;
     autoLock.Unlock();
 
     device()->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
@@ -503,137 +543,31 @@ ImageLayerD3D9::RenderLayer()
   GetContainer()->NotifyPaintedImage(image);
 }
 
-ShadowImageLayerD3D9::ShadowImageLayerD3D9(LayerManagerD3D9* aManager)
-  : ShadowImageLayer(aManager, nsnull)
-  , LayerD3D9(aManager)
+already_AddRefed<IDirect3DTexture9>
+ImageLayerD3D9::GetAsTexture(gfxIntSize* aSize)
 {
-  mImplData = static_cast<LayerD3D9*>(this);
-}  
-
-ShadowImageLayerD3D9::~ShadowImageLayerD3D9()
-{}
-
-void
-ShadowImageLayerD3D9::Swap(const SharedImage& aNewFront,
-                           SharedImage* aNewBack)
-{
-  if (aNewFront.type() == SharedImage::TSurfaceDescriptor) {
-    if (!mBuffer) {
-      mBuffer = new ShadowBufferD3D9(this);
-    }
-    nsRefPtr<gfxASurface> surf =
-      ShadowLayerForwarder::OpenDescriptor(aNewFront.get_SurfaceDescriptor());
-
-    mBuffer->Upload(surf, GetVisibleRegion().GetBounds());
-  } else {
-    const YUVImage& yuv = aNewFront.get_YUVImage();
-
-    nsRefPtr<gfxSharedImageSurface> surfY =
-      gfxSharedImageSurface::Open(yuv.Ydata());
-    nsRefPtr<gfxSharedImageSurface> surfU =
-      gfxSharedImageSurface::Open(yuv.Udata());
-    nsRefPtr<gfxSharedImageSurface> surfV =
-      gfxSharedImageSurface::Open(yuv.Vdata());
-
-    PlanarYCbCrImage::Data data;
-    data.mYChannel = surfY->Data();
-    data.mYStride = surfY->Stride();
-    data.mYSize = surfY->GetSize();
-    data.mCbChannel = surfU->Data();
-    data.mCrChannel = surfV->Data();
-    data.mCbCrStride = surfU->Stride();
-    data.mCbCrSize = surfU->GetSize();
-    data.mPicSize = surfY->GetSize();
-    data.mPicX = 0;
-    data.mPicY = 0;
-
-    if (!mYCbCrImage) {
-      mYCbCrImage = new PlanarYCbCrImage(new BufferRecycleBin());
-    }
-
-    mYCbCrImage->SetData(data);
-
-  }
-  
-  *aNewBack = aNewFront;
-}
-
-void
-ShadowImageLayerD3D9::Disconnect()
-{
-  Destroy();
-}
-
-void
-ShadowImageLayerD3D9::Destroy()
-{
-  mBuffer = nsnull;
-  mYCbCrImage = nsnull;
-}
-
-Layer*
-ShadowImageLayerD3D9::GetLayer()
-{
-  return this;
-}
-
-void
-ShadowImageLayerD3D9::RenderLayer()
-{
-  if (mBuffer) {
-    mBuffer->RenderTo(mD3DManager, GetEffectiveVisibleRegion());
-  } else if (mYCbCrImage) {
-    if (!mYCbCrImage->mBufferSize) {
-      return;
-    }
-
-    if (!mYCbCrImage->GetBackendData(LayerManager::LAYERS_D3D9)) {
-      AllocateTexturesYCbCr(mYCbCrImage, device(), mD3DManager);
-    }
-
-    PlanarYCbCrD3D9BackendData *data =
-      static_cast<PlanarYCbCrD3D9BackendData*>(mYCbCrImage->GetBackendData(LayerManager::LAYERS_D3D9));
-
-    if (!data) {
-      return;
-    }
-
-    if (!mYCbCrImage->mBufferSize) {
-      return;
-    }
-    
-    SetShaderTransformAndOpacity();
-
-    device()->SetVertexShaderConstantF(CBvLayerQuad,
-                                       ShaderConstantRect(0,
-                                                          0,
-                                                          mYCbCrImage->mSize.width,
-                                                          mYCbCrImage->mSize.height),
-                                       1);
-
-    mD3DManager->SetShaderMode(DeviceManagerD3D9::YCBCRLAYER);
-
-    /*
-     * Send 3d control data and metadata
-     */
-    if (mD3DManager->GetNv3DVUtils()) {
-      // TODO Add 3D support
-    }
-
-    // Linear scaling is default here, adhering to mFilter is difficult since
-    // presumably even with point filtering we'll still want chroma upsampling
-    // to be linear. In the current approach we can't.
-    device()->SetTexture(0, data->mYTexture);
-    device()->SetTexture(1, data->mCbTexture);
-    device()->SetTexture(2, data->mCrTexture);
-
-    device()->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
-  } else {
-    NS_ERROR("Unexpected image format.");
+  if (!GetContainer()) {
+    return nullptr;
   }
 
-}
+  AutoLockImage autoLock(GetContainer());
 
+  Image *image = autoLock.GetImage();
+
+  if (!image) {
+    return nullptr;
+  }
+
+  if (image->GetFormat() != CAIRO_SURFACE &&
+      image->GetFormat() != REMOTE_IMAGE_BITMAP) {
+    return nullptr;
+  }
+
+  bool dontCare;
+  *aSize = image->GetSize();
+  nsRefPtr<IDirect3DTexture9> result = GetTexture(image, dontCare);
+  return result.forget();
+}
 
 } /* layers */
 } /* mozilla */
