@@ -35,6 +35,83 @@
 #define X86
 #endif /* _M_IX86 || __i386__ || __i386 || _M_AMD64 || __x86_64__ || __x86_64 */
 
+/**
+ * AltiVec detection for PowerPC CPUs
+ * In case we have a method of detecting do the runtime detection.
+ * Otherwise statically choose the AltiVec path in case the compiler
+ * was told to build with AltiVec support.
+ */
+#if (defined(__POWERPC__) || defined(__powerpc__))
+#if defined(__linux__)
+#include <unistd.h>
+#include <fcntl.h>
+#include <stdio.h>
+#include <elf.h>
+#include <linux/auxvec.h>
+#include <asm/cputable.h>
+#include <link.h>
+
+static inline qcms_bool have_altivec() {
+	static int available = -1;
+	int new_avail = 0;
+        ElfW(auxv_t) auxv;
+	ssize_t count;
+	int fd, i;
+
+	if (available != -1)
+		return (available != 0 ? true : false);
+
+	fd = open("/proc/self/auxv", O_RDONLY);
+	if (fd < 0)
+		goto out;
+	do {
+		count = read(fd, &auxv, sizeof(auxv));
+		if (count < 0)
+			goto out_close;
+
+		if (auxv.a_type == AT_HWCAP) {
+			new_avail = !!(auxv.a_un.a_val & PPC_FEATURE_HAS_ALTIVEC);
+			goto out_close;
+		}
+	} while (auxv.a_type != AT_NULL);
+
+out_close:
+	close(fd);
+out:
+	available = new_avail;
+	return (available != 0 ? true : false);
+}
+#elif defined(__APPLE__) && defined(__MACH__)
+#include <sys/sysctl.h>
+
+/**
+ * rip-off from ffmpeg AltiVec detection code.
+ * this code also appears on Apple's AltiVec pages.
+ */
+static inline qcms_bool have_altivec() {
+	int sels[2] = {CTL_HW, HW_VECTORUNIT};
+	static int available = -1;
+	size_t len = sizeof(available);
+	int err;
+
+	if (available != -1)
+		return (available != 0 ? true : false);
+
+	err = sysctl(sels, 2, &available, &len, NULL, 0);
+
+	if (err == 0)
+		if (available != 0)
+			return true;
+
+	return false;
+}
+#elif defined(__ALTIVEC__) || defined(__APPLE_ALTIVEC__)
+#define have_altivec() true
+#else
+#define have_altivec() false
+#endif
+#endif // (defined(__POWERPC__) || defined(__powerpc__))
+
 // Build a White point, primary chromas transfer matrix from RGB to CIE XYZ
 // This is just an approximation, I am not handling all the non-linear
 // aspects of the RGB to XYZ process, and assumming that the gamma correction
@@ -251,9 +328,10 @@ static void qcms_transform_data_rgb_out_pow(qcms_transform *transform, unsigned
 		float out_device_g = pow(out_linear_g, transform->out_gamma_g);
 		float out_device_b = pow(out_linear_b, transform->out_gamma_b);
 
-		*dest++ = clamp_u8(255*out_device_r);
-		*dest++ = clamp_u8(255*out_device_g);
-		*dest++ = clamp_u8(255*out_device_b);
+		dest[OUTPUT_R_INDEX] = clamp_u8(255*out_device_r);
+		dest[OUTPUT_G_INDEX] = clamp_u8(255*out_device_g);
+		dest[OUTPUT_B_INDEX] = clamp_u8(255*out_device_b);
+		dest += RGB_OUTPUT_COMPONENTS;
 	}
 }
 #endif
@@ -271,9 +349,10 @@ static void qcms_transform_data_gray_out_lut(qcms_transform *transform, unsigned
 		out_device_g = lut_interp_linear(linear, transform->output_gamma_lut_g, transform->output_gamma_lut_g_length);
 		out_device_b = lut_interp_linear(linear, transform->output_gamma_lut_b, transform->output_gamma_lut_b_length);
 
-		*dest++ = clamp_u8(out_device_r*255);
-		*dest++ = clamp_u8(out_device_g*255);
-		*dest++ = clamp_u8(out_device_b*255);
+		dest[OUTPUT_R_INDEX] = clamp_u8(out_device_r*255);
+		dest[OUTPUT_G_INDEX] = clamp_u8(out_device_g*255);
+		dest[OUTPUT_B_INDEX] = clamp_u8(out_device_b*255);
+		dest += RGB_OUTPUT_COMPONENTS;
 	}
 }
 
@@ -297,10 +376,11 @@ static void qcms_transform_data_graya_out_lut(qcms_transform *transform, unsigne
 		out_device_g = lut_interp_linear(linear, transform->output_gamma_lut_g, transform->output_gamma_lut_g_length);
 		out_device_b = lut_interp_linear(linear, transform->output_gamma_lut_b, transform->output_gamma_lut_b_length);
 
-		*dest++ = clamp_u8(out_device_r*255);
-		*dest++ = clamp_u8(out_device_g*255);
-		*dest++ = clamp_u8(out_device_b*255);
-		*dest++ = alpha;
+		dest[OUTPUT_R_INDEX] = clamp_u8(out_device_r*255);
+		dest[OUTPUT_G_INDEX] = clamp_u8(out_device_g*255);
+		dest[OUTPUT_B_INDEX] = clamp_u8(out_device_b*255);
+		dest[OUTPUT_A_INDEX] = alpha;
+		dest += RGBA_OUTPUT_COMPONENTS;
 	}
 }
 
@@ -317,9 +397,10 @@ static void qcms_transform_data_gray_out_precache(qcms_transform *transform, uns
 		/* we could round here... */
 		gray = linear * PRECACHE_OUTPUT_MAX;
 
-		*dest++ = transform->output_table_r->data[gray];
-		*dest++ = transform->output_table_g->data[gray];
-		*dest++ = transform->output_table_b->data[gray];
+		dest[OUTPUT_R_INDEX] = transform->output_table_r->data[gray];
+		dest[OUTPUT_G_INDEX] = transform->output_table_g->data[gray];
+		dest[OUTPUT_B_INDEX] = transform->output_table_b->data[gray];
+		dest += RGB_OUTPUT_COMPONENTS;
 	}
 }
 
@@ -336,10 +417,11 @@ static void qcms_transform_data_graya_out_precache(qcms_transform *transform, un
 		/* we could round here... */
 		gray = linear * PRECACHE_OUTPUT_MAX;
 
-		*dest++ = transform->output_table_r->data[gray];
-		*dest++ = transform->output_table_g->data[gray];
-		*dest++ = transform->output_table_b->data[gray];
-		*dest++ = alpha;
+		dest[OUTPUT_R_INDEX] = transform->output_table_r->data[gray];
+		dest[OUTPUT_G_INDEX] = transform->output_table_g->data[gray];
+		dest[OUTPUT_B_INDEX] = transform->output_table_b->data[gray];
+		dest[OUTPUT_A_INDEX] = alpha;
+		dest += RGBA_OUTPUT_COMPONENTS;
 	}
 }
 
@@ -370,9 +452,10 @@ static void qcms_transform_data_rgb_out_lut_precache(qcms_transform *transform,
 		g = out_linear_g * PRECACHE_OUTPUT_MAX;
 		b = out_linear_b * PRECACHE_OUTPUT_MAX;
 
-		*dest++ = transform->output_table_r->data[r];
-		*dest++ = transform->output_table_g->data[g];
-		*dest++ = transform->output_table_b->data[b];
+		dest[OUTPUT_R_INDEX] = transform->output_table_r->data[r];
+		dest[OUTPUT_G_INDEX] = transform->output_table_g->data[g];
+		dest[OUTPUT_B_INDEX] = transform->output_table_b->data[b];
+		dest += RGB_OUTPUT_COMPONENTS;
 	}
 }
 
@@ -404,10 +487,11 @@ static void qcms_transform_data_rgba_out_lut_precache(qcms_transform *transform,
 		g = out_linear_g * PRECACHE_OUTPUT_MAX;
 		b = out_linear_b * PRECACHE_OUTPUT_MAX;
 
-		*dest++ = transform->output_table_r->data[r];
-		*dest++ = transform->output_table_g->data[g];
-		*dest++ = transform->output_table_b->data[b];
-		*dest++ = alpha;
+		dest[OUTPUT_R_INDEX] = transform->output_table_r->data[r];
+		dest[OUTPUT_G_INDEX] = transform->output_table_g->data[g];
+		dest[OUTPUT_B_INDEX] = transform->output_table_b->data[b];
+		dest[OUTPUT_A_INDEX] = alpha;
+		dest += RGBA_OUTPUT_COMPONENTS;
 	}
 }
 
@@ -428,12 +512,12 @@ static void qcms_transform_data_clut(qcms_transform *transform, unsigned char *s
 		unsigned char in_b = *src++;
 		float linear_r = in_r/255.0f, linear_g=in_g/255.0f, linear_b = in_b/255.0f;
 
-		int x = floor(linear_r * (transform->grid_size-1));
-		int y = floor(linear_g * (transform->grid_size-1));
-		int z = floor(linear_b * (transform->grid_size-1));
-		int x_n = ceil(linear_r * (transform->grid_size-1));
-		int y_n = ceil(linear_g * (transform->grid_size-1));
-		int z_n = ceil(linear_b * (transform->grid_size-1));
+		int x = floorf(linear_r * (transform->grid_size-1));
+		int y = floorf(linear_g * (transform->grid_size-1));
+		int z = floorf(linear_b * (transform->grid_size-1));
+		int x_n = ceilf(linear_r * (transform->grid_size-1));
+		int y_n = ceilf(linear_g * (transform->grid_size-1));
+		int z_n = ceilf(linear_b * (transform->grid_size-1));
 		float x_d = linear_r * (transform->grid_size-1) - x; 
 		float y_d = linear_g * (transform->grid_size-1) - y;
 		float z_d = linear_b * (transform->grid_size-1) - z; 
@@ -469,6 +553,10 @@ static void qcms_transform_data_clut(qcms_transform *transform, unsigned char *s
 }
 */
 
+static int int_div_ceil(int value, int div) {
+	return ((value  + div - 1) / div);
+}
+
 // Using lcms' tetra interpolation algorithm.
 static void qcms_transform_data_tetra_clut_rgba(qcms_transform *transform, unsigned char *src, unsigned char *dest, size_t length) {
 	unsigned int i;
@@ -489,12 +577,12 @@ static void qcms_transform_data_tetra_clut_rgba(qcms_transform *transform, unsig
 		unsigned char in_a = *src++;
 		float linear_r = in_r/255.0f, linear_g=in_g/255.0f, linear_b = in_b/255.0f;
 
-		int x = floor(linear_r * (transform->grid_size-1));
-		int y = floor(linear_g * (transform->grid_size-1));
-		int z = floor(linear_b * (transform->grid_size-1));
-		int x_n = ceil(linear_r * (transform->grid_size-1));
-		int y_n = ceil(linear_g * (transform->grid_size-1));
-		int z_n = ceil(linear_b * (transform->grid_size-1));
+		int x = in_r * (transform->grid_size-1) / 255;
+		int y = in_g * (transform->grid_size-1) / 255;
+		int z = in_b * (transform->grid_size-1) / 255;
+		int x_n = int_div_ceil(in_r * (transform->grid_size-1), 255);
+		int y_n = int_div_ceil(in_g * (transform->grid_size-1), 255);
+		int z_n = int_div_ceil(in_b * (transform->grid_size-1), 255);
 		float rx = linear_r * (transform->grid_size-1) - x; 
 		float ry = linear_g * (transform->grid_size-1) - y;
 		float rz = linear_b * (transform->grid_size-1) - z; 
@@ -577,10 +665,11 @@ static void qcms_transform_data_tetra_clut_rgba(qcms_transform *transform, unsig
 		clut_g = c0_g + c1_g*rx + c2_g*ry + c3_g*rz;
 		clut_b = c0_b + c1_b*rx + c2_b*ry + c3_b*rz;
 
-		*dest++ = clamp_u8(clut_r*255.0f);
-		*dest++ = clamp_u8(clut_g*255.0f);
-		*dest++ = clamp_u8(clut_b*255.0f);
-		*dest++ = in_a;
+		dest[OUTPUT_R_INDEX] = clamp_u8(clut_r*255.0f);
+		dest[OUTPUT_G_INDEX] = clamp_u8(clut_g*255.0f);
+		dest[OUTPUT_B_INDEX] = clamp_u8(clut_b*255.0f);
+		dest[OUTPUT_A_INDEX] = in_a;
+		dest += RGBA_OUTPUT_COMPONENTS;
 	}	
 }
 
@@ -603,15 +692,15 @@ static void qcms_transform_data_tetra_clut(qcms_transform *transform, unsigned c
 		unsigned char in_b = *src++;
 		float linear_r = in_r/255.0f, linear_g=in_g/255.0f, linear_b = in_b/255.0f;
 
-		int x = floor(linear_r * (transform->grid_size-1));
-		int y = floor(linear_g * (transform->grid_size-1));
-		int z = floor(linear_b * (transform->grid_size-1));
-		int x_n = ceil(linear_r * (transform->grid_size-1));
-		int y_n = ceil(linear_g * (transform->grid_size-1));
-		int z_n = ceil(linear_b * (transform->grid_size-1));
-		float rx = linear_r * (transform->grid_size-1) - x; 
+		int x = in_r * (transform->grid_size-1) / 255;
+		int y = in_g * (transform->grid_size-1) / 255;
+		int z = in_b * (transform->grid_size-1) / 255;
+		int x_n = int_div_ceil(in_r * (transform->grid_size-1), 255);
+		int y_n = int_div_ceil(in_g * (transform->grid_size-1), 255);
+		int z_n = int_div_ceil(in_b * (transform->grid_size-1), 255);
+		float rx = linear_r * (transform->grid_size-1) - x;
 		float ry = linear_g * (transform->grid_size-1) - y;
-		float rz = linear_b * (transform->grid_size-1) - z; 
+		float rz = linear_b * (transform->grid_size-1) - z;
 
 		c0_r = CLU(r_table, x, y, z);
 		c0_g = CLU(g_table, x, y, z);
@@ -691,9 +780,10 @@ static void qcms_transform_data_tetra_clut(qcms_transform *transform, unsigned c
 		clut_g = c0_g + c1_g*rx + c2_g*ry + c3_g*rz;
 		clut_b = c0_b + c1_b*rx + c2_b*ry + c3_b*rz;
 
-		*dest++ = clamp_u8(clut_r*255.0f);
-		*dest++ = clamp_u8(clut_g*255.0f);
-		*dest++ = clamp_u8(clut_b*255.0f);
+		dest[OUTPUT_R_INDEX] = clamp_u8(clut_r*255.0f);
+		dest[OUTPUT_G_INDEX] = clamp_u8(clut_g*255.0f);
+		dest[OUTPUT_B_INDEX] = clamp_u8(clut_b*255.0f);
+		dest += RGB_OUTPUT_COMPONENTS;
 	}	
 }
 
@@ -726,9 +816,10 @@ static void qcms_transform_data_rgb_out_lut(qcms_transform *transform, unsigned
 		out_device_b = lut_interp_linear(out_linear_b, 
 				transform->output_gamma_lut_b, transform->output_gamma_lut_b_length);
 
-		*dest++ = clamp_u8(out_device_r*255);
-		*dest++ = clamp_u8(out_device_g*255);
-		*dest++ = clamp_u8(out_device_b*255);
+		dest[OUTPUT_R_INDEX] = clamp_u8(out_device_r*255);
+		dest[OUTPUT_G_INDEX] = clamp_u8(out_device_g*255);
+		dest[OUTPUT_B_INDEX] = clamp_u8(out_device_b*255);
+		dest += RGB_OUTPUT_COMPONENTS;
 	}
 }
 
@@ -762,10 +853,11 @@ static void qcms_transform_data_rgba_out_lut(qcms_transform *transform, unsigned
 		out_device_b = lut_interp_linear(out_linear_b, 
 				transform->output_gamma_lut_b, transform->output_gamma_lut_b_length);
 
-		*dest++ = clamp_u8(out_device_r*255);
-		*dest++ = clamp_u8(out_device_g*255);
-		*dest++ = clamp_u8(out_device_b*255);
-		*dest++ = alpha;
+		dest[OUTPUT_R_INDEX] = clamp_u8(out_device_r*255);
+		dest[OUTPUT_G_INDEX] = clamp_u8(out_device_g*255);
+		dest[OUTPUT_B_INDEX] = clamp_u8(out_device_b*255);
+		dest[OUTPUT_A_INDEX] = alpha;
+		dest += RGBA_OUTPUT_COMPONENTS;
 	}
 }
 
@@ -794,9 +886,17 @@ static void qcms_transform_data_rgb_out_linear(qcms_transform *transform, unsign
 }
 #endif
 
+/*
+ * If users create and destroy objects on different threads, even if the same
+ * objects aren't used on different threads at the same time, we can still run
+ * in to trouble with refcounts if they aren't atomic.
+ *
+ * This can lead to us prematurely deleting the precache if threads get unlucky
+ * and write the wrong value to the ref count.
+ */
 static struct precache_output *precache_reference(struct precache_output *p)
 {
-	p->ref_count++;
+	qcms_atomic_increment(p->ref_count);
 	return p;
 }
 
@@ -810,7 +910,7 @@ static struct precache_output *precache_create()
 
 void precache_release(struct precache_output *p)
 {
-	if (--p->ref_count == 0) {
+	if (qcms_atomic_decrement(p->ref_count) == 0) {
 		free(p);
 	}
 }
@@ -1117,7 +1217,11 @@ qcms_transform* qcms_transform_create(
 		precache = true;
 	}
 
-	if (qcms_supports_iccv4 && (in->A2B0 || out->B2A0 || in->mAB || out->mAB)) {
+	// This precache assumes RGB_SIGNATURE (fails on GRAY_SIGNATURE, for instance)
+	if (qcms_supports_iccv4 &&
+			(in_type == QCMS_DATA_RGB_8 || in_type == QCMS_DATA_RGBA_8) &&
+			(in->A2B0 || out->B2A0 || in->mAB || out->mAB))
+		{
 		// Precache the transformation to a CLUT 33x33x33 in size.
 		// 33 is used by many profiles and works well in pratice. 
 		// This evenly divides 256 into blocks of 8x8x8.
@@ -1178,6 +1282,14 @@ qcms_transform* qcms_transform_create(
 				    transform->transform_fn = qcms_transform_data_rgba_out_lut_sse1;
 #endif
 		    } else
+#endif
+#if (defined(__POWERPC__) || defined(__powerpc__))
+		    if (have_altivec()) {
+			    if (in_type == QCMS_DATA_RGB_8)
+				    transform->transform_fn = qcms_transform_data_rgb_out_lut_altivec;
+			    else
+				    transform->transform_fn = qcms_transform_data_rgba_out_lut_altivec;
+		    } else
 #endif
 			{
 				if (in_type == QCMS_DATA_RGB_8)
