@@ -18,16 +18,10 @@ public:
                              const GrDrawTarget* target,
                              bool antiAlias) const SK_OVERRIDE;
 
-    static bool staticCanDrawPath(bool pathIsConvex,
-                                  GrPathFill fill,
-                                  const GrDrawTarget* target,
-                                  bool antiAlias);
-
 protected:
     virtual bool onDrawPath(const SkPath& path,
                             GrPathFill fill,
                             const GrVec* translate,
                             GrDrawTarget* target,
-                            GrDrawState::StageMask stageMask,
                             bool antiAlias) SK_OVERRIDE;
 };
