@@ -1,48 +1,14 @@
 /* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Foundation code.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Vladimir Vukicevic <vladimir@mozilla.com>
- *   Masayuki Nakano <masayuki@d-toybox.com>
- *   Behdad Esfahbod <behdad@gnome.org>
- *   Mats Palmgren <mats.palmgren@bredband.net>
- *   Karl Tomlinson <karlt+@karlt.net>, Mozilla Corporation
- *   Takuro Ashie <ashie@clear-code.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "gfxFT2FontBase.h"
 #include "gfxFT2Utils.h"
 #include "harfbuzz/hb.h"
+#include "mozilla/Likely.h"
+
+using namespace mozilla::gfx;
 
 gfxFT2FontBase::gfxFT2FontBase(cairo_scaled_font_t *aScaledFont,
                                gfxFontEntry *aFontEntry,
@@ -52,6 +18,7 @@ gfxFT2FontBase::gfxFT2FontBase(cairo_scaled_font_t *aScaledFont,
       mHasMetrics(false)
 {
     cairo_scaled_font_reference(mScaledFont);
+    ConstructFontOptions();
 }
 
 gfxFT2FontBase::~gfxFT2FontBase()
@@ -59,8 +26,8 @@ gfxFT2FontBase::~gfxFT2FontBase()
     cairo_scaled_font_destroy(mScaledFont);
 }
 
-PRUint32
-gfxFT2FontBase::GetGlyph(PRUint32 aCharCode)
+uint32_t
+gfxFT2FontBase::GetGlyph(uint32_t aCharCode)
 {
     // FcFreeTypeCharIndex needs to lock the FT_Face and can end up searching
     // through all the postscript glyph names in the font.  Therefore use a
@@ -79,10 +46,10 @@ gfxFT2FontBase::GetGlyph(PRUint32 aCharCode)
     // scripts with large character sets as used for East Asian languages.
 
     struct CmapCacheSlot {
-        PRUint32 mCharCode;
-        PRUint32 mGlyphIndex;
+        uint32_t mCharCode;
+        uint32_t mGlyphIndex;
     };
-    const PRUint32 kNumSlots = 256;
+    const uint32_t kNumSlots = 256;
     static cairo_user_data_key_t sCmapCacheKey;
 
     CmapCacheSlot *slots = static_cast<CmapCacheSlot*>
@@ -122,7 +89,7 @@ gfxFT2FontBase::GetGlyph(PRUint32 aCharCode)
 }
 
 void
-gfxFT2FontBase::GetGlyphExtents(PRUint32 aGlyph, cairo_text_extents_t* aExtents)
+gfxFT2FontBase::GetGlyphExtents(uint32_t aGlyph, cairo_text_extents_t* aExtents)
 {
     NS_PRECONDITION(aExtents != NULL, "aExtents must not be NULL");
 
@@ -144,7 +111,7 @@ gfxFT2FontBase::GetMetrics()
     if (mHasMetrics)
         return mMetrics;
 
-    if (NS_UNLIKELY(GetStyle()->size <= 0.0)) {
+    if (MOZ_UNLIKELY(GetStyle()->size <= 0.0)) {
         new(&mMetrics) gfxFont::Metrics(); // zero initialize
         mSpaceGlyph = 0;
     } else {
@@ -170,7 +137,7 @@ gfxFT2FontBase::GetMetrics()
 }
 
 // Get the glyphID of a space
-PRUint32
+uint32_t
 gfxFT2FontBase::GetSpaceGlyph()
 {
     NS_ASSERTION(GetStyle()->size != 0,
@@ -179,27 +146,11 @@ gfxFT2FontBase::GetSpaceGlyph()
     return mSpaceGlyph;
 }
 
-hb_blob_t *
-gfxFT2FontBase::GetFontTable(PRUint32 aTag)
-{
-    hb_blob_t *blob;
-    if (mFontEntry->GetExistingFontTable(aTag, &blob))
-        return blob;
-
-    FallibleTArray<PRUint8> buffer;
-    bool haveTable = gfxFT2LockedFace(this).GetFontTable(aTag, buffer);
-
-    // Cache even when there is no table to save having to open the FT_Face
-    // again.
-    return mFontEntry->ShareFontTableAndGetBlob(aTag,
-                                                haveTable ? &buffer : nsnull);
-}
-
-PRUint32
-gfxFT2FontBase::GetGlyph(PRUint32 unicode, PRUint32 variation_selector)
+uint32_t
+gfxFT2FontBase::GetGlyph(uint32_t unicode, uint32_t variation_selector)
 {
     if (variation_selector) {
-        PRUint32 id =
+        uint32_t id =
             gfxFT2LockedFace(this).GetUVSGlyph(unicode, variation_selector);
         if (id)
             return id;
@@ -208,8 +159,8 @@ gfxFT2FontBase::GetGlyph(PRUint32 unicode, PRUint32 variation_selector)
     return GetGlyph(unicode);
 }
 
-PRInt32
-gfxFT2FontBase::GetGlyphWidth(gfxContext *aCtx, PRUint16 aGID)
+int32_t
+gfxFT2FontBase::GetGlyphWidth(gfxContext *aCtx, uint16_t aGID)
 {
     cairo_text_extents_t extents;
     GetGlyphExtents(aGID, &extents);
@@ -257,3 +208,25 @@ gfxFT2FontBase::SetupCairoFont(gfxContext *aContext)
     cairo_set_scaled_font(cr, cairoFont);
     return true;
 }
+
+void
+gfxFT2FontBase::ConstructFontOptions()
+{
+  NS_LossyConvertUTF16toASCII name(this->GetName());
+  mFontOptions.mName = name.get();
+
+  const gfxFontStyle* style = this->GetStyle();
+  if (style->style == NS_FONT_STYLE_ITALIC) {
+    if (style->weight == NS_FONT_WEIGHT_BOLD) {
+      mFontOptions.mStyle = FONT_STYLE_BOLD_ITALIC;
+    } else {
+      mFontOptions.mStyle = FONT_STYLE_ITALIC;
+    }
+  } else {
+    if (style->weight == NS_FONT_WEIGHT_BOLD) {
+      mFontOptions.mStyle = FONT_STYLE_BOLD;
+    } else {
+      mFontOptions.mStyle = FONT_STYLE_NORMAL;
+    }
+  }
+}
