@@ -1,41 +1,9 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  * vim: sw=4 ts=4 et :
  */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Plugin App.
- *
- * The Initial Developer of the Original Code is
- *   Chris Jones <jones.chris.g@gmail.com>
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/ipc/AsyncChannel.h"
 #include "mozilla/ipc/BrowserProcessSubThread.h"
@@ -45,7 +13,8 @@
 #include "nsTraceRefcnt.h"
 #include "nsXULAppAPI.h"
 
-using mozilla::MonitorAutoLock;
+using namespace mozilla;
+using namespace std;
 
 template<>
 struct RunnableMethodTraits<mozilla::ipc::AsyncChannel>
@@ -147,7 +116,6 @@ AsyncChannel::ProcessLink::Open(mozilla::ipc::Transport* aTransport,
     // FIXME need to check for valid channel
 
     mTransport = aTransport;
-    mExistingListener = mTransport->set_listener(this);
 
     // FIXME figure out whether we're in parent or child, grab IO loop
     // appropriately
@@ -164,9 +132,6 @@ AsyncChannel::ProcessLink::Open(mozilla::ipc::Transport* aTransport,
         mChan->mChild = false;
         needOpen = false;
         aIOLoop = XRE_GetIOMessageLoop();
-        // FIXME assuming that the parent waits for the OnConnected event.
-        // FIXME see GeckoChildProcessHost.cpp.  bad assumption!
-        mChan->mChannelState = ChannelConnected;
     }
 
     mIOLoop = aIOLoop;
@@ -174,14 +139,27 @@ AsyncChannel::ProcessLink::Open(mozilla::ipc::Transport* aTransport,
     NS_ASSERTION(mIOLoop, "need an IO loop");
     NS_ASSERTION(mChan->mWorkerLoop, "need a worker loop");
 
-    if (needOpen) {             // child process
+    {
         MonitorAutoLock lock(*mChan->mMonitor);
 
-        mIOLoop->PostTask(FROM_HERE, 
-                          NewRunnableMethod(this, &ProcessLink::OnChannelOpened));
+        if (needOpen) {
+            // Transport::Connect() has not been called.  Call it so
+            // we start polling our pipe and processing outgoing
+            // messages.
+            mIOLoop->PostTask(
+                FROM_HERE,
+                NewRunnableMethod(this, &ProcessLink::OnChannelOpened));
+        } else {
+            // Transport::Connect() has already been called.  Take
+            // over the channel from the previous listener and process
+            // any queued messages.
+            mIOLoop->PostTask(
+                FROM_HERE,
+                NewRunnableMethod(this, &ProcessLink::OnTakeConnectedChannel));
+        }
 
         // FIXME/cjones: handle errors
-        while (mChan->mChannelState != ChannelConnected) {
+        while (!mChan->Connected()) {
             mChan->mMonitor->Wait();
         }
     }
@@ -232,6 +210,25 @@ AsyncChannel::ThreadLink::ThreadLink(AsyncChannel *aChan,
 
 AsyncChannel::ThreadLink::~ThreadLink()
 {
+    MonitorAutoLock lock(*mChan->mMonitor);
+    // Bug 848949: We need to prevent the other side
+    // from sending us any more messages to avoid Use-After-Free.
+    // The setup here is as shown:
+    //
+    //          (Us)         (Them)
+    //       AsyncChannel  AsyncChannel
+    //         |  ^     \ /     ^ |
+    //         |  |      X      | |
+    //         v  |     / \     | v
+    //        ThreadLink   ThreadLink
+    //
+    // We want to null out the diagonal link from their ThreadLink
+    // to our AsyncChannel.  Note that we must hold the monitor so
+    // that we do this atomically with respect to them trying to send
+    // us a message.
+    if (mTargetChan) {
+        static_cast<ThreadLink*>(mTargetChan->mLink)->mTargetChan = 0;
+    }
     mTargetChan = 0;
 }
 
@@ -251,7 +248,8 @@ AsyncChannel::ThreadLink::SendMessage(Message *msg)
     mChan->AssertWorkerThread();
     mChan->mMonitor->AssertCurrentThreadOwns();
 
-    mTargetChan->OnMessageReceivedFromLink(*msg);
+    if (mTargetChan)
+        mTargetChan->OnMessageReceivedFromLink(*msg);
     delete msg;
 }
 
@@ -268,16 +266,18 @@ AsyncChannel::ThreadLink::SendClose()
     // The I/O thread would then invoke OnChannelErrorFromLink().
     // As usual, we skip that process and just invoke the
     // OnChannelErrorFromLink() method directly.
-    mTargetChan->OnChannelErrorFromLink();
+    if (mTargetChan)
+        mTargetChan->OnChannelErrorFromLink();
 }
 
 AsyncChannel::AsyncChannel(AsyncListener* aListener)
-  : mListener(aListener),
+  : mListener(aListener->asWeakPtr()),
     mChannelState(ChannelClosed),
     mWorkerLoop(),
     mChild(false),
     mChannelErrorTask(NULL),
-    mLink(NULL)
+    mLink(NULL),
+    mWorkerLoopID(-1)
 {
     MOZ_COUNT_CTOR(AsyncChannel);
 }
@@ -297,6 +297,7 @@ AsyncChannel::Open(Transport* aTransport,
     NS_PRECONDITION(!mLink, "Open() called > once");
     mMonitor = new RefCountedMonitor();
     mWorkerLoop = MessageLoop::current();
+    mWorkerLoopID = mWorkerLoop->id();
     mLink = link = new ProcessLink(this);
     link->Open(aTransport, aIOLoop, aSide); // n.b.: sets mChild
     return true;
@@ -354,6 +355,7 @@ void
 AsyncChannel::CommonThreadOpenInit(AsyncChannel *aTargetChan, Side aSide)
 {
     mWorkerLoop = MessageLoop::current();
+    mWorkerLoopID = mWorkerLoop->id();
     mLink = new ThreadLink(this, aTargetChan);
     mChild = (aSide == Child); 
 }
@@ -384,12 +386,7 @@ AsyncChannel::Close()
     AssertWorkerThread();
 
     {
-        // n.b.: We increase the ref count of monitor temporarily
-        //       for the duration of this block.  Otherwise, the
-        //       function NotifyMaybeChannelError() will call
-        //       ::Clear() which can free the monitor.
-        nsRefPtr<RefCountedMonitor> monitor(mMonitor);
-        MonitorAutoLock lock(*monitor);
+        MonitorAutoLock lock(*mMonitor);
 
         if (ChannelError == mChannelState ||
             ChannelTimeout == mChannelState) {
@@ -399,7 +396,7 @@ AsyncChannel::Close()
             // also be deleted and the listener will never be notified
             // of the channel error.
             if (mListener) {
-                MonitorAutoUnlock unlock(*monitor);
+                MonitorAutoUnlock unlock(*mMonitor);
                 NotifyMaybeChannelError();
             }
             return;
@@ -496,7 +493,7 @@ AsyncChannel::OnDispatchMessage(const Message& msg)
 }
 
 bool
-AsyncChannel::OnSpecialMessage(uint16 id, const Message& msg)
+AsyncChannel::OnSpecialMessage(uint16_t id, const Message& msg)
 {
     return false;
 }
@@ -514,6 +511,8 @@ AsyncChannel::OnNotifyMaybeChannelError()
     AssertWorkerThread();
     mMonitor->AssertNotCurrentThreadOwns();
 
+    mChannelErrorTask = NULL;
+
     // OnChannelError holds mMonitor when it posts this task and this
     // task cannot be allowed to run until OnChannelError has
     // exited. We enforce that order by grabbing the mutex here which
@@ -574,12 +573,20 @@ AsyncChannel::NotifyMaybeChannelError()
 void
 AsyncChannel::Clear()
 {
-    mListener = 0;
+    // Don't clear mWorkerLoopID; we use it in AssertLinkThread() and
+    // AssertWorkerThread().
+    //
+    // Also don't clear mListener.  If we clear it, then sending a message
+    // through this channel after it's Clear()'ed can cause this process to
+    // crash.
+    //
+    // In practice, mListener owns the channel, so the channel gets deleted
+    // before mListener.  But just to be safe, mListener is a weak pointer.
+
     mWorkerLoop = 0;
 
     delete mLink;
     mLink = 0;
-    mMonitor = 0;
 
     if (mChannelErrorTask) {
         mChannelErrorTask->Cancel();
@@ -590,10 +597,8 @@ AsyncChannel::Clear()
 static void
 PrintErrorMessage(bool isChild, const char* channelName, const char* msg)
 {
-#ifdef DEBUG
-    fprintf(stderr, "\n###!!! [%s][%s] Error: %s\n\n",
-            isChild ? "Child" : "Parent", channelName, msg);
-#endif
+    printf_stderr("\n###!!! [%s][%s] Error: %s\n\n",
+                  isChild ? "Child" : "Parent", channelName, msg);
 }
 
 bool
@@ -602,7 +607,7 @@ AsyncChannel::MaybeHandleError(Result code, const char* channelName)
     if (MsgProcessed == code)
         return true;
 
-    const char* errorMsg;
+    const char* errorMsg = nullptr;
     switch (code) {
     case MsgNotKnown:
         errorMsg = "Unknown message: not processed";
@@ -638,7 +643,7 @@ AsyncChannel::MaybeHandleError(Result code, const char* channelName)
 void
 AsyncChannel::ReportConnectionError(const char* channelName) const
 {
-    const char* errorMsg;
+    const char* errorMsg = nullptr;
     switch (mChannelState) {
     case ChannelClosed:
         errorMsg = "Closed channel: cannot send/recv";
@@ -666,7 +671,7 @@ AsyncChannel::ReportConnectionError(const char* channelName) const
 }
 
 void
-AsyncChannel::DispatchOnChannelConnected(int32 peer_pid)
+AsyncChannel::DispatchOnChannelConnected(int32_t peer_pid)
 {
     AssertWorkerThread();
     if (mListener)
@@ -700,13 +705,49 @@ AsyncChannel::ProcessLink::OnChannelOpened()
     mChan->AssertLinkThread();
     {
         MonitorAutoLock lock(*mChan->mMonitor);
+
+        mExistingListener = mTransport->set_listener(this);
+#ifdef DEBUG
+        if (mExistingListener) {
+            queue<Message> pending;
+            mExistingListener->GetQueuedMessages(pending);
+            MOZ_ASSERT(pending.empty());
+        }
+#endif  // DEBUG
+
         mChan->mChannelState = ChannelOpening;
+        lock.Notify();
     }
     /*assert*/mTransport->Connect();
 }
 
 void
-AsyncChannel::ProcessLink::OnChannelConnected(int32 peer_pid)
+AsyncChannel::ProcessLink::OnTakeConnectedChannel()
+{
+    AssertIOThread();
+
+    queue<Message> pending;
+    {
+        MonitorAutoLock lock(*mChan->mMonitor);
+
+        mChan->mChannelState = ChannelConnected;
+
+        mExistingListener = mTransport->set_listener(this);
+        if (mExistingListener) {
+            mExistingListener->GetQueuedMessages(pending);
+        }
+        lock.Notify();
+    }
+
+    // Dispatch whatever messages the previous listener had queued up.
+    while (!pending.empty()) {
+        OnMessageReceived(pending.front());
+        pending.pop();
+    }
+}
+
+void
+AsyncChannel::ProcessLink::OnChannelConnected(int32_t peer_pid)
 {
     AssertIOThread();
 
@@ -775,13 +816,27 @@ AsyncChannel::OnChannelErrorFromLink()
     PostErrorNotifyTask();
 }
 
+void
+AsyncChannel::CloseWithError()
+{
+    AssertWorkerThread();
+
+    MonitorAutoLock lock(*mMonitor);
+    if (ChannelConnected != mChannelState) {
+        return;
+    }
+    SynchronouslyClose();
+    mChannelState = ChannelError;
+    PostErrorNotifyTask();
+}
+
 void
 AsyncChannel::PostErrorNotifyTask()
 {
-    AssertLinkThread();
     mMonitor->AssertCurrentThreadOwns();
 
-    NS_ASSERTION(!mChannelErrorTask, "OnChannelError called twice?");
+    if (mChannelErrorTask)
+        return;
 
     // This must be the last code that runs on this thread!
     mChannelErrorTask =
