@@ -4,8 +4,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 465460;
-var summary = 'TM: valueOf in a loop: do not assert';
+var BUGNUMBER = 472528;
+var summary = 'Do not assert: !js_IsActiveWithOrBlock(cx, fp->scopeChain, 0)';
 var actual = '';
 var expect = '';
 
@@ -19,11 +19,19 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
+
   jit(true);
 
-  for each (let x in [1, {}, 1, null, 1, {}, 1, null, 1]) { }
-
+  try
+  {
+    for (var i = 0; i < 4; ++i) {
+      for (let j = 0; j < 2; ++j) { }
+      let (x) (function(){});
+    }
+  }
+  catch(ex)
+  {
+  }
   jit(false);
 
   reportCompare(expect, actual, summary);
