@@ -40,13 +40,6 @@ XPCCallContext::IsValid() const
     return mState != INIT_FAILED;
 }
 
-inline nsXPConnect*
-XPCCallContext::GetXPConnect() const
-{
-    CHECK_STATE(HAVE_CONTEXT);
-    return mXPC;
-}
-
 inline XPCJSRuntime*
 XPCCallContext::GetRuntime() const
 {
@@ -68,13 +61,6 @@ XPCCallContext::GetJSContext() const
     return mJSContext;
 }
 
-inline JSBool
-XPCCallContext::GetContextPopRequired() const
-{
-    CHECK_STATE(HAVE_CONTEXT);
-    return mContextPopRequired;
-}
-
 inline XPCContext::LangType
 XPCCallContext::GetCallerLanguage() const
 {
@@ -96,22 +82,6 @@ XPCCallContext::GetPrevCallContext() const
     return mPrevCallContext;
 }
 
-inline JSObject*
-XPCCallContext::GetScopeForNewJSObjects() const
-{
-    CHECK_STATE(HAVE_SCOPE);
-    return mScopeForNewJSObjects;
-}
-
-inline void
-XPCCallContext::SetScopeForNewJSObjects(JSObject *scope)
-{
-    NS_ABORT_IF_FALSE(mState == HAVE_CONTEXT, "wrong call context state");
-    NS_ABORT_IF_FALSE(js::IsObjectInContextCompartment(scope, mJSContext), "wrong compartment");
-    mScopeForNewJSObjects = scope;
-    mState = HAVE_SCOPE;
-}
-
 inline JSObject*
 XPCCallContext::GetFlattenedJSObject() const
 {
@@ -144,9 +114,7 @@ inline XPCWrappedNativeProto*
 XPCCallContext::GetProto() const
 {
     CHECK_STATE(HAVE_OBJECT);
-    if (mWrapper)
-        return mWrapper->GetProto();
-    return mFlattenedJSObject ? GetSlimWrapperProto(mFlattenedJSObject) : nullptr;
+    return mWrapper ? mWrapper->GetProto() : nullptr;
 }
 
 inline JSBool
@@ -259,7 +227,7 @@ XPCCallContext::GetResolveName() const
 }
 
 inline jsid
-XPCCallContext::SetResolveName(jsid name)
+XPCCallContext::SetResolveName(JS::HandleId name)
 {
     CHECK_STATE(HAVE_CONTEXT);
     return XPCJSRuntime::Get()->SetResolveName(name);
@@ -293,20 +261,6 @@ XPCCallContext::SetMethodIndex(uint16_t index)
     mMethodIndex = index;
 }
 
-inline JSBool
-XPCCallContext::GetDestroyJSContextInDestructor() const
-{
-    CHECK_STATE(HAVE_CONTEXT);
-    return mDestroyJSContextInDestructor;
-}
-
-inline void
-XPCCallContext::SetDestroyJSContextInDestructor(JSBool b)
-{
-    CHECK_STATE(HAVE_CONTEXT);
-    mDestroyJSContextInDestructor = b;
-}
-
 /***************************************************************************/
 
 inline const nsIID*
@@ -598,26 +552,10 @@ xpc_ForcePropertyResolve(JSContext* cx, JSObject* obj, jsid id)
     return true;
 }
 
-inline JSObject*
-xpc_NewSystemInheritingJSObject(JSContext *cx, JSClass *clasp, JSObject *proto,
-                                bool uniqueType, JSObject *parent)
-{
-    // Global creation should go through XPCWrappedNative::WrapNewGlobal().
-    MOZ_ASSERT(!(clasp->flags & JSCLASS_IS_GLOBAL));
-
-    JSObject *obj;
-    if (uniqueType) {
-        obj = JS_NewObjectWithUniqueType(cx, clasp, proto, parent);
-    } else {
-        obj = JS_NewObject(cx, clasp, proto, parent);
-    }
-    return obj;
-}
-
 inline jsid
 GetRTIdByIndex(JSContext *cx, unsigned index)
 {
-  XPCJSRuntime *rt = nsXPConnect::FastGetXPConnect()->GetRuntime();
+  XPCJSRuntime *rt = nsXPConnect::XPConnect()->GetRuntime();
   return rt->GetStringID(index);
 }
 
@@ -635,27 +573,6 @@ void ThrowBadResult(nsresult result, XPCCallContext& ccx)
                                result, ccx);
 }
 
-inline void
-XPCLazyCallContext::SetWrapper(XPCWrappedNative* wrapper,
-                               XPCWrappedNativeTearOff* tearoff)
-{
-    mWrapper = wrapper;
-    mTearOff = tearoff;
-    if (mTearOff)
-        mFlattenedJSObject = mTearOff->GetJSObject();
-    else
-        mFlattenedJSObject = mWrapper->GetFlatJSObject();
-}
-inline void
-XPCLazyCallContext::SetWrapper(JSObject* flattenedJSObject)
-{
-    NS_ASSERTION(IS_SLIM_WRAPPER_OBJECT(flattenedJSObject),
-                 "What kind of object is this?");
-    mWrapper = nullptr;
-    mTearOff = nullptr;
-    mFlattenedJSObject = flattenedJSObject;
-}
-
 /***************************************************************************/
 
 #endif /* xpcinlines_h___ */
