@@ -1,42 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   John Bandhauer <jband@netscape.com> (original author)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* Private maps (hashtables). */
 
@@ -49,7 +15,7 @@
 
 // Note that most of the declarations for hash table entries begin with
 // a pointer to something or another. This makes them look enough like
-// the JSDHashEntryStub struct that the default OPs (JS_DHashGetStubOps())
+// the PLDHashEntryStub struct that the default OPs (PL_DHashGetStubOps())
 // just do the right thing for most of our needs.
 
 // no virtuals in the maps - all the common stuff inlined
@@ -59,62 +25,59 @@
 
 class JSObject2WrappedJSMap
 {
+    typedef js::HashMap<JSObject*, nsXPCWrappedJS*, js::PointerHasher<JSObject*, 3>,
+                        js::SystemAllocPolicy> Map;
+
 public:
-    struct Entry : public JSDHashEntryHdr
-    {
-        JSObject*       key;
-        nsXPCWrappedJS* value;
-    };
-
-    static JSObject2WrappedJSMap* newMap(int size);
-
-    inline nsXPCWrappedJS* Find(JSObject* Obj)
-    {
-        NS_PRECONDITION(Obj,"bad param");
-        Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, Obj, JS_DHASH_LOOKUP);
-        if (JS_DHASH_ENTRY_IS_FREE(entry))
-            return nsnull;
-        return entry->value;
+    static JSObject2WrappedJSMap* newMap(int size) {
+        JSObject2WrappedJSMap* map = new JSObject2WrappedJSMap();
+        if (map && map->mTable.init(size))
+            return map;
+        delete map;
+        return nullptr;
     }
 
-    inline nsXPCWrappedJS* Add(nsXPCWrappedJS* wrapper)
-    {
+    inline nsXPCWrappedJS* Find(JSObject* Obj) {
+        NS_PRECONDITION(Obj,"bad param");
+        Map::Ptr p = mTable.lookup(Obj);
+        return p ? p->value : nullptr;
+    }
+
+    inline nsXPCWrappedJS* Add(nsXPCWrappedJS* wrapper) {
         NS_PRECONDITION(wrapper,"bad param");
         JSObject* obj = wrapper->GetJSObjectPreserveColor();
-        Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, obj, JS_DHASH_ADD);
-        if (!entry)
-            return nsnull;
-        if (entry->key)
-            return entry->value;
-        entry->key = obj;
-        entry->value = wrapper;
-        return wrapper;
+        Map::AddPtr p = mTable.lookupForAdd(obj);
+        if (p)
+            return p->value;
+        return mTable.add(p, obj, wrapper) ? wrapper : nullptr;
     }
 
-    inline void Remove(nsXPCWrappedJS* wrapper)
-    {
+    inline void Remove(nsXPCWrappedJS* wrapper) {
         NS_PRECONDITION(wrapper,"bad param");
-        JS_DHashTableOperate(mTable, wrapper->GetJSObjectPreserveColor(),
-                             JS_DHASH_REMOVE);
+        mTable.remove(wrapper->GetJSObjectPreserveColor());
     }
 
-    inline uint32_t Count() {return mTable->entryCount;}
-    inline uint32_t Enumerate(JSDHashEnumerator f, void *arg)
-        {return JS_DHashTableEnumerate(mTable, f, arg);}
+    inline uint32_t Count() {return mTable.count();}
 
-    size_t SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf);
+    inline void Dump(int16_t depth) {
+        for (Map::Range r = mTable.all(); !r.empty(); r.popFront())
+            r.front().value->DebugDump(depth);
+    }
 
-    ~JSObject2WrappedJSMap();
-private:
-    JSObject2WrappedJSMap();    // no implementation
-    JSObject2WrappedJSMap(int size);
+    void FindDyingJSObjects(nsTArray<nsXPCWrappedJS*>* dying);
 
-    static size_t SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr, JSMallocSizeOfFun mallocSizeOf, void *);
+    void ShutdownMarker(JSRuntime* rt);
+
+    size_t SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf) {
+        size_t n = mallocSizeOf(this);
+        n += mTable.sizeOfExcludingThis(mallocSizeOf);
+        return n;
+    }
 
 private:
-    JSDHashTable *mTable;
+    JSObject2WrappedJSMap() {}
+
+    Map mTable;
 };
 
 /*************************/
@@ -122,7 +85,7 @@ private:
 class Native2WrappedNativeMap
 {
 public:
-    struct Entry : public JSDHashEntryHdr
+    struct Entry : public PLDHashEntryHdr
     {
         nsISupports*      key;
         XPCWrappedNative* value;
@@ -134,9 +97,9 @@ public:
     {
         NS_PRECONDITION(Obj,"bad param");
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, Obj, JS_DHASH_LOOKUP);
-        if (JS_DHASH_ENTRY_IS_FREE(entry))
-            return nsnull;
+            PL_DHashTableOperate(mTable, Obj, PL_DHASH_LOOKUP);
+        if (PL_DHASH_ENTRY_IS_FREE(entry))
+            return nullptr;
         return entry->value;
     }
 
@@ -144,10 +107,11 @@ public:
     {
         NS_PRECONDITION(wrapper,"bad param");
         nsISupports* obj = wrapper->GetIdentityObject();
+        MOZ_ASSERT(!Find(obj), "wrapper already in new scope!");
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, obj, JS_DHASH_ADD);
+            PL_DHashTableOperate(mTable, obj, PL_DHASH_ADD);
         if (!entry)
-            return nsnull;
+            return nullptr;
         if (entry->key)
             return entry->value;
         entry->key = obj;
@@ -165,12 +129,12 @@ public:
                      "nsISupports identity! This will most likely cause serious "
                      "problems!");
 #endif
-        JS_DHashTableOperate(mTable, wrapper->GetIdentityObject(), JS_DHASH_REMOVE);
+        PL_DHashTableOperate(mTable, wrapper->GetIdentityObject(), PL_DHASH_REMOVE);
     }
 
     inline uint32_t Count() {return mTable->entryCount;}
-    inline uint32_t Enumerate(JSDHashEnumerator f, void *arg)
-        {return JS_DHashTableEnumerate(mTable, f, arg);}
+    inline uint32_t Enumerate(PLDHashEnumerator f, void *arg)
+        {return PL_DHashTableEnumerate(mTable, f, arg);}
 
     size_t SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf);
 
@@ -179,10 +143,10 @@ private:
     Native2WrappedNativeMap();    // no implementation
     Native2WrappedNativeMap(int size);
 
-    static size_t SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr, JSMallocSizeOfFun mallocSizeOf, void *);
+    static size_t SizeOfEntryExcludingThis(PLDHashEntryHdr *hdr, JSMallocSizeOfFun mallocSizeOf, void *);
 
 private:
-    JSDHashTable *mTable;
+    PLDHashTable *mTable;
 };
 
 /*************************/
@@ -190,12 +154,12 @@ private:
 class IID2WrappedJSClassMap
 {
 public:
-    struct Entry : public JSDHashEntryHdr
+    struct Entry : public PLDHashEntryHdr
     {
         const nsIID*         key;
         nsXPCWrappedJSClass* value;
 
-        static struct JSDHashTableOps sOps;
+        static struct PLDHashTableOps sOps;
     };
 
     static IID2WrappedJSClassMap* newMap(int size);
@@ -203,9 +167,9 @@ public:
     inline nsXPCWrappedJSClass* Find(REFNSIID iid)
     {
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, &iid, JS_DHASH_LOOKUP);
-        if (JS_DHASH_ENTRY_IS_FREE(entry))
-            return nsnull;
+            PL_DHashTableOperate(mTable, &iid, PL_DHASH_LOOKUP);
+        if (PL_DHASH_ENTRY_IS_FREE(entry))
+            return nullptr;
         return entry->value;
     }
 
@@ -214,9 +178,9 @@ public:
         NS_PRECONDITION(clazz,"bad param");
         const nsIID* iid = &clazz->GetIID();
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, iid, JS_DHASH_ADD);
+            PL_DHashTableOperate(mTable, iid, PL_DHASH_ADD);
         if (!entry)
-            return nsnull;
+            return nullptr;
         if (entry->key)
             return entry->value;
         entry->key = iid;
@@ -227,19 +191,19 @@ public:
     inline void Remove(nsXPCWrappedJSClass* clazz)
     {
         NS_PRECONDITION(clazz,"bad param");
-        JS_DHashTableOperate(mTable, &clazz->GetIID(), JS_DHASH_REMOVE);
+        PL_DHashTableOperate(mTable, &clazz->GetIID(), PL_DHASH_REMOVE);
     }
 
     inline uint32_t Count() {return mTable->entryCount;}
-    inline uint32_t Enumerate(JSDHashEnumerator f, void *arg)
-        {return JS_DHashTableEnumerate(mTable, f, arg);}
+    inline uint32_t Enumerate(PLDHashEnumerator f, void *arg)
+        {return PL_DHashTableEnumerate(mTable, f, arg);}
 
     ~IID2WrappedJSClassMap();
 private:
     IID2WrappedJSClassMap();    // no implementation
     IID2WrappedJSClassMap(int size);
 private:
-    JSDHashTable *mTable;
+    PLDHashTable *mTable;
 };
 
 /*************************/
@@ -247,12 +211,12 @@ private:
 class IID2NativeInterfaceMap
 {
 public:
-    struct Entry : public JSDHashEntryHdr
+    struct Entry : public PLDHashEntryHdr
     {
         const nsIID*        key;
         XPCNativeInterface* value;
 
-        static struct JSDHashTableOps sOps;
+        static struct PLDHashTableOps sOps;
     };
 
     static IID2NativeInterfaceMap* newMap(int size);
@@ -260,9 +224,9 @@ public:
     inline XPCNativeInterface* Find(REFNSIID iid)
     {
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, &iid, JS_DHASH_LOOKUP);
-        if (JS_DHASH_ENTRY_IS_FREE(entry))
-            return nsnull;
+            PL_DHashTableOperate(mTable, &iid, PL_DHASH_LOOKUP);
+        if (PL_DHASH_ENTRY_IS_FREE(entry))
+            return nullptr;
         return entry->value;
     }
 
@@ -271,9 +235,9 @@ public:
         NS_PRECONDITION(iface,"bad param");
         const nsIID* iid = iface->GetIID();
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, iid, JS_DHASH_ADD);
+            PL_DHashTableOperate(mTable, iid, PL_DHASH_ADD);
         if (!entry)
-            return nsnull;
+            return nullptr;
         if (entry->key)
             return entry->value;
         entry->key = iid;
@@ -284,12 +248,12 @@ public:
     inline void Remove(XPCNativeInterface* iface)
     {
         NS_PRECONDITION(iface,"bad param");
-        JS_DHashTableOperate(mTable, iface->GetIID(), JS_DHASH_REMOVE);
+        PL_DHashTableOperate(mTable, iface->GetIID(), PL_DHASH_REMOVE);
     }
 
     inline uint32_t Count() {return mTable->entryCount;}
-    inline uint32_t Enumerate(JSDHashEnumerator f, void *arg)
-        {return JS_DHashTableEnumerate(mTable, f, arg);}
+    inline uint32_t Enumerate(PLDHashEnumerator f, void *arg)
+        {return PL_DHashTableEnumerate(mTable, f, arg);}
 
     size_t SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf);
 
@@ -298,10 +262,10 @@ private:
     IID2NativeInterfaceMap();    // no implementation
     IID2NativeInterfaceMap(int size);
 
-    static size_t SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr, JSMallocSizeOfFun mallocSizeOf, void *);
+    static size_t SizeOfEntryExcludingThis(PLDHashEntryHdr *hdr, JSMallocSizeOfFun mallocSizeOf, void *);
 
 private:
-    JSDHashTable *mTable;
+    PLDHashTable *mTable;
 };
 
 /*************************/
@@ -309,7 +273,7 @@ private:
 class ClassInfo2NativeSetMap
 {
 public:
-    struct Entry : public JSDHashEntryHdr
+    struct Entry : public PLDHashEntryHdr
     {
         nsIClassInfo* key;
         XPCNativeSet* value;
@@ -320,9 +284,9 @@ public:
     inline XPCNativeSet* Find(nsIClassInfo* info)
     {
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, info, JS_DHASH_LOOKUP);
-        if (JS_DHASH_ENTRY_IS_FREE(entry))
-            return nsnull;
+            PL_DHashTableOperate(mTable, info, PL_DHASH_LOOKUP);
+        if (PL_DHASH_ENTRY_IS_FREE(entry))
+            return nullptr;
         return entry->value;
     }
 
@@ -330,9 +294,9 @@ public:
     {
         NS_PRECONDITION(info,"bad param");
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, info, JS_DHASH_ADD);
+            PL_DHashTableOperate(mTable, info, PL_DHASH_ADD);
         if (!entry)
-            return nsnull;
+            return nullptr;
         if (entry->key)
             return entry->value;
         entry->key = info;
@@ -343,12 +307,12 @@ public:
     inline void Remove(nsIClassInfo* info)
     {
         NS_PRECONDITION(info,"bad param");
-        JS_DHashTableOperate(mTable, info, JS_DHASH_REMOVE);
+        PL_DHashTableOperate(mTable, info, PL_DHASH_REMOVE);
     }
 
     inline uint32_t Count() {return mTable->entryCount;}
-    inline uint32_t Enumerate(JSDHashEnumerator f, void *arg)
-        {return JS_DHashTableEnumerate(mTable, f, arg);}
+    inline uint32_t Enumerate(PLDHashEnumerator f, void *arg)
+        {return PL_DHashTableEnumerate(mTable, f, arg);}
 
     // ClassInfo2NativeSetMap holds pointers to *some* XPCNativeSets.
     // So we don't want to count those XPCNativeSets, because they are better
@@ -361,7 +325,7 @@ private:
     ClassInfo2NativeSetMap();    // no implementation
     ClassInfo2NativeSetMap(int size);
 private:
-    JSDHashTable *mTable;
+    PLDHashTable *mTable;
 };
 
 /*************************/
@@ -369,7 +333,7 @@ private:
 class ClassInfo2WrappedNativeProtoMap
 {
 public:
-    struct Entry : public JSDHashEntryHdr
+    struct Entry : public PLDHashEntryHdr
     {
         nsIClassInfo*          key;
         XPCWrappedNativeProto* value;
@@ -380,9 +344,9 @@ public:
     inline XPCWrappedNativeProto* Find(nsIClassInfo* info)
     {
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, info, JS_DHASH_LOOKUP);
-        if (JS_DHASH_ENTRY_IS_FREE(entry))
-            return nsnull;
+            PL_DHashTableOperate(mTable, info, PL_DHASH_LOOKUP);
+        if (PL_DHASH_ENTRY_IS_FREE(entry))
+            return nullptr;
         return entry->value;
     }
 
@@ -390,9 +354,9 @@ public:
     {
         NS_PRECONDITION(info,"bad param");
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, info, JS_DHASH_ADD);
+            PL_DHashTableOperate(mTable, info, PL_DHASH_ADD);
         if (!entry)
-            return nsnull;
+            return nullptr;
         if (entry->key)
             return entry->value;
         entry->key = info;
@@ -403,12 +367,12 @@ public:
     inline void Remove(nsIClassInfo* info)
     {
         NS_PRECONDITION(info,"bad param");
-        JS_DHashTableOperate(mTable, info, JS_DHASH_REMOVE);
+        PL_DHashTableOperate(mTable, info, PL_DHASH_REMOVE);
     }
 
     inline uint32_t Count() {return mTable->entryCount;}
-    inline uint32_t Enumerate(JSDHashEnumerator f, void *arg)
-        {return JS_DHashTableEnumerate(mTable, f, arg);}
+    inline uint32_t Enumerate(PLDHashEnumerator f, void *arg)
+        {return PL_DHashTableEnumerate(mTable, f, arg);}
 
     size_t SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf);
 
@@ -417,10 +381,10 @@ private:
     ClassInfo2WrappedNativeProtoMap();    // no implementation
     ClassInfo2WrappedNativeProtoMap(int size);
 
-    static size_t SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr, JSMallocSizeOfFun mallocSizeOf, void *);
+    static size_t SizeOfEntryExcludingThis(PLDHashEntryHdr *hdr, JSMallocSizeOfFun mallocSizeOf, void *);
 
 private:
-    JSDHashTable *mTable;
+    PLDHashTable *mTable;
 };
 
 /*************************/
@@ -428,16 +392,16 @@ private:
 class NativeSetMap
 {
 public:
-    struct Entry : public JSDHashEntryHdr
+    struct Entry : public PLDHashEntryHdr
     {
         XPCNativeSet* key_value;
 
-        static JSBool
-        Match(JSDHashTable *table,
-              const JSDHashEntryHdr *entry,
+        static bool
+        Match(PLDHashTable *table,
+              const PLDHashEntryHdr *entry,
               const void *key);
 
-        static struct JSDHashTableOps sOps;
+        static struct PLDHashTableOps sOps;
     };
 
     static NativeSetMap* newMap(int size);
@@ -445,9 +409,9 @@ public:
     inline XPCNativeSet* Find(XPCNativeSetKey* key)
     {
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, key, JS_DHASH_LOOKUP);
-        if (JS_DHASH_ENTRY_IS_FREE(entry))
-            return nsnull;
+            PL_DHashTableOperate(mTable, key, PL_DHASH_LOOKUP);
+        if (PL_DHASH_ENTRY_IS_FREE(entry))
+            return nullptr;
         return entry->key_value;
     }
 
@@ -456,9 +420,9 @@ public:
         NS_PRECONDITION(key,"bad param");
         NS_PRECONDITION(set,"bad param");
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, key, JS_DHASH_ADD);
+            PL_DHashTableOperate(mTable, key, PL_DHASH_ADD);
         if (!entry)
-            return nsnull;
+            return nullptr;
         if (entry->key_value)
             return entry->key_value;
         entry->key_value = set;
@@ -467,7 +431,7 @@ public:
 
     inline XPCNativeSet* Add(XPCNativeSet* set)
     {
-        XPCNativeSetKey key(set, nsnull, 0);
+        XPCNativeSetKey key(set, nullptr, 0);
         return Add(&key, set);
     }
 
@@ -475,13 +439,13 @@ public:
     {
         NS_PRECONDITION(set,"bad param");
 
-        XPCNativeSetKey key(set, nsnull, 0);
-        JS_DHashTableOperate(mTable, &key, JS_DHASH_REMOVE);
+        XPCNativeSetKey key(set, nullptr, 0);
+        PL_DHashTableOperate(mTable, &key, PL_DHASH_REMOVE);
     }
 
     inline uint32_t Count() {return mTable->entryCount;}
-    inline uint32_t Enumerate(JSDHashEnumerator f, void *arg)
-        {return JS_DHashTableEnumerate(mTable, f, arg);}
+    inline uint32_t Enumerate(PLDHashEnumerator f, void *arg)
+        {return PL_DHashTableEnumerate(mTable, f, arg);}
 
     size_t SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf);
 
@@ -490,10 +454,10 @@ private:
     NativeSetMap();    // no implementation
     NativeSetMap(int size);
 
-    static size_t SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr, JSMallocSizeOfFun mallocSizeOf, void *);
+    static size_t SizeOfEntryExcludingThis(PLDHashEntryHdr *hdr, JSMallocSizeOfFun mallocSizeOf, void *);
 
 private:
-    JSDHashTable *mTable;
+    PLDHashTable *mTable;
 };
 
 /***************************************************************************/
@@ -501,20 +465,20 @@ private:
 class IID2ThisTranslatorMap
 {
 public:
-    struct Entry : public JSDHashEntryHdr
+    struct Entry : public PLDHashEntryHdr
     {
         nsIID                         key;
         nsIXPCFunctionThisTranslator* value;
 
-        static JSBool
-        Match(JSDHashTable *table,
-              const JSDHashEntryHdr *entry,
+        static bool
+        Match(PLDHashTable *table,
+              const PLDHashEntryHdr *entry,
               const void *key);
 
         static void
-        Clear(JSDHashTable *table, JSDHashEntryHdr *entry);
+        Clear(PLDHashTable *table, PLDHashEntryHdr *entry);
 
-        static struct JSDHashTableOps sOps;
+        static struct PLDHashTableOps sOps;
     };
 
     static IID2ThisTranslatorMap* newMap(int size);
@@ -522,9 +486,9 @@ public:
     inline nsIXPCFunctionThisTranslator* Find(REFNSIID iid)
     {
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, &iid, JS_DHASH_LOOKUP);
-        if (JS_DHASH_ENTRY_IS_FREE(entry))
-            return nsnull;
+            PL_DHashTableOperate(mTable, &iid, PL_DHASH_LOOKUP);
+        if (PL_DHASH_ENTRY_IS_FREE(entry))
+            return nullptr;
         return entry->value;
     }
 
@@ -533,9 +497,9 @@ public:
     {
 
         Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, &iid, JS_DHASH_ADD);
+            PL_DHashTableOperate(mTable, &iid, PL_DHASH_ADD);
         if (!entry)
-            return nsnull;
+            return nullptr;
         NS_IF_ADDREF(obj);
         NS_IF_RELEASE(entry->value);
         entry->value = obj;
@@ -545,19 +509,19 @@ public:
 
     inline void Remove(REFNSIID iid)
     {
-        JS_DHashTableOperate(mTable, &iid, JS_DHASH_REMOVE);
+        PL_DHashTableOperate(mTable, &iid, PL_DHASH_REMOVE);
     }
 
     inline uint32_t Count() {return mTable->entryCount;}
-    inline uint32_t Enumerate(JSDHashEnumerator f, void *arg)
-        {return JS_DHashTableEnumerate(mTable, f, arg);}
+    inline uint32_t Enumerate(PLDHashEnumerator f, void *arg)
+        {return PL_DHashTableEnumerate(mTable, f, arg);}
 
     ~IID2ThisTranslatorMap();
 private:
     IID2ThisTranslatorMap();    // no implementation
     IID2ThisTranslatorMap(int size);
 private:
-    JSDHashTable *mTable;
+    PLDHashTable *mTable;
 };
 
 /***************************************************************************/
@@ -565,36 +529,36 @@ private:
 class XPCNativeScriptableSharedMap
 {
 public:
-    struct Entry : public JSDHashEntryHdr
+    struct Entry : public PLDHashEntryHdr
     {
         XPCNativeScriptableShared* key;
 
-        static JSDHashNumber
-        Hash(JSDHashTable *table, const void *key);
+        static PLDHashNumber
+        Hash(PLDHashTable *table, const void *key);
 
-        static JSBool
-        Match(JSDHashTable *table,
-              const JSDHashEntryHdr *entry,
+        static bool
+        Match(PLDHashTable *table,
+              const PLDHashEntryHdr *entry,
               const void *key);
 
-        static struct JSDHashTableOps sOps;
+        static struct PLDHashTableOps sOps;
     };
 
     static XPCNativeScriptableSharedMap* newMap(int size);
 
-    JSBool GetNewOrUsed(uint32_t flags, char* name, PRUint32 interfacesBitmap,
+    JSBool GetNewOrUsed(uint32_t flags, char* name, uint32_t interfacesBitmap,
                         XPCNativeScriptableInfo* si);
 
     inline uint32_t Count() {return mTable->entryCount;}
-    inline uint32_t Enumerate(JSDHashEnumerator f, void *arg)
-        {return JS_DHashTableEnumerate(mTable, f, arg);}
+    inline uint32_t Enumerate(PLDHashEnumerator f, void *arg)
+        {return PL_DHashTableEnumerate(mTable, f, arg);}
 
     ~XPCNativeScriptableSharedMap();
 private:
     XPCNativeScriptableSharedMap();    // no implementation
     XPCNativeScriptableSharedMap(int size);
 private:
-    JSDHashTable *mTable;
+    PLDHashTable *mTable;
 };
 
 /***************************************************************************/
@@ -607,10 +571,10 @@ public:
     inline XPCWrappedNativeProto* Add(XPCWrappedNativeProto* proto)
     {
         NS_PRECONDITION(proto,"bad param");
-        JSDHashEntryStub* entry = (JSDHashEntryStub*)
-            JS_DHashTableOperate(mTable, proto, JS_DHASH_ADD);
+        PLDHashEntryStub* entry = (PLDHashEntryStub*)
+            PL_DHashTableOperate(mTable, proto, PL_DHASH_ADD);
         if (!entry)
-            return nsnull;
+            return nullptr;
         if (entry->key)
             return (XPCWrappedNativeProto*) entry->key;
         entry->key = proto;
@@ -620,93 +584,22 @@ public:
     inline void Remove(XPCWrappedNativeProto* proto)
     {
         NS_PRECONDITION(proto,"bad param");
-        JS_DHashTableOperate(mTable, proto, JS_DHASH_REMOVE);
+        PL_DHashTableOperate(mTable, proto, PL_DHASH_REMOVE);
     }
 
     inline uint32_t Count() {return mTable->entryCount;}
-    inline uint32_t Enumerate(JSDHashEnumerator f, void *arg)
-        {return JS_DHashTableEnumerate(mTable, f, arg);}
+    inline uint32_t Enumerate(PLDHashEnumerator f, void *arg)
+        {return PL_DHashTableEnumerate(mTable, f, arg);}
 
     ~XPCWrappedNativeProtoMap();
 private:
     XPCWrappedNativeProtoMap();    // no implementation
     XPCWrappedNativeProtoMap(int size);
 private:
-    JSDHashTable *mTable;
+    PLDHashTable *mTable;
 };
 
-class WrappedNative2WrapperMap
-{
-    static struct JSDHashTableOps sOps;
-
-    static void ClearLink(JSDHashTable* table, JSDHashEntryHdr* entry);
-    static void MoveLink(JSDHashTable* table, const JSDHashEntryHdr* from,
-                         JSDHashEntryHdr* to);
-
-public:
-    struct Link : public PRCList
-    {
-        JSObject *obj;
-    };
-
-    struct Entry : public JSDHashEntryHdr
-    {
-        // Note: key must be the flat JSObject for a wrapped native.
-        JSObject*         key;
-        Link              value;
-    };
-
-    static WrappedNative2WrapperMap* newMap(int size);
-
-    inline JSObject* Find(JSObject* wrapper)
-    {
-        NS_PRECONDITION(wrapper, "bad param");
-        Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, wrapper, JS_DHASH_LOOKUP);
-        if (JS_DHASH_ENTRY_IS_FREE(entry))
-            return nsnull;
-        return entry->value.obj;
-    }
-
-    // Note: If the entry already exists, then this will overwrite the
-    // existing entry, returning the old value.
-    JSObject* Add(WrappedNative2WrapperMap* head,
-                  JSObject* wrappedObject,
-                  JSObject* wrapper);
-
-    // Function to find a link.
-    Link* FindLink(JSObject* wrappedObject)
-    {
-        Entry* entry = (Entry*)
-            JS_DHashTableOperate(mTable, wrappedObject, JS_DHASH_LOOKUP);
-        if (JS_DHASH_ENTRY_IS_BUSY(entry))
-            return &entry->value;
-        return nsnull;
-    }
-
-    // "Internal" function to add an empty link without doing unnecessary
-    // work.
-    bool AddLink(JSObject* wrappedObject, Link* oldLink);
-
-    inline void Remove(JSObject* wrapper)
-    {
-        NS_PRECONDITION(wrapper,"bad param");
-        JS_DHashTableOperate(mTable, wrapper, JS_DHASH_REMOVE);
-    }
-
-    inline uint32_t Count() {return mTable->entryCount;}
-    inline uint32_t Enumerate(JSDHashEnumerator f, void *arg)
-        {return JS_DHashTableEnumerate(mTable, f, arg);}
-
-    ~WrappedNative2WrapperMap();
-
-private:
-    WrappedNative2WrapperMap();    // no implementation
-    WrappedNative2WrapperMap(int size);
-
-private:
-    JSDHashTable *mTable;
-};
+/***************************************************************************/
 
 class JSObject2JSObjectMap
 {
@@ -714,20 +607,19 @@ class JSObject2JSObjectMap
                         js::SystemAllocPolicy> Map;
 
 public:
-    static JSObject2JSObjectMap* newMap(int size)
-    {
-        JSObject2JSObjectMap* map = new JSObject2JSObjectMap(size);
-        if (map && map->mTable.initialized())
+    static JSObject2JSObjectMap* newMap(int size) {
+        JSObject2JSObjectMap* map = new JSObject2JSObjectMap();
+        if (map && map->mTable.init(size))
             return map;
         delete map;
-        return nsnull;
+        return nullptr;
     }
 
     inline JSObject* Find(JSObject* key) {
         NS_PRECONDITION(key, "bad param");
         if (Map::Ptr p = mTable.lookup(key))
             return p->value;
-        return nsnull;
+        return nullptr;
     }
 
     /* Note: If the entry already exists, return the old value. */
@@ -737,7 +629,7 @@ public:
         if (p)
             return p->value;
         if (!mTable.add(p, key, value))
-            return nsnull;
+            return nullptr;
         return value;
     }
 
@@ -750,19 +642,23 @@ public:
 
     void Sweep() {
         for (Map::Enum e(mTable); !e.empty(); e.popFront()) {
-            if (JS_IsAboutToBeFinalized(e.front().key) || JS_IsAboutToBeFinalized(e.front().value))
+            JSObject *updated = e.front().key;
+            if (JS_IsAboutToBeFinalized(&updated) || JS_IsAboutToBeFinalized(&e.front().value))
                 e.removeFront();
+            else if (updated != e.front().key)
+                e.rekeyFront(updated);
         }
     }
 
-    void Reparent(JSContext *aCx, JSObject *aNewInner) {
+    void Reparent(JSContext *aCx, JSObject *aNewInnerArg) {
+        JS::RootedObject aNewInner(aCx, aNewInnerArg);
         for (Map::Enum e(mTable); !e.empty(); e.popFront()) {
             /*
              * We reparent wrappers that have as their parent an inner window
              * whose outer has the new inner window as its current inner.
              */
-            JSObject *parent = JS_GetParent(e.front().value);
-            JSObject *outer = JS_ObjectToOuterObject(aCx, parent);
+            JS::RootedObject parent(aCx, JS_GetParent(e.front().value));
+            JS::RootedObject outer(aCx, JS_ObjectToOuterObject(aCx, parent));
             if (outer) {
                 JSObject *inner = JS_ObjectToInnerObject(aCx, outer);
                 if (inner == aNewInner && inner != parent)
@@ -774,12 +670,8 @@ public:
     }
 
 private:
-    JSObject2JSObjectMap() MOZ_DELETE;
-    JSObject2JSObjectMap(int size) {
-        mTable.init(size);
-    }
+    JSObject2JSObjectMap() {}
 
-private:
     Map mTable;
 };
 
