@@ -1,55 +1,20 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Mats Palmgren <matspal@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
  * rendering object for the point that anchors out-of-flow rendering
  * objects such as floats and absolutely positioned elements
  */
 
-#include "nsLayoutUtils.h"
 #include "nsPlaceholderFrame.h"
-#include "nsLineLayout.h"
-#include "nsIContent.h"
+
+#include "nsDisplayList.h"
+#include "nsFrameManager.h"
+#include "nsLayoutUtils.h"
 #include "nsPresContext.h"
 #include "nsRenderingContext.h"
-#include "nsGkAtoms.h"
-#include "nsFrameManager.h"
-#include "nsDisplayList.h"
 
 nsIFrame*
 NS_NewPlaceholderFrame(nsIPresShell* aPresShell, nsStyleContext* aContext,
@@ -60,26 +25,6 @@ NS_NewPlaceholderFrame(nsIPresShell* aPresShell, nsStyleContext* aContext,
 
 NS_IMPL_FRAMEARENA_HELPERS(nsPlaceholderFrame)
 
-nsPlaceholderFrame::~nsPlaceholderFrame()
-{
-}
-
-/* virtual */ nscoord
-nsPlaceholderFrame::GetMinWidth(nsRenderingContext *aRenderingContext)
-{
-  nscoord result = 0;
-  DISPLAY_MIN_WIDTH(this, result);
-  return result;
-}
-
-/* virtual */ nscoord
-nsPlaceholderFrame::GetPrefWidth(nsRenderingContext *aRenderingContext)
-{
-  nscoord result = 0;
-  DISPLAY_PREF_WIDTH(this, result);
-  return result;
-}
-
 /* virtual */ nsSize
 nsPlaceholderFrame::GetMinSize(nsBoxLayoutState& aBoxLayoutState)
 {
@@ -105,8 +50,8 @@ nsPlaceholderFrame::GetMaxSize(nsBoxLayoutState& aBoxLayoutState)
 }
 
 /* virtual */ void
-nsPlaceholderFrame::AddInlineMinWidth(nsRenderingContext *aRenderingContext,
-                                      nsIFrame::InlineMinWidthData *aData)
+nsPlaceholderFrame::AddInlineMinWidth(nsRenderingContext* aRenderingContext,
+                                      nsIFrame::InlineMinWidthData* aData)
 {
   // Override AddInlineMinWith so that *nothing* happens.  In
   // particular, we don't want to zero out |aData->trailingWhitespace|,
@@ -115,13 +60,19 @@ nsPlaceholderFrame::AddInlineMinWidth(nsRenderingContext *aRenderingContext,
   // false.
 
   // ...but push floats onto the list
-  if (mOutOfFlowFrame->GetStyleDisplay()->mFloats != NS_STYLE_FLOAT_NONE)
-    aData->floats.AppendElement(mOutOfFlowFrame);
+  if (mOutOfFlowFrame->IsFloating()) {
+    nscoord floatWidth =
+      nsLayoutUtils::IntrinsicForContainer(aRenderingContext,
+                                           mOutOfFlowFrame,
+                                           nsLayoutUtils::MIN_WIDTH);
+    aData->floats.AppendElement(
+      InlineIntrinsicWidthData::FloatInfo(mOutOfFlowFrame, floatWidth));
+  }
 }
 
 /* virtual */ void
-nsPlaceholderFrame::AddInlinePrefWidth(nsRenderingContext *aRenderingContext,
-                                       nsIFrame::InlinePrefWidthData *aData)
+nsPlaceholderFrame::AddInlinePrefWidth(nsRenderingContext* aRenderingContext,
+                                       nsIFrame::InlinePrefWidthData* aData)
 {
   // Override AddInlinePrefWith so that *nothing* happens.  In
   // particular, we don't want to zero out |aData->trailingWhitespace|,
@@ -130,16 +81,54 @@ nsPlaceholderFrame::AddInlinePrefWidth(nsRenderingContext *aRenderingContext,
   // false.
 
   // ...but push floats onto the list
-  if (mOutOfFlowFrame->GetStyleDisplay()->mFloats != NS_STYLE_FLOAT_NONE)
-    aData->floats.AppendElement(mOutOfFlowFrame);
+  if (mOutOfFlowFrame->IsFloating()) {
+    nscoord floatWidth =
+      nsLayoutUtils::IntrinsicForContainer(aRenderingContext,
+                                           mOutOfFlowFrame,
+                                           nsLayoutUtils::PREF_WIDTH);
+    aData->floats.AppendElement(
+      InlineIntrinsicWidthData::FloatInfo(mOutOfFlowFrame, floatWidth));
+  }
 }
 
 NS_IMETHODIMP
-nsPlaceholderFrame::Reflow(nsPresContext*          aPresContext,
+nsPlaceholderFrame::Reflow(nsPresContext*           aPresContext,
                            nsHTMLReflowMetrics&     aDesiredSize,
                            const nsHTMLReflowState& aReflowState,
                            nsReflowStatus&          aStatus)
 {
+#ifdef DEBUG
+  // We should be getting reflowed before our out-of-flow.
+  // If this is our first reflow, and our out-of-flow has already received its
+  // first reflow (before us), complain.
+  // XXXdholbert This "look for a previous continuation or IB-split sibling"
+  // code could use nsLayoutUtils::GetPrevContinuationOrSpecialSibling(), if
+  // we ever add a function like that. (We currently have a "Next" version.)
+  if ((GetStateBits() & NS_FRAME_FIRST_REFLOW) &&
+      !(mOutOfFlowFrame->GetStateBits() & NS_FRAME_FIRST_REFLOW)) {
+
+    // Unfortunately, this can currently happen when the placeholder is in a
+    // later continuation or later IB-split sibling than its out-of-flow (as
+    // is the case in some of our existing unit tests). So for now, in that
+    // case, we'll warn instead of asserting.
+    bool isInContinuationOrIBSplit = false;
+    nsIFrame* ancestor = this;
+    while ((ancestor = ancestor->GetParent())) {
+      if (ancestor->GetPrevContinuation() ||
+          ancestor->Properties().Get(IBSplitSpecialPrevSibling())) {
+        isInContinuationOrIBSplit = true;
+        break;
+      }
+    }
+
+    if (isInContinuationOrIBSplit) {
+      NS_WARNING("Out-of-flow frame got reflowed before its placeholder");
+    } else {
+      NS_ERROR("Out-of-flow frame got reflowed before its placeholder");
+    }
+  }
+#endif
+
   DO_GLOBAL_REFLOW_COUNT("nsPlaceholderFrame");
   DISPLAY_REFLOW(aPresContext, this, aReflowState, aDesiredSize, aStatus);
   aDesiredSize.width = 0;
@@ -153,21 +142,19 @@ nsPlaceholderFrame::Reflow(nsPresContext*          aPresContext,
 void
 nsPlaceholderFrame::DestroyFrom(nsIFrame* aDestructRoot)
 {
-  nsIPresShell* shell = PresContext()->GetPresShell();
   nsIFrame* oof = mOutOfFlowFrame;
   if (oof) {
-    oof->InvalidateFrameSubtree();
     // Unregister out-of-flow frame
-    shell->FrameManager()->UnregisterPlaceholderFrame(this);
-    mOutOfFlowFrame = nsnull;
+    nsFrameManager* fm = PresContext()->GetPresShell()->FrameManager();
+    fm->UnregisterPlaceholderFrame(this);
+    mOutOfFlowFrame = nullptr;
     // If aDestructRoot is not an ancestor of the out-of-flow frame,
     // then call RemoveFrame on it here.
     // Also destroy it here if it's a popup frame. (Bug 96291)
-    if (shell->FrameManager() &&
-        ((GetStateBits() & PLACEHOLDER_FOR_POPUP) ||
-         !nsLayoutUtils::IsProperAncestorFrame(aDestructRoot, oof))) {
+    if ((GetStateBits() & PLACEHOLDER_FOR_POPUP) ||
+        !nsLayoutUtils::IsProperAncestorFrame(aDestructRoot, oof)) {
       ChildListID listId = nsLayoutUtils::GetChildListNameFor(oof);
-      shell->FrameManager()->RemoveFrame(listId, oof, false);
+      fm->RemoveFrame(listId, oof);
     }
     // else oof will be destroyed by its parent
   }
@@ -178,7 +165,7 @@ nsPlaceholderFrame::DestroyFrom(nsIFrame* aDestructRoot)
 nsIAtom*
 nsPlaceholderFrame::GetType() const
 {
-  return nsGkAtoms::placeholderFrame; 
+  return nsGkAtoms::placeholderFrame;
 }
 
 /* virtual */ bool
@@ -212,16 +199,18 @@ PaintDebugPlaceholder(nsIFrame* aFrame, nsRenderingContext* aCtx,
   aCtx->SetColor(NS_RGB(0, 255, 255));
   nscoord x = nsPresContext::CSSPixelsToAppUnits(-5);
   aCtx->FillRect(aPt.x + x, aPt.y,
-                 nsPresContext::CSSPixelsToAppUnits(13), nsPresContext::CSSPixelsToAppUnits(3));
+                 nsPresContext::CSSPixelsToAppUnits(13),
+                 nsPresContext::CSSPixelsToAppUnits(3));
   nscoord y = nsPresContext::CSSPixelsToAppUnits(-10);
   aCtx->FillRect(aPt.x, aPt.y + y,
-                 nsPresContext::CSSPixelsToAppUnits(3), nsPresContext::CSSPixelsToAppUnits(10));
+                 nsPresContext::CSSPixelsToAppUnits(3),
+                 nsPresContext::CSSPixelsToAppUnits(10));
 }
 #endif // DEBUG
 
 #if defined(DEBUG) || (defined(MOZ_REFLOW_PERF_DSP) && defined(MOZ_REFLOW_PERF))
 
-NS_IMETHODIMP
+void
 nsPlaceholderFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
                                      const nsRect&           aDirtyRect,
                                      const nsDisplayListSet& aLists)
@@ -229,15 +218,13 @@ nsPlaceholderFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
   DO_GLOBAL_REFLOW_COUNT_DSP("nsPlaceholderFrame");
   
 #ifdef DEBUG
-  if (!GetShowFrameBorders())
-    return NS_OK;
-  
-  return aLists.Outlines()->AppendNewToTop(new (aBuilder)
-      nsDisplayGeneric(aBuilder, this, PaintDebugPlaceholder, "DebugPlaceholder",
-                       nsDisplayItem::TYPE_DEBUG_PLACEHOLDER));
-#else // DEBUG
-  return NS_OK;
-#endif // DEBUG
+  if (GetShowFrameBorders()) {
+    aLists.Outlines()->AppendNewToTop(
+      new (aBuilder) nsDisplayGeneric(aBuilder, this, PaintDebugPlaceholder,
+                                      "DebugPlaceholder",
+                                      nsDisplayItem::TYPE_DEBUG_PLACEHOLDER));
+  }
+#endif
 }
 #endif // DEBUG || (MOZ_REFLOW_PERF_DSP && MOZ_REFLOW_PERF)
 
@@ -248,40 +235,15 @@ nsPlaceholderFrame::GetFrameName(nsAString& aResult) const
   return MakeFrameName(NS_LITERAL_STRING("Placeholder"), aResult);
 }
 
-NS_IMETHODIMP
-nsPlaceholderFrame::List(FILE* out, PRInt32 aIndent) const
+void
+nsPlaceholderFrame::List(FILE* out, int32_t aIndent, uint32_t aFlags) const
 {
-  IndentBy(out, aIndent);
-  ListTag(out);
-#ifdef DEBUG_waterson
-  fprintf(out, " [parent=%p]", static_cast<void*>(mParent));
-#endif
-  if (HasView()) {
-    fprintf(out, " [view=%p]", (void*)GetView());
-  }
-  fprintf(out, " {%d,%d,%d,%d}", mRect.x, mRect.y, mRect.width, mRect.height);
-  if (0 != mState) {
-    fprintf(out, " [state=%016llx]", (unsigned long long)mState);
-  }
-  nsIFrame* prevInFlow = GetPrevInFlow();
-  nsIFrame* nextInFlow = GetNextInFlow();
-  if (nsnull != prevInFlow) {
-    fprintf(out, " prev-in-flow=%p", static_cast<void*>(prevInFlow));
-  }
-  if (nsnull != nextInFlow) {
-    fprintf(out, " next-in-flow=%p", static_cast<void*>(nextInFlow));
-  }
-  if (nsnull != mContent) {
-    fprintf(out, " [content=%p]", static_cast<void*>(mContent));
-  }
-  if (nsnull != mStyleContext) {
-    fprintf(out, " [sc=%p]", static_cast<void*>(mStyleContext));
-  }
+  ListGeneric(out, aIndent, aFlags);
+
   if (mOutOfFlowFrame) {
     fprintf(out, " outOfFlowFrame=");
     nsFrame::ListTag(out, mOutOfFlowFrame);
   }
   fputs("\n", out);
-  return NS_OK;
 }
-#endif
+#endif // DEBUG
