@@ -1,51 +1,15 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* representation of length values in computed style data */
 
 #ifndef nsStyleCoord_h___
 #define nsStyleCoord_h___
 
-#include "nscore.h"
 #include "nsCoord.h"
-#include "nsCRT.h"
 #include "nsStyleConsts.h"
-class nsString;
-class nsStyleContext;
 
 enum nsStyleUnit {
   eStyleUnit_Null         = 0,      // (no value) value is not specified
@@ -69,7 +33,7 @@ enum nsStyleUnit {
 };
 
 typedef union {
-  PRInt32     mInt;   // nscoord is a PRInt32 for now
+  int32_t     mInt;   // nscoord is a int32_t for now
   float       mFloat;
   // An mPointer is a weak pointer to a value that is guaranteed to
   // outlive the nsStyleCoord.  In the case of nsStyleCoord::Calc*, it
@@ -106,7 +70,7 @@ public:
   nsStyleCoord(nsStyleUnit aUnit = eStyleUnit_Null);
   enum CoordConstructorType { CoordConstructor };
   inline nsStyleCoord(nscoord aValue, CoordConstructorType);
-  nsStyleCoord(PRInt32 aValue, nsStyleUnit aUnit);
+  nsStyleCoord(int32_t aValue, nsStyleUnit aUnit);
   nsStyleCoord(float aValue, nsStyleUnit aUnit);
   inline nsStyleCoord(const nsStyleCoord& aCopy);
   inline nsStyleCoord(const nsStyleUnion& aValue, nsStyleUnit aUnit);
@@ -160,17 +124,18 @@ public:
   }
 
   nscoord     GetCoordValue() const;
-  PRInt32     GetIntValue() const;
+  int32_t     GetIntValue() const;
   float       GetPercentValue() const;
   float       GetFactorValue() const;
   float       GetAngleValue() const;
   double      GetAngleValueInRadians() const;
   Calc*       GetCalcValue() const;
   void        GetUnionValue(nsStyleUnion& aValue) const;
+  uint32_t    HashValue(uint32_t aHash) const;
 
   void  Reset();  // sets to null
   void  SetCoordValue(nscoord aValue);
-  void  SetIntValue(PRInt32 aValue, nsStyleUnit aUnit);
+  void  SetIntValue(int32_t aValue, nsStyleUnit aUnit);
   void  SetPercentValue(float aValue);
   void  SetFactorValue(float aValue);
   void  SetAngleValue(float aValue, nsStyleUnit aUnit);
@@ -218,7 +183,7 @@ public:
   inline void SetBottom(const nsStyleCoord& aCoord);
 
 protected:
-  PRUint8       mUnits[4];
+  uint8_t       mUnits[4];
   nsStyleUnion  mValues[4];
 };
 
@@ -237,16 +202,16 @@ public:
   bool           operator!=(const nsStyleCorners& aOther) const;
 
   // aCorner is always one of NS_CORNER_* defined in nsStyleConsts.h
-  inline nsStyleUnit GetUnit(PRUint8 aHalfCorner) const;
+  inline nsStyleUnit GetUnit(uint8_t aHalfCorner) const;
 
-  inline nsStyleCoord Get(PRUint8 aHalfCorner) const;
+  inline nsStyleCoord Get(uint8_t aHalfCorner) const;
 
   void  Reset();
 
-  inline void Set(PRUint8 aHalfCorner, const nsStyleCoord& aCoord);
+  inline void Set(uint8_t aHalfCorner, const nsStyleCoord& aCoord);
 
 protected:
-  PRUint8       mUnits[8];
+  uint8_t       mUnits[8];
   nsStyleUnion  mValues[8];
 };
 
@@ -296,7 +261,7 @@ inline nscoord nsStyleCoord::GetCoordValue() const
   return 0;
 }
 
-inline PRInt32 nsStyleCoord::GetIntValue() const
+inline int32_t nsStyleCoord::GetIntValue() const
 {
   NS_ASSERTION((mUnit == eStyleUnit_Enumerated) ||
                (mUnit == eStyleUnit_Integer), "not an int value");
@@ -341,7 +306,7 @@ inline nsStyleCoord::Calc* nsStyleCoord::GetCalcValue() const
   if (IsCalcUnit()) {
     return static_cast<Calc*>(mValue.mPointer);
   }
-  return nsnull;
+  return nullptr;
 }
 
 
@@ -442,17 +407,17 @@ inline bool nsStyleCorners::operator!=(const nsStyleCorners& aOther) const
   return !((*this) == aOther);
 }
 
-inline nsStyleUnit nsStyleCorners::GetUnit(PRUint8 aCorner) const
+inline nsStyleUnit nsStyleCorners::GetUnit(uint8_t aCorner) const
 {
   return (nsStyleUnit)mUnits[aCorner];
 }
 
-inline nsStyleCoord nsStyleCorners::Get(PRUint8 aCorner) const
+inline nsStyleCoord nsStyleCorners::Get(uint8_t aCorner) const
 {
   return nsStyleCoord(mValues[aCorner], nsStyleUnit(mUnits[aCorner]));
 }
 
-inline void nsStyleCorners::Set(PRUint8 aCorner, const nsStyleCoord& aCoord)
+inline void nsStyleCorners::Set(uint8_t aCorner, const nsStyleCoord& aCoord)
 {
   mUnits[aCorner] = aCoord.GetUnit();
   aCoord.GetUnionValue(mValues[aCorner]);
