@@ -7,15 +7,22 @@
 #undef NDEBUG
 #include <assert.h>
 #include <pthread.h>
-#include <stdio.h>
 #include <stdlib.h>
-#include <CoreServices/CoreServices.h>
 #include <AudioUnit/AudioUnit.h>
+#include <CoreAudio/AudioHardware.h>
 #include "cubeb/cubeb.h"
+#include "cubeb-internal.h"
 
 #define NBUFS 4
 
+static struct cubeb_ops const audiounit_ops;
+
+struct cubeb {
+  struct cubeb_ops const * ops;
+};
+
 struct cubeb_stream {
+  cubeb * context;
   AudioUnit unit;
   cubeb_data_callback data_callback;
   cubeb_state_callback state_callback;
@@ -29,9 +36,9 @@ struct cubeb_stream {
 };
 
 static OSStatus
-audio_unit_output_callback(void * user_ptr, AudioUnitRenderActionFlags * flags,
-                           AudioTimeStamp const * tstamp, UInt32 bus, UInt32 nframes,
-                           AudioBufferList * bufs)
+audiounit_output_callback(void * user_ptr, AudioUnitRenderActionFlags * flags,
+                          AudioTimeStamp const * tstamp, UInt32 bus, UInt32 nframes,
+                          AudioBufferList * bufs)
 {
   cubeb_stream * stm;
   unsigned char * buf;
@@ -65,7 +72,7 @@ audio_unit_output_callback(void * user_ptr, AudioUnitRenderActionFlags * flags,
     return noErr;
   }
 
-  if (got < nframes) {
+  if ((UInt32) got < nframes) {
     size_t got_bytes = got * stm->sample_spec.mBytesPerFrame;
     size_t rem_bytes = (nframes - got) * stm->sample_spec.mBytesPerFrame;
 
@@ -81,48 +88,111 @@ audio_unit_output_callback(void * user_ptr, AudioUnitRenderActionFlags * flags,
   return noErr;
 }
 
-int
-cubeb_init(cubeb ** context, char const * context_name)
+/*static*/ int
+audiounit_init(cubeb ** context, char const * context_name)
 {
-  *context = (void *) 0xdeadbeef;
+  cubeb * ctx;
+
+  *context = NULL;
+
+  ctx = calloc(1, sizeof(*ctx));
+  assert(ctx);
+
+  ctx->ops = &audiounit_ops;
+
+  *context = ctx;
+
   return CUBEB_OK;
 }
 
-char const *
-cubeb_get_backend_id(cubeb * ctx)
+static char const *
+audiounit_get_backend_id(cubeb * ctx)
 {
   return "audiounit";
 }
 
-void
-cubeb_destroy(cubeb * ctx)
-{
-  assert(ctx == (void *) 0xdeadbeef);
-}
 
 int
-cubeb_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_name,
-                  cubeb_stream_params stream_params, unsigned int latency,
-                  cubeb_data_callback data_callback, cubeb_state_callback state_callback,
-                  void * user_ptr)
+audiounit_get_max_channel_count(cubeb * ctx, uint32_t * max_channels)
+{
+  UInt32 size;
+  OSStatus r;
+  AudioDeviceID output_device_id;
+  AudioStreamBasicDescription stream_format;
+  AudioObjectPropertyAddress output_device_address = {
+    kAudioHardwarePropertyDefaultOutputDevice,
+    kAudioObjectPropertyScopeGlobal,
+    kAudioObjectPropertyElementMaster
+  };
+  AudioObjectPropertyAddress stream_format_address = {
+    kAudioDevicePropertyStreamFormat,
+    kAudioDevicePropertyScopeOutput,
+    kAudioObjectPropertyElementMaster
+  };
+
+  assert(ctx && max_channels);
+
+  /* Get the output device. */
+  size = sizeof(output_device_id);
+
+  r = AudioObjectGetPropertyData(kAudioObjectSystemObject,
+                                 &output_device_address,
+                                 0,
+                                 0,
+                                 &size,
+                                 &output_device_id);
+
+  if (r != noErr) {
+    return CUBEB_ERROR;
+  }
+
+  size = sizeof(stream_format);
+
+  r = AudioObjectGetPropertyData(output_device_id,
+                                 &stream_format_address,
+                                 0,
+                                 NULL,
+                                 &size,
+                                 &stream_format);
+  if (r != noErr) {
+    return CUBEB_ERROR;
+  }
+
+  *max_channels = stream_format.mChannelsPerFrame;
+
+  return CUBEB_OK;
+}
+
+static void
+audiounit_destroy(cubeb * ctx)
+{
+  free(ctx);
+}
+
+static void audiounit_stream_destroy(cubeb_stream * stm);
+
+static int
+audiounit_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_name,
+                      cubeb_stream_params stream_params, unsigned int latency,
+                      cubeb_data_callback data_callback, cubeb_state_callback state_callback,
+                      void * user_ptr)
 {
   AudioStreamBasicDescription ss;
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
   ComponentDescription desc;
-  cubeb_stream * stm;
   Component comp;
+#else
+  AudioComponentDescription desc;
+  AudioComponent comp;
+#endif
+  cubeb_stream * stm;
   AURenderCallbackStruct input;
   unsigned int buffer_size;
   OSStatus r;
 
-  assert(context == (void *) 0xdeadbeef);
+  assert(context);
   *stream = NULL;
 
-  if (stream_params.rate < 1 || stream_params.rate > 192000 ||
-      stream_params.channels < 1 || stream_params.channels > 32 ||
-      latency < 1 || latency > 2000) {
-    return CUBEB_ERROR_INVALID_FORMAT;
-  }
-
   memset(&ss, 0, sizeof(ss));
   ss.mFormatFlags = 0;
 
@@ -163,12 +233,17 @@ cubeb_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_n
   desc.componentManufacturer = kAudioUnitManufacturer_Apple;
   desc.componentFlags = 0;
   desc.componentFlagsMask = 0;
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
   comp = FindNextComponent(NULL, &desc);
+#else
+  comp = AudioComponentFindNext(NULL, &desc);
+#endif
   assert(comp);
 
   stm = calloc(1, sizeof(*stm));
   assert(stm);
 
+  stm->context = context;
   stm->data_callback = data_callback;
   stm->state_callback = state_callback;
   stm->user_ptr = user_ptr;
@@ -181,21 +256,31 @@ cubeb_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_n
   stm->frames_played = 0;
   stm->frames_queued = 0;
 
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
   r = OpenAComponent(comp, &stm->unit);
+#else
+  r = AudioComponentInstanceNew(comp, &stm->unit);
+#endif
   if (r != 0) {
-    fprintf(stderr, "cubeb_audiounit: FATAL: OpenAComponent returned %ld\n", (long) r);
+    audiounit_stream_destroy(stm);
+    return CUBEB_ERROR;
   }
-  assert(r == 0);
 
-  input.inputProc = audio_unit_output_callback;
+  input.inputProc = audiounit_output_callback;
   input.inputProcRefCon = stm;
   r = AudioUnitSetProperty(stm->unit, kAudioUnitProperty_SetRenderCallback,
                            kAudioUnitScope_Global, 0, &input, sizeof(input));
-  assert(r == 0);
+  if (r != 0) {
+    audiounit_stream_destroy(stm);
+    return CUBEB_ERROR;
+  }
 
   r = AudioUnitSetProperty(stm->unit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input,
                            0, &ss, sizeof(ss));
-  assert(r == 0);
+  if (r != 0) {
+    audiounit_stream_destroy(stm);
+    return CUBEB_ERROR;
+  }
 
   buffer_size = ss.mSampleRate / 1000.0 * latency * ss.mBytesPerFrame / NBUFS;
   if (buffer_size % ss.mBytesPerFrame != 0) {
@@ -204,28 +289,32 @@ cubeb_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_n
   assert(buffer_size % ss.mBytesPerFrame == 0);
 
   r = AudioUnitInitialize(stm->unit);
-  assert(r == 0);
+  if (r != 0) {
+    audiounit_stream_destroy(stm);
+    return CUBEB_ERROR;
+  }
 
   *stream = stm;
 
   return CUBEB_OK;
 }
 
-void
-cubeb_stream_destroy(cubeb_stream * stm)
+static void
+audiounit_stream_destroy(cubeb_stream * stm)
 {
-  OSStatus r;
+  int r;
 
   stm->shutdown = 1;
 
-  r = AudioOutputUnitStop(stm->unit);
-  assert(r == 0);
-
-  r = AudioUnitUninitialize(stm->unit);
-  assert(r == 0);
-
-  r = CloseComponent(stm->unit);
-  assert(r == 0);
+  if (stm->unit) {
+    AudioOutputUnitStop(stm->unit);
+    AudioUnitUninitialize(stm->unit);
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
+    CloseComponent(stm->unit);
+#else
+    AudioComponentInstanceDispose(stm->unit);
+#endif
+  }
 
   r = pthread_mutex_destroy(&stm->mutex);
   assert(r == 0);
@@ -233,8 +322,8 @@ cubeb_stream_destroy(cubeb_stream * stm)
   free(stm);
 }
 
-int
-cubeb_stream_start(cubeb_stream * stm)
+static int
+audiounit_stream_start(cubeb_stream * stm)
 {
   OSStatus r;
   r = AudioOutputUnitStart(stm->unit);
@@ -243,8 +332,8 @@ cubeb_stream_start(cubeb_stream * stm)
   return CUBEB_OK;
 }
 
-int
-cubeb_stream_stop(cubeb_stream * stm)
+static int
+audiounit_stream_stop(cubeb_stream * stm)
 {
   OSStatus r;
   r = AudioOutputUnitStop(stm->unit);
@@ -253,11 +342,23 @@ cubeb_stream_stop(cubeb_stream * stm)
   return CUBEB_OK;
 }
 
-int
-cubeb_stream_get_position(cubeb_stream * stm, uint64_t * position)
+static int
+audiounit_stream_get_position(cubeb_stream * stm, uint64_t * position)
 {
   pthread_mutex_lock(&stm->mutex);
   *position = stm->frames_played;
   pthread_mutex_unlock(&stm->mutex);
   return CUBEB_OK;
 }
+
+static struct cubeb_ops const audiounit_ops = {
+  .init = audiounit_init,
+  .get_backend_id = audiounit_get_backend_id,
+  .get_max_channel_count = audiounit_get_max_channel_count,
+  .destroy = audiounit_destroy,
+  .stream_init = audiounit_stream_init,
+  .stream_destroy = audiounit_stream_destroy,
+  .stream_start = audiounit_stream_start,
+  .stream_stop = audiounit_stream_stop,
+  .stream_get_position = audiounit_stream_get_position
+};
