@@ -1,10 +1,14 @@
 /*
- * Copyright  2011 Mozilla Foundation
+ * Copyright © 2011 Mozilla Foundation
  *
  * This program is made available under an ISC-style license.  See the
  * accompanying file LICENSE for details.
  */
 #undef NDEBUG
+#define __MSVCRT_VERSION__ 0x0700
+#define WINVER 0x0501
+#define WIN32_LEAN_AND_MEAN
+#include <malloc.h>
 #include <assert.h>
 #include <windows.h>
 #include <mmreg.h>
@@ -12,6 +16,12 @@
 #include <process.h>
 #include <stdlib.h>
 #include "cubeb/cubeb.h"
+#include "cubeb-internal.h"
+
+/* This is missing from the MinGW headers. Use a safe fallback. */
+#if !defined(MEMORY_ALLOCATION_ALIGNMENT)
+#define MEMORY_ALLOCATION_ALIGNMENT 16
+#endif
 
 #define CUBEB_STREAM_MAX 32
 #define NBUFS 4
@@ -26,13 +36,17 @@ struct cubeb_stream_item {
   cubeb_stream * stream;
 };
 
+static struct cubeb_ops const winmm_ops;
+
 struct cubeb {
+  struct cubeb_ops const * ops;
   HANDLE event;
   HANDLE thread;
   int shutdown;
   PSLIST_HEADER work;
   CRITICAL_SECTION lock;
   unsigned int active_streams;
+  unsigned int minimum_latency;
 };
 
 struct cubeb_stream {
@@ -72,7 +86,7 @@ bytes_per_frame(cubeb_stream_params params)
 }
 
 static WAVEHDR *
-cubeb_get_next_buffer(cubeb_stream * stm)
+winmm_get_next_buffer(cubeb_stream * stm)
 {
   WAVEHDR * hdr = NULL;
 
@@ -87,7 +101,7 @@ cubeb_get_next_buffer(cubeb_stream * stm)
 }
 
 static void
-cubeb_refill_stream(cubeb_stream * stm)
+winmm_refill_stream(cubeb_stream * stm)
 {
   WAVEHDR * hdr;
   long got;
@@ -113,7 +127,7 @@ cubeb_refill_stream(cubeb_stream * stm)
     return;
   }
 
-  hdr = cubeb_get_next_buffer(stm);
+  hdr = winmm_get_next_buffer(stm);
 
   wanted = (DWORD) stm->buffer_size / bytes_per_frame(stm->params);
 
@@ -123,6 +137,7 @@ cubeb_refill_stream(cubeb_stream * stm)
   got = stm->data_callback(stm, stm->user_ptr, hdr->lpData, wanted);
   EnterCriticalSection(&stm->lock);
   if (got < 0) {
+    LeaveCriticalSection(&stm->lock);
     /* XXX handle this case */
     assert(0);
     return;
@@ -146,7 +161,7 @@ cubeb_refill_stream(cubeb_stream * stm)
 }
 
 static unsigned __stdcall
-cubeb_buffer_thread(void * user_ptr)
+winmm_buffer_thread(void * user_ptr)
 {
   cubeb * ctx = (cubeb *) user_ptr;
   assert(ctx);
@@ -158,9 +173,15 @@ cubeb_buffer_thread(void * user_ptr)
     rv = WaitForSingleObject(ctx->event, INFINITE);
     assert(rv == WAIT_OBJECT_0);
 
-    while ((item = InterlockedPopEntrySList(ctx->work)) != NULL) {
-      cubeb_refill_stream(((struct cubeb_stream_item *) item)->stream);
-      _aligned_free(item);
+    /* Process work items in batches so that a single stream can't
+       starve the others by continuously adding new work to the top of
+       the work item stack. */
+    item = InterlockedFlushSList(ctx->work);
+    while (item != NULL) {
+      PSLIST_ENTRY tmp = item;
+      winmm_refill_stream(((struct cubeb_stream_item *) tmp)->stream);
+      item = item->Next;
+      _aligned_free(tmp);
     }
 
     if (ctx->shutdown) {
@@ -172,7 +193,7 @@ cubeb_buffer_thread(void * user_ptr)
 }
 
 static void CALLBACK
-cubeb_buffer_callback(HWAVEOUT waveout, UINT msg, DWORD_PTR user_ptr, DWORD_PTR p1, DWORD_PTR p2)
+winmm_buffer_callback(HWAVEOUT waveout, UINT msg, DWORD_PTR user_ptr, DWORD_PTR p1, DWORD_PTR p2)
 {
   cubeb_stream * stm = (cubeb_stream *) user_ptr;
   struct cubeb_stream_item * item;
@@ -189,8 +210,38 @@ cubeb_buffer_callback(HWAVEOUT waveout, UINT msg, DWORD_PTR user_ptr, DWORD_PTR
   SetEvent(stm->context->event);
 }
 
-int
-cubeb_init(cubeb ** context, char const * context_name)
+static unsigned int
+calculate_minimum_latency(void)
+{
+  OSVERSIONINFOEX osvi;
+  DWORDLONG mask;
+
+  /* Running under Terminal Services results in underruns with low latency. */
+  if (GetSystemMetrics(SM_REMOTESESSION) == TRUE) {
+    return 500;
+  }
+
+  /* Vista's WinMM implementation underruns when less than 200ms of audio is buffered. */
+  memset(&osvi, 0, sizeof(OSVERSIONINFOEX));
+  osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
+  osvi.dwMajorVersion = 6;
+  osvi.dwMinorVersion = 0;
+
+  mask = 0;
+  VER_SET_CONDITION(mask, VER_MAJORVERSION, VER_EQUAL);
+  VER_SET_CONDITION(mask, VER_MINORVERSION, VER_EQUAL);
+
+  if (VerifyVersionInfo(&osvi, VER_MAJORVERSION | VER_MINORVERSION, mask) != 0) {
+    return 200;
+  }
+
+  return 0;
+}
+
+static void winmm_destroy(cubeb * ctx);
+
+/*static*/ int
+winmm_init(cubeb ** context, char const * context_name)
 {
   cubeb * ctx;
 
@@ -200,32 +251,44 @@ cubeb_init(cubeb ** context, char const * context_name)
   ctx = calloc(1, sizeof(*ctx));
   assert(ctx);
 
+  ctx->ops = &winmm_ops;
+
   ctx->work = _aligned_malloc(sizeof(*ctx->work), MEMORY_ALLOCATION_ALIGNMENT);
   assert(ctx->work);
   InitializeSListHead(ctx->work);
 
   ctx->event = CreateEvent(NULL, FALSE, FALSE, NULL);
   if (!ctx->event) {
-    cubeb_destroy(ctx);
+    winmm_destroy(ctx);
     return CUBEB_ERROR;
   }
 
-  ctx->thread = (HANDLE) _beginthreadex(NULL, 64 * 1024, cubeb_buffer_thread, ctx, 0, NULL);
+  ctx->thread = (HANDLE) _beginthreadex(NULL, 64 * 1024, winmm_buffer_thread, ctx, 0, NULL);
   if (!ctx->thread) {
-    cubeb_destroy(ctx);
+    winmm_destroy(ctx);
     return CUBEB_ERROR;
   }
 
+  SetThreadPriority(ctx->thread, THREAD_PRIORITY_TIME_CRITICAL);
+
   InitializeCriticalSection(&ctx->lock);
   ctx->active_streams = 0;
 
+  ctx->minimum_latency = calculate_minimum_latency();
+
   *context = ctx;
 
   return CUBEB_OK;
 }
 
-void
-cubeb_destroy(cubeb * ctx)
+static char const *
+winmm_get_backend_id(cubeb * ctx)
+{
+  return "winmm";
+}
+
+static void
+winmm_destroy(cubeb * ctx)
 {
   DWORD rv;
 
@@ -251,8 +314,10 @@ cubeb_destroy(cubeb * ctx)
   free(ctx);
 }
 
-int
-cubeb_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_name,
+static void winmm_stream_destroy(cubeb_stream * stm);
+
+static int
+winmm_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_name,
                   cubeb_stream_params stream_params, unsigned int latency,
                   cubeb_data_callback data_callback,
                   cubeb_state_callback state_callback,
@@ -269,12 +334,6 @@ cubeb_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_n
 
   *stream = NULL;
 
-  if (stream_params.rate < 1 || stream_params.rate > 192000 ||
-      stream_params.channels < 1 || stream_params.channels > 32 ||
-      latency < 1 || latency > 2000) {
-    return CUBEB_ERROR_INVALID_FORMAT;
-  }
-
   memset(&wfx, 0, sizeof(wfx));
   if (stream_params.channels > 2) {
     wfx.Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE;
@@ -307,9 +366,7 @@ cubeb_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_n
 
   wfx.Format.nBlockAlign = (wfx.Format.wBitsPerSample * wfx.Format.nChannels) / 8;
   wfx.Format.nAvgBytesPerSec = wfx.Format.nSamplesPerSec * wfx.Format.nBlockAlign;
-  wfx.Samples.wValidBitsPerSample = 0;
-  wfx.Samples.wSamplesPerBlock = 0;
-  wfx.Samples.wReserved = 0;
+  wfx.Samples.wValidBitsPerSample = wfx.Format.wBitsPerSample;
 
   EnterCriticalSection(&context->lock);
   /* CUBEB_STREAM_MAX is a horrible hack to avoid a situation where, when
@@ -333,6 +390,10 @@ cubeb_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_n
   stm->state_callback = state_callback;
   stm->user_ptr = user_ptr;
 
+  if (latency < context->minimum_latency) {
+    latency = context->minimum_latency;
+  }
+
   bufsz = (size_t) (stm->params.rate / 1000.0 * latency * bytes_per_frame(stm->params) / NBUFS);
   if (bufsz % bytes_per_frame(stm->params) != 0) {
     bufsz += bytes_per_frame(stm->params) - (bufsz % bytes_per_frame(stm->params));
@@ -345,23 +406,23 @@ cubeb_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_n
 
   stm->event = CreateEvent(NULL, FALSE, FALSE, NULL);
   if (!stm->event) {
-    cubeb_stream_destroy(stm);
+    winmm_stream_destroy(stm);
     return CUBEB_ERROR;
   }
 
-  /* cubeb_buffer_callback will be called during waveOutOpen, so all
+  /* winmm_buffer_callback will be called during waveOutOpen, so all
      other initialization must be complete before calling it. */
   r = waveOutOpen(&stm->waveout, WAVE_MAPPER, &wfx.Format,
-                  (DWORD_PTR) cubeb_buffer_callback, (DWORD_PTR) stm,
+                  (DWORD_PTR) winmm_buffer_callback, (DWORD_PTR) stm,
                   CALLBACK_FUNCTION);
   if (r != MMSYSERR_NOERROR) {
-    cubeb_stream_destroy(stm);
+    winmm_stream_destroy(stm);
     return CUBEB_ERROR;
   }
 
   r = waveOutPause(stm->waveout);
   if (r != MMSYSERR_NOERROR) {
-    cubeb_stream_destroy(stm);
+    winmm_stream_destroy(stm);
     return CUBEB_ERROR;
   }
 
@@ -375,11 +436,11 @@ cubeb_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_n
 
     r = waveOutPrepareHeader(stm->waveout, hdr, sizeof(*hdr));
     if (r != MMSYSERR_NOERROR) {
-      cubeb_stream_destroy(stm);
+      winmm_stream_destroy(stm);
       return CUBEB_ERROR;
     }
 
-    cubeb_refill_stream(stm);
+    winmm_refill_stream(stm);
   }
 
   *stream = stm;
@@ -387,8 +448,8 @@ cubeb_stream_init(cubeb * context, cubeb_stream ** stream, char const * stream_n
   return CUBEB_OK;
 }
 
-void
-cubeb_stream_destroy(cubeb_stream * stm)
+static void
+winmm_stream_destroy(cubeb_stream * stm)
 {
   DWORD rv;
   int i;
@@ -445,7 +506,20 @@ cubeb_stream_destroy(cubeb_stream * stm)
 }
 
 int
-cubeb_stream_start(cubeb_stream * stm)
+winmm_get_max_channel_count(cubeb * ctx, uint32_t * max_channels)
+{
+  MMRESULT rv;
+  LPWAVEOUTCAPS waveout_caps;
+  assert(ctx && max_channels);
+
+  /* We don't support more than two channels in this backend. */
+  *max_channels = 2;
+
+  return CUBEB_OK;
+}
+
+static int
+winmm_stream_start(cubeb_stream * stm)
 {
   MMRESULT r;
 
@@ -462,8 +536,8 @@ cubeb_stream_start(cubeb_stream * stm)
   return CUBEB_OK;
 }
 
-int
-cubeb_stream_stop(cubeb_stream * stm)
+static int
+winmm_stream_stop(cubeb_stream * stm)
 {
   MMRESULT r;
 
@@ -480,8 +554,8 @@ cubeb_stream_stop(cubeb_stream * stm)
   return CUBEB_OK;
 }
 
-int
-cubeb_stream_get_position(cubeb_stream * stm, uint64_t * position)
+static int
+winmm_stream_get_position(cubeb_stream * stm, uint64_t * position)
 {
   MMRESULT r;
   MMTIME time;
@@ -500,3 +574,14 @@ cubeb_stream_get_position(cubeb_stream * stm, uint64_t * position)
   return CUBEB_OK;
 }
 
+static struct cubeb_ops const winmm_ops = {
+  /*.init =*/ winmm_init,
+  /*.get_backend_id =*/ winmm_get_backend_id,
+  /*.get_max_channel_count=*/ winmm_get_max_channel_count,
+  /*.destroy =*/ winmm_destroy,
+  /*.stream_init =*/ winmm_stream_init,
+  /*.stream_destroy =*/ winmm_stream_destroy,
+  /*.stream_start =*/ winmm_stream_start,
+  /*.stream_stop =*/ winmm_stream_stop,
+  /*.stream_get_position =*/ winmm_stream_get_position
+};
