@@ -2,9 +2,17 @@
 set -e
 
 cp $1/include/cubeb/cubeb.h include
+cp $1/src/cubeb.c src
+cp $1/src/cubeb-internal.h src
 cp $1/src/cubeb_alsa.c src
 cp $1/src/cubeb_winmm.c src
 cp $1/src/cubeb_audiounit.c src
+cp $1/src/cubeb_pulse.c src
+cp $1/src/cubeb_sndio.c src
+cp $1/src/cubeb_opensl.c src
+cp $1/src/cubeb_audiotrack.c src
+cp $1/src/android/audiotrack_definitions.h src/android
+cp $1/src/android/sles_definitions.h src/android
 cp $1/LICENSE .
 cp $1/README .
 cp $1/AUTHORS .
@@ -19,7 +27,7 @@ if [ -n "$rev" ]; then
     version=$version-dirty
     echo "WARNING: updating from a dirty git repository."
   fi
-  sed -i.bak -e "/The git commit ID used was/ s/[0-9a-f]\+\(-dirty\)\?\./$version./" README_MOZILLA
+  sed -i.bak -e "/The git commit ID used was/ s/[0-9a-f]\{40\}\(-dirty\)\{0,1\}\./$version./" README_MOZILLA
   rm README_MOZILLA.bak
 else
   echo "Remember to update README_MOZILLA with the version details."
