@@ -4,6 +4,8 @@
 
 package org.mozilla.gecko;
 
+import org.mozilla.gecko.mozglue.GeckoLoader;
+
 import android.content.Context;
 
 public class NSSBridge {
@@ -15,7 +17,7 @@ public class NSSBridge {
     static public String encrypt(Context context, String aValue)
       throws Exception {
         String resourcePath = context.getPackageResourcePath();
-        GeckoAppShell.loadNSSLibs(context, resourcePath);
+        GeckoLoader.loadNSSLibs(context, resourcePath);
 
         String path = GeckoProfile.get(context).getDir().toString();
         return nativeEncrypt(path, aValue);
@@ -24,7 +26,7 @@ public class NSSBridge {
     static public String encrypt(Context context, String profilePath, String aValue)
       throws Exception {
         String resourcePath = context.getPackageResourcePath();
-        GeckoAppShell.loadNSSLibs(context, resourcePath);
+        GeckoLoader.loadNSSLibs(context, resourcePath);
 
         return nativeEncrypt(profilePath, aValue);
     }
@@ -32,7 +34,7 @@ public class NSSBridge {
     static public String decrypt(Context context, String aValue)
       throws Exception {
         String resourcePath = context.getPackageResourcePath();
-        GeckoAppShell.loadNSSLibs(context, resourcePath);
+        GeckoLoader.loadNSSLibs(context, resourcePath);
 
         String path = GeckoProfile.get(context).getDir().toString();
         return nativeDecrypt(path, aValue);
@@ -41,7 +43,7 @@ public class NSSBridge {
     static public String decrypt(Context context, String profilePath, String aValue)
       throws Exception {
         String resourcePath = context.getPackageResourcePath();
-        GeckoAppShell.loadNSSLibs(context, resourcePath);
+        GeckoLoader.loadNSSLibs(context, resourcePath);
 
         return nativeDecrypt(profilePath, aValue);
     }
