@@ -9,15 +9,12 @@ import org.mozilla.gecko.db.BrowserContract;
 import org.mozilla.gecko.db.BrowserContract.Bookmarks;
 import org.mozilla.gecko.db.BrowserContract.Passwords;
 import org.mozilla.gecko.db.LocalBrowserDB;
+import org.mozilla.gecko.mozglue.GeckoLoader;
 import org.mozilla.gecko.sqlite.SQLiteBridge;
 import org.mozilla.gecko.sqlite.SQLiteBridgeException;
 import org.mozilla.gecko.sync.setup.SyncAccounts;
 import org.mozilla.gecko.sync.setup.SyncAccounts.SyncAccountParameters;
-import org.mozilla.gecko.util.GeckoEventListener;
-
-import org.json.JSONArray;
-import org.json.JSONException;
-import org.json.JSONObject;
+import org.mozilla.gecko.util.ThreadUtils;
 
 import android.accounts.Account;
 import android.content.ContentProviderOperation;
@@ -532,39 +529,26 @@ public class ProfileMigrator {
         }
     }
 
-    private class SyncTask implements Runnable, GeckoEventListener {
-        private List<String> mSyncSettingsList;
+    private class SyncTask implements Runnable {
         private Map<String, String> mSyncSettingsMap;
 
-        // Initialize preferences by sending the "Preferences:Get" command to Gecko
         protected void requestValues() {
-            mSyncSettingsList = Arrays.asList(SYNC_SETTINGS_LIST);
             mSyncSettingsMap = new HashMap<String, String>();
-            JSONArray jsonPrefs = new JSONArray(mSyncSettingsList);
-            Log.d(LOGTAG, "Sending: " + jsonPrefs.toString());
-            GeckoEvent event =
-                GeckoEvent.createBroadcastEvent("Preferences:Get",
-                                                jsonPrefs.toString());
-            GeckoAppShell.sendEventToGecko(event);
-        }
+            PrefsHelper.getPrefs(SYNC_SETTINGS_LIST, new PrefsHelper.PrefHandlerBase() {
+                @Override public void prefValue(String pref, boolean value) {
+                    mSyncSettingsMap.put(pref, value ? "1" : "0");
+                }
 
-        // Receive settings reply from Gecko, do the rest of the setup
-        public void handleMessage(String event, JSONObject message) {
-            Log.d(LOGTAG, "Received event: " + event);
-            try {
-                if (event.equals("Preferences:Data")) {
-                    // Receive most settings from Gecko's service.
-                    // This includes personal info, so don't log.
-                    // Log.d(LOGTAG, "Message: " + message.toString());
-                    JSONArray jsonPrefs = message.getJSONArray("preferences");
-
-                    // Check that the batch of preferences we got notified of are in
-                    // the ones we requested and not those requested by other java code.
-                    if (!parsePrefs(jsonPrefs))
-                        return;
-
-                    unregisterEventListener("Preferences:Data", this);
+                @Override public void prefValue(String pref, String value) {
+                    if (!TextUtils.isEmpty(value)) {
+                        mSyncSettingsMap.put(pref, value);
+                    } else {
+                        Log.w(LOGTAG, "Could not recover setting for = " + pref);
+                        mSyncSettingsMap.put(pref, null);
+                    }
+                }
 
+                @Override public void finish() {
                     // Now call the password provider to fill in the rest.
                     for (String location: SYNC_REALM_LIST) {
                         Log.d(LOGTAG, "Checking: " + location);
@@ -581,9 +565,7 @@ public class ProfileMigrator {
                     // Call Sync and transfer settings.
                     configureSync();
                 }
-            } catch (Exception e) {
-                Log.e(LOGTAG, "Exception handling message \"" + event + "\":", e);
-            }
+            });
         }
 
         protected String getPassword(String realm) {
@@ -619,41 +601,6 @@ public class ProfileMigrator {
             return result;
         }
 
-        // Returns true if we sucessfully got the preferences we requested.
-        protected boolean parsePrefs(JSONArray jsonPrefs) {
-            try {
-                final int length = jsonPrefs.length();
-                for (int i = 0; i < length; i++) {
-                    JSONObject jPref = jsonPrefs.getJSONObject(i);
-                    final String prefName = jPref.getString("name");
-
-                    // Check to make sure we're working with preferences we requested.
-                    if (!mSyncSettingsList.contains(prefName))
-                        return false;
-
-                    final String prefType = jPref.getString("type");
-                    if ("bool".equals(prefType)) {
-                        final boolean value = jPref.getBoolean("value");
-                        mSyncSettingsMap.put(prefName, value ? "1" : "0");
-                    } else {
-                        final String value = jPref.getString("value");
-                        if (!TextUtils.isEmpty(value)) {
-                            mSyncSettingsMap.put(prefName, value);
-                        } else {
-                            Log.w(LOGTAG, "Could not recover setting for = " + prefName);
-                            mSyncSettingsMap.put(prefName, null);
-                        }
-                    }
-                }
-            } catch (JSONException e) {
-                Log.e(LOGTAG, "Exception handling preferences answer: "
-                      + e.getMessage());
-                return false;
-            }
-
-            return true;
-        }
-
         protected void configureSync() {
             final String userName = mSyncSettingsMap.get("services.sync.account");
             final String syncKey = mSyncSettingsMap.get("Mozilla Services Encryption Passphrase");
@@ -663,7 +610,8 @@ public class ProfileMigrator {
             final String clientName = mSyncSettingsMap.get("services.sync.client.name");
             final String clientGuid = mSyncSettingsMap.get("services.sync.client.GUID");
 
-            GeckoAppShell.getHandler().post(new Runnable() {
+            ThreadUtils.postToBackgroundThread(new Runnable() {
+                @Override
                 public void run() {
                     if (userName == null || syncKey == null || syncPass == null) {
                         // This isn't going to work. Give up.
@@ -690,9 +638,9 @@ public class ProfileMigrator {
         }
 
         protected void registerAndRequest() {
-            GeckoAppShell.getHandler().post(new Runnable() {
+            ThreadUtils.postToBackgroundThread(new Runnable() {
+                @Override
                 public void run() {
-                    registerEventListener("Preferences:Data", SyncTask.this);
                     requestValues();
                 }
             });
@@ -705,7 +653,8 @@ public class ProfileMigrator {
                 @Override
                 protected void onPostExecute(Boolean result) {
                     if (result.booleanValue()) {
-                        GeckoAppShell.getHandler().post(new Runnable() {
+                        ThreadUtils.postToBackgroundThread(new Runnable() {
+                            @Override
                             public void run() {
                                 Log.i(LOGTAG, "Sync account already configured, skipping.");
                                 setMigratedSync();
@@ -718,19 +667,11 @@ public class ProfileMigrator {
                 }
             }.execute(mContext);
         }
-
-        private void registerEventListener(String event, GeckoEventListener listener) {
-            GeckoAppShell.getEventDispatcher().registerEventListener(event, listener);
-        }
-
-        private void unregisterEventListener(String event, GeckoEventListener listener) {
-            GeckoAppShell.getEventDispatcher().unregisterEventListener(event, listener);
-        }
     }
 
     private class MiscTask implements Runnable {
         protected void cleanupXULLibCache() {
-            File cacheFile = GeckoAppShell.getCacheDir(mContext);
+            File cacheFile = GeckoLoader.getCacheDir(mContext);
             File[] files = cacheFile.listFiles();
             if (files != null) {
                 Iterator<File> cacheFiles = Arrays.asList(files).iterator();
@@ -904,7 +845,7 @@ public class ProfileMigrator {
 
                 mDB.updateFaviconInBatch(mCr, mOperations, url, faviconUrl, faviconGuid, newData);
             } catch (SQLException e) {
-                Log.i(LOGTAG, "Migrating favicon failed: " + mime + " URL: " + url
+                Log.i(LOGTAG, "Migrating favicon failed: " + mime
                       + " error:" + e.getMessage());
             }
         }
@@ -993,7 +934,8 @@ public class ProfileMigrator {
 
             // GlobalHistory access communicates with Gecko
             // and must run on its thread.
-            GeckoAppShell.getHandler().post(new Runnable() {
+            ThreadUtils.postToBackgroundThread(new Runnable() {
+                    @Override
                     public void run() {
                         for (String url : placesHistory) {
                             GlobalHistory.getInstance().addToGeckoOnly(url);
@@ -1337,7 +1279,7 @@ public class ProfileMigrator {
             File dbFileShm = new File(dbPathShm);
 
             SQLiteBridge db = null;
-            GeckoAppShell.loadSQLiteLibs(mContext, mContext.getPackageResourcePath());
+            GeckoLoader.loadSQLiteLibs(mContext, mContext.getPackageResourcePath());
             try {
                 db = new SQLiteBridge(dbPath);
                 if (!checkPlacesSchema(db)) {
