@@ -7,12 +7,18 @@ package org.mozilla.gecko;
 
 import org.mozilla.gecko.AwesomeBar.ContextMenuSubject;
 import org.mozilla.gecko.db.BrowserContract.Bookmarks;
+import org.mozilla.gecko.db.BrowserContract.Combined;
 import org.mozilla.gecko.db.BrowserDB;
 import org.mozilla.gecko.db.BrowserDB.URLColumns;
+import org.mozilla.gecko.gfx.BitmapUtils;
+import org.mozilla.gecko.util.GamepadUtils;
+import org.mozilla.gecko.util.ThreadUtils;
+import org.mozilla.gecko.widget.FaviconView;
 
 import android.app.Activity;
 import android.content.Context;
 import android.database.Cursor;
+import android.graphics.Bitmap;
 import android.os.AsyncTask;
 import android.util.Log;
 import android.util.Pair;
@@ -23,10 +29,8 @@ import android.view.MenuInflater;
 import android.view.View;
 import android.view.ViewGroup;
 import android.widget.AdapterView;
-import android.widget.ImageView;
 import android.widget.ListView;
 import android.widget.SimpleCursorAdapter;
-import android.widget.TabHost.TabContentFactory;
 import android.widget.TextView;
 
 import java.util.LinkedList;
@@ -40,10 +44,12 @@ public class BookmarksTab extends AwesomeBarTab {
     private BookmarksQueryTask mQueryTask = null;
     private boolean mShowReadingList = false;
 
+    @Override
     public int getTitleStringId() {
         return R.string.awesomebar_bookmarks_title;
     }
 
+    @Override
     public String getTag() {
         return TAG;
     }
@@ -52,23 +58,10 @@ public class BookmarksTab extends AwesomeBarTab {
         super(context);
     }
 
-    public TabContentFactory getFactory() {
-        return new TabContentFactory() {
-             public View createTabContent(String tag) {
-                 final ListView list = getListView();
-                 list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
-                     public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
-                         handleItemClick(parent, view, position, id);
-                     }
-                 });
-                 return list;
-             }
-        };
-    }
-
-    public ListView getListView() {
+    @Override
+    public View getView() {
         if (mView == null) {
-            mView = (ListView) (LayoutInflater.from(mContext).inflate(R.layout.awesomebar_list, null));
+            mView = new ListView(mContext, null);
             ((Activity)mContext).registerForContextMenu(mView);
             mView.setTag(TAG);
             mView.setOnTouchListener(mListListener);
@@ -77,6 +70,13 @@ public class BookmarksTab extends AwesomeBarTab {
             ListView list = (ListView)mView;
             list.setAdapter(null);
             list.setAdapter(getCursorAdapter());
+            list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
+                @Override
+                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
+                    handleItemClick(parent, view, position, id);
+                }
+            });
+            list.setOnKeyListener(GamepadUtils.getListItemClickDispatcher());
 
             if (mShowReadingList) {
                 String title = getResources().getString(R.string.bookmarks_folder_reading_list);
@@ -93,23 +93,32 @@ public class BookmarksTab extends AwesomeBarTab {
         mShowReadingList = showReadingList;
     }
 
+    @Override
     public void destroy() {
-        BookmarksListAdapter adapter = getCursorAdapter();
-        if (adapter == null) {
-            return;
+        super.destroy();
+        // Can't use getters for adapter. It will create one if null.
+        if (mCursorAdapter != null && mView != null) {
+            ListView list = (ListView)mView;
+            list.setAdapter(null);
+            final Cursor cursor = mCursorAdapter.getCursor();
+            // Gingerbread locks the DB when closing a cursor, so do it in the
+            // background.
+            ThreadUtils.postToBackgroundThread(new Runnable() {
+                @Override
+                public void run() {
+                    if (cursor != null && !cursor.isClosed())
+                        cursor.close();
+                }
+            });
         }
-
-        Cursor cursor = adapter.getCursor();
-        if (cursor != null)
-            cursor.close();
     }
 
+    @Override
     public boolean onBackPressed() {
         // If the soft keyboard is visible in the bookmarks or history tab, the user
         // must have explictly brought it up, so we should try hiding it instead of
         // exiting the activity or going up a bookmarks folder level.
-        ListView view = getListView();
-        if (hideSoftInput(view))
+        if (hideSoftInput(getView()))
             return true;
 
         return moveToParentFolder();
@@ -161,7 +170,7 @@ public class BookmarksTab extends AwesomeBarTab {
     }
 
     public void handleItemClick(AdapterView<?> parent, View view, int position, long id) {
-        ListView list = getListView();
+        ListView list = (ListView)getView();
         if (list == null)
             return;
 
@@ -195,17 +204,13 @@ public class BookmarksTab extends AwesomeBarTab {
         }
 
         // Otherwise, just open the URL
-        AwesomeBarTabs.OnUrlOpenListener listener = getUrlListener();
-        if (listener == null) {
-            return;
-        }
-
         String url = cursor.getString(cursor.getColumnIndexOrThrow(URLColumns.URL));
+        String title = cursor.getString(cursor.getColumnIndexOrThrow(URLColumns.TITLE));
         long parentId = cursor.getLong(cursor.getColumnIndexOrThrow(Bookmarks.PARENT));
         if (parentId == Bookmarks.FIXED_READING_LIST_ID) {
             url = ReaderModeUtils.getAboutReaderForUrl(url, true);
         }
-        listener.onUrlOpen(url);
+        sendToListener(url, title);
     }
 
     private class BookmarksListAdapter extends SimpleCursorAdapter {
@@ -260,6 +265,7 @@ public class BookmarksTab extends AwesomeBarTab {
             return (folderPair.first == Bookmarks.FIXED_READING_LIST_ID);
         }
 
+        @Override
         public int getItemViewType(int position) {
             Cursor c = getCursor();
  
@@ -317,7 +323,7 @@ public class BookmarksTab extends AwesomeBarTab {
 
                 viewHolder = new AwesomeEntryViewHolder();
                 viewHolder.titleView = (TextView) convertView.findViewById(R.id.title);
-                viewHolder.faviconView = (ImageView) convertView.findViewById(R.id.favicon);
+                viewHolder.faviconView = (FaviconView) convertView.findViewById(R.id.favicon);
 
                 if (viewType == VIEW_TYPE_ITEM)
                     viewHolder.urlView = (TextView) convertView.findViewById(R.id.url);
@@ -333,8 +339,18 @@ public class BookmarksTab extends AwesomeBarTab {
 
             if (viewType == VIEW_TYPE_ITEM) {
                 updateTitle(viewHolder.titleView, cursor);
-                updateUrl(viewHolder.urlView, cursor);
-                updateFavicon(viewHolder.faviconView, cursor);
+                updateUrl(viewHolder, cursor);
+
+                byte[] b = cursor.getBlob(cursor.getColumnIndexOrThrow(URLColumns.FAVICON));
+                Bitmap favicon = null;
+                if (b != null) {
+                    Bitmap bitmap = BitmapUtils.decodeByteArray(b);
+                    if (bitmap != null) {
+                        favicon = Favicons.getInstance().scaleImage(bitmap);
+                    }
+                }
+                String url = cursor.getString(cursor.getColumnIndexOrThrow(URLColumns.URL));
+                updateFavicon(viewHolder.faviconView, favicon, url);
             } else {
                 viewHolder.titleView.setText(getFolderTitle(position));
             }
@@ -370,7 +386,8 @@ public class BookmarksTab extends AwesomeBarTab {
         @Override
         protected void onPostExecute(final Cursor cursor) {
             // Hack: force this to the main thread, even though it should already be on it
-            GeckoApp.mAppContext.mMainHandler.post(new Runnable() {
+            ThreadUtils.postToUiThread(new Runnable() {
+                @Override
                 public void run() {
                     // this will update the cursorAdapter to use the new one if it already exists
                     // We need to add the header before we set the adapter, hence make it null
@@ -404,6 +421,7 @@ public class BookmarksTab extends AwesomeBarTab {
         return mCursorAdapter.isInReadingList();
     }
 
+    @Override
     public ContextMenuSubject getSubject(ContextMenu menu, View view, ContextMenuInfo menuInfo) {
         ContextMenuSubject subject = null;
 
@@ -436,17 +454,17 @@ public class BookmarksTab extends AwesomeBarTab {
                                             cursor.getString(cursor.getColumnIndexOrThrow(URLColumns.URL)),
                                             cursor.getBlob(cursor.getColumnIndexOrThrow(URLColumns.FAVICON)),
                                             cursor.getString(cursor.getColumnIndexOrThrow(URLColumns.TITLE)),
-                                            keyword);
+                                            keyword,
+                                            isInReadingList() ? Combined.DISPLAY_READER : Combined.DISPLAY_NORMAL);
         }
 
         if (subject == null)
             return subject;
 
-        MenuInflater inflater = new MenuInflater(mContext);
-        inflater.inflate(R.menu.awesomebar_contextmenu, menu);
+        setupMenu(menu, subject);
         
         menu.findItem(R.id.remove_history).setVisible(false);
-        menu.setHeaderTitle(subject.title);
+        menu.findItem(R.id.open_in_reader).setVisible(false);
 
         return subject;
     }
