@@ -1,82 +1,43 @@
 /* -*- Mode: Java; c-basic-offset: 4; tab-width: 20; indent-tabs-mode: nil; -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Android code.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2009-2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Patrick Walton <pcwalton@mozilla.com>
- *   Chris Lord <chrislord.net@gmail.com>
- *   Arkady Blyakher <rkadyb@mit.edu>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 package org.mozilla.gecko.gfx;
 
-import org.mozilla.gecko.gfx.BufferedCairoImage;
-import org.mozilla.gecko.gfx.IntSize;
-import org.mozilla.gecko.gfx.Layer.RenderContext;
-import org.mozilla.gecko.gfx.LayerController;
-import org.mozilla.gecko.gfx.NinePatchTileLayer;
-import org.mozilla.gecko.gfx.SingleTileLayer;
-import org.mozilla.gecko.gfx.TextureReaper;
-import org.mozilla.gecko.gfx.TextureGenerator;
-import org.mozilla.gecko.gfx.TextLayer;
-import org.mozilla.gecko.gfx.TileLayer;
 import org.mozilla.gecko.GeckoAppShell;
+import org.mozilla.gecko.R;
+import org.mozilla.gecko.Tab;
+import org.mozilla.gecko.Tabs;
+import org.mozilla.gecko.gfx.Layer.RenderContext;
+import org.mozilla.gecko.mozglue.DirectBufferAllocator;
+
 import android.content.Context;
 import android.content.SharedPreferences;
 import android.graphics.Bitmap;
+import android.graphics.Canvas;
+import android.graphics.Color;
+import android.graphics.Matrix;
 import android.graphics.Point;
 import android.graphics.PointF;
 import android.graphics.Rect;
 import android.graphics.RectF;
-import android.graphics.Region;
-import android.graphics.RegionIterator;
 import android.opengl.GLES20;
-import android.opengl.GLSurfaceView;
 import android.os.SystemClock;
-import android.util.DisplayMetrics;
 import android.util.Log;
-import android.view.WindowManager;
-import javax.microedition.khronos.egl.EGLConfig;
-import javax.microedition.khronos.opengles.GL10;
+
 import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
 import java.nio.FloatBuffer;
 import java.nio.IntBuffer;
 import java.util.concurrent.CopyOnWriteArrayList;
 
+import javax.microedition.khronos.egl.EGLConfig;
+
 /**
  * The layer renderer implements the rendering logic for a layer view.
  */
-public class LayerRenderer {
+public class LayerRenderer implements Tabs.OnTabsChangedListener {
     private static final String LOGTAG = "GeckoLayerRenderer";
     private static final String PROFTAG = "GeckoLayerRendererProf";
 
@@ -90,17 +51,17 @@ public class LayerRenderer {
     private static final int FRAME_RATE_METER_HEIGHT = 32;
 
     private final LayerView mView;
-    private final SingleTileLayer mBackgroundLayer;
-    private final ScreenshotLayer mCheckerboardLayer;
     private final NinePatchTileLayer mShadowLayer;
     private TextLayer mFrameRateLayer;
     private final ScrollbarLayer mHorizScrollLayer;
     private final ScrollbarLayer mVertScrollLayer;
     private final FadeRunnable mFadeRunnable;
-    private final FloatBuffer mCoordBuffer;
+    private ByteBuffer mCoordByteBuffer;
+    private FloatBuffer mCoordBuffer;
     private RenderContext mLastPageContext;
     private int mMaxTextureSize;
     private int mBackgroundColor;
+    private int mOverscrollColor;
 
     private CopyOnWriteArrayList<Layer> mExtraLayers = new CopyOnWriteArrayList<Layer>();
 
@@ -138,6 +99,11 @@ public class LayerRenderer {
 
     // The shaders run on the GPU directly, the vertex shader is only applying the
     // matrix transform detailed above
+
+    // Note we flip the y-coordinate in the vertex shader from a
+    // coordinate system with (0,0) in the top left to one with (0,0) in
+    // the bottom left.
+
     public static final String DEFAULT_VERTEX_SHADER =
         "uniform mat4 uTMatrix;\n" +
         "attribute vec4 vPosition;\n" +
@@ -145,63 +111,35 @@ public class LayerRenderer {
         "varying vec2 vTexCoord;\n" +
         "void main() {\n" +
         "    gl_Position = uTMatrix * vPosition;\n" +
-        "    vTexCoord = aTexCoord;\n" +
+        "    vTexCoord.x = aTexCoord.x;\n" +
+        "    vTexCoord.y = 1.0 - aTexCoord.y;\n" +
         "}\n";
 
-    // Note we flip the y-coordinate in the fragment shader from a
-    // coordinate system with (0,0) in the top left to one with (0,0) in
-    // the bottom left.
+    // We use highp because the screenshot textures
+    // we use are large and we stretch them alot
+    // so we need all the precision we can get.
+    // Unfortunately, highp is not required by ES 2.0
+    // so on GPU's like Mali we end up getting mediump
     public static final String DEFAULT_FRAGMENT_SHADER =
-        "precision mediump float;\n" +
+        "precision highp float;\n" +
         "varying vec2 vTexCoord;\n" +
         "uniform sampler2D sTexture;\n" +
         "void main() {\n" +
-        "    gl_FragColor = texture2D(sTexture, vec2(vTexCoord.x, 1.0 - vTexCoord.y));\n" +
+        "    gl_FragColor = texture2D(sTexture, vTexCoord);\n" +
         "}\n";
 
-    public void setCheckerboardBitmap(Bitmap bitmap, RectF pageRect) {
-        mCheckerboardLayer.setBitmap(bitmap);
-        mCheckerboardLayer.beginTransaction();
-        try {
-            mCheckerboardLayer.setPosition(RectUtils.round(pageRect));
-            mCheckerboardLayer.invalidate();
-        } finally {
-            mCheckerboardLayer.endTransaction();
-        }
-    }
-
-    public void updateCheckerboardBitmap(Bitmap bitmap, float x, float y,
-                                         float width, float height,
-                                         RectF pageRect) {
-        mCheckerboardLayer.updateBitmap(bitmap, x, y, width, height);
-        mCheckerboardLayer.beginTransaction();
-        try {
-            mCheckerboardLayer.setPosition(RectUtils.round(pageRect));
-            mCheckerboardLayer.invalidate();
-        } finally {
-            mCheckerboardLayer.endTransaction();
-        }
-    }
-
-    public void resetCheckerboard() {
-        mCheckerboardLayer.reset();
-    }
-
     public LayerRenderer(LayerView view) {
         mView = view;
+        mOverscrollColor = view.getContext().getResources().getColor(R.color.background_normal);
 
-        LayerController controller = view.getController();
-
-        CairoImage backgroundImage = new BufferedCairoImage(controller.getBackgroundPattern());
-        mBackgroundLayer = new SingleTileLayer(true, backgroundImage);
-
-        mCheckerboardLayer = ScreenshotLayer.create();
-
-        CairoImage shadowImage = new BufferedCairoImage(controller.getShadowPattern());
+        CairoImage shadowImage = new BufferedCairoImage(view.getShadowPattern());
         mShadowLayer = new NinePatchTileLayer(shadowImage);
 
-        mHorizScrollLayer = ScrollbarLayer.create(this, false);
-        mVertScrollLayer = ScrollbarLayer.create(this, true);
+        Bitmap scrollbarImage = view.getScrollbarImage();
+        IntSize size = new IntSize(scrollbarImage.getWidth(), scrollbarImage.getHeight());
+        scrollbarImage = expandCanvasToPowerOfTwo(scrollbarImage, size);
+        mVertScrollLayer = new ScrollbarLayer(this, scrollbarImage, size, true);
+        mHorizScrollLayer = new ScrollbarLayer(this, diagonalFlip(scrollbarImage), new IntSize(size.height, size.width), false);
         mFadeRunnable = new FadeRunnable();
 
         mFrameTimings = new int[60];
@@ -209,12 +147,45 @@ public class LayerRenderer {
 
         // Initialize the FloatBuffer that will be used to store all vertices and texture
         // coordinates in draw() commands.
-        ByteBuffer byteBuffer = GeckoAppShell.allocateDirectBuffer(COORD_BUFFER_SIZE * 4);
-        byteBuffer.order(ByteOrder.nativeOrder());
-        mCoordBuffer = byteBuffer.asFloatBuffer();
+        mCoordByteBuffer = DirectBufferAllocator.allocate(COORD_BUFFER_SIZE * 4);
+        mCoordByteBuffer.order(ByteOrder.nativeOrder());
+        mCoordBuffer = mCoordByteBuffer.asFloatBuffer();
+
+        Tabs.registerOnTabsChangedListener(this);
     }
 
-    void onSurfaceCreated(GL10 gl, EGLConfig config) {
+    private Bitmap expandCanvasToPowerOfTwo(Bitmap image, IntSize size) {
+        IntSize potSize = size.nextPowerOfTwo();
+        if (size.equals(potSize)) {
+            return image;
+        }
+        // make the bitmap size a power-of-two in both dimensions if it's not already.
+        Bitmap potImage = Bitmap.createBitmap(potSize.width, potSize.height, image.getConfig());
+        new Canvas(potImage).drawBitmap(image, new Matrix(), null);
+        return potImage;
+    }
+
+    private Bitmap diagonalFlip(Bitmap image) {
+        Matrix rotation = new Matrix();
+        rotation.setValues(new float[] { 0, 1, 0, 1, 0, 0, 0, 0, 1 }); // transform (x,y) into (y,x)
+        Bitmap rotated = Bitmap.createBitmap(image, 0, 0, image.getWidth(), image.getHeight(), rotation, true);
+        return rotated;
+    }
+
+    public void destroy() {
+        DirectBufferAllocator.free(mCoordByteBuffer);
+        mCoordByteBuffer = null;
+        mCoordBuffer = null;
+        mShadowLayer.destroy();
+        mHorizScrollLayer.destroy();
+        mVertScrollLayer.destroy();
+        if (mFrameRateLayer != null) {
+            mFrameRateLayer.destroy();
+        }
+        Tabs.unregisterOnTabsChangedListener(this);
+    }
+
+    void onSurfaceCreated(EGLConfig config) {
         checkMonitoringEnabled();
         createDefaultProgram();
         activateDefaultProgram();
@@ -271,8 +242,6 @@ public class LayerRenderer {
     }
 
     public void addLayer(Layer layer) {
-        LayerController controller = mView.getController();
-
         synchronized (mExtraLayers) {
             if (mExtraLayers.contains(layer)) {
                 mExtraLayers.remove(layer);
@@ -283,8 +252,6 @@ public class LayerRenderer {
     }
 
     public void removeLayer(Layer layer) {
-        LayerController controller = mView.getController();
-
         synchronized (mExtraLayers) {
             mExtraLayers.remove(layer);
         }
@@ -311,21 +278,23 @@ public class LayerRenderer {
         return pixelBuffer;
     }
 
-    private RenderContext createScreenContext(ImmutableViewportMetrics metrics) {
+    private RenderContext createScreenContext(ImmutableViewportMetrics metrics, PointF offset) {
         RectF viewport = new RectF(0.0f, 0.0f, metrics.getWidth(), metrics.getHeight());
-        RectF pageRect = new RectF(metrics.getPageRect());
-        return createContext(viewport, pageRect, 1.0f);
+        RectF pageRect = metrics.getPageRect();
+
+        return createContext(viewport, pageRect, 1.0f, offset);
     }
 
-    private RenderContext createPageContext(ImmutableViewportMetrics metrics) {
-        Rect viewport = RectUtils.round(metrics.getViewport());
+    private RenderContext createPageContext(ImmutableViewportMetrics metrics, PointF offset) {
+        RectF viewport = metrics.getViewport();
         RectF pageRect = metrics.getPageRect();
         float zoomFactor = metrics.zoomFactor;
-        return createContext(new RectF(viewport), pageRect, zoomFactor);
+
+        return createContext(new RectF(RectUtils.round(viewport)), pageRect, zoomFactor, offset);
     }
 
-    private RenderContext createContext(RectF viewport, RectF pageRect, float zoomFactor) {
-        return new RenderContext(viewport, pageRect, zoomFactor, mPositionHandle, mTextureHandle,
+    private RenderContext createContext(RectF viewport, RectF pageRect, float zoomFactor, PointF offset) {
+        return new RenderContext(viewport, pageRect, zoomFactor, offset, mPositionHandle, mTextureHandle,
                                  mCoordBuffer);
     }
 
@@ -419,6 +388,7 @@ public class LayerRenderer {
             return !mStarted;
         }
 
+        @Override
         public void run() {
             long timeDelta = mRunAt - SystemClock.elapsedRealtime();
             if (timeDelta > 0) {
@@ -442,12 +412,28 @@ public class LayerRenderer {
         // Whether a layer was updated.
         private boolean mUpdated;
         private final Rect mPageRect;
+        private final Rect mAbsolutePageRect;
+        private final PointF mRenderOffset;
 
         public Frame(ImmutableViewportMetrics metrics) {
             mFrameMetrics = metrics;
-            mPageContext = createPageContext(metrics);
-            mScreenContext = createScreenContext(metrics);
-            mPageRect = getPageRect();
+
+            // Work out the offset due to margins
+            Layer rootLayer = mView.getLayerClient().getRoot();
+            mRenderOffset = mFrameMetrics.getMarginOffset();
+            float scaleDiff = mFrameMetrics.zoomFactor / rootLayer.getResolution();
+            mRenderOffset.set(mRenderOffset.x * scaleDiff,
+                              mRenderOffset.y * scaleDiff);
+
+            mPageContext = createPageContext(metrics, mRenderOffset);
+            mScreenContext = createScreenContext(metrics, mRenderOffset);
+
+            RectF pageRect = mFrameMetrics.getPageRect();
+            mAbsolutePageRect = RectUtils.round(pageRect);
+
+            PointF origin = mFrameMetrics.getOrigin();
+            pageRect.offset(-origin.x, -origin.y);
+            mPageRect = RectUtils.round(pageRect);
         }
 
         private void setScissorRect() {
@@ -465,15 +451,11 @@ public class LayerRenderer {
             int right = Math.min(screenSize.width, rect.right);
             int bottom = Math.min(screenSize.height, rect.bottom);
 
-            return new Rect(left, screenSize.height - bottom, right,
-                            (screenSize.height - bottom) + (bottom - top));
-        }
+            Rect scissorRect = new Rect(left, screenSize.height - bottom, right,
+                                        (screenSize.height - bottom) + (bottom - top));
+            scissorRect.offset(Math.round(-mRenderOffset.x), Math.round(-mRenderOffset.y));
 
-        private Rect getPageRect() {
-            Point origin = PointUtils.round(mFrameMetrics.getOrigin());
-            Rect pageRect = RectUtils.round(mFrameMetrics.getPageRect());
-            pageRect.offset(-origin.x, -origin.y);
-            return pageRect;
+            return scissorRect;
         }
 
         /** This function is invoked via JNI; be careful when modifying signature. */
@@ -485,11 +467,11 @@ public class LayerRenderer {
 
             mUpdated = true;
 
-            Layer rootLayer = mView.getController().getRoot();
+            Layer rootLayer = mView.getLayerClient().getRoot();
 
-            if (!mPageContext.fuzzyEquals(mLastPageContext)) {
-                // the viewport or page changed, so show the scrollbars again
-                // as per UX decision
+            if (!mPageContext.fuzzyEquals(mLastPageContext) && !mView.isFullScreen()) {
+                // The viewport or page changed, so show the scrollbars again
+                // as per UX decision. Don't do this if we're in full-screen mode though.
                 mVertScrollLayer.unfade();
                 mHorizScrollLayer.unfade();
                 mFadeRunnable.scheduleStartFade(ScrollbarLayer.FADE_DELAY);
@@ -503,9 +485,7 @@ public class LayerRenderer {
 
             /* Update layers. */
             if (rootLayer != null) mUpdated &= rootLayer.update(mPageContext);  // called on compositor thread
-            mUpdated &= mBackgroundLayer.update(mScreenContext);    // called on compositor thread
             mUpdated &= mShadowLayer.update(mPageContext);  // called on compositor thread
-            mUpdated &= mCheckerboardLayer.update(mPageContext);   // called on compositor thread
             if (mFrameRateLayer != null) mUpdated &= mFrameRateLayer.update(mScreenContext); // called on compositor thread
             mUpdated &= mVertScrollLayer.update(mPageContext);  // called on compositor thread
             mUpdated &= mHorizScrollLayer.update(mPageContext); // called on compositor thread
@@ -553,52 +533,42 @@ public class LayerRenderer {
             return mask;
         }
 
+        private void clear(int color) {
+            GLES20.glClearColor(((color >> 16) & 0xFF) / 255.0f,
+                                ((color >> 8) & 0xFF) / 255.0f,
+                                (color & 0xFF) / 255.0f,
+                                0.0f);
+            // The bits set here need to match up with those used
+            // in gfx/layers/opengl/LayerManagerOGL.cpp.
+            GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT |
+                           GLES20.GL_DEPTH_BUFFER_BIT);
+        }
+
         /** This function is invoked via JNI; be careful when modifying signature. */
         public void drawBackground() {
             GLES20.glDisable(GLES20.GL_SCISSOR_TEST);
 
-            /* Update background color. */
-            mBackgroundColor = mView.getController().getCheckerboardColor();
+            // Draw the overscroll background area as a solid color
+            clear(mOverscrollColor);
 
-            /* Clear to the page background colour. The bits set here need to
-             * match up with those used in gfx/layers/opengl/LayerManagerOGL.cpp.
-             */
-            GLES20.glClearColor(((mBackgroundColor>>16)&0xFF) / 255.0f,
-                                ((mBackgroundColor>>8)&0xFF) / 255.0f,
-                                (mBackgroundColor&0xFF) / 255.0f,
-                                0.0f);
-            GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT |
-                           GLES20.GL_DEPTH_BUFFER_BIT);
+            // Update background color.
+            mBackgroundColor = mView.getBackgroundColor();
 
-            /* Draw the background. */
-            mBackgroundLayer.setMask(mPageRect);
-            mBackgroundLayer.draw(mScreenContext);
+            // Clear the page area to the page background colour.
+            setScissorRect();
+            clear(mBackgroundColor);
+            GLES20.glDisable(GLES20.GL_SCISSOR_TEST);
 
-            /* Draw the drop shadow, if we need to. */
-            RectF untransformedPageRect = new RectF(0.0f, 0.0f, mPageRect.width(),
-                                                    mPageRect.height());
-            if (!untransformedPageRect.contains(mView.getController().getViewport()))
+            // Draw the drop shadow, if we need to.
+            RectF offsetAbsPageRect = new RectF(mAbsolutePageRect);
+            offsetAbsPageRect.offset(mRenderOffset.x, mRenderOffset.y);
+            if (!offsetAbsPageRect.contains(mFrameMetrics.getViewport()))
                 mShadowLayer.draw(mPageContext);
-
-            /* Draw the 'checkerboard'. We use gfx.show_checkerboard_pattern to
-             * determine whether to draw the screenshot layer.
-             */
-            if (mView.getController().checkerboardShouldShowChecks()) {
-                /* Find the area the root layer will render into, to mask the checkerboard layer */
-                Rect rootMask = getMaskForLayer(mView.getController().getRoot());
-                mCheckerboardLayer.setMask(rootMask);
-
-                /* Scissor around the page-rect, in case the page has shrunk
-                 * since the screenshot layer was last updated.
-                 */
-                setScissorRect(); // Calls glEnable(GL_SCISSOR_TEST))
-                mCheckerboardLayer.draw(mPageContext);
-            }
         }
 
         // Draws the layer the client added to us.
         void drawRootLayer() {
-            Layer rootLayer = mView.getController().getRoot();
+            Layer rootLayer = mView.getLayerClient().getRoot();
             if (rootLayer == null) {
                 return;
             }
@@ -611,13 +581,7 @@ public class LayerRenderer {
             /* Draw any extra layers that were added (likely plugins) */
             if (mExtraLayers.size() > 0) {
                 for (Layer layer : mExtraLayers) {
-                    if (!layer.usesDefaultProgram())
-                        deactivateDefaultProgram();
-
                     layer.draw(mPageContext);
-
-                    if (!layer.usesDefaultProgram())
-                        activateDefaultProgram();
                 }
             }
 
@@ -630,34 +594,16 @@ public class LayerRenderer {
                 mHorizScrollLayer.draw(mPageContext);
 
             /* Measure how much of the screen is checkerboarding */
-            Layer rootLayer = mView.getController().getRoot();
+            Layer rootLayer = mView.getLayerClient().getRoot();
             if ((rootLayer != null) &&
                 (mProfileRender || PanningPerfAPI.isRecordingCheckerboard())) {
-                // Find out how much of the viewport area is valid
-                Rect viewport = RectUtils.round(mPageContext.viewport);
-                Region validRegion = rootLayer.getValidRegion(mPageContext);
-                validRegion.op(viewport, Region.Op.INTERSECT);
-
-                float checkerboard = 0.0f;
-                if (!(validRegion.isRect() && validRegion.getBounds().equals(viewport))) {
-                    int screenArea = viewport.width() * viewport.height();
-                    validRegion.op(viewport, Region.Op.REVERSE_DIFFERENCE);
-
-                    // XXX The assumption here is that a Region never has overlapping
-                    //     rects. This is true, as evidenced by reading the SkRegion
-                    //     source, but is not mentioned in the Android documentation,
-                    //     and so is liable to change.
-                    //     If it does change, this code will need to be reevaluated.
-                    Rect r = new Rect();
-                    int checkerboardArea = 0;
-                    for (RegionIterator i = new RegionIterator(validRegion); i.next(r);) {
-                        checkerboardArea += r.width() * r.height();
-                    }
-
-                    checkerboard = checkerboardArea / (float)screenArea;
-                }
+                // Calculate the incompletely rendered area of the page
+                float checkerboard =  1.0f - GeckoAppShell.computeRenderIntegrity();
 
                 PanningPerfAPI.recordCheckerboard(checkerboard);
+                if (checkerboard < 0.0f || checkerboard > 1.0f) {
+                    Log.e(LOGTAG, "Checkerboard value out of bounds: " + checkerboard);
+                }
 
                 mCompleteFramesRendered += 1.0f - checkerboard;
                 mFramesRendered ++;
@@ -698,15 +644,30 @@ public class LayerRenderer {
                 }
             }
 
-            // Remove white screen once we've painted
+            // Remove background color once we've painted. GeckoLayerClient is
+            // responsible for setting this flag before current document is
+            // composited.
             if (mView.getPaintState() == LayerView.PAINT_BEFORE_FIRST) {
-                GeckoAppShell.getMainHandler().postAtFrontOfQueue(new Runnable() {
+                mView.post(new Runnable() {
+                    @Override
                     public void run() {
-                        mView.setBackgroundColor(android.graphics.Color.TRANSPARENT);
+                        mView.getChildAt(0).setBackgroundColor(Color.TRANSPARENT);
                     }
                 });
                 mView.setPaintState(LayerView.PAINT_AFTER_FIRST);
             }
         }
     }
+
+    @Override
+    public void onTabChanged(final Tab tab, Tabs.TabEvents msg, Object data) {
+        // Sets the background of the newly selected tab. This background color
+        // gets cleared in endDrawing(). This function runs on the UI thread,
+        // but other code that touches the paint state is run on the compositor
+        // thread, so this may need to be changed if any problems appear.
+        if (msg == Tabs.TabEvents.SELECTED) {
+            mView.getChildAt(0).setBackgroundColor(tab.getBackgroundColor());
+            mView.setPaintState(LayerView.PAINT_START);
+        }
+    }
 }
