@@ -5,17 +5,15 @@
 
 package org.mozilla.gecko.gfx;
 
-import org.mozilla.gecko.OnInterceptTouchListener;
 import org.mozilla.gecko.Tab;
 import org.mozilla.gecko.Tabs;
-import org.mozilla.gecko.ui.PanZoomController;
-import org.mozilla.gecko.ui.SimpleScaleGestureDetector;
 
 import android.content.Context;
 import android.os.SystemClock;
 import android.util.Log;
 import android.view.GestureDetector;
 import android.view.MotionEvent;
+import android.view.View;
 
 import java.util.LinkedList;
 import java.util.Queue;
@@ -50,26 +48,23 @@ import java.util.Queue;
  * to respond to user actions a timely manner regardless of default-prevention,
  * and fix issues like bug 749384.
  */
-public final class TouchEventHandler implements Tabs.OnTabsChangedListener {
+final class TouchEventHandler implements Tabs.OnTabsChangedListener {
     private static final String LOGTAG = "GeckoTouchEventHandler";
 
     // The time limit for listeners to respond with preventDefault on touchevents
     // before we begin panning the page
     private final int EVENT_LISTENER_TIMEOUT = 200;
 
-    private final LayerView mView;
+    private final View mView;
     private final GestureDetector mGestureDetector;
     private final SimpleScaleGestureDetector mScaleGestureDetector;
-    private final PanZoomController mPanZoomController;
+    private final JavaPanZoomController mPanZoomController;
 
     // the queue of events that we are holding on to while waiting for a preventDefault
     // notification
     private final Queue<MotionEvent> mEventQueue;
     private final ListenerTimeoutProcessor mListenerTimeoutProcessor;
 
-    // the listener we use to notify gecko of touch events
-    private OnInterceptTouchListener mOnTouchListener;
-
     // whether or not we should wait for touch listeners to respond (this state is
     // per-tab and is updated when we switch tabs).
     private boolean mWaitForTouchListeners;
@@ -125,11 +120,11 @@ public final class TouchEventHandler implements Tabs.OnTabsChangedListener {
     //   processed. (n is the absolute value of the balance.)
     private int mProcessingBalance;
 
-    TouchEventHandler(Context context, LayerView view, GeckoLayerClient layerClient) {
+    TouchEventHandler(Context context, View view, JavaPanZoomController panZoomController) {
         mView = view;
 
         mEventQueue = new LinkedList<MotionEvent>();
-        mPanZoomController = layerClient.getPanZoomController();
+        mPanZoomController = panZoomController;
         mGestureDetector = new GestureDetector(context, mPanZoomController);
         mScaleGestureDetector = new SimpleScaleGestureDetector(mPanZoomController);
         mListenerTimeoutProcessor = new ListenerTimeoutProcessor();
@@ -140,25 +135,12 @@ public final class TouchEventHandler implements Tabs.OnTabsChangedListener {
         Tabs.registerOnTabsChangedListener(this);
     }
 
+    public void destroy() {
+        Tabs.unregisterOnTabsChangedListener(this);
+    }
+
     /* This function MUST be called on the UI thread */
     public boolean handleEvent(MotionEvent event) {
-        // if we don't have gecko listeners, just dispatch the event
-        // and be done with it, no extra work needed.
-        if (mOnTouchListener == null) {
-            dispatchEvent(event);
-            return true;
-        }
-
-        if (mOnTouchListener.onInterceptTouchEvent(mView, event)) {
-            return true;
-        }
-
-        // if this is a hover event just notify gecko, we don't have any interest in the java layer.
-        if (isHoverEvent(event)) {
-            mOnTouchListener.onTouch(mView, event);
-            return true;
-        }
-
         if (isDownEvent(event)) {
             // this is the start of a new block of events! whee!
             mHoldInQueue = mWaitForTouchListeners;
@@ -201,10 +183,7 @@ public final class TouchEventHandler implements Tabs.OnTabsChangedListener {
             mPanZoomController.preventedTouchFinished();
         }
 
-        // notify gecko of the event
-        mOnTouchListener.onTouch(mView, event);
-
-        return true;
+        return false;
     }
 
     /**
@@ -232,16 +211,6 @@ public final class TouchEventHandler implements Tabs.OnTabsChangedListener {
         mWaitForTouchListeners = aValue;
     }
 
-    /* This function MUST be called on the UI thread. */
-    public void setOnTouchListener(OnInterceptTouchListener onTouchListener) {
-        mOnTouchListener = onTouchListener;
-    }
-
-    private boolean isHoverEvent(MotionEvent event) {
-        int action = (event.getAction() & MotionEvent.ACTION_MASK);
-        return (action == MotionEvent.ACTION_HOVER_ENTER || action == MotionEvent.ACTION_HOVER_MOVE || action == MotionEvent.ACTION_HOVER_EXIT);
-    }
-
     private boolean isDownEvent(MotionEvent event) {
         int action = (event.getAction() & MotionEvent.ACTION_MASK);
         return (action == MotionEvent.ACTION_DOWN || action == MotionEvent.ACTION_POINTER_DOWN);
@@ -263,7 +232,7 @@ public final class TouchEventHandler implements Tabs.OnTabsChangedListener {
         if (mScaleGestureDetector.isInProgress()) {
             return;
         }
-        mPanZoomController.onTouchEvent(event);
+        mPanZoomController.handleEvent(event);
     }
 
     /**
@@ -329,6 +298,7 @@ public final class TouchEventHandler implements Tabs.OnTabsChangedListener {
 
     private class ListenerTimeoutProcessor implements Runnable {
         /* This MUST be run on the UI thread */
+        @Override
         public void run() {
             if (mProcessingBalance < 0) {
                 // gecko already responded with default-prevented notification, and so
@@ -343,6 +313,7 @@ public final class TouchEventHandler implements Tabs.OnTabsChangedListener {
 
     // Tabs.OnTabsChangedListener implementation
 
+    @Override
     public void onTabChanged(Tab tab, Tabs.TabEvents msg, Object data) {
         if ((Tabs.getInstance().isSelectedTab(tab) && msg == Tabs.TabEvents.STOP) || msg == Tabs.TabEvents.SELECTED) {
             mWaitForTouchListeners = tab.getHasTouchListeners();
