@@ -1,6 +1,6 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this file,
- * You can obtain one at http://mozilla.org/MPL/2.0/. */
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 package org.mozilla.gecko.sync;
 
@@ -8,7 +8,6 @@ import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 
 import org.json.simple.JSONObject;
-import org.json.simple.parser.JSONParser;
 import org.json.simple.parser.ParseException;
 import org.mozilla.apache.commons.codec.binary.Base64;
 import org.mozilla.gecko.sync.crypto.CryptoException;
@@ -131,7 +130,7 @@ public class CryptoRecord extends Record {
   public static CryptoRecord fromJSONRecord(String jsonRecord)
       throws ParseException, NonObjectJSONException, IOException {
     byte[] bytes = jsonRecord.getBytes("UTF-8");
-    ExtendedJSONObject object = CryptoRecord.parseUTF8AsJSONObject(bytes);
+    ExtendedJSONObject object = ExtendedJSONObject.parseUTF8AsJSONObject(bytes);
 
     return CryptoRecord.fromJSONRecord(object);
   }
@@ -141,7 +140,10 @@ public class CryptoRecord extends Record {
       throws IOException, ParseException, NonObjectJSONException {
     String id                  = (String) jsonRecord.get(KEY_ID);
     String collection          = (String) jsonRecord.get(KEY_COLLECTION);
-    ExtendedJSONObject payload = jsonRecord.getJSONObject(KEY_PAYLOAD);
+    String jsonEncodedPayload  = (String) jsonRecord.get(KEY_PAYLOAD);
+
+    ExtendedJSONObject payload = ExtendedJSONObject.parseJSONObject(jsonEncodedPayload);
+
     CryptoRecord record = new CryptoRecord(payload);
     record.guid         = id;
     record.collection   = collection;
@@ -164,16 +166,6 @@ public class CryptoRecord extends Record {
     this.keyBundle = bundle;
   }
 
-  private static ExtendedJSONObject parseUTF8AsJSONObject(byte[] in)
-      throws UnsupportedEncodingException, ParseException, NonObjectJSONException {
-    Object obj = new JSONParser().parse(new String(in, "UTF-8"));
-    if (obj instanceof JSONObject) {
-      return new ExtendedJSONObject((JSONObject) obj);
-    } else {
-      throw new NonObjectJSONException(obj);
-    }
-  }
-
   public CryptoRecord decrypt() throws CryptoException, IOException, ParseException,
                        NonObjectJSONException {
     if (keyBundle == null) {
@@ -190,7 +182,7 @@ public class CryptoRecord extends Record {
     // There's no difference between handling the crypto/keys object and
     // anything else; we just get this.keyBundle from a different source.
     byte[] cleartext = decryptPayload(payload, keyBundle);
-    payload = CryptoRecord.parseUTF8AsJSONObject(cleartext);
+    payload = ExtendedJSONObject.parseUTF8AsJSONObject(cleartext);
     return this;
   }
 
