@@ -1,20 +1,22 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this file,
- * You can obtain one at http://mozilla.org/MPL/2.0/. */
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 package org.mozilla.gecko.sync.net;
 
+import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.io.Reader;
 import java.util.Scanner;
 
 import org.json.simple.parser.ParseException;
+import org.mozilla.gecko.background.common.log.Logger;
 import org.mozilla.gecko.sync.ExtendedJSONObject;
 import org.mozilla.gecko.sync.NonObjectJSONException;
 import org.mozilla.gecko.sync.Utils;
 
-import android.util.Log;
 import ch.boye.httpclientandroidlib.Header;
 import ch.boye.httpclientandroidlib.HttpEntity;
 import ch.boye.httpclientandroidlib.HttpResponse;
@@ -58,34 +60,6 @@ public class SyncResponse {
     return body;
   }
 
-  /**
-   * Return the body as an Object.
-   *
-   * @return null if there is no body, or an Object if it successfully parses.
-   *         The return value will be an ExtendedJSONObject if it's a JSON object.
-   * @throws IllegalStateException
-   * @throws IOException
-   * @throws ParseException
-   */
-  public Object jsonBody() throws IllegalStateException, IOException,
-                          ParseException {
-    if (body != null) {
-      // Do it from the cached String.
-      return ExtendedJSONObject.parse(body);
-    }
-
-    HttpEntity entity = this.response.getEntity();
-    if (entity == null) {
-      return null;
-    }
-    InputStream content = entity.getContent();
-    try {
-      return ExtendedJSONObject.parse(content);
-    } finally {
-      content.close();
-    }
-  }
-
   /**
    * Return the body as a <b>non-null</b> <code>ExtendedJSONObject</code>.
    *
@@ -99,11 +73,23 @@ public class SyncResponse {
   public ExtendedJSONObject jsonObjectBody() throws IllegalStateException,
                                             IOException, ParseException,
                                             NonObjectJSONException {
-    Object body = this.jsonBody();
-    if (body instanceof ExtendedJSONObject) {
-      return (ExtendedJSONObject) body;
+    if (body != null) {
+      // Do it from the cached String.
+      return ExtendedJSONObject.parseJSONObject(body);
+    }
+
+    HttpEntity entity = this.response.getEntity();
+    if (entity == null) {
+      throw new IOException("no entity");
+    }
+
+    InputStream content = entity.getContent();
+    try {
+      Reader in = new BufferedReader(new InputStreamReader(content, "UTF-8"));
+      return ExtendedJSONObject.parseJSONObject(in);
+    } finally {
+      content.close();
     }
-    throw new NonObjectJSONException(body);
   }
 
   private boolean hasHeader(String h) {
@@ -120,7 +106,7 @@ public class SyncResponse {
       Header header = this.response.getFirstHeader(h);
       String value  = header.getValue();
       if (missingHeader(value)) {
-        Log.w(LOG_TAG, h + " header present but empty.");
+        Logger.warn(LOG_TAG, h + " header present but empty.");
         return -1;
       }
       return Integer.parseInt(value, 10);
@@ -139,7 +125,7 @@ public class SyncResponse {
     Header header = this.response.getFirstHeader(HEADER_RETRY_AFTER);
     String retryAfter = header.getValue();
     if (missingHeader(retryAfter)) {
-      Log.w(LOG_TAG, "Retry-After header present but empty.");
+      Logger.warn(LOG_TAG, "Retry-After header present but empty.");
       return -1;
     }
 
@@ -154,7 +140,7 @@ public class SyncResponse {
       final long now  = System.currentTimeMillis();
       return (int)((then - now) / 1000);     // Convert milliseconds to seconds.
     } catch (DateParseException e) {
-      Log.w(LOG_TAG, "Retry-After header neither integer nor date: " + retryAfter);
+      Logger.warn(LOG_TAG, "Retry-After header neither integer nor date: " + retryAfter);
       return -1;
     }
   }
