@@ -5,7 +5,7 @@
 package org.mozilla.gecko.sync.stage;
 
 import org.mozilla.gecko.sync.CryptoRecord;
-import org.mozilla.gecko.sync.GlobalSession;
+import org.mozilla.gecko.sync.delegates.ClientsDataDelegate;
 import org.mozilla.gecko.sync.repositories.RecordFactory;
 import org.mozilla.gecko.sync.repositories.Repository;
 import org.mozilla.gecko.sync.repositories.android.FennecTabsRepository;
@@ -16,10 +16,6 @@ import org.mozilla.gecko.sync.repositories.domain.VersionConstants;
 public class FennecTabsServerSyncStage extends ServerSyncStage {
   private static final String COLLECTION = "tabs";
 
-  public FennecTabsServerSyncStage(GlobalSession session) {
-    super(session);
-  }
-
   public class FennecTabsRecordFactory extends RecordFactory {
     @Override
     public Record createRecord(Record record) {
@@ -46,7 +42,8 @@ public class FennecTabsServerSyncStage extends ServerSyncStage {
 
   @Override
   protected Repository getLocalRepository() {
-    return new FennecTabsRepository();
+    final ClientsDataDelegate clientsDelegate = session.getClientsDelegate();
+    return new FennecTabsRepository(clientsDelegate.getClientName(), clientsDelegate.getAccountGUID());
   }
 
   @Override
