@@ -13,13 +13,12 @@ import java.util.concurrent.atomic.AtomicInteger;
 
 import org.json.simple.JSONArray;
 import org.json.simple.JSONObject;
+import org.mozilla.gecko.background.common.log.Logger;
 import org.mozilla.gecko.sync.CommandProcessor;
 import org.mozilla.gecko.sync.CommandProcessor.Command;
 import org.mozilla.gecko.sync.CryptoRecord;
 import org.mozilla.gecko.sync.ExtendedJSONObject;
-import org.mozilla.gecko.sync.GlobalSession;
 import org.mozilla.gecko.sync.HTTPFailureException;
-import org.mozilla.gecko.sync.Logger;
 import org.mozilla.gecko.sync.NoCollectionKeysSetException;
 import org.mozilla.gecko.sync.Utils;
 import org.mozilla.gecko.sync.crypto.CryptoException;
@@ -40,7 +39,7 @@ import org.mozilla.gecko.sync.repositories.domain.VersionConstants;
 
 import ch.boye.httpclientandroidlib.HttpStatus;
 
-public class SyncClientsEngineStage implements GlobalSyncStage {
+public class SyncClientsEngineStage extends AbstractSessionManagingSyncStage {
   private static final String LOG_TAG = "SyncClientsEngineStage";
 
   public static final String COLLECTION_NAME       = "clients";
@@ -48,7 +47,6 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
   public static final int CLIENTS_TTL_REFRESH      = 604800000;   // 7 days in milliseconds.
   public static final int MAX_UPLOAD_FAILURE_COUNT = 5;
 
-  protected final GlobalSession session;
   protected final ClientRecordFactory factory = new ClientRecordFactory();
   protected ClientUploadDelegate clientUploadDelegate;
   protected ClientDownloadDelegate clientDownloadDelegate;
@@ -61,13 +59,6 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
   protected final AtomicInteger uploadAttemptsCount = new AtomicInteger();
   protected final List<ClientRecord> toUpload = new ArrayList<ClientRecord>();
 
-  public SyncClientsEngineStage(GlobalSession session) {
-    if (session == null) {
-      throw new IllegalArgumentException("session must not be null.");
-    }
-    this.session = session;
-  }
-
   protected int getClientsCount() {
     return getClientsDatabaseAccessor().clientsCount();
   }
@@ -342,7 +333,7 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
   }
 
   @Override
-  public void resetLocal() {
+  protected void resetLocal() {
     // Clear timestamps and local data.
     session.config.persistServerClientRecordTimestamp(0L);   // TODO: roll these into one.
     session.config.persistServerClientsTimestamp(0L);
@@ -356,7 +347,7 @@ public class SyncClientsEngineStage implements GlobalSyncStage {
   }
 
   @Override
-  public void wipeLocal() throws Exception {
+  protected void wipeLocal() throws Exception {
     // Nothing more to do.
     this.resetLocal();
   }
