@@ -23,6 +23,73 @@ XPCOMUtils.defineLazyGetter(window, "gChromeWin", function()
 
 XPCOMUtils.defineLazyGetter(window, "SelectHelper", function() gChromeWin.SelectHelper);
 
+var ContextMenus = {
+  target: null,
+
+  init: function() {
+    document.addEventListener("contextmenu", this, false);
+  },
+
+  handleEvent: function(event) {
+    // store the target of context menu events so that we know which app to act on
+    this.target = event.target;
+    while (!this.target.hasAttribute("contextmenu")) {
+      this.target = this.target.parentNode;
+    }
+
+    if (!this.target) {
+      document.getElementById("contextmenu-enable").setAttribute("hidden", "true");
+      document.getElementById("contextmenu-disable").setAttribute("hidden", "true");
+      document.getElementById("contextmenu-uninstall").setAttribute("hidden", "true");
+      document.getElementById("contextmenu-default").setAttribute("hidden", "true");
+      return;
+    }
+
+    let addon = this.target.addon;
+    if (addon.scope == AddonManager.SCOPE_APPLICATION) {
+      document.getElementById("contextmenu-uninstall").setAttribute("hidden", "true");
+    } else {
+      document.getElementById("contextmenu-uninstall").removeAttribute("hidden");
+    }
+
+    let enabled = this.target.getAttribute("isDisabled") != "true";
+    if (enabled) {
+      document.getElementById("contextmenu-enable").setAttribute("hidden", "true");
+      document.getElementById("contextmenu-disable").removeAttribute("hidden");
+    } else {
+      document.getElementById("contextmenu-enable").removeAttribute("hidden");
+      document.getElementById("contextmenu-disable").setAttribute("hidden", "true");
+    }
+
+    // Only show the "Set as Default" menuitem for enabled non-default search engines.
+    if (addon.type == "search" && enabled && addon.id != Services.search.defaultEngine.name) {
+      document.getElementById("contextmenu-default").removeAttribute("hidden");
+    } else {
+      document.getElementById("contextmenu-default").setAttribute("hidden", "true");
+    }
+  },
+
+  enable: function(event) {
+    Addons.setEnabled(true, this.target.addon);
+    this.target = null;
+  },
+  
+  disable: function (event) {
+    Addons.setEnabled(false, this.target.addon);
+    this.target = null;
+  },
+  
+  uninstall: function (event) {
+    Addons.uninstall(this.target.addon);
+    this.target = null;
+  },
+
+  setDefaultSearch: function(event) {
+    Addons.setDefaultSearch(this.target.addon);
+    this.target = null;
+  }
+}
+
 function init() {
   window.addEventListener("popstate", onPopState, false);
   Services.obs.addObserver(Addons, "browser-search-engine-modified", false);
@@ -31,6 +98,7 @@ function init() {
   AddonManager.addAddonListener(Addons);
   Addons.getAddons();
   showList();
+  ContextMenus.init();
 }
 
 function uninit() {
@@ -76,15 +144,16 @@ var Addons = {
   _createItem: function _createItem(aAddon) {
     let outer = document.createElement("div");
     outer.setAttribute("addonID", aAddon.id);
-    outer.className = "addon-item";
+    outer.className = "addon-item list-item";
     outer.setAttribute("role", "button");
+    outer.setAttribute("contextmenu", "addonmenu");
     outer.addEventListener("click", function() {
       this.showDetails(outer);
       history.pushState({ id: aAddon.id }, document.title);
     }.bind(this), true);
 
     let img = document.createElement("img");
-    img.className = "favicon";
+    img.className = "icon";
     img.setAttribute("src", aAddon.iconURL);
     outer.appendChild(img);
 
@@ -154,17 +223,17 @@ var Addons = {
 
   _getElementForAddon: function(aKey) {
     let list = document.getElementById("addons-list");
-    let element = list.querySelector("div[addonID='" + aKey + "']");
+    let element = list.querySelector("div[addonID=" + aKey.quote() + "]");
     return element;
   },
 
   getAddons: function getAddons() {
-    // Clear all content before filling the addons
-    let list = document.getElementById("addons-list");
-    list.innerHTML = "";
-
     let self = this;
     AddonManager.getAddonsByTypes(["extension", "theme", "locale"], function(aAddons) {
+      // Clear all content before filling the addons
+      let list = document.getElementById("addons-list");
+      list.innerHTML = "";
+
       for (let i=0; i<aAddons.length; i++) {
         let item = self._createItemForAddon(aAddons[i]);
         list.appendChild(item);
@@ -187,6 +256,7 @@ var Addons = {
         addon.version = "";
         addon.description = engine.description || defaultDescription;
         addon.iconURL = engine.iconURI ? engine.iconURI.spec : "";
+        addon.optionsURL = "";
         addon.appDisabled = false;
         addon.scope = isDefault(engine) ? AddonManager.SCOPE_APPLICATION : AddonManager.SCOPE_PROFILE;
         addon.engine = engine;
@@ -195,6 +265,7 @@ var Addons = {
         item.setAttribute("isDisabled", engine.hidden);
         item.setAttribute("updateable", "false");
         item.setAttribute("opType", "");
+        item.setAttribute("optionsURL", "");
         item.addon = addon;
         list.appendChild(item);
       }
@@ -234,18 +305,20 @@ var Addons = {
     detailItem.setAttribute("optionsURL", aListItem.getAttribute("optionsURL"));
     let addon = detailItem.addon = aListItem.addon;
 
-    let favicon = document.querySelector("#addons-details > .addon-item .favicon");
+    let favicon = document.querySelector("#addons-details > .addon-item .icon");
     if (addon.iconURL)
       favicon.setAttribute("src", addon.iconURL);
     else
       favicon.removeAttribute("src");
 
-    document.querySelector("#addons-details > .addon-item .title").textContent = addon.name;
-    document.querySelector("#addons-details > .addon-item .version").textContent = addon.version;
-    document.querySelector("#addons-details > .addon-item .tag").textContent = gStringBundle.GetStringFromName("addonType." + addon.type);
-    document.querySelector("#addons-details > .addon-item .description-full").textContent = addon.description;
+    detailItem.querySelector(".title").textContent = addon.name;
+    detailItem.querySelector(".version").textContent = addon.version;
+    detailItem.querySelector(".tag").textContent = gStringBundle.GetStringFromName("addonType." + addon.type);
+    detailItem.querySelector(".description-full").textContent = addon.description;
+    detailItem.querySelector(".status-uninstalled").textContent =
+      gStringBundle.formatStringFromName("addonStatus.uninstalled", [addon.name], 1);
 
-    let enableBtn = document.getElementById("uninstall-btn");
+    let enableBtn = document.getElementById("enable-btn");
     if (addon.appDisabled)
       enableBtn.setAttribute("disabled", "true");
     else
@@ -257,6 +330,18 @@ var Addons = {
     else
       uninstallBtn.removeAttribute("disabled");
 
+    let defaultButton = document.getElementById("default-btn");
+    if (addon.type == "search") {
+      if (addon.id == Services.search.defaultEngine.name)
+        defaultButton.setAttribute("disabled", "true");
+      else
+        defaultButton.removeAttribute("disabled");
+
+      defaultButton.removeAttribute("hidden");
+    } else {
+      defaultButton.setAttribute("hidden", "true");
+    }
+
     let box = document.querySelector("#addons-details > .addon-item .options-box");
     box.innerHTML = "";
 
@@ -269,28 +354,30 @@ var Addons = {
       if (xhr.responseXML) {
         // Only allow <setting> for now
         let settings = xhr.responseXML.querySelectorAll(":root > setting");
-        for (let i = 0; i < settings.length; i++) {
-          var setting = settings[i];
-          var desc = stripTextNodes(setting).trim();
-          if (!setting.hasAttribute("desc"))
-            setting.setAttribute("desc", desc);
-          box.appendChild(setting);
+        if (settings.length > 0) {
+          for (let i = 0; i < settings.length; i++) {
+            var setting = settings[i];
+            var desc = stripTextNodes(setting).trim();
+            if (!setting.hasAttribute("desc"))
+              setting.setAttribute("desc", desc);
+            box.appendChild(setting);
+          }
+          // Send an event so add-ons can prepopulate any non-preference based
+          // settings
+          let event = document.createEvent("Events");
+          event.initEvent("AddonOptionsLoad", true, false);
+          window.dispatchEvent(event);
+  
+          // Also send a notification to match the behavior of desktop Firefox
+          let id = aListItem.getAttribute("addonID");
+          Services.obs.notifyObservers(document, AddonManager.OPTIONS_NOTIFICATION_DISPLAYED, id);
+        } else {
+          // No options, so hide the header and reset the list item
+          detailItem.setAttribute("optionsURL", "");
+          aListItem.setAttribute("optionsURL", "");
         }
-/*
-        // Send an event so add-ons can prepopulate any non-preference based
-        // settings
-        let event = document.createEvent("Events");
-        event.initEvent("AddonOptionsLoad", true, false);
-        this.dispatchEvent(event);
-
-        // Also send a notification to match the behavior of desktop Firefox
-        let id = this.id.substring(17); // length of |urn:mozilla:item:|
-        Services.obs.notifyObservers(document, "addon-options-displayed", id);
-*/
       }
-    } catch (e) {
-      Cu.reportError(e)
-    }
+    } catch (e) { }
 
     let list = document.querySelector("#addons-list");
     list.style.display = "none";
@@ -317,7 +404,8 @@ var Addons = {
         let item = list.firstElementChild;
         while (item) {
           if (item.addon && (item.addon.type == "theme") && (item.addon.isActive)) {
-            this.setEnabled(false, item);
+            item.addon.userDisabled = true;
+            item.setAttribute("isDisabled", true);
             break;
           }
           item = item.nextSibling;
@@ -364,11 +452,11 @@ var Addons = {
     this.setEnabled(false);
   },
 
-  uninstall: function uninstall() {
+  uninstall: function uninstall(aAddon) {
     let list = document.getElementById("addons-list");
     let detailItem = document.querySelector("#addons-details > .addon-item");
 
-    let addon = detailItem.addon;
+    let addon = aAddon || detailItem.addon;
     if (!addon)
       return;
 
@@ -417,6 +505,20 @@ var Addons = {
     listItem.setAttribute("opType", opType);
   },
 
+  setDefaultSearch: function setDefaultSearch(aAddon) {
+    let addon = aAddon || document.querySelector("#addons-details > .addon-item").addon;
+    if (addon.type != "search")
+      return;
+
+    let engine = Services.search.getEngineByName(addon.id);
+
+    // Move the new default search engine to the top of the search engine list.
+    Services.search.moveEngine(engine, 0);
+    Services.search.defaultEngine = engine;
+
+    document.getElementById("default-btn").setAttribute("disabled", "true");
+  },
+
   showRestart: function showRestart() {
     this._restartCount++;
     gChromeWin.XPInstallObserver.showRestartPrompt();
