@@ -1,42 +1,8 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim:set ts=2 sw=2 sts=2 et cindent: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is Google Inc.
- * Portions created by the Initial Developer are Copyright (C) 2005
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *  Darin Fisher <darin@meer.net>
- *  Jason Duell <jduell.mcbugs@gmail.com>
- *  Michal Novotny <michal.novotny@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsDeleteDir.h"
 #include "nsIFile.h"
@@ -47,6 +13,7 @@
 #include "nsAutoPtr.h"
 #include "nsThreadUtils.h"
 #include "nsISupportsPriority.h"
+#include "nsCacheUtils.h"
 #include <time.h>
 
 using namespace mozilla;
@@ -62,22 +29,8 @@ public:
   }
 };
 
-class nsDestroyThreadEvent : public nsRunnable {
-public:
-  nsDestroyThreadEvent(nsIThread *thread)
-    : mThread(thread)
-  {}
-  NS_IMETHOD Run()
-  {
-    mThread->Shutdown();
-    return NS_OK;
-  }
-private:
-  nsCOMPtr<nsIThread> mThread;
-};
 
-
-nsDeleteDir * nsDeleteDir::gInstance = nsnull;
+nsDeleteDir * nsDeleteDir::gInstance = nullptr;
 
 nsDeleteDir::nsDeleteDir()
   : mLock("nsDeleteDir.mLock"),
@@ -85,12 +38,12 @@ nsDeleteDir::nsDeleteDir()
     mShutdownPending(false),
     mStopDeleting(false)
 {
-  NS_ASSERTION(gInstance==nsnull, "multiple nsCacheService instances!");
+  NS_ASSERTION(gInstance==nullptr, "multiple nsCacheService instances!");
 }
 
 nsDeleteDir::~nsDeleteDir()
 {
-  gInstance = nsnull;
+  gInstance = nullptr;
 }
 
 nsresult
@@ -121,7 +74,7 @@ nsDeleteDir::Shutdown(bool finishDeleting)
       gInstance->mStopDeleting = true;
 
     // remove all pending timers
-    for (PRInt32 i = gInstance->mTimers.Count(); i > 0; i--) {
+    for (int32_t i = gInstance->mTimers.Count(); i > 0; i--) {
       nsCOMPtr<nsITimer> timer = gInstance->mTimers[i-1];
       gInstance->mTimers.RemoveObjectAt(i-1);
       timer->Cancel();
@@ -154,7 +107,7 @@ nsDeleteDir::Shutdown(bool finishDeleting)
 
   delete gInstance;
 
-  for (PRInt32 i = 0; i < dirsToRemove.Count(); i++)
+  for (int32_t i = 0; i < dirsToRemove.Count(); i++)
     dirsToRemove[i]->Remove(true);
 
   return NS_OK;
@@ -166,7 +119,7 @@ nsDeleteDir::InitThread()
   if (mThread)
     return NS_OK;
 
-  nsresult rv = NS_NewThread(getter_AddRefs(mThread));
+  nsresult rv = NS_NewNamedThread("Cache Deleter", getter_AddRefs(mThread));
   if (NS_FAILED(rv)) {
     NS_WARNING("Can't create background thread");
     return rv;
@@ -189,8 +142,8 @@ nsDeleteDir::DestroyThread()
     // more work to do, so don't delete thread.
     return;
 
-  NS_DispatchToMainThread(new nsDestroyThreadEvent(mThread));
-  mThread = nsnull;
+  nsShutdownThread::Shutdown(mThread);
+  mThread = nullptr;
 }
 
 void
@@ -200,7 +153,7 @@ nsDeleteDir::TimerCallback(nsITimer *aTimer, void *arg)
   {
     MutexAutoLock lock(gInstance->mLock);
 
-    PRInt32 idx = gInstance->mTimers.IndexOf(aTimer);
+    int32_t idx = gInstance->mTimers.IndexOf(aTimer);
     if (idx == -1) {
       // Timer was canceled and removed during shutdown.
       return;
@@ -213,8 +166,16 @@ nsDeleteDir::TimerCallback(nsITimer *aTimer, void *arg)
   dirList = static_cast<nsCOMArray<nsIFile> *>(arg);
 
   bool shuttingDown = false;
-  for (PRInt32 i = 0; i < dirList->Count() && !shuttingDown; i++) {
-    gInstance->RemoveDir((*dirList)[i], &shuttingDown);
+
+  // Intentional extra braces to control variable sope.
+  {
+    // Low IO priority can only be set when running in the context of the
+    // current thread.  So this shouldn't be moved to where we set the priority
+    // of the Cache deleter thread using the nsThread's NSPR priority constants.
+    nsAutoLowPriorityIO autoLowPriority;
+    for (int32_t i = 0; i < dirList->Count() && !shuttingDown; i++) {
+      gInstance->RemoveDir((*dirList)[i], &shuttingDown);
+    }
   }
 
   {
@@ -224,7 +185,7 @@ nsDeleteDir::TimerCallback(nsITimer *aTimer, void *arg)
 }
 
 nsresult
-nsDeleteDir::DeleteDir(nsIFile *dirIn, bool moveToTrash, PRUint32 delay)
+nsDeleteDir::DeleteDir(nsIFile *dirIn, bool moveToTrash, uint32_t delay)
 {
   Telemetry::AutoTimer<Telemetry::NETWORK_DISK_CACHE_TRASHRENAME> timer;
 
@@ -244,15 +205,15 @@ nsDeleteDir::DeleteDir(nsIFile *dirIn, bool moveToTrash, PRUint32 delay)
     rv = GetTrashDir(dir, &trash);
     if (NS_FAILED(rv))
       return rv;
-    nsCAutoString origLeaf;
+    nsAutoCString origLeaf;
     rv = trash->GetNativeLeafName(origLeaf);
     if (NS_FAILED(rv))
       return rv;
 
     // Append random number to the trash directory and check if it exists.
-    srand(PR_Now());
-    nsCAutoString leaf;
-    for (PRInt32 i = 0; i < 10; i++) {
+    srand(static_cast<unsigned>(PR_Now()));
+    nsAutoCString leaf;
+    for (int32_t i = 0; i < 10; i++) {
       leaf = origLeaf;
       leaf.AppendInt(rand());
       rv = trash->SetNativeLeafName(leaf);
@@ -281,7 +242,7 @@ nsDeleteDir::DeleteDir(nsIFile *dirIn, bool moveToTrash, PRUint32 delay)
     // Important: must rename directory w/o changing parent directory: else on
     // NTFS we'll wait (with cache lock) while nsIFile's ACL reset walks file
     // tree: was hanging GUI for *minutes* on large cache dirs.
-    rv = dir->MoveToNative(nsnull, leaf);
+    rv = dir->MoveToNative(nullptr, leaf);
 #endif
     if (NS_FAILED(rv))
       return rv;
@@ -310,12 +271,12 @@ nsDeleteDir::GetTrashDir(nsIFile *target, nsCOMPtr<nsIFile> *result)
   char* cachePath = getenv("CACHE_DIRECTORY");
   if (cachePath) {
     rv = NS_NewNativeLocalFile(nsDependentCString(cachePath),
-                               true, (nsILocalFile**)(nsIFile**)getter_AddRefs(*result));
+                               true, getter_AddRefs(*result));
     if (NS_FAILED(rv))
       return rv;
 
     // Add a sub folder with the cache folder name
-    nsCAutoString leaf;
+    nsAutoCString leaf;
     rv = target->GetNativeLeafName(leaf);
     (*result)->AppendNative(leaf);
   } else
@@ -326,7 +287,7 @@ nsDeleteDir::GetTrashDir(nsIFile *target, nsCOMPtr<nsIFile> *result)
   if (NS_FAILED(rv))
     return rv;
 
-  nsCAutoString leaf;
+  nsAutoCString leaf;
   rv = (*result)->GetNativeLeafName(leaf);
   if (NS_FAILED(rv))
     return rv;
@@ -412,7 +373,7 @@ nsDeleteDir::RemoveOldTrashes(nsIFile *cacheDir)
 }
 
 nsresult
-nsDeleteDir::PostTimer(void *arg, PRUint32 delay)
+nsDeleteDir::PostTimer(void *arg, uint32_t delay)
 {
   nsresult rv;
 
