@@ -81,7 +81,7 @@ struct nsDiskCacheEntry;
 
 // The minimum amount of milliseconds to wait before re-attempting to
 // revalidate the cache.
-#define kRevalidateCacheTimeout 5000
+#define kRevalidateCacheTimeout 3000
 #define kRevalidateCacheTimeoutTolerance 10
 #define kRevalidateCacheErrorTimeout 1000
 
@@ -408,7 +408,8 @@ public:
  *  Returns error if it detects change in format or cache wasn't closed.
  */
     nsresult  Open( nsIFile *  cacheDirectory,
-                    nsDiskCache::CorruptCacheInfo *  corruptInfo);
+                    nsDiskCache::CorruptCacheInfo *  corruptInfo,
+                    bool reportCacheCleanTelemetryData);
     nsresult  Close(bool flush);
     nsresult  Trim();
 
@@ -484,6 +485,8 @@ public:
     
     int32_t  EntryCount()  { return mHeader.mEntryCount; }
 
+    size_t  SizeOfExcludingThis(nsMallocSizeOfFun aMallocSizeOf);
+
 
 private:
 
@@ -539,7 +542,8 @@ private:
 
     // Initializes the _CACHE_CLEAN_ related functionality
     nsresult InitCacheClean(nsIFile *  cacheDirectory,
-                            nsDiskCache::CorruptCacheInfo *  corruptInfo);
+                            nsDiskCache::CorruptCacheInfo *  corruptInfo,
+                            bool reportCacheCleanTelemetryData);
     // Writes out a value of '0' or '1' in the _CACHE_CLEAN_ file
     nsresult WriteCacheClean(bool clean);
     // Resets the timout for revalidating the cache
