@@ -7,14 +7,9 @@
 #define _NS_NSSCERTIFICATECHILD_H_
 
 #include "nsIX509Cert.h"
-#include "nsNSSShutDown.h"
 #include "nsISerializable.h"
 #include "nsIClassInfo.h"
-
-#include "nsNSSCertHeader.h"
-
-class nsINSSComponent;
-class nsIASN1Sequence;
+#include "secitem.h"
 
 /* Certificate */
 class nsNSSCertificateFakeTransport : public nsIX509Cert,
