@@ -1,8 +1,11 @@
+/* Any copyright is dedicated to the Public Domain.
+   http://creativecommons.org/publicdomain/zero/1.0/ */
+
 Cu.import("resource://services-sync/engines.js");
-Cu.import("resource://services-sync/engines/clients.js");
-Cu.import("resource://services-sync/record.js");
 Cu.import("resource://services-sync/service.js");
 Cu.import("resource://services-sync/util.js");
+Cu.import("resource://testing-common/services/sync/rotaryengine.js");
+Cu.import("resource://testing-common/services/sync/utils.js");
 
 // Track HMAC error counts.
 let hmacErrorCount = 0;
@@ -23,20 +26,20 @@ function shared_setup() {
   Service.clusterURL = TEST_CLUSTER_URL;
 
   // Make sure RotaryEngine is the only one we sync.
-  Engines._engines = {};
-  Engines.register(RotaryEngine);
-  let engine = Engines.get("rotary");
+  Service.engineManager._engines = {};
+  Service.engineManager.register(RotaryEngine);
+  let engine = Service.engineManager.get("rotary");
   engine.enabled = true;
   engine.lastSync = 123; // Needs to be non-zero so that tracker is queried.
   engine._store.items = {flying: "LNER Class A3 4472",
                          scotsman: "Flying Scotsman"};
   engine._tracker.addChangedID('scotsman', 0);
-  do_check_eq(1, Engines.getEnabled().length);
+  do_check_eq(1, Service.engineManager.getEnabled().length);
 
   let engines = {rotary:  {version: engine.version,
                            syncID:  engine.syncID},
-                 clients: {version: Clients.version,
-                           syncID:  Clients.syncID}};
+                 clients: {version: Service.clientsEngine.version,
+                           syncID:  Service.clientsEngine.syncID}};
 
   // Common server objects.
   let global      = new ServerWBO("global", {engines: engines});
@@ -82,7 +85,7 @@ add_test(function hmac_error_during_404() {
     _("Syncing.");
     Service.sync();
     _("Partially resetting client, as if after a restart, and forcing redownload.");
-    CollectionKeys.clear();
+    Service.collectionKeys.clear();
     engine.lastSync = 0;        // So that we redownload records.
     key404Counter = 1;
     _("---------------------------");
@@ -93,7 +96,7 @@ add_test(function hmac_error_during_404() {
     do_check_eq(hmacErrorCount, 0)
   } finally {
     Svc.Prefs.resetBranch("");
-    Records.clearCache();
+    Service.recordManager.clearCache();
     server.stop(run_next_test);
   }
 });
@@ -212,7 +215,7 @@ add_test(function hmac_error_during_node_reassignment() {
       Utils.nextTick(function() {
         _("Now a fresh sync will get no HMAC errors.");
         _("Partially resetting client, as if after a restart, and forcing redownload.");
-        CollectionKeys.clear();
+        Service.collectionKeys.clear();
         engine.lastSync = 0;
         hmacErrorCount = 0;
 
@@ -222,9 +225,9 @@ add_test(function hmac_error_during_node_reassignment() {
 
           Svc.Obs.remove("weave:service:sync:finish", obs);
           Svc.Obs.remove("weave:service:sync:error", obs);
-                  
+
           Svc.Prefs.resetBranch("");
-          Records.clearCache();
+          Service.recordManager.clearCache();
           server.stop(run_next_test);
         };
 
