@@ -1,20 +1,24 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
+Cu.import("resource://services-sync/constants.js");
 Cu.import("resource://services-sync/engines.js");
 Cu.import("resource://services-sync/engines/clients.js");
-Cu.import("resource://services-sync/constants.js");
-Cu.import("resource://services-sync/policies.js");
+Cu.import("resource://services-sync/util.js");
+Cu.import("resource://testing-common/services/sync/utils.js");
 
 Svc.DefaultPrefs.set("registerEngines", "");
 Cu.import("resource://services-sync/service.js");
 
+let scheduler = Service.scheduler;
+let clientsEngine = Service.clientsEngine;
+
 function sync_httpd_setup() {
   let global = new ServerWBO("global", {
     syncID: Service.syncID,
     storageVersion: STORAGE_VERSION,
-    engines: {clients: {version: Clients.version,
-                        syncID: Clients.syncID}}
+    engines: {clients: {version: clientsEngine.version,
+                        syncID: clientsEngine.syncID}}
   });
   let clientsColl = new ServerCollection({}, true);
 
@@ -36,10 +40,10 @@ function setUp() {
   Service.serverURL = TEST_SERVER_URL;
   Service.clusterURL = TEST_CLUSTER_URL;
 
-  generateNewKeys();
-  let serverKeys = CollectionKeys.asWBO("crypto", "keys");
-  serverKeys.encrypt(Identity.syncKeyBundle);
-  return serverKeys.upload(Service.cryptoKeysURL);
+  generateNewKeys(Service.collectionKeys);
+  let serverKeys = Service.collectionKeys.asWBO("crypto", "keys");
+  serverKeys.encrypt(Service.identity.syncKeyBundle);
+  return serverKeys.upload(Service.resource(Service.cryptoKeysURL));
 }
 
 function run_test() {
@@ -64,86 +68,86 @@ add_test(function test_successful_sync_adjustSyncInterval() {
   setUp();
 
   // Confirm defaults
-  do_check_false(SyncScheduler.idle);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
-  do_check_false(SyncScheduler.hasIncomingItems);
+  do_check_false(scheduler.idle);
+  do_check_false(scheduler.numClients > 1);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
+  do_check_false(scheduler.hasIncomingItems);
 
-  _("Test as long as numClients <= 1 our sync interval is SINGLE_USER."); 
+  _("Test as long as numClients <= 1 our sync interval is SINGLE_USER.");
   // idle == true && numClients <= 1 && hasIncomingItems == false
-  SyncScheduler.idle = true;
+  scheduler.idle = true;
   Service.sync();
   do_check_eq(syncSuccesses, 1);
-  do_check_true(SyncScheduler.idle);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_false(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
-  
+  do_check_true(scheduler.idle);
+  do_check_false(scheduler.numClients > 1);
+  do_check_false(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
+
   // idle == false && numClients <= 1 && hasIncomingItems == false
-  SyncScheduler.idle = false;
+  scheduler.idle = false;
   Service.sync();
   do_check_eq(syncSuccesses, 2);
-  do_check_false(SyncScheduler.idle);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_false(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
+  do_check_false(scheduler.idle);
+  do_check_false(scheduler.numClients > 1);
+  do_check_false(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
 
   // idle == false && numClients <= 1 && hasIncomingItems == true
-  SyncScheduler.hasIncomingItems = true;
+  scheduler.hasIncomingItems = true;
   Service.sync();
   do_check_eq(syncSuccesses, 3);
-  do_check_false(SyncScheduler.idle);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_true(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
+  do_check_false(scheduler.idle);
+  do_check_false(scheduler.numClients > 1);
+  do_check_true(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
 
   // idle == true && numClients <= 1 && hasIncomingItems == true
-  SyncScheduler.idle = true;
+  scheduler.idle = true;
   Service.sync();
   do_check_eq(syncSuccesses, 4);
-  do_check_true(SyncScheduler.idle);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_true(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
+  do_check_true(scheduler.idle);
+  do_check_false(scheduler.numClients > 1);
+  do_check_true(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
 
   _("Test as long as idle && numClients > 1 our sync interval is idleInterval.");
   // idle == true && numClients > 1 && hasIncomingItems == true
-  Clients._store.create({id: "foo", cleartext: "bar"});
+  Service.clientsEngine._store.create({id: "foo", cleartext: "bar"});
   Service.sync();
   do_check_eq(syncSuccesses, 5);
-  do_check_true(SyncScheduler.idle);
-  do_check_true(SyncScheduler.numClients > 1);
-  do_check_true(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.idleInterval);
+  do_check_true(scheduler.idle);
+  do_check_true(scheduler.numClients > 1);
+  do_check_true(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.idleInterval);
 
   // idle == true && numClients > 1 && hasIncomingItems == false
-  SyncScheduler.hasIncomingItems = false;
+  scheduler.hasIncomingItems = false;
   Service.sync();
   do_check_eq(syncSuccesses, 6);
-  do_check_true(SyncScheduler.idle);
-  do_check_true(SyncScheduler.numClients > 1);
-  do_check_false(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.idleInterval);
+  do_check_true(scheduler.idle);
+  do_check_true(scheduler.numClients > 1);
+  do_check_false(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.idleInterval);
 
   _("Test non-idle, numClients > 1, no incoming items => activeInterval.");
   // idle == false && numClients > 1 && hasIncomingItems == false
-  SyncScheduler.idle = false;
+  scheduler.idle = false;
   Service.sync();
   do_check_eq(syncSuccesses, 7);
-  do_check_false(SyncScheduler.idle);
-  do_check_true(SyncScheduler.numClients > 1);
-  do_check_false(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.activeInterval);
-  
+  do_check_false(scheduler.idle);
+  do_check_true(scheduler.numClients > 1);
+  do_check_false(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.activeInterval);
+
   _("Test non-idle, numClients > 1, incoming items => immediateInterval.");
   // idle == false && numClients > 1 && hasIncomingItems == true
-  SyncScheduler.hasIncomingItems = true;
+  scheduler.hasIncomingItems = true;
   Service.sync();
   do_check_eq(syncSuccesses, 8);
-  do_check_false(SyncScheduler.idle);
-  do_check_true(SyncScheduler.numClients > 1);
-  do_check_false(SyncScheduler.hasIncomingItems); //gets reset to false
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.immediateInterval);
+  do_check_false(scheduler.idle);
+  do_check_true(scheduler.numClients > 1);
+  do_check_false(scheduler.hasIncomingItems); //gets reset to false
+  do_check_eq(scheduler.syncInterval, scheduler.immediateInterval);
 
   Svc.Obs.remove("weave:service:sync:finish", onSyncFinish);
   Service.startOver();
@@ -159,96 +163,96 @@ add_test(function test_unsuccessful_sync_adjustSyncInterval() {
     syncFailures++;
   }
   Svc.Obs.add("weave:service:sync:error", onSyncError);
-    
+
   _("Test unsuccessful sync calls adjustSyncInterval");
   // Force sync to fail.
   Svc.Prefs.set("firstSync", "notReady");
-  
+
   let server = sync_httpd_setup();
   setUp();
 
   // Confirm defaults
-  do_check_false(SyncScheduler.idle);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
-  do_check_false(SyncScheduler.hasIncomingItems);
+  do_check_false(scheduler.idle);
+  do_check_false(scheduler.numClients > 1);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
+  do_check_false(scheduler.hasIncomingItems);
 
-  _("Test as long as numClients <= 1 our sync interval is SINGLE_USER."); 
+  _("Test as long as numClients <= 1 our sync interval is SINGLE_USER.");
   // idle == true && numClients <= 1 && hasIncomingItems == false
-  SyncScheduler.idle = true;
+  scheduler.idle = true;
   Service.sync();
   do_check_eq(syncFailures, 1);
-  do_check_true(SyncScheduler.idle);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_false(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
-  
+  do_check_true(scheduler.idle);
+  do_check_false(scheduler.numClients > 1);
+  do_check_false(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
+
   // idle == false && numClients <= 1 && hasIncomingItems == false
-  SyncScheduler.idle = false;
+  scheduler.idle = false;
   Service.sync();
   do_check_eq(syncFailures, 2);
-  do_check_false(SyncScheduler.idle);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_false(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
+  do_check_false(scheduler.idle);
+  do_check_false(scheduler.numClients > 1);
+  do_check_false(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
 
   // idle == false && numClients <= 1 && hasIncomingItems == true
-  SyncScheduler.hasIncomingItems = true;
+  scheduler.hasIncomingItems = true;
   Service.sync();
   do_check_eq(syncFailures, 3);
-  do_check_false(SyncScheduler.idle);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_true(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
+  do_check_false(scheduler.idle);
+  do_check_false(scheduler.numClients > 1);
+  do_check_true(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
 
   // idle == true && numClients <= 1 && hasIncomingItems == true
-  SyncScheduler.idle = true;
+  scheduler.idle = true;
   Service.sync();
   do_check_eq(syncFailures, 4);
-  do_check_true(SyncScheduler.idle);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_true(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
-  
+  do_check_true(scheduler.idle);
+  do_check_false(scheduler.numClients > 1);
+  do_check_true(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
+
   _("Test as long as idle && numClients > 1 our sync interval is idleInterval.");
   // idle == true && numClients > 1 && hasIncomingItems == true
-  Clients._store.create({id: "foo", cleartext: "bar"});
+  Service.clientsEngine._store.create({id: "foo", cleartext: "bar"});
 
   Service.sync();
   do_check_eq(syncFailures, 5);
-  do_check_true(SyncScheduler.idle);
-  do_check_true(SyncScheduler.numClients > 1);
-  do_check_true(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.idleInterval);
+  do_check_true(scheduler.idle);
+  do_check_true(scheduler.numClients > 1);
+  do_check_true(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.idleInterval);
 
   // idle == true && numClients > 1 && hasIncomingItems == false
-  SyncScheduler.hasIncomingItems = false;
+  scheduler.hasIncomingItems = false;
   Service.sync();
   do_check_eq(syncFailures, 6);
-  do_check_true(SyncScheduler.idle);
-  do_check_true(SyncScheduler.numClients > 1);
-  do_check_false(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.idleInterval);
+  do_check_true(scheduler.idle);
+  do_check_true(scheduler.numClients > 1);
+  do_check_false(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.idleInterval);
 
   _("Test non-idle, numClients > 1, no incoming items => activeInterval.");
   // idle == false && numClients > 1 && hasIncomingItems == false
-  SyncScheduler.idle = false;
+  scheduler.idle = false;
   Service.sync();
   do_check_eq(syncFailures, 7);
-  do_check_false(SyncScheduler.idle);
-  do_check_true(SyncScheduler.numClients > 1);
-  do_check_false(SyncScheduler.hasIncomingItems);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.activeInterval);
-  
+  do_check_false(scheduler.idle);
+  do_check_true(scheduler.numClients > 1);
+  do_check_false(scheduler.hasIncomingItems);
+  do_check_eq(scheduler.syncInterval, scheduler.activeInterval);
+
   _("Test non-idle, numClients > 1, incoming items => immediateInterval.");
   // idle == false && numClients > 1 && hasIncomingItems == true
-  SyncScheduler.hasIncomingItems = true;
+  scheduler.hasIncomingItems = true;
   Service.sync();
   do_check_eq(syncFailures, 8);
-  do_check_false(SyncScheduler.idle);
-  do_check_true(SyncScheduler.numClients > 1);
-  do_check_false(SyncScheduler.hasIncomingItems); //gets reset to false
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.immediateInterval);
+  do_check_false(scheduler.idle);
+  do_check_true(scheduler.numClients > 1);
+  do_check_false(scheduler.hasIncomingItems); //gets reset to false
+  do_check_eq(scheduler.syncInterval, scheduler.immediateInterval);
 
   Service.startOver();
   Svc.Obs.remove("weave:service:sync:error", onSyncError);
@@ -260,29 +264,29 @@ add_test(function test_back_triggers_sync() {
   setUp();
 
   // Single device: no sync triggered.
-  SyncScheduler.idle = true;
-  SyncScheduler.observe(null, "back", Svc.Prefs.get("scheduler.idleTime"));
-  do_check_false(SyncScheduler.idle);
+  scheduler.idle = true;
+  scheduler.observe(null, "back", Svc.Prefs.get("scheduler.idleTime"));
+  do_check_false(scheduler.idle);
 
   // Multiple devices: sync is triggered.
-  Clients._store.create({id: "foo", cleartext: "bar"});
-  SyncScheduler.updateClientMode();
+  clientsEngine._store.create({id: "foo", cleartext: "bar"});
+  scheduler.updateClientMode();
 
   Svc.Obs.add("weave:service:sync:finish", function onSyncFinish() {
     Svc.Obs.remove("weave:service:sync:finish", onSyncFinish);
 
-    Records.clearCache();
+    Service.recordManager.clearCache();
     Svc.Prefs.resetBranch("");
-    SyncScheduler.setDefaults();
-    Clients.resetClient();
+    scheduler.setDefaults();
+    clientsEngine.resetClient();
 
     Service.startOver();
     server.stop(run_next_test);
   });
 
-  SyncScheduler.idle = true;
-  SyncScheduler.observe(null, "back", Svc.Prefs.get("scheduler.idleTime"));
-  do_check_false(SyncScheduler.idle);
+  scheduler.idle = true;
+  scheduler.observe(null, "back", Svc.Prefs.get("scheduler.idleTime"));
+  do_check_false(scheduler.idle);
 });
 
 add_test(function test_adjust_interval_on_sync_error() {
@@ -301,15 +305,15 @@ add_test(function test_adjust_interval_on_sync_error() {
   Svc.Prefs.set("firstSync", "notReady");
 
   do_check_eq(syncFailures, 0);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
+  do_check_false(scheduler.numClients > 1);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
 
-  Clients._store.create({id: "foo", cleartext: "bar"});
+  clientsEngine._store.create({id: "foo", cleartext: "bar"});
   Service.sync();
 
   do_check_eq(syncFailures, 1);
-  do_check_true(SyncScheduler.numClients > 1);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.activeInterval);
+  do_check_true(scheduler.numClients > 1);
+  do_check_eq(scheduler.syncInterval, scheduler.activeInterval);
 
   Svc.Obs.remove("weave:service:sync:error", onSyncError);
   Service.startOver();
@@ -318,7 +322,7 @@ add_test(function test_adjust_interval_on_sync_error() {
 
 add_test(function test_bug671378_scenario() {
   // Test scenario similar to bug 671378. This bug appeared when a score
-  // update occurred that wasn't large enough to trigger a sync so 
+  // update occurred that wasn't large enough to trigger a sync so
   // scheduleNextSync() was called without a time interval parameter,
   // setting nextSync to a non-zero value and preventing the timer from
   // being adjusted in the next call to scheduleNextSync().
@@ -335,72 +339,72 @@ add_test(function test_bug671378_scenario() {
   // After first sync call, syncInterval & syncTimer are singleDeviceInterval.
   Service.sync();
   do_check_eq(syncSuccesses, 1);
-  do_check_false(SyncScheduler.numClients > 1);
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
-  do_check_eq(SyncScheduler.syncTimer.delay, SyncScheduler.singleDeviceInterval);
+  do_check_false(scheduler.numClients > 1);
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
+  do_check_eq(scheduler.syncTimer.delay, scheduler.singleDeviceInterval);
 
   // Wrap scheduleNextSync so we are notified when it is finished.
-  SyncScheduler._scheduleNextSync = SyncScheduler.scheduleNextSync;
-  SyncScheduler.scheduleNextSync = function() {
-    SyncScheduler._scheduleNextSync();
+  scheduler._scheduleNextSync = scheduler.scheduleNextSync;
+  scheduler.scheduleNextSync = function() {
+    scheduler._scheduleNextSync();
 
     // Check on sync:finish scheduleNextSync sets the appropriate
     // syncInterval and syncTimer values.
     if (syncSuccesses == 2) {
-      do_check_neq(SyncScheduler.nextSync, 0);
-      do_check_eq(SyncScheduler.syncInterval, SyncScheduler.activeInterval);
-      do_check_true(SyncScheduler.syncTimer.delay <= SyncScheduler.activeInterval);
- 
-      SyncScheduler.scheduleNextSync = SyncScheduler._scheduleNextSync;
+      do_check_neq(scheduler.nextSync, 0);
+      do_check_eq(scheduler.syncInterval, scheduler.activeInterval);
+      do_check_true(scheduler.syncTimer.delay <= scheduler.activeInterval);
+
+      scheduler.scheduleNextSync = scheduler._scheduleNextSync;
       Svc.Obs.remove("weave:service:sync:finish", onSyncFinish);
       Service.startOver();
       server.stop(run_next_test);
     }
   };
-  
-  // Set nextSync != 0 
+
+  // Set nextSync != 0
   // syncInterval still hasn't been set by call to updateClientMode.
-  // Explicitly trying to invoke scheduleNextSync during a sync 
+  // Explicitly trying to invoke scheduleNextSync during a sync
   // (to immitate a score update that isn't big enough to trigger a sync).
   Svc.Obs.add("weave:service:sync:start", function onSyncStart() {
-    // Wait for other sync:start observers to be called so that 
+    // Wait for other sync:start observers to be called so that
     // nextSync is set to 0.
     Utils.nextTick(function() {
       Svc.Obs.remove("weave:service:sync:start", onSyncStart);
 
-      SyncScheduler.scheduleNextSync();
-      do_check_neq(SyncScheduler.nextSync, 0);
-      do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
-      do_check_eq(SyncScheduler.syncTimer.delay, SyncScheduler.singleDeviceInterval);
+      scheduler.scheduleNextSync();
+      do_check_neq(scheduler.nextSync, 0);
+      do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
+      do_check_eq(scheduler.syncTimer.delay, scheduler.singleDeviceInterval);
     });
   });
 
-  Clients._store.create({id: "foo", cleartext: "bar"});
+  clientsEngine._store.create({id: "foo", cleartext: "bar"});
   Service.sync();
 });
 
 add_test(function test_adjust_timer_larger_syncInterval() {
   _("Test syncInterval > current timout period && nextSync != 0, syncInterval is NOT used.");
-  Clients._store.create({id: "foo", cleartext: "bar"});
-  SyncScheduler.updateClientMode();
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.activeInterval);
+  clientsEngine._store.create({id: "foo", cleartext: "bar"});
+  scheduler.updateClientMode();
+  do_check_eq(scheduler.syncInterval, scheduler.activeInterval);
 
-  SyncScheduler.scheduleNextSync();
+  scheduler.scheduleNextSync();
 
   // Ensure we have a small interval.
-  do_check_neq(SyncScheduler.nextSync, 0);
-  do_check_eq(SyncScheduler.syncTimer.delay, SyncScheduler.activeInterval);
+  do_check_neq(scheduler.nextSync, 0);
+  do_check_eq(scheduler.syncTimer.delay, scheduler.activeInterval);
 
   // Make interval large again
-  Clients._wipeClient();
-  SyncScheduler.updateClientMode();
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.singleDeviceInterval);
+  clientsEngine._wipeClient();
+  scheduler.updateClientMode();
+  do_check_eq(scheduler.syncInterval, scheduler.singleDeviceInterval);
 
-  SyncScheduler.scheduleNextSync();
+  scheduler.scheduleNextSync();
 
   // Ensure timer delay remains as the small interval.
-  do_check_neq(SyncScheduler.nextSync, 0);
-  do_check_true(SyncScheduler.syncTimer.delay <= SyncScheduler.activeInterval);
+  do_check_neq(scheduler.nextSync, 0);
+  do_check_true(scheduler.syncTimer.delay <= scheduler.activeInterval);
 
   //SyncSchedule.
   Service.startOver();
@@ -409,22 +413,22 @@ add_test(function test_adjust_timer_larger_syncInterval() {
 
 add_test(function test_adjust_timer_smaller_syncInterval() {
   _("Test current timout > syncInterval period && nextSync != 0, syncInterval is used.");
-  SyncScheduler.scheduleNextSync();
+  scheduler.scheduleNextSync();
 
   // Ensure we have a large interval.
-  do_check_neq(SyncScheduler.nextSync, 0);
-  do_check_eq(SyncScheduler.syncTimer.delay, SyncScheduler.singleDeviceInterval);
+  do_check_neq(scheduler.nextSync, 0);
+  do_check_eq(scheduler.syncTimer.delay, scheduler.singleDeviceInterval);
 
   // Make interval smaller
-  Clients._store.create({id: "foo", cleartext: "bar"});
-  SyncScheduler.updateClientMode();
-  do_check_eq(SyncScheduler.syncInterval, SyncScheduler.activeInterval);
+  clientsEngine._store.create({id: "foo", cleartext: "bar"});
+  scheduler.updateClientMode();
+  do_check_eq(scheduler.syncInterval, scheduler.activeInterval);
 
-  SyncScheduler.scheduleNextSync();
+  scheduler.scheduleNextSync();
 
   // Ensure smaller timer delay is used.
-  do_check_neq(SyncScheduler.nextSync, 0);
-  do_check_true(SyncScheduler.syncTimer.delay <= SyncScheduler.activeInterval);
+  do_check_neq(scheduler.nextSync, 0);
+  do_check_true(scheduler.syncTimer.delay <= scheduler.activeInterval);
 
   //SyncSchedule.
   Service.startOver();
