@@ -1,41 +1,10 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  * vim: sw=2 ts=2 et lcs=trail\:.,tab\:>~ :
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozStorage.
- *
- * The Initial Developer of the Original Code is
- * the Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Marco Bonardo <mak77@bonardo.net> (Original Author)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
+#include "mozilla/DebugOnly.h"
 
 #include "VacuumManager.h"
 
@@ -94,14 +63,14 @@ NS_IMETHODIMP
 BaseCallback::HandleError(mozIStorageError *aError)
 {
 #ifdef DEBUG
-  PRInt32 result;
+  int32_t result;
   nsresult rv = aError->GetResult(&result);
   NS_ENSURE_SUCCESS(rv, rv);
-  nsCAutoString message;
+  nsAutoCString message;
   rv = aError->GetMessage(message);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCAutoString warnMsg;
+  nsAutoCString warnMsg;
   warnMsg.AppendLiteral("An error occured during async execution: ");
   warnMsg.AppendInt(result);
   warnMsg.AppendLiteral(" ");
@@ -119,7 +88,7 @@ BaseCallback::HandleResult(mozIStorageResultSet *aResultSet)
 }
 
 NS_IMETHODIMP
-BaseCallback::HandleCompletion(PRUint16 aReason)
+BaseCallback::HandleCompletion(uint16_t aReason)
 {
   // By default BaseCallback will just be silent on completion.
   return NS_OK;
@@ -174,12 +143,12 @@ Vacuumer::execute()
   // Ask for the expected page size.  Vacuum can change the page size, unless
   // the database is using WAL journaling.
   // TODO Bug 634374: figure out a strategy to fix page size with WAL.
-  PRInt32 expectedPageSize = 0;
+  int32_t expectedPageSize = 0;
   rv = mParticipant->GetExpectedDatabasePageSize(&expectedPageSize);
-  if (NS_FAILED(rv) || expectedPageSize < 512 || expectedPageSize > 65536) {
+  if (NS_FAILED(rv) || !Service::pageSizeIsValid(expectedPageSize)) {
     NS_WARNING("Invalid page size requested for database, will use default ");
     NS_WARNING(mDBFilename.get());
-    expectedPageSize = mozIStorageConnection::DEFAULT_PAGE_SIZE;
+    expectedPageSize = Service::getDefaultPageSize();
   }
 
   // Get the database filename.  Last vacuum time is stored under this name
@@ -197,9 +166,9 @@ Vacuumer::execute()
   MOZ_ASSERT(!mDBFilename.IsEmpty(), "Database filename cannot be empty");
 
   // Check interval from last vacuum.
-  PRInt32 now = static_cast<PRInt32>(PR_Now() / PR_USEC_PER_SEC);
-  PRInt32 lastVacuum;
-  nsCAutoString prefName(PREF_VACUUM_BRANCH);
+  int32_t now = static_cast<int32_t>(PR_Now() / PR_USEC_PER_SEC);
+  int32_t lastVacuum;
+  nsAutoCString prefName(PREF_VACUUM_BRANCH);
   prefName += mDBFilename;
   rv = Preferences::GetInt(prefName.get(), &lastVacuum);
   if (NS_SUCCEEDED(rv) && (now - lastVacuum) < VACUUM_INTERVAL_SECONDS) {
@@ -221,7 +190,7 @@ Vacuumer::execute()
   nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
   if (os) {
     DebugOnly<nsresult> rv =
-      os->NotifyObservers(nsnull, OBSERVER_TOPIC_HEAVY_IO,
+      os->NotifyObservers(nullptr, OBSERVER_TOPIC_HEAVY_IO,
                           OBSERVER_DATA_VACUUM_BEGIN.get());
     MOZ_ASSERT(NS_SUCCEEDED(rv), "Should be able to notify");
   }
@@ -229,7 +198,7 @@ Vacuumer::execute()
   // Execute the statements separately, since the pragma may conflict with the
   // vacuum, if they are executed in the same transaction.
   nsCOMPtr<mozIStorageAsyncStatement> pageSizeStmt;
-  nsCAutoString pageSizeQuery(MOZ_STORAGE_UNIQUIFY_QUERY_STR
+  nsAutoCString pageSizeQuery(MOZ_STORAGE_UNIQUIFY_QUERY_STR
                               "PRAGMA page_size = ");
   pageSizeQuery.AppendInt(expectedPageSize);
   rv = mDBConn->CreateAsyncStatement(pageSizeQuery,
@@ -258,14 +227,14 @@ NS_IMETHODIMP
 Vacuumer::HandleError(mozIStorageError *aError)
 {
 #ifdef DEBUG
-  PRInt32 result;
+  int32_t result;
   nsresult rv = aError->GetResult(&result);
   NS_ENSURE_SUCCESS(rv, rv);
-  nsCAutoString message;
+  nsAutoCString message;
   rv = aError->GetMessage(message);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCAutoString warnMsg;
+  nsAutoCString warnMsg;
   warnMsg.AppendLiteral("Unable to vacuum database: ");
   warnMsg.Append(mDBFilename);
   warnMsg.AppendLiteral(" - ");
@@ -277,10 +246,10 @@ Vacuumer::HandleError(mozIStorageError *aError)
 
 #ifdef PR_LOGGING
   {
-    PRInt32 result;
+    int32_t result;
     nsresult rv = aError->GetResult(&result);
     NS_ENSURE_SUCCESS(rv, rv);
-    nsCAutoString message;
+    nsAutoCString message;
     rv = aError->GetMessage(message);
     NS_ENSURE_SUCCESS(rv, rv);
     PR_LOG(gStorageLog, PR_LOG_ERROR,
@@ -299,13 +268,13 @@ Vacuumer::HandleResult(mozIStorageResultSet *aResultSet)
 }
 
 NS_IMETHODIMP
-Vacuumer::HandleCompletion(PRUint16 aReason)
+Vacuumer::HandleCompletion(uint16_t aReason)
 {
   if (aReason == REASON_FINISHED) {
     // Update last vacuum time.
-    PRInt32 now = static_cast<PRInt32>(PR_Now() / PR_USEC_PER_SEC);
+    int32_t now = static_cast<int32_t>(PR_Now() / PR_USEC_PER_SEC);
     MOZ_ASSERT(!mDBFilename.IsEmpty(), "Database filename cannot be empty");
-    nsCAutoString prefName(PREF_VACUUM_BRANCH);
+    nsAutoCString prefName(PREF_VACUUM_BRANCH);
     prefName += mDBFilename;
     DebugOnly<nsresult> rv = Preferences::SetInt(prefName.get(), now);
     MOZ_ASSERT(NS_SUCCEEDED(rv), "Should be able to set a preference"); 
@@ -321,7 +290,7 @@ Vacuumer::notifyCompletion(bool aSucceeded)
 {
   nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
   if (os) {
-    os->NotifyObservers(nsnull, OBSERVER_TOPIC_HEAVY_IO,
+    os->NotifyObservers(nullptr, OBSERVER_TOPIC_HEAVY_IO,
                         OBSERVER_DATA_VACUUM_END.get());
   }
 
@@ -342,7 +311,7 @@ NS_IMPL_ISUPPORTS1(
 )
 
 VacuumManager *
-VacuumManager::gVacuumManager = nsnull;
+VacuumManager::gVacuumManager = nullptr;
 
 VacuumManager *
 VacuumManager::getSingleton()
@@ -373,7 +342,7 @@ VacuumManager::~VacuumManager()
   MOZ_ASSERT(gVacuumManager == this,
              "Deleting a non-singleton instance of the service");
   if (gVacuumManager == this) {
-    gVacuumManager = nsnull;
+    gVacuumManager = nullptr;
   }
 }
 
@@ -393,11 +362,11 @@ VacuumManager::Observe(nsISupports *aSubject,
     // If there are more entries than what a month can contain, we could end up
     // skipping some, since we run daily.  So we use a starting index.
     static const char* kPrefName = PREF_VACUUM_BRANCH "index";
-    PRInt32 startIndex = Preferences::GetInt(kPrefName, 0);
+    int32_t startIndex = Preferences::GetInt(kPrefName, 0);
     if (startIndex >= entries.Count()) {
       startIndex = 0;
     }
-    PRInt32 index;
+    int32_t index;
     for (index = startIndex; index < entries.Count(); ++index) {
       nsCOMPtr<Vacuumer> vacuum = new Vacuumer(entries[index]);
       // Only vacuum one database per day.
