@@ -1,45 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Blake Ross <blaker@netscape.com>
- *   Ben Goodger <ben@netscape.com>
- *   Shawn Wilsher <me@shawnwilsher.com>
- *   Srirang G Doddihal <brahmana@doddihal.com>
- *   Edward Lee <edward.lee@engineering.uiuc.edu>
- *   Ehsan Akhgari <ehsan.akhgari@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef downloadmanager___h___
 #define downloadmanager___h___
@@ -51,7 +13,7 @@
 #include "nsIDownload.h"
 #include "nsIDownloadManager.h"
 #include "nsIDownloadProgressListener.h"
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 #include "nsIMIMEInfo.h"
 #include "nsINavHistoryService.h"
 #include "nsIObserver.h"
@@ -66,8 +28,8 @@
 #include "nsAutoPtr.h"
 #include "nsCOMArray.h"
 
-typedef PRInt16 DownloadState;
-typedef PRInt16 DownloadType;
+typedef int16_t DownloadState;
+typedef int16_t DownloadType;
 
 class nsDownload;
 
@@ -91,30 +53,24 @@ public:
   static nsDownloadManager *GetSingleton();
 
   virtual ~nsDownloadManager();
-  nsDownloadManager() :
-      mDBType(DATABASE_DISK)
-    , mInPrivateBrowsing(false)
+  nsDownloadManager()
 #ifdef DOWNLOAD_SCANNER
-    , mScanner(nsnull)
+    : mScanner(nullptr)
 #endif
   {
   }
 
 protected:
-  enum DatabaseType
-  {
-    DATABASE_DISK = 0, // default
-    DATABASE_MEMORY
-  };
-
   nsresult InitDB();
   nsresult InitFileDB();
-  void CloseDB();
-  nsresult InitMemoryDB();
+  void CloseAllDBs();
+  void CloseDB(mozIStorageConnection* aDBConn,
+               mozIStorageStatement* aUpdateStmt,
+               mozIStorageStatement* aGetIdsStmt);
+  nsresult InitPrivateDB();
   already_AddRefed<mozIStorageConnection> GetFileDBConnection(nsIFile *dbFile) const;
-  already_AddRefed<mozIStorageConnection> GetMemoryDBConnection() const;
-  nsresult SwitchDatabaseTypeTo(enum DatabaseType aType);
-  nsresult CreateTable();
+  already_AddRefed<mozIStorageConnection> GetPrivateDBConnection() const;
+  nsresult CreateTable(mozIStorageConnection* aDBConn);
 
   /**
    * Fix up the database after a crash such as dealing with previously-active
@@ -129,7 +85,11 @@ protected:
    */
   nsresult RestoreActiveDownloads();
 
-  nsresult GetDownloadFromDB(PRUint32 aID, nsDownload **retVal);
+  nsresult GetDownloadFromDB(const nsACString& aGUID, nsDownload **retVal);
+  nsresult GetDownloadFromDB(uint32_t aID, nsDownload **retVal);
+  nsresult GetDownloadFromDB(mozIStorageConnection* aDBConn,
+                             mozIStorageStatement* stmt,
+                             nsDownload **retVal);
 
   /**
    * Specially track the active downloads so that we don't need to check
@@ -144,32 +104,35 @@ protected:
    *
    * @return The id of the download, or 0 if there was an error.
    */
-  PRInt64 AddDownloadToDB(const nsAString &aName,
+  int64_t AddDownloadToDB(const nsAString &aName,
                           const nsACString &aSource,
                           const nsACString &aTarget,
                           const nsAString &aTempPath,
-                          PRInt64 aStartTime,
-                          PRInt64 aEndTime,
+                          int64_t aStartTime,
+                          int64_t aEndTime,
                           const nsACString &aMimeType,
                           const nsACString &aPreferredApp,
-                          nsHandlerInfoAction aPreferredAction);
+                          nsHandlerInfoAction aPreferredAction,
+                          bool aPrivate,
+                          nsACString &aNewGUID);
 
-  void NotifyListenersOnDownloadStateChange(PRInt16 aOldState,
-                                            nsIDownload *aDownload);
+  void NotifyListenersOnDownloadStateChange(int16_t aOldState,
+                                            nsDownload *aDownload);
   void NotifyListenersOnProgressChange(nsIWebProgress *aProgress,
                                        nsIRequest *aRequest,
-                                       PRInt64 aCurSelfProgress,
-                                       PRInt64 aMaxSelfProgress,
-                                       PRInt64 aCurTotalProgress,
-                                       PRInt64 aMaxTotalProgress,
-                                       nsIDownload *aDownload);
+                                       int64_t aCurSelfProgress,
+                                       int64_t aMaxSelfProgress,
+                                       int64_t aCurTotalProgress,
+                                       int64_t aMaxTotalProgress,
+                                       nsDownload *aDownload);
   void NotifyListenersOnStateChange(nsIWebProgress *aProgress,
                                     nsIRequest *aRequest,
-                                    PRUint32 aStateFlags,
+                                    uint32_t aStateFlags,
                                     nsresult aStatus,
-                                    nsIDownload *aDownload);
+                                    nsDownload *aDownload);
 
-  nsDownload *FindDownload(PRUint32 aID);
+  nsDownload *FindDownload(const nsACString& aGUID);
+  nsDownload *FindDownload(uint32_t aID);
 
   /**
    * First try to resume the download, and if that fails, retry it.
@@ -223,14 +186,14 @@ protected:
   static void ResumeOnWakeCallback(nsITimer *aTimer, void *aClosure);
   nsCOMPtr<nsITimer> mResumeOnWakeTimer;
 
-  void ConfirmCancelDownloads(PRInt32 aCount,
+  void ConfirmCancelDownloads(int32_t aCount,
                               nsISupportsPRBool *aCancelDownloads,
                               const PRUnichar *aTitle,
                               const PRUnichar *aCancelMessageMultiple,
                               const PRUnichar *aCancelMessageSingle,
                               const PRUnichar *aDontCancelButton);
 
-  PRInt32 GetRetentionBehavior();
+  int32_t GetRetentionBehavior();
 
   /**
    * Type to indicate possible behaviors for active downloads across sessions.
@@ -256,6 +219,13 @@ protected:
   void OnEnterPrivateBrowsingMode();
   void OnLeavePrivateBrowsingMode();
 
+  nsresult RetryDownload(const nsACString& aGUID);
+  nsresult RetryDownload(nsDownload* dl);
+
+  nsresult RemoveDownload(const nsACString& aGUID);
+
+  nsresult NotifyDownloadRemoval(nsDownload* aRemoved);
+
   // Virus scanner for windows
 #ifdef DOWNLOAD_SCANNER
 private:
@@ -263,18 +233,29 @@ private:
 #endif
 
 private:
+  nsresult CleanUp(mozIStorageConnection* aDBConn);
+  nsresult InitStatements(mozIStorageConnection* aDBConn,
+                          mozIStorageStatement** aUpdateStatement,
+                          mozIStorageStatement** aGetIdsStatement);
+  nsresult RemoveAllDownloads(nsCOMArray<nsDownload>& aDownloads);
+  nsresult PauseAllDownloads(nsCOMArray<nsDownload>& aDownloads, bool aSetResume);
+  nsresult ResumeAllDownloads(nsCOMArray<nsDownload>& aDownloads, bool aResumeAll);
+  nsresult RemoveDownloadsForURI(mozIStorageStatement* aStatement, nsIURI *aURI);
+
   nsCOMArray<nsIDownloadProgressListener> mListeners;
+  nsCOMArray<nsIDownloadProgressListener> mPrivacyAwareListeners;
   nsCOMPtr<nsIStringBundle> mBundle;
   nsCOMPtr<mozIStorageConnection> mDBConn;
+  nsCOMPtr<mozIStorageConnection> mPrivateDBConn;
   nsCOMArray<nsDownload> mCurrentDownloads;
+  nsCOMArray<nsDownload> mCurrentPrivateDownloads;
   nsCOMPtr<nsIObserverService> mObserverService;
   nsCOMPtr<mozIStorageStatement> mUpdateDownloadStatement;
+  nsCOMPtr<mozIStorageStatement> mUpdatePrivateDownloadStatement;
   nsCOMPtr<mozIStorageStatement> mGetIdsForURIStatement;
+  nsCOMPtr<mozIStorageStatement> mGetPrivateIdsForURIStatement;
   nsAutoPtr<mozStorageTransaction> mHistoryTransaction;
 
-  enum DatabaseType mDBType;
-  bool mInPrivateBrowsing;
-
   static nsDownloadManager *gDownloadManagerService;
 
   friend class nsDownload;
@@ -325,30 +306,19 @@ protected:
   /**
    * Update the start time which also implies the last update time is the same.
    */
-  void SetStartTime(PRInt64 aStartTime);
+  void SetStartTime(int64_t aStartTime);
 
   /**
    * Update the amount of bytes transferred and max bytes; and recalculate the
    * download percent.
    */
-  void SetProgressBytes(PRInt64 aCurrBytes, PRInt64 aMaxBytes);
-
-  /**
-   * Pause the download, but in certain cases it might get fake-paused instead
-   * of real-paused.
-   */
-  nsresult Pause();
+  void SetProgressBytes(int64_t aCurrBytes, int64_t aMaxBytes);
 
   /**
    * All this does is cancel the connection that the download is using. It does
    * not remove it from the download manager.
    */
-  nsresult Cancel();
-
-  /**
-   * Resume the download.
-   */
-  nsresult Resume();
+  nsresult CancelTransfer();
 
   /**
    * Download is not transferring?
@@ -389,7 +359,7 @@ protected:
 
   /**
    * Fail a download because of a failure status and prompt the provided
-   * message or use a generic download failure message if nsnull.
+   * message or use a generic download failure message if nullptr.
    */
   nsresult FailDownload(nsresult aStatus, const PRUnichar *aMessage);
 
@@ -411,34 +381,35 @@ protected:
 private:
   nsString mDisplayName;
   nsCString mEntityID;
+  nsCString mGUID;
 
   nsCOMPtr<nsIURI> mSource;
   nsCOMPtr<nsIURI> mReferrer;
   nsCOMPtr<nsICancelable> mCancelable;
   nsCOMPtr<nsIRequest> mRequest;
-  nsCOMPtr<nsILocalFile> mTempFile;
+  nsCOMPtr<nsIFile> mTempFile;
   nsCOMPtr<nsIMIMEInfo> mMIMEInfo;
 
   DownloadState mDownloadState;
-  DownloadType mDownloadType;
 
-  PRUint32 mID;
-  PRInt32 mPercentComplete;
+  uint32_t mID;
+  int32_t mPercentComplete;
 
   /**
    * These bytes are based on the position of where the request started, so 0
    * doesn't necessarily mean we have nothing. Use GetAmountTransferred and
    * GetSize for the real transferred amount and size.
    */
-  PRInt64 mCurrBytes;
-  PRInt64 mMaxBytes;
+  int64_t mCurrBytes;
+  int64_t mMaxBytes;
 
   PRTime mStartTime;
   PRTime mLastUpdate;
-  PRInt64 mResumedAt;
+  int64_t mResumedAt;
   double mSpeed;
 
   bool mHasMultipleFiles;
+  bool mPrivate;
 
   /**
    * Track various states of the download trying to auto-resume when starting
@@ -450,6 +421,11 @@ private:
   enum AutoResume { DONT_RESUME, AUTO_RESUME };
   AutoResume mAutoResume;
 
+  /**
+   * Stores the SHA-256 hash associated with the downloaded file.
+   */
+  nsAutoCString mHash;
+
   friend class nsDownloadManager;
 };
 
