@@ -12,7 +12,6 @@
 #include "nsIObserverService.h"
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
-#include "nsIProfileChangeStatus.h"
 #include "nsIPromptService.h"
 #include "nsIStringBundle.h"
 #include "nsISupportsPrimitives.h"
@@ -26,15 +25,14 @@
 #include "nsAutoPtr.h"
 #include "nsStringGlue.h"
 #include "mozilla/Preferences.h"
+#include "GeckoProfiler.h"
 
 #include "prprf.h"
-#include "nsCRT.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsWidgetsCID.h"
 #include "nsAppShellCID.h"
 #include "nsXPCOMCIDInternal.h"
 #include "mozilla/Services.h"
-#include "mozilla/FunctionTimer.h"
 #include "nsIXPConnect.h"
 #include "jsapi.h"
 #include "prenv.h"
@@ -42,21 +40,8 @@
 #include "mozilla/mozPoisonWrite.h"
 
 #if defined(XP_WIN)
-#include <windows.h>
-// windows.h can go to hell 
+// Prevent collisions with nsAppStartup::GetStartupInfo()
 #undef GetStartupInfo
-#elif defined(XP_UNIX)
-#include <unistd.h>
-#include <sys/syscall.h>
-#endif
-
-#ifdef XP_MACOSX
-#include <sys/sysctl.h>
-#endif
-
-#ifdef __OpenBSD__
-#include <sys/param.h>
-#include <sys/sysctl.h>
 #endif
 
 #include "mozilla/Telemetry.h"
@@ -67,6 +52,7 @@ static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 #define kPrefLastSuccess "toolkit.startup.last_success"
 #define kPrefMaxResumedCrashes "toolkit.startup.max_resumed_crashes"
 #define kPrefRecentCrashes "toolkit.startup.recent_crashes"
+#define kPrefAlwaysUseSafeMode "toolkit.startup.always_use_safe_mode"
 
 #if defined(XP_WIN)
 #include "mozilla/perfprobe.h"
@@ -128,6 +114,23 @@ public:
   }
 };
 
+/**
+ * Computes an approximation of the absolute time represented by @a stamp
+ * which is comparable to those obtained via PR_Now(). If the current absolute
+ * time varies a lot (e.g. DST adjustments) since the first call then the
+ * resulting times may be inconsistent.
+ *
+ * @param stamp The timestamp to be converted
+ * @returns The converted timestamp
+ */
+uint64_t ComputeAbsoluteTimestamp(PRTime prnow, TimeStamp now, TimeStamp stamp)
+{
+  static PRTime sAbsoluteNow = PR_Now();
+  static TimeStamp sMonotonicNow = TimeStamp::Now();
+
+  return sAbsoluteNow - (sMonotonicNow - stamp).ToMicroseconds();
+}
+
 //
 // nsAppStartup
 //
@@ -140,31 +143,24 @@ nsAppStartup::nsAppStartup() :
   mRestart(false),
   mInterrupted(false),
   mIsSafeModeNecessary(false),
-  mStartupCrashTrackingEnded(false),
-  mCachedShutdownTime(false),
-  mLastShutdownTime(0)
+  mStartupCrashTrackingEnded(false)
 { }
 
 
 nsresult
 nsAppStartup::Init()
 {
-  NS_TIME_FUNCTION;
   nsresult rv;
 
   // Create widget application shell
   mAppShell = do_GetService(kAppShellCID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  NS_TIME_FUNCTION_MARK("Got AppShell service");
-
   nsCOMPtr<nsIObserverService> os =
     mozilla::services::GetObserverService();
   if (!os)
     return NS_ERROR_FAILURE;
 
-  NS_TIME_FUNCTION_MARK("Got Observer service");
-
   os->AddObserver(this, "quit-application-forced", true);
   os->AddObserver(this, "sessionstore-windows-restored", true);
   os->AddObserver(this, "profile-change-teardown", true);
@@ -278,83 +274,7 @@ nsAppStartup::Run(void)
   return mRestart ? NS_SUCCESS_RESTART_APP : NS_OK;
 }
 
-static TimeStamp gRecordedShutdownStartTime;
-static bool gAlreadyFreedShutdownTimeFileName = false;
-static char *gRecordedShutdownTimeFileName = NULL;
 
-static char *
-GetShutdownTimeFileName()
-{
-  if (gAlreadyFreedShutdownTimeFileName) {
-    return NULL;
-  }
-
-  if (!gRecordedShutdownTimeFileName) {
-    nsCOMPtr<nsIFile> mozFile;
-    NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(mozFile));
-    if (!mozFile)
-      return NULL;
-
-    mozFile->AppendNative(NS_LITERAL_CSTRING("Telemetry.ShutdownTime.txt"));
-    nsCAutoString nativePath;
-    nsresult rv = mozFile->GetNativePath(nativePath);
-    if (!NS_SUCCEEDED(rv))
-      return NULL;
-
-    gRecordedShutdownTimeFileName = PL_strdup(nativePath.get());
-  }
-
-  return gRecordedShutdownTimeFileName;
-}
-
-static void
-RecordShutdownStartTimeStamp() {
-  if (!Telemetry::CanRecord())
-    return;
-
-  gRecordedShutdownStartTime = TimeStamp::Now();
-
-  GetShutdownTimeFileName();
-}
-
-namespace mozilla {
-void
-RecordShutdownEndTimeStamp() {
-  if (!gRecordedShutdownTimeFileName || gAlreadyFreedShutdownTimeFileName)
-    return;
-
-  nsCString name(gRecordedShutdownTimeFileName);
-  PL_strfree(gRecordedShutdownTimeFileName);
-  gRecordedShutdownTimeFileName = NULL;
-  gAlreadyFreedShutdownTimeFileName = true;
-
-  nsCString tmpName = name;
-  tmpName += ".tmp";
-  FILE *f = fopen(tmpName.get(), "w");
-  if (!f)
-    return;
-  // On a normal release build this should be called just before
-  // calling _exit, but on a debug build or when the user forces a full
-  // shutdown this is called as late as possible, so we have to
-  // white list this write as write poisoning will be enabled.
-  int fd = fileno(f);
-  MozillaRegisterDebugFD(fd);
-
-  TimeStamp now = TimeStamp::Now();
-  MOZ_ASSERT(now >= gRecordedShutdownStartTime);
-  TimeDuration diff = now - gRecordedShutdownStartTime;
-  uint32_t diff2 = diff.ToMilliseconds();
-  int written = fprintf(f, "%d\n", diff2);
-  MozillaUnRegisterDebugFILE(f);
-  int rv = fclose(f);
-  if (written < 0 || rv != 0) {
-    PR_Delete(tmpName.get());
-    return;
-  }
-  PR_Delete(name.get());
-  PR_Rename(tmpName.get(), name.get());
-}
-}
 
 NS_IMETHODIMP
 nsAppStartup::Quit(uint32_t aMode)
@@ -370,19 +290,25 @@ nsAppStartup::Quit(uint32_t aMode)
   if (mShuttingDown)
     return NS_OK;
 
-  RecordShutdownStartTimeStamp();
-
   // If we're considering quitting, we will only do so if:
   if (ferocity == eConsiderQuit) {
+#ifdef XP_MACOSX
+    nsCOMPtr<nsIAppShellService> appShell
+      (do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
+    bool hasHiddenPrivateWindow = false;
+    if (appShell) {
+      appShell->GetHasHiddenPrivateWindow(&hasHiddenPrivateWindow);
+    }
+    int32_t suspiciousCount = hasHiddenPrivateWindow ? 2 : 1;
+#endif
+
     if (mConsiderQuitStopper == 0) {
       // there are no windows...
       ferocity = eAttemptQuit;
     }
 #ifdef XP_MACOSX
-    else if (mConsiderQuitStopper == 1) {
+    else if (mConsiderQuitStopper == suspiciousCount) {
       // ... or there is only a hiddenWindow left, and it's useless:
-      nsCOMPtr<nsIAppShellService> appShell
-        (do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
 
       // Failure shouldn't be fatal, but will abort quit attempt:
       if (!appShell)
@@ -392,9 +318,15 @@ nsAppStartup::Quit(uint32_t aMode)
       appShell->GetApplicationProvidedHiddenWindow(&usefulHiddenWindow);
       nsCOMPtr<nsIXULWindow> hiddenWindow;
       appShell->GetHiddenWindow(getter_AddRefs(hiddenWindow));
-      // If the one window is useful, we won't quit:
-      if (!hiddenWindow || usefulHiddenWindow)
+      // If the remaining windows are useful, we won't quit:
+      nsCOMPtr<nsIXULWindow> hiddenPrivateWindow;
+      if (hasHiddenPrivateWindow) {
+        appShell->GetHiddenPrivateWindow(getter_AddRefs(hiddenPrivateWindow));
+        if ((!hiddenWindow && !hiddenPrivateWindow) || usefulHiddenWindow)
+          return NS_OK;
+      } else if (!hiddenWindow || usefulHiddenWindow) {
         return NS_OK;
+      }
 
       ferocity = eAttemptQuit;
     }
@@ -422,6 +354,8 @@ nsAppStartup::Quit(uint32_t aMode)
       }
     }
 
+    PROFILER_MARKER("Shutdown start");
+    mozilla::RecordShutdownStartTimeStamp();
     mShuttingDown = true;
     if (!mRestart) {
       mRestart = (aMode & eRestart) != 0;
@@ -429,8 +363,9 @@ nsAppStartup::Quit(uint32_t aMode)
     }
 
     if (mRestart) {
-      // Firefox-restarts reuse the process. Process start-time isn't a useful indicator of startup time
-      PR_SetEnv(PR_smprintf("MOZ_APP_RESTART=%lld", (int64_t) PR_Now() / PR_USEC_PER_MSEC));
+      /* Firefox-restarts reuse the process so regular process start-time isn't
+         a useful indicator of startup time anymore. */
+      TimeStamp::RecordProcessRestart();
     }
 
     obsService = mozilla::services::GetObserverService();
@@ -569,47 +504,6 @@ nsAppStartup::ExitLastWindowClosingSurvivalArea(void)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsAppStartup::GetLastShutdownDuration(uint32_t *aResult)
-{
-  // We make this check so that GetShutdownTimeFileName() doesn't get
-  // called; calling that function without telemetry enabled violates
-  // assumptions that the write-the-shutdown-timestamp machinery makes.
-  if (!Telemetry::CanRecord()) {
-    *aResult = 0;
-    return NS_OK;
-  }
-
-  if (!mCachedShutdownTime) {
-    const char *filename = GetShutdownTimeFileName();
-
-    if (!filename) {
-      *aResult = 0;
-      return NS_OK;
-    }
-
-    FILE *f = fopen(filename, "r");
-    if (!f) {
-      *aResult = 0;
-      return NS_OK;
-    }
-
-    int shutdownTime;
-    int r = fscanf(f, "%d\n", &shutdownTime);
-    fclose(f);
-    if (r != 1) {
-      *aResult = 0;
-      return NS_OK;
-    }
-
-    mLastShutdownTime = shutdownTime;
-    mCachedShutdownTime = true;
-  }
-
-  *aResult = mLastShutdownTime;
-  return NS_OK;
-}
-
 //
 // nsAppStartup->nsIAppStartup2
 //
@@ -753,169 +647,53 @@ nsAppStartup::Observe(nsISupports *aSubject,
   return NS_OK;
 }
 
-#if defined(LINUX) || defined(ANDROID)
-static uint64_t 
-JiffiesSinceBoot(const char *file)
-{
-  char stat[512];
-  FILE *f = fopen(file, "r");
-  if (!f)
-    return 0;
-  int n = fread(&stat, 1, sizeof(stat) - 1, f);
-  fclose(f);
-  if (n <= 0)
-    return 0;
-  stat[n] = 0;
-  
-  long long unsigned starttime = 0; // instead of uint64_t to keep GCC quiet
-  
-  char *s = strrchr(stat, ')');
-  if (!s)
-    return 0;
-  int ret = sscanf(s + 2,
-                   "%*c %*d %*d %*d %*d %*d %*u %*u %*u %*u "
-                   "%*u %*u %*u %*u %*u %*d %*d %*d %*d %llu",
-                   &starttime);
-  if (ret != 1 || !starttime)
-    return 0;
-  return starttime;
-}
-
-static void
-ThreadedCalculateProcessCreationTimestamp(void *aClosure)
-{
-  PR_SetCurrentThreadName("Startup Timer");
-
-  PRTime now = PR_Now();
-  long hz = sysconf(_SC_CLK_TCK);
-  if (!hz)
-    return;
-
-  char thread_stat[40];
-  sprintf(thread_stat, "/proc/self/task/%d/stat", (pid_t) syscall(__NR_gettid));
-  
-  uint64_t thread_jiffies = JiffiesSinceBoot(thread_stat);
-  uint64_t self_jiffies = JiffiesSinceBoot("/proc/self/stat");
-  
-  if (!thread_jiffies || !self_jiffies)
-    return;
-
-  PRTime interval = (thread_jiffies - self_jiffies) * PR_USEC_PER_SEC / hz;
-  StartupTimeline::Record(StartupTimeline::PROCESS_CREATION, now - interval);
-}
-
-static PRTime
-CalculateProcessCreationTimestamp()
-{
- PRThread *thread = PR_CreateThread(PR_USER_THREAD,
-                                    ThreadedCalculateProcessCreationTimestamp,
-                                    NULL,
-                                    PR_PRIORITY_NORMAL,
-                                    PR_LOCAL_THREAD,
-                                    PR_JOINABLE_THREAD,
-                                    0);
-
-  PR_JoinThread(thread);
-  return StartupTimeline::Get(StartupTimeline::PROCESS_CREATION);
-}
-#elif defined(XP_WIN)
-static PRTime
-CalculateProcessCreationTimestamp()
-{
-  FILETIME start, foo, bar, baz;
-  bool success = GetProcessTimes(GetCurrentProcess(), &start, &foo, &bar, &baz);
-  if (!success)
-    return 0;
-  // copied from NSPR _PR_FileTimeToPRTime
-  uint64_t timestamp = 0;
-  CopyMemory(&timestamp, &start, sizeof(PRTime));
-#ifdef __GNUC__
-  timestamp = (timestamp - 116444736000000000LL) / 10LL;
-#else
-  timestamp = (timestamp - 116444736000000000i64) / 10i64;
-#endif
-  return timestamp;
-}
-#elif defined(XP_MACOSX)
-static PRTime
-CalculateProcessCreationTimestamp()
-{
-  int mib[4] = { CTL_KERN, KERN_PROC, KERN_PROC_PID, getpid() };
-  size_t buffer_size;
-  if (sysctl(mib, 4, NULL, &buffer_size, NULL, 0))
-    return 0;
-
-  struct kinfo_proc *proc = (kinfo_proc*) malloc(buffer_size);  
-  if (sysctl(mib, 4, proc, &buffer_size, NULL, 0)) {
-    free(proc);
-    return 0;
-  }
-  PRTime starttime = static_cast<PRTime>(proc->kp_proc.p_un.__p_starttime.tv_sec) * PR_USEC_PER_SEC;
-  starttime += proc->kp_proc.p_un.__p_starttime.tv_usec;
-  free(proc);
-  return starttime;
-}
-#elif defined(__OpenBSD__)
-static PRTime
-CalculateProcessCreationTimestamp()
-{
-  int mib[6] = { CTL_KERN, KERN_PROC, KERN_PROC_PID, getpid(), sizeof(struct kinfo_proc), 1 };
-  size_t buffer_size;
-  if (sysctl(mib, 6, NULL, &buffer_size, NULL, 0))
-    return 0;
-
-  struct kinfo_proc *proc = (struct kinfo_proc*) malloc(buffer_size);
-  if (sysctl(mib, 6, proc, &buffer_size, NULL, 0)) {
-    free(proc);
-    return 0;
-  }
-  PRTime starttime = static_cast<PRTime>(proc->p_ustart_sec) * PR_USEC_PER_SEC;
-  starttime += proc->p_ustart_usec;
-  free(proc);
-  return starttime;
-}
-#else
-static PRTime
-CalculateProcessCreationTimestamp()
-{
-  return 0;
-}
-#endif
- 
 NS_IMETHODIMP
 nsAppStartup::GetStartupInfo(JSContext* aCx, JS::Value* aRetval)
 {
-  JSObject *obj = JS_NewObject(aCx, NULL, NULL, NULL);
+  JS::Rooted<JSObject*> obj(aCx, JS_NewObject(aCx, NULL, NULL, NULL));
   *aRetval = OBJECT_TO_JSVAL(obj);
 
-  PRTime ProcessCreationTimestamp = StartupTimeline::Get(StartupTimeline::PROCESS_CREATION);
+  TimeStamp procTime = StartupTimeline::Get(StartupTimeline::PROCESS_CREATION);
+  TimeStamp now = TimeStamp::Now();
+  PRTime absNow = PR_Now();
 
-  if (!ProcessCreationTimestamp) {
-    char *moz_app_restart = PR_GetEnv("MOZ_APP_RESTART");
-    if (moz_app_restart) {
-      ProcessCreationTimestamp = nsCRT::atoll(moz_app_restart) * PR_USEC_PER_MSEC;
-    } else {
-      ProcessCreationTimestamp = CalculateProcessCreationTimestamp();
+  if (procTime.IsNull()) {
+    bool error = false;
+
+    procTime = TimeStamp::ProcessCreation(error);
+
+    if (error) {
+      Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS,
+        StartupTimeline::PROCESS_CREATION);
     }
-    // Bug 670008: Avoid obviously invalid process creation times
-    if (PR_Now() <= ProcessCreationTimestamp) {
-      ProcessCreationTimestamp = -1;
-      Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS, StartupTimeline::PROCESS_CREATION);
-    }
-    StartupTimeline::Record(StartupTimeline::PROCESS_CREATION, ProcessCreationTimestamp);
+
+    StartupTimeline::Record(StartupTimeline::PROCESS_CREATION, procTime);
   }
 
-  for (int i = StartupTimeline::PROCESS_CREATION; i < StartupTimeline::MAX_EVENT_ID; ++i) {
+  for (int i = StartupTimeline::PROCESS_CREATION;
+       i < StartupTimeline::MAX_EVENT_ID;
+       ++i)
+  {
     StartupTimeline::Event ev = static_cast<StartupTimeline::Event>(i);
-    if (StartupTimeline::Get(ev) > 0) {
-      // always define main to aid with bug 689256
-      if ((ev != StartupTimeline::MAIN) &&
-          (StartupTimeline::Get(ev) < StartupTimeline::Get(StartupTimeline::PROCESS_CREATION))) {
-        Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS, i);
-        StartupTimeline::Record(ev, -1);
+    TimeStamp stamp = StartupTimeline::Get(ev);
+
+    if (stamp.IsNull() && (ev == StartupTimeline::MAIN)) {
+      // Always define main to aid with bug 689256.
+      stamp = procTime;
+      MOZ_ASSERT(!stamp.IsNull());
+      Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS,
+        StartupTimeline::MAIN);
+    }
+
+    if (!stamp.IsNull()) {
+      if (stamp >= procTime) {
+        PRTime prStamp = ComputeAbsoluteTimestamp(absNow, now, stamp)
+          / PR_USEC_PER_MSEC;
+        JS::Rooted<JSObject*> date(aCx, JS_NewDateObjectMsec(aCx, prStamp));
+        JS_DefineProperty(aCx, obj, StartupTimeline::Describe(ev),
+          OBJECT_TO_JSVAL(date), NULL, NULL, JSPROP_ENUMERATE);
       } else {
-        JSObject *date = JS_NewDateObjectMsec(aCx, StartupTimeline::Get(ev) / PR_USEC_PER_MSEC);
-        JS_DefineProperty(aCx, obj, StartupTimeline::Describe(ev), OBJECT_TO_JSVAL(date), NULL, NULL, JSPROP_ENUMERATE);
+        Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS, ev);
       }
     }
   }
@@ -928,6 +706,17 @@ nsAppStartup::GetAutomaticSafeModeNecessary(bool *_retval)
 {
   NS_ENSURE_ARG_POINTER(_retval);
 
+  bool alwaysSafe = false;
+  Preferences::GetBool(kPrefAlwaysUseSafeMode, &alwaysSafe);
+
+  if (!alwaysSafe) {
+#if DEBUG
+    mIsSafeModeNecessary = false;
+#else
+    mIsSafeModeNecessary &= !PR_GetEnv("MOZ_DISABLE_AUTO_SAFE_MODE");
+#endif
+  }
+
   *_retval = mIsSafeModeNecessary;
   return NS_OK;
 }
@@ -1052,14 +841,21 @@ nsAppStartup::TrackStartupCrashEnd()
 
   // Use the timestamp of XRE_main as an approximation for the lock file timestamp.
   // See MAX_STARTUP_BUFFER for the buffer time period.
+  TimeStamp mainTime = StartupTimeline::Get(StartupTimeline::MAIN);
+  TimeStamp now = TimeStamp::Now();
+  PRTime prNow = PR_Now();
   nsresult rv;
-  PRTime mainTime = StartupTimeline::Get(StartupTimeline::MAIN);
-  if (mainTime <= 0) {
+
+  if (mainTime.IsNull()) {
     NS_WARNING("Could not get StartupTimeline::MAIN time.");
   } else {
-    int32_t lockFileTime = (int32_t)(mainTime / PR_USEC_PER_SEC);
-    rv = Preferences::SetInt(kPrefLastSuccess, lockFileTime);
-    if (NS_FAILED(rv)) NS_WARNING("Could not set startup crash detection pref.");
+    uint64_t lockFileTime = ComputeAbsoluteTimestamp(prNow, now, mainTime);
+
+    rv = Preferences::SetInt(kPrefLastSuccess,
+      (int32_t)(lockFileTime / PR_USEC_PER_SEC));
+
+    if (NS_FAILED(rv))
+      NS_WARNING("Could not set startup crash detection pref.");
   }
 
   if (inSafeMode && mIsSafeModeNecessary) {
