@@ -6,7 +6,7 @@
 
 Components.utils.import("resource://gre/modules/AddonRepository.jsm");
 
-do_load_httpd_js();
+Components.utils.import("resource://testing-common/httpd.js");
 let gServer;
 
 const PORT      = 4444;
@@ -35,7 +35,7 @@ const PREF_ADDON1_CACHE_ENABLED = "extensions." + ADDON_IDS[1] + ".getAddons.cac
 const ADDON_PROPERTIES = ["id", "type", "name", "version", "creator",
                           "developers", "translators", "contributors",
                           "description", "fullDescription",
-                          "developerComments", "eula", "iconURL",
+                          "developerComments", "eula", "iconURL", "icons",
                           "screenshots", "homepageURL", "supportURL",
                           "optionsURL", "aboutURL", "contributionURL",
                           "contributionAmount", "averageRating", "reviewCount",
@@ -81,6 +81,7 @@ const REPOSITORY_ADDONS = [{
   developerComments:      "Repo Add-on 1\nDeveloper Comments",
   eula:                   "Repo Add-on 1 - EULA",
   iconURL:                BASE_URL + "/repo/1/icon.png",
+  icons:                  { "32": BASE_URL + "/repo/1/icon.png" },
   homepageURL:            BASE_URL + "/repo/1/homepage.html",
   supportURL:             BASE_URL + "/repo/1/support.html",
   contributionURL:        BASE_URL + "/repo/1/meetDevelopers.html",
@@ -129,6 +130,7 @@ const REPOSITORY_ADDONS = [{
   developerComments:      "Repo Add-on 2 - Developer Comments",
   eula:                   "Repo Add-on 2 - EULA",
   iconURL:                BASE_URL + "/repo/2/icon.png",
+  icons:                  { "32": BASE_URL + "/repo/2/icon.png" },
   screenshots:            [{
                             url:          BASE_URL + "/repo/2/firstFull.png",
                             thumbnailURL: BASE_URL + "/repo/2/firstThumbnail.png",
@@ -155,6 +157,7 @@ const REPOSITORY_ADDONS = [{
   name:                   "Repo Add-on 3",
   version:                "2.3",
   iconURL:                BASE_URL + "/repo/3/icon.png",
+  icons:                  { "32": BASE_URL + "/repo/3/icon.png" },
   screenshots:            [{
                             url:          BASE_URL + "/repo/3/firstFull.png",
                             thumbnailURL: BASE_URL + "/repo/3/firstThumbnail.png",
@@ -182,6 +185,7 @@ const WITHOUT_CACHE = [{
                            { name: "XPI Add-on 1 - Second Contributor" }],
   description:            "XPI Add-on 1 - Description",
   iconURL:                BASE_URL + "/xpi/1/icon.png",
+  icons:                  { "32": BASE_URL + "/xpi/1/icon.png" },
   homepageURL:            BASE_URL + "/xpi/1/homepage.html",
   optionsURL:             BASE_URL + "/xpi/1/options.html",
   aboutURL:               BASE_URL + "/xpi/1/about.html",
@@ -191,7 +195,8 @@ const WITHOUT_CACHE = [{
   type:                   "theme",
   name:                   "XPI Add-on 2",
   version:                "1.2",
-  sourceURI:              NetUtil.newURI(ADDON_FILES[1]).spec
+  sourceURI:              NetUtil.newURI(ADDON_FILES[1]).spec,
+  icons:                  {}
 }, {
   id:                     ADDON_IDS[2],
   type:                   "theme",
@@ -200,6 +205,9 @@ const WITHOUT_CACHE = [{
   get iconURL () {
     return get_subfile_uri(ADDON_IDS[2], "icon.png");
   },
+  get icons () {
+    return { "32": get_subfile_uri(ADDON_IDS[2], "icon.png") };
+  },
   screenshots:            [{ get url () { return get_subfile_uri(ADDON_IDS[2], "preview.png"); } }],
   sourceURI:              NetUtil.newURI(ADDON_FILES[2]).spec
 }];
@@ -226,6 +234,7 @@ const WITH_CACHE = [{
   developerComments:      "Repo Add-on 1\nDeveloper Comments",
   eula:                   "Repo Add-on 1 - EULA",
   iconURL:                BASE_URL + "/xpi/1/icon.png",
+  icons:                  { "32": BASE_URL + "/xpi/1/icon.png" },
   homepageURL:            BASE_URL + "/xpi/1/homepage.html",
   supportURL:             BASE_URL + "/repo/1/support.html",
   optionsURL:             BASE_URL + "/xpi/1/options.html",
@@ -276,6 +285,7 @@ const WITH_CACHE = [{
   developerComments:      "Repo Add-on 2 - Developer Comments",
   eula:                   "Repo Add-on 2 - EULA",
   iconURL:                BASE_URL + "/repo/2/icon.png",
+  icons:                  { "32": BASE_URL + "/repo/2/icon.png" },
   screenshots:            [{
                             url:          BASE_URL + "/repo/2/firstFull.png",
                             thumbnailURL: BASE_URL + "/repo/2/firstThumbnail.png",
@@ -305,6 +315,9 @@ const WITH_CACHE = [{
   get iconURL () {
     return get_subfile_uri(ADDON_IDS[2], "icon.png");
   },
+  get icons () {
+    return { "32": get_subfile_uri(ADDON_IDS[2], "icon.png") };
+  },
   screenshots:            [{
                             url:          BASE_URL + "/repo/3/firstFull.png",
                             thumbnailURL: BASE_URL + "/repo/3/firstThumbnail.png",
@@ -338,6 +351,7 @@ const WITH_EXTENSION_CACHE = [{
   developerComments:      "Repo Add-on 1\nDeveloper Comments",
   eula:                   "Repo Add-on 1 - EULA",
   iconURL:                BASE_URL + "/xpi/1/icon.png",
+  icons:                  { "32": BASE_URL + "/xpi/1/icon.png" },
   homepageURL:            BASE_URL + "/xpi/1/homepage.html",
   supportURL:             BASE_URL + "/repo/1/support.html",
   optionsURL:             BASE_URL + "/xpi/1/options.html",
@@ -372,7 +386,8 @@ const WITH_EXTENSION_CACHE = [{
   type:                   "theme",
   name:                   "XPI Add-on 2",
   version:                "1.2",
-  sourceURI:              NetUtil.newURI(ADDON_FILES[1]).spec
+  sourceURI:              NetUtil.newURI(ADDON_FILES[1]).spec,
+  icons:                  {}
 }, {
   id:                     ADDON_IDS[2],
   type:                   "theme",
@@ -381,6 +396,9 @@ const WITH_EXTENSION_CACHE = [{
   get iconURL () {
     return get_subfile_uri(ADDON_IDS[2], "icon.png");
   },
+  get icons () {
+    return { "32": get_subfile_uri(ADDON_IDS[2], "icon.png") };
+  },
   screenshots:            [{ get url () { return get_subfile_uri(ADDON_IDS[2], "preview.png"); } }],
   sourceURI:              NetUtil.newURI(ADDON_FILES[2]).spec
 }];
@@ -515,7 +533,7 @@ function run_test() {
   installAllFiles(ADDON_FILES, function() {
     restartManager();
 
-    gServer = new nsHttpServer();
+    gServer = new HttpServer();
     gServer.registerDirectory("/data/", do_get_file("data"));
     gServer.start(PORT);
 
@@ -674,6 +692,7 @@ function run_test_12() {
 // database, and that XPI add-ons still do not use any of repository properties
 function run_test_13() {
   check_database_exists(true);
+  Services.prefs.setCharPref(PREF_GETADDONS_BYIDS_PERF, GETADDONS_EMPTY);
 
   trigger_background_update(function() {
     // Database should have been deleted
@@ -690,7 +709,6 @@ function run_test_13() {
 // enabled but has no information
 function run_test_14() {
   Services.prefs.setBoolPref(PREF_GETADDONS_CACHE_ENABLED, true);
-  Services.prefs.setCharPref(PREF_GETADDONS_BYIDS, GETADDONS_EMPTY);
 
   trigger_background_update(function() {
     check_database_exists(true);
