@@ -14,7 +14,7 @@
 
 #include "nsOSHelperAppService.h"
 #include "nsMIMEInfoUnix.h"
-#ifdef MOZ_WIDGET_GTK2
+#ifdef MOZ_WIDGET_GTK
 #include "nsGNOMERegistry.h"
 #endif
 #include "nsISupports.h"
@@ -327,7 +327,7 @@ nsOSHelperAppService::GetTypeAndDescriptionFromMimetypesFile(const nsAString& aF
   nsCOMPtr<nsILineInputStream> mimeTypes;
   bool netscapeFormat;
   nsAutoString buf;
-  nsCAutoString cBuf;
+  nsAutoCString cBuf;
   bool more = false;
   rv = CreateInputStream(aFilename, getter_AddRefs(mimeFile), getter_AddRefs(mimeTypes),
                          cBuf, &netscapeFormat, &more);
@@ -494,7 +494,7 @@ nsOSHelperAppService::GetExtensionsAndDescriptionFromMimetypesFile(const nsAStri
   nsCOMPtr<nsIFileInputStream> mimeFile;
   nsCOMPtr<nsILineInputStream> mimeTypes;
   bool netscapeFormat;
-  nsCAutoString cBuf;
+  nsAutoCString cBuf;
   nsAutoString buf;
   bool more = false;
   rv = CreateInputStream(aFilename, getter_AddRefs(mimeFile), getter_AddRefs(mimeTypes),
@@ -969,7 +969,7 @@ nsOSHelperAppService::GetHandlerAndDescriptionFromMailcapFile(const nsAString& a
   }
 
   nsString entry, buffer;
-  nsCAutoString cBuffer;
+  nsAutoCString cBuffer;
   entry.SetCapacity(128);
   cBuffer.SetCapacity(80);
   rv = mailcap->ReadLine(cBuffer, &more);
@@ -1077,7 +1077,7 @@ nsOSHelperAppService::GetHandlerAndDescriptionFromMailcapFile(const nsAString& a
                 } else if (optionName.EqualsLiteral("x-mozilla-flags")) {
                   aMozillaFlags = Substring(++equal_sign_iter, semicolon_iter);
                 } else if (optionName.EqualsLiteral("test")) {
-                  nsCAutoString testCommand;
+                  nsAutoCString testCommand;
                   rv = UnescapeCommand(Substring(++equal_sign_iter, semicolon_iter),
                                        aMajorType,
                                        aMinorType,
@@ -1160,7 +1160,7 @@ nsresult nsOSHelperAppService::OSProtocolHandlerExists(const char * aProtocolSch
     *aHandlerExists = true;
 #endif
 
-#ifdef MOZ_WIDGET_GTK2
+#ifdef MOZ_WIDGET_GTK
   // Check the GConf registry for a protocol handler
   *aHandlerExists = nsGNOMERegistry::HandlerExists(aProtocolScheme);
 #if (MOZ_PLATFORM_MAEMO == 5) && defined (MOZ_ENABLE_GNOMEVFS)
@@ -1173,7 +1173,7 @@ nsresult nsOSHelperAppService::OSProtocolHandlerExists(const char * aProtocolSch
 
 NS_IMETHODIMP nsOSHelperAppService::GetApplicationDescription(const nsACString& aScheme, nsAString& _retval)
 {
-#ifdef MOZ_WIDGET_GTK2
+#ifdef MOZ_WIDGET_GTK
   nsGNOMERegistry::GetAppDescForScheme(aScheme, _retval);
   return _retval.IsEmpty() ? NS_ERROR_NOT_AVAILABLE : NS_OK;
 #else
@@ -1210,7 +1210,7 @@ nsresult nsOSHelperAppService::GetFileTokenForPath(const PRUnichar * platformApp
   bool exists = false;
   // ugly hack.  Walk the PATH variable...
   char* unixpath = PR_GetEnv("PATH");
-  nsCAutoString path(unixpath);
+  nsAutoCString path(unixpath);
 
   const char* start_iter = path.BeginReading(start_iter);
   const char* colon_iter = start_iter;
@@ -1268,12 +1268,13 @@ nsOSHelperAppService::GetFromExtension(const nsCString& aFileExt) {
 
   if (NS_FAILED(rv) || majorType.IsEmpty()) {
     
-#ifdef MOZ_WIDGET_GTK2
+#ifdef MOZ_WIDGET_GTK
     LOG(("Looking in GNOME registry\n"));
-    nsMIMEInfoBase *gnomeInfo = nsGNOMERegistry::GetFromExtension(aFileExt).get();
+    nsRefPtr<nsMIMEInfoBase> gnomeInfo =
+      nsGNOMERegistry::GetFromExtension(aFileExt);
     if (gnomeInfo) {
       LOG(("Got MIMEInfo from GNOME registry\n"));
-      return gnomeInfo;
+      return gnomeInfo.forget();
     }
 #endif
 
@@ -1300,11 +1301,8 @@ nsOSHelperAppService::GetFromExtension(const nsCString& aFileExt) {
     return nullptr;
   }
 
-  nsCAutoString mimeType(asciiMajorType + NS_LITERAL_CSTRING("/") + asciiMinorType);
-  nsMIMEInfoUnix* mimeInfo = new nsMIMEInfoUnix(mimeType);
-  if (!mimeInfo)
-    return nullptr;
-  NS_ADDREF(mimeInfo);
+  nsAutoCString mimeType(asciiMajorType + NS_LITERAL_CSTRING("/") + asciiMinorType);
+  nsRefPtr<nsMIMEInfoUnix> mimeInfo = new nsMIMEInfoUnix(mimeType);
   
   mimeInfo->AppendExtension(aFileExt);
   nsHashtable typeOptions; // empty hash table
@@ -1342,7 +1340,7 @@ nsOSHelperAppService::GetFromExtension(const nsCString& aFileExt) {
     mimeInfo->SetPreferredAction(nsIMIMEInfo::saveToDisk);
   }
 
-  return mimeInfo;
+  return mimeInfo.forget();
 }
 
 already_AddRefed<nsMIMEInfoBase>
@@ -1388,18 +1386,18 @@ nsOSHelperAppService::GetFromType(const nsCString& aMIMEType) {
           NS_LossyConvertUTF16toASCII(handler).get(),
           NS_LossyConvertUTF16toASCII(mailcap_description).get()));
 
-#ifdef MOZ_WIDGET_GTK2
-  nsMIMEInfoBase *gnomeInfo = nullptr;
+#ifdef MOZ_WIDGET_GTK
+  nsRefPtr<nsMIMEInfoBase> gnomeInfo;
   if (handler.IsEmpty()) {
     // No useful data yet.  Check the GNOME registry.  Unfortunately, newer
     // GNOME versions no longer have type-to-extension mappings, so we might
     // get back a MIMEInfo without any extensions set.  In that case we'll have
     // to look in our mime.types files for the extensions.    
     LOG(("Looking in GNOME registry\n"));
-    gnomeInfo = nsGNOMERegistry::GetFromType(aMIMEType).get();
+    gnomeInfo = nsGNOMERegistry::GetFromType(aMIMEType);
     if (gnomeInfo && gnomeInfo->HasExtensions()) {
       LOG(("Got MIMEInfo from GNOME registry, and it has extensions set\n"));
-      return gnomeInfo;
+      return gnomeInfo.forget();
     }
   }
 #endif
@@ -1411,14 +1409,14 @@ nsOSHelperAppService::GetFromType(const nsCString& aMIMEType) {
                                  extensions,
                                  mime_types_description);
 
-#ifdef MOZ_WIDGET_GTK2
+#ifdef MOZ_WIDGET_GTK
   if (gnomeInfo) {
     LOG(("Got MIMEInfo from GNOME registry without extensions; setting them "
          "to %s\n", NS_LossyConvertUTF16toASCII(extensions).get()));
 
     NS_ASSERTION(!gnomeInfo->HasExtensions(), "How'd that happen?");
     gnomeInfo->SetFileExtensions(NS_ConvertUTF16toUTF8(extensions));
-    return gnomeInfo;
+    return gnomeInfo.forget();
   }
 #endif
 
@@ -1466,10 +1464,7 @@ nsOSHelperAppService::GetFromType(const nsCString& aMIMEType) {
     return nullptr;
   }
   
-  nsMIMEInfoUnix* mimeInfo = new nsMIMEInfoUnix(aMIMEType);
-  if (!mimeInfo)
-    return nullptr;
-  NS_ADDREF(mimeInfo);
+  nsRefPtr<nsMIMEInfoUnix> mimeInfo = new nsMIMEInfoUnix(aMIMEType);
 
   mimeInfo->SetFileExtensions(NS_ConvertUTF16toUTF8(extensions));
   if (! mime_types_description.IsEmpty()) {
@@ -1492,7 +1487,7 @@ nsOSHelperAppService::GetFromType(const nsCString& aMIMEType) {
     mimeInfo->SetPreferredAction(nsIMIMEInfo::saveToDisk);
   }
 
-  return mimeInfo;
+  return mimeInfo.forget();
 }
 
 
@@ -1501,7 +1496,7 @@ nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aType,
                                         const nsACString& aFileExt,
                                         bool       *aFound) {
   *aFound = true;
-  nsMIMEInfoBase* retval = GetFromType(PromiseFlatCString(aType)).get();
+  nsRefPtr<nsMIMEInfoBase> retval = GetFromType(PromiseFlatCString(aType));
   bool hasDefault = false;
   if (retval)
     retval->GetHasDefaultHandler(&hasDefault);
@@ -1509,7 +1504,7 @@ nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aType,
     nsRefPtr<nsMIMEInfoBase> miByExt = GetFromExtension(PromiseFlatCString(aFileExt));
     // If we had no extension match, but a type match, use that
     if (!miByExt && retval)
-      return retval;
+      return retval.forget();
     // If we had an extension match but no type match, set the mimetype and use
     // it
     if (!retval && miByExt) {
@@ -1517,19 +1512,18 @@ nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aType,
         miByExt->SetMIMEType(aType);
       miByExt.swap(retval);
 
-      return retval;
+      return retval.forget();
     }
     // If we got nothing, make a new mimeinfo
     if (!retval) {
       *aFound = false;
       retval = new nsMIMEInfoUnix(aType);
       if (retval) {
-        NS_ADDREF(retval);
         if (!aFileExt.IsEmpty())
           retval->AppendExtension(aFileExt);
       }
       
-      return retval;
+      return retval.forget();
     }
 
     // Copy the attributes of retval (mimeinfo from type) onto miByExt, to
@@ -1542,7 +1536,7 @@ nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aType,
 
     miByExt.swap(retval);
   }
-  return retval;
+  return retval.forget();
 }
 
 NS_IMETHODIMP
