@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Josh Aas <josh@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsMenuItemX.h"
 #include "nsMenuBarX.h"
@@ -52,16 +19,14 @@
 #include "nsIWidget.h"
 #include "nsIDocument.h"
 #include "nsIDOMDocument.h"
-#include "nsIPrivateDOMEvent.h"
-#include "nsIDOMEventTarget.h"
 #include "nsIDOMElement.h"
 
 nsMenuItemX::nsMenuItemX()
 {
   mType           = eRegularMenuItemType;
   mNativeMenuItem = nil;
-  mMenuParent     = nsnull;
-  mMenuGroupOwner = nsnull;
+  mMenuParent     = nullptr;
+  mMenuGroupOwner = nullptr;
   mIsChecked      = false;
 
   MOZ_COUNT_CTOR(nsMenuItemX);
@@ -92,7 +57,7 @@ nsMenuItemX::~nsMenuItemX()
 struct macKeyCodeData {
   const char* str;
   size_t strlength;
-  PRUint32 keycode;
+  uint32_t keycode;
 };
 
 static const macKeyCodeData gMacKeyCodes[] = {
@@ -219,20 +184,20 @@ static const macKeyCodeData gMacKeyCodes[] = {
 
 };
 
-PRUint32 nsMenuItemX::ConvertGeckoToMacKeyCode(nsAString& aKeyCodeName)
+uint32_t nsMenuItemX::ConvertGeckoToMacKeyCode(nsAString& aKeyCodeName)
 {
   if (aKeyCodeName.IsEmpty()) {
     return 0;
   }
 
-  nsCAutoString keyCodeName;
+  nsAutoCString keyCodeName;
   keyCodeName.AssignWithConversion(aKeyCodeName);
   // We want case-insensitive comparison with data stored as uppercase.
   ToUpperCase(keyCodeName);
 
-  PRUint32 keyCodeNameLength = keyCodeName.Length();
+  uint32_t keyCodeNameLength = keyCodeName.Length();
   const char* keyCodeNameStr = keyCodeName.get();
-  for (PRUint16 i = 0; i < (sizeof(gMacKeyCodes) / sizeof(gMacKeyCodes[0])); ++i) {
+  for (uint16_t i = 0; i < (sizeof(gMacKeyCodes) / sizeof(gMacKeyCodes[0])); ++i) {
     if (keyCodeNameLength == gMacKeyCodes[i].strlength &&
         nsCRT::strcmp(gMacKeyCodes[i].str, keyCodeNameStr) == 0) {
       return gMacKeyCodes[i].keycode;
@@ -375,18 +340,16 @@ nsresult nsMenuItemX::DispatchDOMEvent(const nsString &eventName, bool *preventD
   event->InitEvent(eventName, true, true);
 
   // mark DOM event as trusted
-  nsCOMPtr<nsIPrivateDOMEvent> privateEvent(do_QueryInterface(event));
-  privateEvent->SetTrusted(true);
+  event->SetTrusted(true);
 
   // send DOM event
-  nsCOMPtr<nsIDOMEventTarget> eventTarget = do_QueryInterface(mContent);
-  rv = eventTarget->DispatchEvent(event, preventDefaultCalled);
+  rv = mContent->DispatchEvent(event, preventDefaultCalled);
   if (NS_FAILED(rv)) {
-    NS_WARNING("Failed to send DOM event via nsIDOMEventTarget");
+    NS_WARNING("Failed to send DOM event via EventTarget");
     return rv;
   }
 
-  return NS_OK;  
+  return NS_OK;
 }
 
 // Walk the sibling list looking for nodes with the same name and
@@ -397,14 +360,14 @@ void nsMenuItemX::UncheckRadioSiblings(nsIContent* inCheckedContent)
   inCheckedContent->GetAttr(kNameSpaceID_None, nsGkAtoms::name, myGroupName);
   if (!myGroupName.Length()) // no groupname, nothing to do
     return;
-  
+
   nsCOMPtr<nsIContent> parent = inCheckedContent->GetParent();
   if (!parent)
     return;
 
   // loop over siblings
-  PRUint32 count = parent->GetChildCount();
-  for (PRUint32 i = 0; i < count; i++) {
+  uint32_t count = parent->GetChildCount();
+  for (uint32_t i = 0; i < count; i++) {
     nsIContent *sibling = parent->GetChildAt(i);
     if (sibling) {      
       if (sibling != inCheckedContent) { // skip this node
@@ -433,7 +396,7 @@ void nsMenuItemX::SetKeyEquiv()
       if (!hasKey || keyChar.IsEmpty()) {
         nsAutoString keyCodeName;
         keyContent->GetAttr(kNameSpaceID_None, nsGkAtoms::keycode, keyCodeName);
-        PRUint32 keycode = ConvertGeckoToMacKeyCode(keyCodeName);
+        uint32_t keycode = ConvertGeckoToMacKeyCode(keyCodeName);
         if (keycode) {
           keyChar.Assign(keycode);
         }
@@ -444,7 +407,7 @@ void nsMenuItemX::SetKeyEquiv()
 
       nsAutoString modifiersStr;
       keyContent->GetAttr(kNameSpaceID_None, nsGkAtoms::modifiers, modifiersStr);
-      PRUint8 modifiers = nsMenuUtilsX::GeckoModifiersForNodeAttribute(modifiersStr);
+      uint8_t modifiers = nsMenuUtilsX::GeckoModifiersForNodeAttribute(modifiersStr);
 
       unsigned int macModifiers = nsMenuUtilsX::MacModifiersForGeckoModifiers(modifiers);
       [mNativeMenuItem setKeyEquivalentModifierMask:macModifiers];
@@ -534,11 +497,11 @@ nsMenuItemX::ObserveAttributeChanged(nsIDocument *aDocument, nsIContent *aConten
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-void nsMenuItemX::ObserveContentRemoved(nsIDocument *aDocument, nsIContent *aChild, PRInt32 aIndexInContainer)
+void nsMenuItemX::ObserveContentRemoved(nsIDocument *aDocument, nsIContent *aChild, int32_t aIndexInContainer)
 {
   if (aChild == mCommandContent) {
     mMenuGroupOwner->UnregisterForContentChanges(mCommandContent);
-    mCommandContent = nsnull;
+    mCommandContent = nullptr;
   }
 
   mMenuParent->SetRebuild(true);
