@@ -1,40 +1,9 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* vim:expandtab:shiftwidth=4:tabstop=4:
  */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is Christopher Blizzard
- * <blizzard@mozilla.org>.  Portions created by the Initial Developer
- * are Copyright (C) 2001 the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef __nsGdkKeyUtils_h__
 #define __nsGdkKeyUtils_h__
@@ -64,14 +33,19 @@ public:
     /**
      * Compute an our DOM keycode from a GDK keyval.
      */
-    static PRUint32 ComputeDOMKeyCode(const GdkEventKey* aGdkKeyEvent);
+    static uint32_t ComputeDOMKeyCode(const GdkEventKey* aGdkKeyEvent);
+
+    /**
+     * Compute a DOM key name index from aGdkKeyEvent.
+     */
+    KeyNameIndex ComputeDOMKeyNameIndex(const GdkEventKey* aGdkKeyEvent);
 
     /**
      * Returns a GDK keyval which is related to the aDOMKeyCode.  However,
      * it may not be same as original value since there are some lost
      * information.
      */
-    static guint GuessGDKKeyval(PRUint32 aDOMKeyCode);
+    static guint GuessGDKKeyval(uint32_t aDOMKeyCode);
 
     /**
      * Modifier is list of modifiers which we support in widget level.
@@ -84,17 +58,18 @@ public:
         SHIFT              = 0x0008,
         CTRL               = 0x0010,
         ALT                = 0x0020,
-        SUPER              = 0x0040,
-        HYPER              = 0x0080,
-        META               = 0x0100,
-        ALTGR              = 0x0200
+        META               = 0x0040,
+        SUPER              = 0x0080,
+        HYPER              = 0x0100,
+        LEVEL3             = 0x0200,
+        LEVEL5             = 0x0400
     };
 
     /**
      * Modifiers is used for combination of Modifier.
      * E.g., |Modifiers modifiers = (SHIFT | CTRL);| means Shift and Ctrl.
      */
-    typedef PRUint32 Modifiers;
+    typedef uint32_t Modifiers;
 
     /**
      * GetCurrentModifierState() returns current modifier key state.
@@ -201,10 +176,11 @@ protected:
         INDEX_NUM_LOCK,
         INDEX_SCROLL_LOCK,
         INDEX_ALT,
+        INDEX_META,
         INDEX_SUPER,
         INDEX_HYPER,
-        INDEX_META,
-        INDEX_ALTGR,
+        INDEX_LEVEL3,
+        INDEX_LEVEL5,
         COUNT_OF_MODIFIER_INDEX
     };
     guint mModifierMasks[COUNT_OF_MODIFIER_INDEX];
@@ -252,8 +228,8 @@ protected:
      * @return                  charCode which is inputted by aGdkKeyEvent.
      *                          If failed, this returns 0.
      */
-    static PRUint32 GetCharCodeFor(const GdkEventKey *aGdkKeyEvent);
-    PRUint32 GetCharCodeFor(const GdkEventKey *aGdkKeyEvent,
+    static uint32_t GetCharCodeFor(const GdkEventKey *aGdkKeyEvent);
+    uint32_t GetCharCodeFor(const GdkEventKey *aGdkKeyEvent,
                             guint aModifierState,
                             gint aGroup);
 
@@ -266,6 +242,24 @@ protected:
      */
     gint GetKeyLevel(GdkEventKey *aGdkKeyEvent);
 
+    /**
+     * GetFirstLatinGroup() returns group of mGdkKeymap which can input an
+     * ASCII character by GDK_A.
+     *
+     * @return                  group value of GdkEventKey.
+     */
+    gint GetFirstLatinGroup();
+
+    /**
+     * IsLatinGroup() checkes whether the keyboard layout of aGroup is
+     * ASCII alphabet inputtable or not.
+     *
+     * @param aGroup            The group value of GdkEventKey.
+     * @return                  TRUE if the keyboard layout can input
+     *                          ASCII alphabet.  Otherwise, FALSE.
+     */
+    bool IsLatinGroup(guint8 aGroup);
+
     /**
      * IsBasicLatinLetterOrNumeral() Checks whether the aCharCode is an
      * alphabet or a numeric character in ASCII.
@@ -274,7 +268,20 @@ protected:
      * @return                  TRUE if aCharCode is an alphabet or a numeric
      *                          in ASCII range.  Otherwise, FALSE.
      */
-    static bool IsBasicLatinLetterOrNumeral(PRUint32 aCharCode);
+    static bool IsBasicLatinLetterOrNumeral(uint32_t aCharCode);
+
+    /**
+     * GetGDKKeyvalWithoutModifier() returns the keyval for aGdkKeyEvent when
+     * ignoring the modifier state except NumLock. (NumLock is a key to change
+     * some key's meaning.)
+     */
+    static guint GetGDKKeyvalWithoutModifier(const GdkEventKey *aGdkKeyEvent);
+
+    /**
+     * GetDOMKeyCodeFromKeyPairs() returns DOM keycode for aGdkKeyval if
+     * it's in KeyPair table.
+     */
+    static uint32_t GetDOMKeyCodeFromKeyPairs(guint aGdkKeyval);
 
     /**
      * InitKeypressEvent() intializes keyCode, charCode and
