@@ -1,42 +1,9 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* vim:expandtab:shiftwidth=4:tabstop=4:
  */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2000
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Stuart Parmenter <pavlov@netscape.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include <string.h>
 
@@ -63,7 +30,7 @@
 #include <unistd.h>
 
 #include <gtk/gtk.h>
-static PRLibrary *libcanberra = nsnull;
+static PRLibrary *libcanberra = nullptr;
 
 /* used to play sounds with libcanberra. */
 typedef struct _ca_context ca_context;
@@ -102,7 +69,7 @@ static ca_proplist_sets_fn ca_proplist_sets;
 static ca_context_play_full_fn ca_context_play_full;
 
 struct ScopedCanberraFile {
-    ScopedCanberraFile(nsILocalFile *file): mFile(file) {};
+    ScopedCanberraFile(nsIFile *file): mFile(file) {};
 
     ~ScopedCanberraFile() {
         if (mFile) {
@@ -113,10 +80,10 @@ struct ScopedCanberraFile {
     void forget() {
         mFile.forget();
     }
-    nsILocalFile* operator->() { return mFile; }
-    operator nsILocalFile*() { return mFile; }
+    nsIFile* operator->() { return mFile; }
+    operator nsIFile*() { return mFile; }
 
-    nsCOMPtr<nsILocalFile> mFile;
+    nsCOMPtr<nsIFile> mFile;
 };
 
 static ca_context*
@@ -134,7 +101,7 @@ ca_context_get_default()
 
     ca_context_create(&ctx);
     if (!ctx) {
-        return nsnull;
+        return nullptr;
     }
 
     g_static_private_set(&ctx_static_private, ctx, (GDestroyNotify) ca_context_destroy);
@@ -142,7 +109,7 @@ ca_context_get_default()
     GtkSettings* settings = gtk_settings_get_default();
     if (g_object_class_find_property(G_OBJECT_GET_CLASS(settings),
                                      "gtk-sound-theme-name")) {
-        gchar* sound_theme_name = nsnull;
+        gchar* sound_theme_name = nullptr;
         g_object_get(settings, "gtk-sound-theme-name", &sound_theme_name, NULL);
 
         if (sound_theme_name) {
@@ -169,7 +136,7 @@ ca_context_get_default()
 
     nsCOMPtr<nsIXULAppInfo> appInfo = do_GetService("@mozilla.org/xre/app-info;1");
     if (appInfo) {
-        nsCAutoString version;
+        nsAutoCString version;
         appInfo->GetVersion(version);
 
         ca_context_change_props(ctx, "application.version", version.get(), NULL);
@@ -186,7 +153,7 @@ ca_finish_cb(ca_context *c,
              int error_code,
              void *userdata)
 {
-    nsILocalFile *file = reinterpret_cast<nsILocalFile *>(userdata);
+    nsIFile *file = reinterpret_cast<nsIFile *>(userdata);
     if (file) {
         file->Remove(false);
         NS_RELEASE(file);
@@ -221,7 +188,7 @@ nsSound::Init()
             ca_context_create = (ca_context_create_fn) PR_FindFunctionSymbol(libcanberra, "ca_context_create");
             if (!ca_context_create) {
                 PR_UnloadLibrary(libcanberra);
-                libcanberra = nsnull;
+                libcanberra = nullptr;
             } else {
                 // at this point we know we have a good libcanberra library
                 ca_context_destroy = (ca_context_destroy_fn) PR_FindFunctionSymbol(libcanberra, "ca_context_destroy");
@@ -243,15 +210,15 @@ nsSound::Shutdown()
 {
     if (libcanberra) {
         PR_UnloadLibrary(libcanberra);
-        libcanberra = nsnull;
+        libcanberra = nullptr;
     }
 }
 
 NS_IMETHODIMP nsSound::OnStreamComplete(nsIStreamLoader *aLoader,
                                         nsISupports *context,
                                         nsresult aStatus,
-                                        PRUint32 dataLen,
-                                        const PRUint8 *data)
+                                        uint32_t dataLen,
+                                        const uint8_t *data)
 {
     // print a load error on bad status, and return
     if (NS_FAILED(aStatus)) {
@@ -265,7 +232,7 @@ NS_IMETHODIMP nsSound::OnStreamComplete(nsIStreamLoader *aLoader,
                 if (channel) {
                       channel->GetURI(getter_AddRefs(uri));
                       if (uri) {
-                            nsCAutoString uriSpec;
+                            nsAutoCString uriSpec;
                             uri->GetSpec(uriSpec);
                             printf("Failed to load %s\n", uriSpec.get());
                       }
@@ -276,8 +243,8 @@ NS_IMETHODIMP nsSound::OnStreamComplete(nsIStreamLoader *aLoader,
         return aStatus;
     }
 
-    nsCOMPtr<nsILocalFile> tmpFile;
-    nsDirectoryService::gService->Get(NS_OS_TEMP_DIR, NS_GET_IID(nsILocalFile),
+    nsCOMPtr<nsIFile> tmpFile;
+    nsDirectoryService::gService->Get(NS_OS_TEMP_DIR, NS_GET_IID(nsIFile),
                                       getter_AddRefs(tmpFile));
 
     nsresult rv = tmpFile->AppendNative(nsDependentCString("mozilla_audio_sample"));
@@ -300,9 +267,9 @@ NS_IMETHODIMP nsSound::OnStreamComplete(nsIStreamLoader *aLoader,
     }
 
     // XXX: Should we do this on another thread?
-    PRUint32 length = dataLen;
+    uint32_t length = dataLen;
     while (length > 0) {
-        PRInt32 amount = PR_Write(fd, data, length);
+        int32_t amount = PR_Write(fd, data, length);
         if (amount < 0) {
             return NS_ERROR_FAILURE;
         }
@@ -321,7 +288,7 @@ NS_IMETHODIMP nsSound::OnStreamComplete(nsIStreamLoader *aLoader,
         return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    nsCAutoString path;
+    nsAutoCString path;
     rv = canberraFile->GetNativePath(path);
     if (NS_FAILED(rv)) {
         return rv;
@@ -359,7 +326,7 @@ NS_METHOD nsSound::Play(nsIURL *aURL)
             return NS_ERROR_OUT_OF_MEMORY;
         }
 
-        nsCAutoString spec;
+        nsAutoCString spec;
         rv = aURL->GetSpec(spec);
         if (NS_FAILED(rv)) {
             return rv;
@@ -379,7 +346,7 @@ NS_METHOD nsSound::Play(nsIURL *aURL)
     return rv;
 }
 
-NS_IMETHODIMP nsSound::PlayEventSound(PRUint32 aEventId)
+NS_IMETHODIMP nsSound::PlayEventSound(uint32_t aEventId)
 {
     if (!mInited)
         Init();
@@ -429,7 +396,7 @@ NS_IMETHODIMP nsSound::PlaySystemSound(const nsAString &aSoundAlias)
 {
     if (NS_IsMozAliasSound(aSoundAlias)) {
         NS_WARNING("nsISound::playSystemSound is called with \"_moz_\" events, they are obsolete, use nsISound::playEventSound instead");
-        PRUint32 eventId;
+        uint32_t eventId;
         if (aSoundAlias.Equals(NS_SYSSOUND_ALERT_DIALOG))
             eventId = EVENT_ALERT_DIALOG_OPEN;
         else if (aSoundAlias.Equals(NS_SYSSOUND_CONFIRM_DIALOG))
@@ -448,8 +415,8 @@ NS_IMETHODIMP nsSound::PlaySystemSound(const nsAString &aSoundAlias)
     nsresult rv;
     nsCOMPtr <nsIURI> fileURI;
 
-    // create a nsILocalFile and then a nsIFileURL from that
-    nsCOMPtr <nsILocalFile> soundFile;
+    // create a nsIFile and then a nsIFileURL from that
+    nsCOMPtr <nsIFile> soundFile;
     rv = NS_NewLocalFile(aSoundAlias, true, 
                          getter_AddRefs(soundFile));
     NS_ENSURE_SUCCESS(rv,rv);
