@@ -1,39 +1,8 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* vim: set ts=4 et sw=4 tw=80: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Nokia.
- *
- * The Initial Developer of the Original Code is Nokia Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include <QGraphicsSceneHoverEvent>
 #include <QGraphicsSceneMouseEvent>
@@ -41,7 +10,8 @@
 #include <QInputContext>
 #endif
 #include <QtCore/QTimer>
-
+// Solve conflict of qgl.h and GLDefs.h
+#define GLdouble_defined 1
 #include "mozqwidget.h"
 #include "nsWindow.h"
 
@@ -107,7 +77,8 @@ MozQWidget::~MozQWidget()
 
 void MozQWidget::paint(QPainter* aPainter, const QStyleOptionGraphicsItem* aOption, QWidget* aWidget /*= 0*/)
 {
-    mReceiver->DoPaint(aPainter, aOption, aWidget);
+    if (mReceiver)
+        mReceiver->DoPaint(aPainter, aOption, aWidget);
 }
 
 void MozQWidget::activate()
@@ -435,17 +406,21 @@ void MozQWidget::closeEvent(QCloseEvent* aEvent)
 
 void MozQWidget::hideEvent(QHideEvent* aEvent)
 {
-    mReceiver->hideEvent(aEvent);
+    if (mReceiver) {
+        mReceiver->hideEvent(aEvent);
+    }
     QGraphicsWidget::hideEvent(aEvent);
 }
 
 void MozQWidget::showEvent(QShowEvent* aEvent)
 {
-    mReceiver->showEvent(aEvent);
+    if (mReceiver) {
+        mReceiver->showEvent(aEvent);
+    }
     QGraphicsWidget::showEvent(aEvent);
 }
 
-bool MozQWidget::SetCursor(nsCursor aCursor)
+void MozQWidget::SetCursor(nsCursor aCursor)
 {
     Qt::CursorShape cursor = Qt::ArrowCursor;
     switch(aCursor) {
@@ -498,16 +473,12 @@ bool MozQWidget::SetCursor(nsCursor aCursor)
     }
 
     setCursor(cursor);
-
-    return NS_OK;
 }
 
-bool MozQWidget::SetCursor(const QPixmap& aCursor, int aHotX, int aHotY)
+void MozQWidget::SetCursor(const QPixmap& aCursor, int aHotX, int aHotY)
 {
     QCursor bitmapCursor(aCursor, aHotX, aHotY);
     setCursor(bitmapCursor);
-
-    return NS_OK;
 }
 
 void MozQWidget::setModal(bool modal)
@@ -655,7 +626,7 @@ MozQWidget::NotifyVKB(const QRect& rect)
     if (observerService) {
         QString rect = QString("{\"left\": %1, \"top\": %2, \"right\": %3, \"bottom\": %4}")
                                .arg(bounds.x()).arg(bounds.y()).arg(bounds.width()).arg(bounds.height());
-        observerService->NotifyObservers(nsnull, "softkb-change", rect.utf16());
+        observerService->NotifyObservers(nullptr, "softkb-change", rect.utf16());
     }
 }
 
@@ -664,7 +635,7 @@ void MozQWidget::SwitchToForeground()
     nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
     if (!os)
         return;
-    os->NotifyObservers(nsnull, "application-foreground", nsnull);
+    os->NotifyObservers(nullptr, "application-foreground", nullptr);
 }
 
 void MozQWidget::SwitchToBackground()
@@ -672,6 +643,6 @@ void MozQWidget::SwitchToBackground()
     nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
     if (!os)
         return;
-    os->NotifyObservers(nsnull, "application-background", nsnull);
+    os->NotifyObservers(nullptr, "application-background", nullptr);
 }
 
