@@ -1,42 +1,6 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * David Hyatt (hyatt@netscape.com).
- * Portions created by the Initial Developer are Copyright (C) 2001
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Lars Knoll <knoll@kde.org>
- *   Zack Rusin <zack@kde.org>
- *   Tim Hill (tim@prismelite.com)
- *   Vladimir Vukicevic <vladimir@pobox.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include <QApplication>
 #include <QStyle>
@@ -61,7 +25,6 @@
 #include "nsThemeConstants.h"
 #include "nsIServiceManager.h"
 #include "nsIDOMHTMLInputElement.h"
-#include <malloc.h>
 
 
 #include "gfxASurface.h"
@@ -86,7 +49,7 @@ nsNativeThemeQt::~nsNativeThemeQt()
 NS_IMPL_ISUPPORTS_INHERITED1(nsNativeThemeQt, nsNativeTheme, nsITheme)
 
 static inline QRect qRectInPixels(const nsRect &aRect,
-                                  const PRInt32 p2a)
+                                  const int32_t p2a)
 {
     return QRect(NSAppUnitsToIntPixels(aRect.x, p2a),
                  NSAppUnitsToIntPixels(aRect.y, p2a),
@@ -122,7 +85,7 @@ _qimage_from_gfximage_format (gfxASurface::gfxImageFormat aFormat)
 NS_IMETHODIMP
 nsNativeThemeQt::DrawWidgetBackground(nsRenderingContext* aContext,
                                       nsIFrame* aFrame,
-                                      PRUint8 aWidgetType,
+                                      uint8_t aWidgetType,
                                       const nsRect& aRect,
                                       const nsRect& aClipRect)
 {
@@ -171,7 +134,7 @@ nsresult
 nsNativeThemeQt::DrawWidgetBackground(QPainter *qPainter,
                                       nsRenderingContext* aContext,
                                       nsIFrame* aFrame,
-                                      PRUint8 aWidgetType,
+                                      uint8_t aWidgetType,
                                       const nsRect& aRect,
                                       const nsRect& aClipRect)
 
@@ -197,7 +160,7 @@ nsNativeThemeQt::DrawWidgetBackground(QPainter *qPainter,
     QMatrix qctm(ctm.xx, ctm.yx, ctm.xy, ctm.yy, ctm.x0, ctm.y0);
     qPainter->setWorldMatrix(qctm, true);
 
-    PRInt32 p2a = aContext->AppUnitsPerDevPixel();
+    int32_t p2a = aContext->AppUnitsPerDevPixel();
 
     QRect r = qRectInPixels(aRect, p2a);
     QRect cr = qRectInPixels(aClipRect, p2a);
@@ -346,7 +309,7 @@ nsNativeThemeQt::DrawWidgetBackground(QPainter *qPainter,
 NS_IMETHODIMP
 nsNativeThemeQt::GetWidgetBorder(nsDeviceContext* ,
                                  nsIFrame* aFrame,
-                                 PRUint8 aWidgetType,
+                                 uint8_t aWidgetType,
                                  nsIntMargin* aResult)
 {
     (*aResult).top = (*aResult).bottom = (*aResult).left = (*aResult).right = 0;
@@ -368,7 +331,7 @@ nsNativeThemeQt::GetWidgetBorder(nsDeviceContext* ,
 
 bool
 nsNativeThemeQt::GetWidgetPadding(nsDeviceContext* ,
-                                  nsIFrame*, PRUint8 aWidgetType,
+                                  nsIFrame*, uint8_t aWidgetType,
                                   nsIntMargin* aResult)
 {
     // XXX: Where to get padding values, framewidth?
@@ -384,7 +347,7 @@ nsNativeThemeQt::GetWidgetPadding(nsDeviceContext* ,
 
 NS_IMETHODIMP
 nsNativeThemeQt::GetMinimumWidgetSize(nsRenderingContext* aContext, nsIFrame* aFrame,
-                                      PRUint8 aWidgetType,
+                                      uint8_t aWidgetType,
                                       nsIntSize* aResult, bool* aIsOverridable)
 {
     (*aResult).width = (*aResult).height = 0;
@@ -392,7 +355,7 @@ nsNativeThemeQt::GetMinimumWidgetSize(nsRenderingContext* aContext, nsIFrame* aF
 
     QStyle *s = qApp->style();
 
-    PRInt32 p2a = aContext->AppUnitsPerDevPixel();
+    int32_t p2a = aContext->AppUnitsPerDevPixel();
 
     switch (aWidgetType) {
     case NS_THEME_RADIO:
@@ -546,10 +509,10 @@ nsNativeThemeQt::GetMinimumWidgetSize(nsRenderingContext* aContext, nsIFrame* aF
 }
 
 NS_IMETHODIMP
-nsNativeThemeQt::WidgetStateChanged(nsIFrame* aFrame, PRUint8 aWidgetType,
+nsNativeThemeQt::WidgetStateChanged(nsIFrame* aFrame, uint8_t aWidgetType,
                                     nsIAtom* aAttribute, bool* aShouldRepaint)
 {
-    *aShouldRepaint = TRUE;
+    *aShouldRepaint = true;
     return NS_OK;
 }
 
@@ -566,7 +529,7 @@ nsNativeThemeQt::ThemeChanged()
 bool
 nsNativeThemeQt::ThemeSupportsWidget(nsPresContext* aPresContext,
                                      nsIFrame* aFrame,
-                                     PRUint8 aWidgetType)
+                                     uint8_t aWidgetType)
 {
     switch (aWidgetType) {
     case NS_THEME_SCROLLBAR:
@@ -600,7 +563,7 @@ nsNativeThemeQt::ThemeSupportsWidget(nsPresContext* aPresContext,
 }
 
 bool
-nsNativeThemeQt::WidgetIsContainer(PRUint8 aWidgetType)
+nsNativeThemeQt::WidgetIsContainer(uint8_t aWidgetType)
 {
 //     if (aWidgetType == NS_THEME_DROPDOWN_BUTTON ||
 //         aWidgetType == NS_THEME_RADIO ||
@@ -612,7 +575,7 @@ nsNativeThemeQt::WidgetIsContainer(PRUint8 aWidgetType)
 }
 
 bool
-nsNativeThemeQt::ThemeDrawsFocusForWidget(nsPresContext* aPresContext, nsIFrame* aFrame, PRUint8 aWidgetType)
+nsNativeThemeQt::ThemeDrawsFocusForWidget(uint8_t aWidgetType)
 {
     if (aWidgetType == NS_THEME_DROPDOWN ||
         aWidgetType == NS_THEME_BUTTON || 
@@ -630,7 +593,7 @@ nsNativeThemeQt::ThemeNeedsComboboxDropmarker()
 }
 
 void
-nsNativeThemeQt::InitButtonStyle(PRUint8 aWidgetType,
+nsNativeThemeQt::InitButtonStyle(uint8_t aWidgetType,
                                  nsIFrame* aFrame,
                                  QRect rect,
                                  QStyleOptionButton &opt)
@@ -669,7 +632,7 @@ nsNativeThemeQt::InitButtonStyle(PRUint8 aWidgetType,
 }
 
 void
-nsNativeThemeQt::InitPlainStyle(PRUint8 aWidgetType,
+nsNativeThemeQt::InitPlainStyle(uint8_t aWidgetType,
                                 nsIFrame* aFrame,
                                 QRect rect,
                                 QStyleOption &opt,
@@ -690,7 +653,7 @@ nsNativeThemeQt::InitPlainStyle(PRUint8 aWidgetType,
 }
 
 void
-nsNativeThemeQt::InitComboStyle(PRUint8 aWidgetType,
+nsNativeThemeQt::InitComboStyle(uint8_t aWidgetType,
                                 nsIFrame* aFrame,
                                 QRect rect,
                                 QStyleOptionComboBox &opt)
