@@ -1,42 +1,9 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* vim:expandtab:shiftwidth=4:tabstop=4:
  */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is Christopher Blizzard
- * <blizzard@mozilla.org>.  Portions created by the Initial Developer
- * are Copyright (C) 2001 the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Masayuki Nakano <masayuki@d-toybox.com>
- *   Vladimir Vukicevic <vladimir@pobox.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef __nsWindow_h__
 #define __nsWindow_h__
@@ -54,6 +21,9 @@
 #include "nsWeakReference.h"
 
 #include "nsGkAtoms.h"
+#include "nsIIdleServiceInternal.h"
+#include "nsIRunnable.h"
+#include "nsThreadUtils.h"
 
 #ifdef MOZ_LOGGING
 
@@ -112,7 +82,7 @@ public:
     nsWindow();
     virtual ~nsWindow();
 
-    nsEventStatus DoPaint( QPainter* aPainter, const QStyleOptionGraphicsItem * aOption, QWidget* aWidget);
+    bool DoPaint( QPainter* aPainter, const QStyleOptionGraphicsItem * aOption, QWidget* aWidget);
 
     static void ReleaseGlobals();
 
@@ -127,15 +97,13 @@ public:
     NS_IMETHOD         Create(nsIWidget        *aParent,
                               nsNativeWidget   aNativeParent,
                               const nsIntRect  &aRect,
-                              EVENT_CALLBACK   aHandleEventFunction,
                               nsDeviceContext *aContext,
                               nsWidgetInitData *aInitData);
 
     virtual already_AddRefed<nsIWidget>
     CreateChild(const nsIntRect&  aRect,
-                EVENT_CALLBACK    aHandleEventFunction,
                 nsDeviceContext* aContext,
-                nsWidgetInitData* aInitData = nsnull,
+                nsWidgetInitData* aInitData = nullptr,
                 bool              aForceUseIWidgetParent = true);
 
     NS_IMETHOD         Destroy(void);
@@ -144,24 +112,24 @@ public:
     virtual float      GetDPI();
     NS_IMETHOD         Show(bool aState);
     NS_IMETHOD         SetModal(bool aModal);
-    NS_IMETHOD         IsVisible(bool & aState);
+    virtual bool       IsVisible() const;
     NS_IMETHOD         ConstrainPosition(bool aAllowSlop,
-                                         PRInt32 *aX,
-                                         PRInt32 *aY);
-    NS_IMETHOD         Move(PRInt32 aX,
-                            PRInt32 aY);
-    NS_IMETHOD         Resize(PRInt32 aWidth,
-                              PRInt32 aHeight,
-                              bool    aRepaint);
-    NS_IMETHOD         Resize(PRInt32 aX,
-                              PRInt32 aY,
-                              PRInt32 aWidth,
-                              PRInt32 aHeight,
-                              bool     aRepaint);
+                                         int32_t *aX,
+                                         int32_t *aY);
+    NS_IMETHOD         Move(double aX,
+                            double aY);
+    NS_IMETHOD         Resize(double aWidth,
+                              double aHeight,
+                              bool   aRepaint);
+    NS_IMETHOD         Resize(double aX,
+                              double aY,
+                              double aWidth,
+                              double aHeight,
+                              bool   aRepaint);
     NS_IMETHOD         PlaceBehind(nsTopLevelWidgetZPlacement  aPlacement,
                                    nsIWidget                  *aWidget,
                                    bool                        aActivate);
-    NS_IMETHOD         SetSizeMode(PRInt32 aMode);
+    NS_IMETHOD         SetSizeMode(int32_t aMode);
     NS_IMETHOD         Enable(bool aState);
     NS_IMETHOD         SetFocus(bool aRaise = false);
     NS_IMETHOD         GetScreenBounds(nsIntRect &aRect);
@@ -169,14 +137,14 @@ public:
     NS_IMETHOD         SetBackgroundColor(const nscolor &aColor);
     NS_IMETHOD         SetCursor(nsCursor aCursor);
     NS_IMETHOD         SetCursor(imgIContainer* aCursor,
-                                 PRUint32 aHotspotX, PRUint32 aHotspotY);
+                                 uint32_t aHotspotX, uint32_t aHotspotY);
     NS_IMETHOD         SetHasTransparentBackground(bool aTransparent);
     NS_IMETHOD         GetHasTransparentBackground(bool& aTransparent);
     NS_IMETHOD         HideWindowChrome(bool aShouldHide);
     NS_IMETHOD         MakeFullScreen(bool aFullScreen);
     NS_IMETHOD         Invalidate(const nsIntRect &aRect);
 
-    virtual void*      GetNativeData(PRUint32 aDataType);
+    virtual void*      GetNativeData(uint32_t aDataType);
     NS_IMETHOD         SetTitle(const nsAString& aTitle);
     NS_IMETHOD         SetIcon(const nsAString& aIconSpec);
     virtual nsIntPoint WidgetToScreenOffset();
@@ -185,13 +153,12 @@ public:
     NS_IMETHOD         EnableDragDrop(bool aEnable);
     NS_IMETHOD         CaptureMouse(bool aCapture);
     NS_IMETHOD         CaptureRollupEvents(nsIRollupListener *aListener,
-                                           bool aDoCapture,
-                                           bool aConsumeRollupEvent);
+                                           bool aDoCapture);
 
     NS_IMETHOD         SetWindowClass(const nsAString& xulWinType);
 
-    NS_IMETHOD         GetAttention(PRInt32 aCycleCount);
-    NS_IMETHOD         BeginResizeDrag   (nsGUIEvent* aEvent, PRInt32 aHorizontal, PRInt32 aVertical);
+    NS_IMETHOD         GetAttention(int32_t aCycleCount);
+    NS_IMETHOD         BeginResizeDrag   (nsGUIEvent* aEvent, int32_t aHorizontal, int32_t aVertical);
 
     NS_IMETHOD_(void) SetInputContext(const InputContext& aContext,
                                       const InputContextAction& aAction);
@@ -220,7 +187,7 @@ public:
     }
 
     // Some of the nsIWidget methods
-    NS_IMETHOD         IsEnabled        (bool *aState);
+    virtual bool IsEnabled() const;
 
     // called when we are destroyed
     void OnDestroy(void);
@@ -231,7 +198,7 @@ public:
     NS_IMETHOD         ReparentNativeWidget(nsIWidget* aNewParent);
 
     QWidget* GetViewWidget();
-    virtual PRUint32 GetGLFrameBufferFormat() MOZ_OVERRIDE;
+    virtual uint32_t GetGLFrameBufferFormat() MOZ_OVERRIDE;
 
 protected:
     nsCOMPtr<nsIWidget> mParent;
@@ -292,20 +259,20 @@ protected:
     virtual nsEventStatus OnTouchEvent(QTouchEvent *event, bool &handled);
 
     virtual nsEventStatus OnGestureEvent(QGestureEvent *event, bool &handled);
-    nsEventStatus DispatchGestureEvent(PRUint32 aMsg, PRUint32 aDirection,
+    nsEventStatus DispatchGestureEvent(uint32_t aMsg, uint32_t aDirection,
                                        double aDelta, const nsIntPoint& aRefPoint);
 
     double DistanceBetweenPoints(const QPointF &aFirstPoint, const QPointF &aSecondPoint);
 #endif
 
-    void               NativeResize(PRInt32 aWidth,
-                                    PRInt32 aHeight,
+    void               NativeResize(int32_t aWidth,
+                                    int32_t aHeight,
                                     bool    aRepaint);
 
-    void               NativeResize(PRInt32 aX,
-                                    PRInt32 aY,
-                                    PRInt32 aWidth,
-                                    PRInt32 aHeight,
+    void               NativeResize(int32_t aX,
+                                    int32_t aY,
+                                    int32_t aWidth,
+                                    int32_t aHeight,
                                     bool    aRepaint);
 
     void               NativeShow  (bool    aAction);
@@ -338,66 +305,68 @@ private:
         bool needDispatch;
     } MozCachedMoveEvent;
 
+    bool               CheckForRollup(double aMouseX, double aMouseY, bool aIsWheel);
     void*              SetupPluginPort(void);
     nsresult           SetWindowIconList(const nsTArray<nsCString> &aIconList);
     void               SetDefaultIcon(void);
     void               InitButtonEvent(nsMouseEvent &event, QGraphicsSceneMouseEvent *aEvent, int aClickCount = 1);
     nsEventStatus      DispatchCommandEvent(nsIAtom* aCommand);
-    nsEventStatus      DispatchContentCommandEvent(PRInt32 aMsg);
+    nsEventStatus      DispatchContentCommandEvent(int32_t aMsg);
     MozQWidget*        createQWidget(MozQWidget* parent,
                                      nsNativeWidget nativeParent,
                                      nsWidgetInitData* aInitData);
     void               SetSoftwareKeyboardState(bool aOpen, const InputContextAction& aAction);
+    void               ClearCachedResources();
 
     MozQWidget*        mWidget;
 
-    PRUint32           mIsVisible : 1,
+    uint32_t           mIsVisible : 1,
                        mActivatePending : 1;
-    PRInt32            mSizeState;
+    int32_t            mSizeState;
     PluginType         mPluginType;
 
     nsRefPtr<gfxASurface> mThebesSurface;
-    nsCOMPtr<nsIdleService> mIdleService;
+    nsCOMPtr<nsIIdleServiceInternal> mIdleService;
 
     bool         mIsTransparent;
- 
+
     // all of our DND stuff
     // this is the last window that had a drag event happen on it.
     void   InitDragEvent         (nsMouseEvent &aEvent);
 
     // this is everything we need to be able to fire motion events
     // repeatedly
-    PRUint32 mKeyDownFlags[8];
+    uint32_t mKeyDownFlags[8];
 
     /* Helper methods for DOM Key Down event suppression. */
-    PRUint32* GetFlagWord32(PRUint32 aKeyCode, PRUint32* aMask) {
+    uint32_t* GetFlagWord32(uint32_t aKeyCode, uint32_t* aMask) {
         /* Mozilla DOM Virtual Key Code is from 0 to 224. */
         NS_ASSERTION((aKeyCode <= 0xFF), "Invalid DOM Key Code");
         aKeyCode &= 0xFF;
 
         /* 32 = 2^5 = 0x20 */
-        *aMask = PRUint32(1) << (aKeyCode & 0x1F);
+        *aMask = uint32_t(1) << (aKeyCode & 0x1F);
         return &mKeyDownFlags[(aKeyCode >> 5)];
     }
 
-    bool IsKeyDown(PRUint32 aKeyCode) {
-        PRUint32 mask;
-        PRUint32* flag = GetFlagWord32(aKeyCode, &mask);
+    bool IsKeyDown(uint32_t aKeyCode) {
+        uint32_t mask;
+        uint32_t* flag = GetFlagWord32(aKeyCode, &mask);
         return ((*flag) & mask) != 0;
     }
 
-    void SetKeyDownFlag(PRUint32 aKeyCode) {
-        PRUint32 mask;
-        PRUint32* flag = GetFlagWord32(aKeyCode, &mask);
+    void SetKeyDownFlag(uint32_t aKeyCode) {
+        uint32_t mask;
+        uint32_t* flag = GetFlagWord32(aKeyCode, &mask);
         *flag |= mask;
     }
 
-    void ClearKeyDownFlag(PRUint32 aKeyCode) {
-        PRUint32 mask;
-        PRUint32* flag = GetFlagWord32(aKeyCode, &mask);
+    void ClearKeyDownFlag(uint32_t aKeyCode) {
+        uint32_t mask;
+        uint32_t* flag = GetFlagWord32(aKeyCode, &mask);
         *flag &= ~mask;
     }
-    PRInt32 mQCursor;
+    int32_t mQCursor;
 
     // Call this function when the users activity is the direct cause of an
     // event (like a keypress or mouse click).
@@ -419,10 +388,10 @@ private:
             event.refPoint.x = nscoord(mMoveEvent.pos.x());
             event.refPoint.y = nscoord(mMoveEvent.pos.y());
 
-            event.isShift         = ((mMoveEvent.modifiers & Qt::ShiftModifier) != 0);
-            event.isControl       = ((mMoveEvent.modifiers & Qt::ControlModifier) != 0);
-            event.isAlt           = ((mMoveEvent.modifiers & Qt::AltModifier) != 0);
-            event.isMeta          = ((mMoveEvent.modifiers & Qt::MetaModifier) != 0);
+            event.InitBasicModifiers(mMoveEvent.modifiers & Qt::ControlModifier,
+                                     mMoveEvent.modifiers & Qt::AltModifier,
+                                     mMoveEvent.modifiers & Qt::ShiftModifier,
+                                     mMoveEvent.modifiers & Qt::MetaModifier);
             event.clickCount      = 0;
 
             DispatchEvent(&event);
@@ -464,7 +433,7 @@ public:
     nsChildWindow();
     ~nsChildWindow();
 
-    PRInt32 mChildID;
+    int32_t mChildID;
 };
 
 class nsPopupWindow : public nsWindow
@@ -473,7 +442,7 @@ public:
     nsPopupWindow ();
     ~nsPopupWindow ();
 
-    PRInt32 mChildID;
+    int32_t mChildID;
 };
 #endif /* __nsWindow_h__ */
 
