@@ -1,44 +1,10 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Mozilla browser.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Chris Jones <jones.chris.g@gmail.com>
- *   Oleg Romashin <romaxa@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#if defined(MOZ_WIDGET_GTK2) || defined(MOZ_WIDGET_GTK3)
+#if defined(MOZ_WIDGET_GTK)
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #elif defined(MOZ_WIDGET_QT)
@@ -71,18 +37,18 @@ nsShmImage::Create(const gfxIntSize& aSize,
 
     nsRefPtr<nsShmImage> shm = new nsShmImage();
     shm->mImage = XShmCreateImage(dpy, aVisual, aDepth,
-                                  ZPixmap, nsnull,
+                                  ZPixmap, nullptr,
                                   &(shm->mInfo),
                                   aSize.width, aSize.height);
     if (!shm->mImage) {
-        return nsnull;
+        return nullptr;
     }
 
     size_t size = SharedMemory::PageAlignedSize(
         shm->mImage->bytes_per_line * shm->mImage->height);
     shm->mSegment = new SharedMemorySysV();
     if (!shm->mSegment->Create(size) || !shm->mSegment->Map(size)) {
-        return nsnull;
+        return nullptr;
     }
 
     shm->mInfo.shmid = shm->mSegment->GetHandle();
@@ -91,7 +57,7 @@ nsShmImage::Create(const gfxIntSize& aSize,
     shm->mInfo.readOnly = False;
 
     int xerror = 0;
-#if defined(MOZ_WIDGET_GTK2) || defined(MOZ_WIDGET_GTK3)
+#if defined(MOZ_WIDGET_GTK)
     gdk_error_trap_push();
     Status attachOk = XShmAttach(dpy, &shm->mInfo);
     XSync(dpy, False);
@@ -104,7 +70,7 @@ nsShmImage::Create(const gfxIntSize& aSize,
         // Assume XShm isn't available, and don't attempt to use it
         // again.
         gShmAvailable = false;
-        return nsnull;
+        return nullptr;
     }
 
     shm->mXAttached = true;
@@ -125,7 +91,7 @@ nsShmImage::Create(const gfxIntSize& aSize,
     default:
         NS_WARNING("Unsupported XShm Image format!");
         gShmAvailable = false;
-        return nsnull;
+        return nullptr;
     }
     return shm.forget();
 }
@@ -141,7 +107,7 @@ nsShmImage::AsSurface()
         ).forget();
 }
 
-#if defined(MOZ_WIDGET_GTK2)
+#if (MOZ_WIDGET_GTK == 2)
 void
 nsShmImage::Put(GdkWindow* aWindow, GdkRectangle* aRects, GdkRectangle* aEnd)
 {
@@ -152,7 +118,7 @@ nsShmImage::Put(GdkWindow* aWindow, GdkRectangle* aRects, GdkRectangle* aEnd)
     Display* dpy = gdk_x11_get_default_xdisplay();
     Drawable d = GDK_DRAWABLE_XID(gd);
 
-    GC gc = XCreateGC(dpy, d, 0, nsnull);
+    GC gc = XCreateGC(dpy, d, 0, nullptr);
     for (GdkRectangle* r = aRects; r < aEnd; r++) {
         XShmPutImage(dpy, d, gc, mImage,
                      r->x, r->y,
@@ -171,7 +137,7 @@ nsShmImage::Put(GdkWindow* aWindow, GdkRectangle* aRects, GdkRectangle* aEnd)
     XSync(dpy, False);
 }
 
-#elif defined(MOZ_WIDGET_GTK3)
+#elif (MOZ_WIDGET_GTK == 3)
 void
 nsShmImage::Put(GdkWindow* aWindow, cairo_rectangle_list_t* aRects)
 {
@@ -179,7 +145,7 @@ nsShmImage::Put(GdkWindow* aWindow, cairo_rectangle_list_t* aRects)
     Drawable d = GDK_WINDOW_XID(aWindow);
     int dx = 0, dy = 0;
 
-    GC gc = XCreateGC(dpy, d, 0, nsnull);
+    GC gc = XCreateGC(dpy, d, 0, nullptr);
     cairo_rectangle_t r;
     for (int i = 0; i < aRects->num_rectangles; i++) {
         r = aRects->rectangles[i];
@@ -208,7 +174,7 @@ nsShmImage::Put(QWidget* aWindow, QRect& aRect)
     Display* dpy = gfxQtPlatform::GetXDisplay(aWindow);
     Drawable d = aWindow->winId();
 
-    GC gc = XCreateGC(dpy, d, 0, nsnull);
+    GC gc = XCreateGC(dpy, d, 0, nullptr);
     // Avoid out of bounds painting
     QRect inter = aRect.intersected(aWindow->rect());
     XShmPutImage(dpy, d, gc, mImage,
@@ -232,7 +198,7 @@ nsShmImage::EnsureShmImage(const gfxIntSize& aSize, Visual* aVisual, unsigned in
         // ordered after the Puts.
         aImage = nsShmImage::Create(aSize, aVisual, aDepth);
     }
-    return !aImage ? nsnull : aImage->AsSurface();
+    return !aImage ? nullptr : aImage->AsSurface();
 }
 
 #endif  // defined(MOZ_X11) && defined(MOZ_HAVE_SHAREDMEMORYSYSV)
