@@ -1,42 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Mike Pinkerton (pinkerton@netscape.com)
- *   Mark Hammond (MarkH@ActiveState.com)
- *   David Gardiner <david.gardiner@unisa.edu.au>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include <ole2.h>
 #include <oleidl.h>
@@ -74,6 +39,7 @@
 #include "gfxContext.h"
 #include "nsRect.h"
 #include "nsMathUtils.h"
+#include "gfxWindowsPlatform.h"
 
 //-------------------------------------------------------------------------
 //
@@ -81,7 +47,7 @@
 //
 //-------------------------------------------------------------------------
 nsDragService::nsDragService()
-  : mNativeDragSrc(nsnull), mNativeDragTarget(nsnull), mDataObject(nsnull), mSentLocalDropEvent(false)
+  : mDataObject(nullptr), mSentLocalDropEvent(false)
 {
 }
 
@@ -92,8 +58,6 @@ nsDragService::nsDragService()
 //-------------------------------------------------------------------------
 nsDragService::~nsDragService()
 {
-  NS_IF_RELEASE(mNativeDragSrc);
-  NS_IF_RELEASE(mNativeDragTarget);
   NS_IF_RELEASE(mDataObject);
 }
 
@@ -119,7 +83,7 @@ nsDragService::CreateDragImage(nsIDOMNode *aDOMNode,
   if (!surface)
     return false;
 
-  PRUint32 bmWidth = dragRect.width, bmHeight = dragRect.height;
+  uint32_t bmWidth = dragRect.width, bmHeight = dragRect.height;
 
   if (bmWidth == 0 || bmHeight == 0)
     return false;
@@ -144,7 +108,7 @@ nsDragService::CreateDragImage(nsIDOMNode *aDOMNode,
   memset((void*)&bmih, 0, sizeof(BITMAPV5HEADER));
   bmih.bV5Size        = sizeof(BITMAPV5HEADER);
   bmih.bV5Width       = bmWidth;
-  bmih.bV5Height      = -(PRInt32)bmHeight; // flip vertical
+  bmih.bV5Height      = -(int32_t)bmHeight; // flip vertical
   bmih.bV5Planes      = 1;
   bmih.bV5BitCount    = 32;
   bmih.bV5Compression = BI_BITFIELDS;
@@ -168,10 +132,10 @@ nsDragService::CreateDragImage(nsIDOMNode *aDOMNode,
 
     // Mouse position in center
     if (mScreenX == -1 || mScreenY == -1) {
-      psdi->ptOffset.x = (PRUint32)((float)bmWidth/2.0f);
-      psdi->ptOffset.y = (PRUint32)((float)bmHeight/2.0f);
+      psdi->ptOffset.x = (uint32_t)((float)bmWidth/2.0f);
+      psdi->ptOffset.y = (uint32_t)((float)bmHeight/2.0f);
     } else {
-      PRInt32 sx = mScreenX, sy = mScreenY;
+      int32_t sx = mScreenX, sy = mScreenY;
       ConvertToUnscaledDevPixels(pc, &sx, &sy);
       psdi->ptOffset.x = sx - dragRect.x;
       psdi->ptOffset.y = sy - dragRect.y;
@@ -188,7 +152,7 @@ NS_IMETHODIMP
 nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode,
                                  nsISupportsArray *anArrayTransferables,
                                  nsIScriptableRegion *aRegion,
-                                 PRUint32 aActionType)
+                                 uint32_t aActionType)
 {
   nsresult rv = nsBaseDragService::InvokeDragSession(aDOMNode,
                                                      anArrayTransferables,
@@ -197,14 +161,14 @@ nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode,
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Try and get source URI of the items that are being dragged
-  nsIURI *uri = nsnull;
+  nsIURI *uri = nullptr;
 
   nsCOMPtr<nsIDocument> doc(do_QueryInterface(mSourceDocument));
   if (doc) {
     uri = doc->GetDocumentURI();
   }
 
-  PRUint32 numItemsToDrag = 0;
+  uint32_t numItemsToDrag = 0;
   rv = anArrayTransferables->Count(&numItemsToDrag);
   if (!numItemsToDrag)
     return NS_ERROR_FAILURE;
@@ -222,7 +186,7 @@ nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode,
     if (!dataObjCollection)
       return NS_ERROR_OUT_OF_MEMORY;
     itemToDrag = dataObjCollection;
-    for (PRUint32 i=0; i<numItemsToDrag; ++i) {
+    for (uint32_t i=0; i<numItemsToDrag; ++i) {
       nsCOMPtr<nsISupports> supports;
       anArrayTransferables->GetElementAt(i, getter_AddRefs(supports));
       nsCOMPtr<nsITransferable> trans(do_QueryInterface(supports));
@@ -270,17 +234,12 @@ nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode,
 //-------------------------------------------------------------------------
 NS_IMETHODIMP
 nsDragService::StartInvokingDragSession(IDataObject * aDataObj,
-                                        PRUint32 aActionType)
+                                        uint32_t aActionType)
 {
   // To do the drag we need to create an object that
   // implements the IDataObject interface (for OLE)
-  nsNativeDragSource* nativeDragSource = new nsNativeDragSource(mDataTransfer);
-  if (!nativeDragSource)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  NS_IF_RELEASE(mNativeDragSrc);
-  mNativeDragSrc = (IDropSource *)nativeDragSource;
-  mNativeDragSrc->AddRef();
+  nsRefPtr<nsNativeDragSource> nativeDragSrc =
+    new nsNativeDragSource(mDataTransfer);
 
   // Now figure out what the native drag effect should be
   DWORD winDropRes;
@@ -314,12 +273,12 @@ nsDragService::StartInvokingDragSession(IDataObject * aDataObj,
   }
 
   // Call the native D&D method
-  HRESULT res = ::DoDragDrop(aDataObj, mNativeDragSrc, effects, &winDropRes);
+  HRESULT res = ::DoDragDrop(aDataObj, nativeDragSrc, effects, &winDropRes);
 
   // In  cases where the drop operation completed outside the application, update
   // the source node's nsIDOMDataTransfer dropEffect value so it is up to date.  
   if (!mSentLocalDropEvent) {
-    PRUint32 dropResult;
+    uint32_t dropResult;
     // Order is important, since multiple flags can be returned.
     if (winDropRes & DROPEFFECT_COPY)
         dropResult = DRAGDROP_ACTION_COPY;
@@ -338,16 +297,18 @@ nsDragService::StartInvokingDragSession(IDataObject * aDataObj,
     }
   }
 
-  mUserCancelled = nativeDragSource->UserCancelled();
+  mUserCancelled = nativeDragSrc->UserCancelled();
 
   // We're done dragging, get the cursor position and end the drag
   // Use GetMessagePos to get the position of the mouse at the last message
   // seen by the event loop. (Bug 489729)
+  // Note that we must convert this from device pixels back to Windows logical
+  // pixels (bug 818927).
   DWORD pos = ::GetMessagePos();
-  POINT cpos;
-  cpos.x = GET_X_LPARAM(pos);
-  cpos.y = GET_Y_LPARAM(pos);
-  SetDragEndPoint(nsIntPoint(cpos.x, cpos.y));
+  FLOAT dpiScale = gfxWindowsPlatform::GetPlatform()->GetDPIScale();
+  nsIntPoint logPos(NSToIntRound(GET_X_LPARAM(pos) / dpiScale),
+                    NSToIntRound(GET_Y_LPARAM(pos) / dpiScale));
+  SetDragEndPoint(logPos);
   EndDragSession(true);
 
   mDoingDrag = false;
@@ -360,7 +321,7 @@ nsDragService::StartInvokingDragSession(IDataObject * aDataObj,
 nsDataObjCollection*
 nsDragService::GetDataObjCollection(IDataObject* aDataObj)
 {
-  nsDataObjCollection * dataObjCol = nsnull;
+  nsDataObjCollection * dataObjCol = nullptr;
   if (aDataObj) {
     nsIDataObjCollection* dataObj;
     if (aDataObj->QueryInterface(IID_IDataObjCollection,
@@ -375,7 +336,7 @@ nsDragService::GetDataObjCollection(IDataObject* aDataObj)
 
 //-------------------------------------------------------------------------
 NS_IMETHODIMP
-nsDragService::GetNumDropItems(PRUint32 * aNumItems)
+nsDragService::GetNumDropItems(uint32_t * aNumItems)
 {
   if (!mDataObject) {
     *aNumItems = 0;
@@ -409,6 +370,9 @@ nsDragService::GetNumDropItems(PRUint32 * aNumItems)
         *aNumItems = ::DragQueryFileW(hdrop, 0xFFFFFFFF, NULL, 0);
         ::GlobalUnlock(stm.hGlobal);
         ::ReleaseStgMedium(&stm);
+        // Data may be provided later, so assume we have 1 item
+        if (*aNumItems == 0)
+          *aNumItems = 1;
       }
       else
         *aNumItems = 1;
@@ -422,7 +386,7 @@ nsDragService::GetNumDropItems(PRUint32 * aNumItems)
 
 //-------------------------------------------------------------------------
 NS_IMETHODIMP
-nsDragService::GetData(nsITransferable * aTransferable, PRUint32 anItem)
+nsDragService::GetData(nsITransferable * aTransferable, uint32_t anItem)
 {
   // This typcially happens on a drop, the target would be asking
   // for it's transferable to be filled in
@@ -435,10 +399,10 @@ nsDragService::GetData(nsITransferable * aTransferable, PRUint32 anItem)
   if (IsCollectionObject(mDataObject)) {
     // multiple items, use |anItem| as an index into our collection
     nsDataObjCollection * dataObjCol = GetDataObjCollection(mDataObject);
-    PRUint32 cnt = dataObjCol->GetNumDataObjects();
+    uint32_t cnt = dataObjCol->GetNumDataObjects();
     if (anItem >= 0 && anItem < cnt) {
       IDataObject * dataObj = dataObjCol->GetDataObjectAt(anItem);
-      dataFound = nsClipboard::GetDataFromDataObject(dataObj, 0, nsnull,
+      dataFound = nsClipboard::GetDataFromDataObject(dataObj, 0, nullptr,
                                                      aTransferable);
     }
     else
@@ -448,14 +412,14 @@ nsDragService::GetData(nsITransferable * aTransferable, PRUint32 anItem)
     // If they are asking for item "0", we can just get it...
     if (anItem == 0) {
        dataFound = nsClipboard::GetDataFromDataObject(mDataObject, anItem,
-                                                      nsnull, aTransferable);
+                                                      nullptr, aTransferable);
     } else {
       // It better be a file drop, or else non-zero indexes are invalid!
       FORMATETC fe2;
       SET_FORMATETC(fe2, CF_HDROP, 0, DVASPECT_CONTENT, -1, TYMED_HGLOBAL);
       if (mDataObject->QueryGetData(&fe2) == S_OK)
         dataFound = nsClipboard::GetDataFromDataObject(mDataObject, anItem,
-                                                       nsnull, aTransferable);
+                                                       nullptr, aTransferable);
       else
         NS_WARNING("Reqesting non-zero index, but clipboard data is not a collection!");
     }
@@ -493,7 +457,7 @@ nsDragService::IsDataFlavorSupported(const char *aDataFlavor, bool *_retval)
   if (!aDataFlavor || !mDataObject || !_retval)
     return NS_ERROR_FAILURE;
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
   if (strcmp(aDataFlavor, kTextMime) == 0)
     NS_WARNING("DO NOT USE THE text/plain DATA FLAVOR ANY MORE. USE text/unicode INSTEAD");
 #endif
@@ -513,8 +477,8 @@ nsDragService::IsDataFlavorSupported(const char *aDataFlavor, bool *_retval)
     // this data type
     nsDataObjCollection* dataObjCol = GetDataObjCollection(mDataObject);
     if (dataObjCol) {
-      PRUint32 cnt = dataObjCol->GetNumDataObjects();
-      for (PRUint32 i=0;i<cnt;++i) {
+      uint32_t cnt = dataObjCol->GetNumDataObjects();
+      for (uint32_t i=0;i<cnt;++i) {
         IDataObject * dataObj = dataObjCol->GetDataObjectAt(i);
         if (S_OK == dataObj->QueryGetData(&fe))
           *_retval = true;             // found it!
