@@ -1,61 +1,26 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Michael Lowe <michael.lowe@bigfoot.com>
- *   Jens Bannmann <jens.b@web.de>
- *   Ryan Jones <sciguyryan@gmail.com>
- *   Ehsan Akhgari <ehsan.akhgari@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsLookAndFeel.h"
 #include <windows.h>
 #include <shellapi.h>
-#include "nsWindow.h"
 #include "nsStyleConsts.h"
 #include "nsUXThemeData.h"
 #include "nsUXThemeConstants.h"
 #include "gfxFont.h"
+#include "gfxWindowsPlatform.h"
 #include "WinUtils.h"
+#include "mozilla/Telemetry.h"
 
 using namespace mozilla::widget;
 using mozilla::LookAndFeel;
 
 static nsresult GetColorFromTheme(nsUXThemeClass cls,
-                           PRInt32 aPart,
-                           PRInt32 aState,
-                           PRInt32 aPropId,
+                           int32_t aPart,
+                           int32_t aState,
+                           int32_t aPropId,
                            nscolor &aColor)
 {
   COLORREF color;
@@ -68,14 +33,28 @@ static nsresult GetColorFromTheme(nsUXThemeClass cls,
   return NS_ERROR_FAILURE;
 }
 
-static PRInt32 GetSystemParam(long flag, PRInt32 def)
+static int32_t GetSystemParam(long flag, int32_t def)
 {
     DWORD value; 
     return ::SystemParametersInfo(flag, 0, &value, 0) ? value : def;
 }
 
+namespace mozilla {
+namespace widget {
+// This is in use here and in nsDOMTouchEvent.cpp
+int32_t IsTouchDeviceSupportPresent()
+{
+  int32_t touchCapabilities;
+  touchCapabilities = ::GetSystemMetrics(SM_DIGITIZER);
+  return ((touchCapabilities & NID_READY) && 
+          (touchCapabilities & (NID_EXTERNAL_TOUCH | NID_INTEGRATED_TOUCH)));
+}
+} }
+
 nsLookAndFeel::nsLookAndFeel() : nsXPLookAndFeel()
 {
+  mozilla::Telemetry::Accumulate(mozilla::Telemetry::TOUCH_ENABLED_DEVICE,
+                                 IsTouchDeviceSupportPresent());
 }
 
 nsLookAndFeel::~nsLookAndFeel()
@@ -322,7 +301,7 @@ nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor)
 }
 
 nsresult
-nsLookAndFeel::GetIntImpl(IntID aID, PRInt32 &aResult)
+nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
 {
   nsresult res = nsXPLookAndFeel::GetIntImpl(aID, aResult);
   if (NS_SUCCEEDED(res))
@@ -331,7 +310,7 @@ nsLookAndFeel::GetIntImpl(IntID aID, PRInt32 &aResult)
 
   switch (aID) {
     case eIntID_CaretBlinkTime:
-        aResult = (PRInt32)::GetCaretBlinkTime();
+        aResult = (int32_t)::GetCaretBlinkTime();
         break;
     case eIntID_CaretWidth:
         aResult = 1;
@@ -401,13 +380,7 @@ nsLookAndFeel::GetIntImpl(IntID aID, PRInt32 &aResult)
         aResult = !IsAppThemed();
         break;
     case eIntID_TouchEnabled:
-        aResult = 0;
-        PRInt32 touchCapabilities;
-        touchCapabilities = ::GetSystemMetrics(SM_DIGITIZER);
-        if ((touchCapabilities & NID_READY) && 
-           (touchCapabilities & (NID_EXTERNAL_TOUCH | NID_INTEGRATED_TOUCH))) {
-            aResult = 1;
-        }
+        aResult = IsTouchDeviceSupportPresent();
         break;
     case eIntID_WindowsDefaultTheme:
         aResult = nsUXThemeData::IsDefaultWindowTheme();
@@ -424,6 +397,11 @@ nsLookAndFeel::GetIntImpl(IntID aID, PRInt32 &aResult)
     case eIntID_DWMCompositor:
         aResult = nsUXThemeData::CheckForCompositor();
         break;
+    case eIntID_WindowsGlass:
+        // Aero Glass is only available prior to Windows 8 when DWM is used.
+        aResult = (nsUXThemeData::CheckForCompositor() &&
+                   WinUtils::GetWindowsVersion() < WinUtils::WIN8_VERSION);
+        break;
     case eIntID_AlertNotificationOrigin:
         aResult = 0;
         {
@@ -477,6 +455,9 @@ nsLookAndFeel::GetIntImpl(IntID aID, PRInt32 &aResult)
     case eIntID_ScrollbarButtonAutoRepeatBehavior:
         aResult = 0;
         break;
+    case eIntID_SwipeAnimationEnabled:
+        aResult = 0;
+        break;
     default:
         aResult = 0;
         res = NS_ERROR_FAILURE;
@@ -578,18 +559,19 @@ GetSysFontInfo(HDC aHDC, LookAndFeel::FontID anID,
     break;
   }
 
-  // FIXME?: mPixelScale is currently hardcoded to 1.
-  float mPixelScale = 1.0f;
+  // Get scaling factor from physical to logical pixels
+  float pixelScale = 1.0f / gfxWindowsPlatform::GetPlatform()->GetDPIScale();
 
   // The lfHeight is in pixels, and it needs to be adjusted for the
   // device it will be displayed on.
   // Screens and Printers will differ in DPI
   //
   // So this accounts for the difference in the DeviceContexts
-  // The mPixelScale will be a "1" for the screen and could be
-  // any value when going to a printer, for example mPixleScale is
+  // The pixelScale will typically be 1.0 for the screen
+  // (though larger for hi-dpi screens where the Windows resolution
+  // scale factor is 125% or 150% or even more), and could be
+  // any value when going to a printer, for example pixelScale is
   // 6.25 when going to a 600dpi printer.
-  // round, but take into account whether it is negative
   float pixelHeight = -ptrLogFont->lfHeight;
   if (pixelHeight < 0) {
     HFONT hFont = ::CreateFontIndirectW(ptrLogFont);
@@ -602,7 +584,7 @@ GetSysFontInfo(HDC aHDC, LookAndFeel::FontID anID,
     ::DeleteObject(hFont);
     pixelHeight = tm.tmAscent;
   }
-  pixelHeight *= mPixelScale;
+  pixelHeight *= pixelScale;
 
   // we have problem on Simplified Chinese system because the system
   // report the default font size is 8 points. but if we use 8, the text
@@ -636,11 +618,14 @@ GetSysFontInfo(HDC aHDC, LookAndFeel::FontID anID,
 
 bool
 nsLookAndFeel::GetFontImpl(FontID anID, nsString &aFontName,
-                           gfxFontStyle &aFontStyle)
+                           gfxFontStyle &aFontStyle,
+                           float aDevPixPerCSSPixel)
 {
   HDC tdc = GetDC(NULL);
   bool status = GetSysFontInfo(tdc, anID, aFontName, aFontStyle);
   ReleaseDC(NULL, tdc);
+  // now convert the logical font size from GetSysFontInfo into device pixels for layout
+  aFontStyle.size *= aDevPixPerCSSPixel;
   return status;
 }
 
