@@ -43,6 +43,8 @@ GfxInfo::Init()
     mIsFGLRX = false;
     mIsNouveau = false;
     mIsIntel = false;
+    mIsOldSwrast = false;
+    mIsLlvmpipe = false;
     mHasTextureFromPixmap = false;
     return GfxInfoBase::Init();
 }
@@ -184,7 +186,7 @@ GfxInfo::GetData()
     mAdapterDescription.AppendLiteral(" -- ");
     mAdapterDescription.Append(mRenderer);
 
-    nsCAutoString note;
+    nsAutoCString note;
     note.Append("OpenGL: ");
     note.Append(mAdapterDescription);
     note.Append(" -- ");
@@ -212,6 +214,10 @@ GfxInfo::GetData()
             mIsNouveau = true;
         if (strcasestr(mRenderer.get(), "intel")) // yes, intel is in the renderer string
             mIsIntel = true;
+        if (strcasestr(mRenderer.get(), "llvmpipe"))
+            mIsLlvmpipe = true;
+        if (strcasestr(mRenderer.get(), "software rasterizer"))
+            mIsOldSwrast = true;
     } else if (strstr(mVendor.get(), "NVIDIA Corporation")) {
         mIsNVIDIA = true;
         // with the NVIDIA driver, the version string contains "NVIDIA major.minor"
@@ -326,11 +332,15 @@ GfxInfo::GetFeatureStatusImpl(int32_t aFeature,
         if (mIsNouveau && version(mMajorVersion, mMinorVersion) < version(8,0)) {
           *aStatus = nsIGfxInfo::FEATURE_BLOCKED_DRIVER_VERSION;
           aSuggestedDriverVersion.AssignLiteral("Mesa 8.0");
-        } else if (version(mMajorVersion, mMinorVersion, mRevisionVersion) < version(7,10,3)) {
+        }
+        else if (version(mMajorVersion, mMinorVersion, mRevisionVersion) < version(7,10,3)) {
           *aStatus = nsIGfxInfo::FEATURE_BLOCKED_DRIVER_VERSION;
           aSuggestedDriverVersion.AssignLiteral("Mesa 7.10.3");
         }
-        if (aFeature == nsIGfxInfo::FEATURE_WEBGL_MSAA)
+        else if (mIsOldSwrast || mIsLlvmpipe) {
+          *aStatus = nsIGfxInfo::FEATURE_BLOCKED_DRIVER_VERSION;
+        }
+        else if (aFeature == nsIGfxInfo::FEATURE_WEBGL_MSAA)
         {
           if (mIsIntel && version(mMajorVersion, mMinorVersion) < version(8,1))
             *aStatus = nsIGfxInfo::FEATURE_BLOCKED_DRIVER_VERSION;
