@@ -13,7 +13,9 @@
 #include "nscore.h"
 #include <mozilla/StandardInteger.h>
 
-PR_BEGIN_EXTERN_C
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 // aSP will be the best approximation possible of what the stack pointer will be
 // pointing to when the execution returns to executing that at that PC.
@@ -29,24 +31,38 @@ typedef void
  * @param aSkipFrames  Number of initial frames to skip.  0 means that
  *                     the first callback will be for the caller of
  *                     NS_StackWalk.
+ * @param aMaxFrames   Maximum number of frames to trace.  0 means no limit.
  * @param aClosure     Caller-supplied data passed through to aCallback.
  * @param aThread      The thread for which the stack is to be retrieved.
  *                     Passing null causes us to walk the stack of the
  *                     current thread. On Windows, this is a thread HANDLE.
  *                     It is currently not supported on any other platform.
+ * @param aPlatformData Platform specific data that can help in walking the
+ *                      stack, this should be NULL unless you really know
+ *                      what you're doing! This needs to be a pointer to a
+ *                      CONTEXT on Windows and should not be passed on other
+ *                      platforms.
  *
- * Returns NS_ERROR_NOT_IMPLEMENTED on platforms where it is
- * unimplemented.
- * Returns NS_ERROR_UNEXPECTED when the stack indicates that the thread
- * is in a very dangerous situation (e.g., holding sem_pool_lock in 
- * Mac OS X pthreads code). Callers should then bail out immediately.
+ * Return values:
+ * - NS_ERROR_NOT_IMPLEMENTED.  Occurs on platforms where it is unimplemented.
+ *
+ * - NS_ERROR_UNEXPECTED.  Occurs when the stack indicates that the thread
+ *   is in a very dangerous situation (e.g., holding sem_pool_lock in Mac OS X
+ *   pthreads code).  Callers should then bail out immediately.
+ *
+ * - NS_ERROR_FAILURE.  Occurs when stack walking completely failed, i.e.
+ *   aCallback was never called.
+ *
+ * - NS_OK.  Occurs when stack walking succeeded, i.e. aCallback was called at
+ *   least once (and there was no need to exit with NS_ERROR_UNEXPECTED).
  *
  * May skip some stack frames due to compiler optimizations or code
  * generation.
  */
 XPCOM_API(nsresult)
 NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
-             void *aClosure, uintptr_t aThread);
+             uint32_t aMaxFrames, void *aClosure, uintptr_t aThread,
+             void *aPlatformData);
 
 typedef struct {
     /*
@@ -55,7 +71,7 @@ typedef struct {
      * string and zero if unknown.
      */
     char library[256];
-    PRUptrdiff loffset;
+    ptrdiff_t loffset;
     /*
      * The name of the file name and line number of the code
      * corresponding to the address, or empty string and zero if
@@ -68,7 +84,7 @@ typedef struct {
      * offset within that function, or empty string and zero if unknown.
      */
     char function[256];
-    PRUptrdiff foffset;
+    ptrdiff_t foffset;
 } nsCodeAddressDetails;
 
 /**
@@ -102,6 +118,8 @@ XPCOM_API(nsresult)
 NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
                             char *aBuffer, uint32_t aBufferSize);
 
-PR_END_EXTERN_C
+#ifdef __cplusplus
+}
+#endif
 
 #endif /* !defined(nsStackWalk_h_) */
