@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef nscore_h___
 #define nscore_h___
@@ -53,10 +21,12 @@
 #endif
 
 /**
- * Incorporate the core NSPR data types which XPCOM uses.
+ * Incorporate the integer data types which XPCOM uses.
  */
-#include "prtypes.h"
 #include "mozilla/StandardInteger.h"
+#include "stddef.h"
+
+#include "mozilla/NullPtr.h"
 
 /*
  * This is for functions that are like malloc_usable_size.  Such functions are
@@ -137,8 +107,8 @@ typedef size_t(*nsMallocSizeOfFun)(const void *p);
  *  - Adding this to a public function _will_ break binary compatibility.
  *  - This may be used on virtual functions but you must ensure it is applied
  *    to all implementations - the compiler will _not_ warn but it will crash.
- *  - This has no effect for inline functions or functions which take a
- *    variable number of arguments.
+ *  - This has no effect for functions which take a variable number of
+ *    arguments.
  *  - __fastcall on windows should not be applied to class
  *    constructors/destructors - use the NS_CONSTRUCTOR_FASTCALL macro for
  *    constructors/destructors.
@@ -147,8 +117,7 @@ typedef size_t(*nsMallocSizeOfFun)(const void *p);
  *           NS_HIDDEN_(int) NS_FASTCALL func2(char *foo);
  */
 
-#if defined(__i386__) && defined(__GNUC__) && \
-    (__GNUC__ >= 3) && !defined(XP_OS2)
+#if defined(__i386__) && defined(__GNUC__) && !defined(XP_OS2)
 #define NS_FASTCALL __attribute__ ((regparm (3), stdcall))
 #define NS_CONSTRUCTOR_FASTCALL __attribute__ ((regparm (3), stdcall))
 #elif defined(XP_WIN) && !defined(_WIN64)
@@ -242,7 +211,7 @@ typedef size_t(*nsMallocSizeOfFun)(const void *p);
 /**
  * Deprecated declarations.
  */
-#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1))
+#ifdef __GNUC__
 # define MOZ_DEPRECATED __attribute__((deprecated))
 #elif defined(_MSC_VER)
 # define MOZ_DEPRECATED __declspec(deprecated)
@@ -340,7 +309,7 @@ typedef size_t(*nsMallocSizeOfFun)(const void *p);
 /**
  * Generic XPCOM result data type
  */
-typedef PRUint32 nsresult;
+#include "nsError.h"
 
 /**
  * Reference count values
@@ -350,25 +319,12 @@ typedef PRUint32 nsresult;
  * The following ifdef exists to maintain binary compatibility with
  * IUnknown.
  */
-#if defined(XP_WIN) && PR_BYTES_PER_LONG == 4
+#ifdef XP_WIN
 typedef unsigned long nsrefcnt;
 #else
-typedef PRUint32 nsrefcnt;
+typedef uint32_t nsrefcnt;
 #endif
 
-/**
- * The preferred symbol for null.  Make sure this is the same size as
- * void* on the target.  See bug 547964.
- */
-#if defined(_WIN64)
-# define nsnull 0LL
-#else
-# define nsnull 0L
-#endif
-
-
-#include "nsError.h"
-
 /* ------------------------------------------------------------------------ */
 /* Casting macros for hiding C++ features from older compilers */
 
@@ -386,7 +342,7 @@ typedef PRUint32 nsrefcnt;
   #if defined(HAVE_CPP_2BYTE_WCHAR_T) && defined(XP_WIN)
     typedef wchar_t PRUnichar;
   #else
-    typedef PRUint16 PRUnichar;
+    typedef uint16_t PRUnichar;
   #endif
 #endif
 
@@ -394,8 +350,8 @@ typedef PRUint32 nsrefcnt;
  * Use these macros to do 64bit safe pointer conversions.
  */
 
-#define NS_PTR_TO_INT32(x)  ((PRInt32)  (intptr_t) (x))
-#define NS_PTR_TO_UINT32(x) ((PRUint32) (intptr_t) (x))
+#define NS_PTR_TO_INT32(x)  ((int32_t)  (intptr_t) (x))
+#define NS_PTR_TO_UINT32(x) ((uint32_t) (intptr_t) (x))
 #define NS_INT32_TO_PTR(x)  ((void *)   (intptr_t) (x))
 
 /*
@@ -404,31 +360,6 @@ typedef PRUint32 nsrefcnt;
 #define NS_STRINGIFY_HELPER(x_) #x_
 #define NS_STRINGIFY(x_) NS_STRINGIFY_HELPER(x_)
 
-/*
- * These macros allow you to give a hint to the compiler about branch
- * probability so that it can better optimize.  Use them like this:
- *
- *  if (NS_LIKELY(v == 1)) {
- *    ... expected code path ...
- *  }
- *
- *  if (NS_UNLIKELY(v == 0)) {
- *    ... non-expected code path ...
- *  }
- *
- * These macros are guaranteed to always return 0 or 1.
- * The NS_FAILED/NS_SUCCEEDED macros depends on this.
- * @return 0 or 1
- */
-
-#if defined(__GNUC__) && (__GNUC__ > 2)
-#define NS_LIKELY(x)    (__builtin_expect(!!(x), 1))
-#define NS_UNLIKELY(x)  (__builtin_expect(!!(x), 0))
-#else
-#define NS_LIKELY(x)    (!!(x))
-#define NS_UNLIKELY(x)  (!!(x))
-#endif
-
  /*
   * If we're being linked as standalone glue, we don't want a dynamic
   * dependency on NSPR libs, so we skip the debug thread-safety
@@ -442,51 +373,6 @@ typedef PRUint32 nsrefcnt;
 #define NS_TLS __thread
 #endif
 
-/**
- * Static type annotations, enforced when static-checking is enabled:
- *
- * NS_STACK_CLASS: a class which must only be instantiated on the stack
- *
- * NS_MUST_OVERRIDE:
- *   a method which every immediate subclass of this class must
- *   override.  A subclass override can itself be NS_MUST_OVERRIDE, in
- *   which case its own subclasses must override the method as well.
- *
- *   This is similar to, but not the same as, marking a method pure
- *   virtual.  It has no effect on the class in which the annotation
- *   appears, you can still provide a definition for the method, and
- *   it objects to the mere existence of a subclass that doesn't
- *   override the method.  See examples in analysis/must-override.js.
- */
-#ifdef NS_STATIC_CHECKING
-#define NS_STACK_CLASS __attribute__((user("NS_stack")))
-#define NS_OKONHEAP    __attribute__((user("NS_okonheap")))
-#define NS_SUPPRESS_STACK_CHECK __attribute__((user("NS_suppress_stackcheck")))
-#define NS_MUST_OVERRIDE __attribute__((user("NS_must_override")))
-#else
-#define NS_STACK_CLASS
-#define NS_OKONHEAP
-#define NS_SUPPRESS_STACK_CHECK
-#define NS_MUST_OVERRIDE
-#endif
-
-/**
- * Attributes defined to help Dehydra GCC analysis.
- */
-#ifdef NS_STATIC_CHECKING
-# define NS_SCRIPTABLE __attribute__((user("NS_script")))
-# define NS_INPARAM __attribute__((user("NS_inparam")))
-# define NS_OUTPARAM  __attribute__((user("NS_outparam")))
-# define NS_INOUTPARAM __attribute__((user("NS_inoutparam")))
-# define NS_OVERRIDE __attribute__((user("NS_override")))
-#else
-# define NS_SCRIPTABLE
-# define NS_INPARAM
-# define NS_OUTPARAM
-# define NS_INOUTPARAM
-# define NS_OVERRIDE
-#endif
-
 /*
  * SEH exception macros.
  */
