@@ -1,42 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 // vim:cindent:ts=2:et:sw=2:
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/Assertions.h"
+#include "mozilla/Attributes.h"
 #include "mozilla/HashFunctions.h"
 
 #include "nsAtomTable.h"
@@ -93,7 +62,7 @@ public:
 
   // This is currently only used during startup when creating a permanent atom
   // from NS_RegisterStaticAtoms
-  AtomImpl(nsStringBuffer* aData, PRUint32 aLength, PLDHashNumber aKeyHash);
+  AtomImpl(nsStringBuffer* aData, uint32_t aLength, PLDHashNumber aKeyHash);
 
 protected:
   // This is only intended to be used when a normal atom is turned into a
@@ -115,7 +84,7 @@ public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSIATOM
 
-  enum { REFCNT_PERMANENT_SENTINEL = PR_UINT32_MAX };
+  enum { REFCNT_PERMANENT_SENTINEL = UINT32_MAX };
 
   virtual bool IsPermanent();
 
@@ -134,12 +103,12 @@ public:
  * A non-refcounted implementation of nsIAtom.
  */
 
-class PermanentAtomImpl : public AtomImpl {
+class PermanentAtomImpl MOZ_FINAL : public AtomImpl {
 public:
   PermanentAtomImpl(const nsAString& aString, PLDHashNumber aKeyHash)
     : AtomImpl(aString, aKeyHash)
   {}
-  PermanentAtomImpl(nsStringBuffer* aData, PRUint32 aLength,
+  PermanentAtomImpl(nsStringBuffer* aData, uint32_t aLength,
                     PLDHashNumber aKeyHash)
     : AtomImpl(aData, aLength, aKeyHash)
   {}
@@ -170,15 +139,15 @@ struct AtomTableEntry : public PLDHashEntryHdr {
 
 struct AtomTableKey
 {
-  AtomTableKey(const PRUnichar* aUTF16String, PRUint32 aLength)
+  AtomTableKey(const PRUnichar* aUTF16String, uint32_t aLength)
     : mUTF16String(aUTF16String),
-      mUTF8String(nsnull),
+      mUTF8String(nullptr),
       mLength(aLength)
   {
   }
 
-  AtomTableKey(const char* aUTF8String, PRUint32 aLength)
-    : mUTF16String(nsnull),
+  AtomTableKey(const char* aUTF8String, uint32_t aLength)
+    : mUTF16String(nullptr),
       mUTF8String(aUTF8String),
       mLength(aLength)
   {
@@ -186,7 +155,7 @@ struct AtomTableKey
 
   const PRUnichar* mUTF16String;
   const char* mUTF8String;
-  PRUint32 mLength;
+  uint32_t mLength;
 };
 
 static PLDHashNumber
@@ -196,10 +165,10 @@ AtomTableGetHash(PLDHashTable *table, const void *key)
 
   if (k->mUTF8String) {
     bool err;
-    PRUint32 hash = HashUTF8AsUTF16(k->mUTF8String, k->mLength, &err);
+    uint32_t hash = HashUTF8AsUTF16(k->mUTF8String, k->mLength, &err);
     if (err) {
       AtomTableKey* mutableKey = const_cast<AtomTableKey*>(k);
-      mutableKey->mUTF8String = nsnull;
+      mutableKey->mUTF8String = nullptr;
       mutableKey->mLength = 0;
       hash = 0;
     }
@@ -223,7 +192,7 @@ AtomTableMatchKey(PLDHashTable *table, const PLDHashEntryHdr *entry,
                          nsDependentAtomString(he->mAtom)) == 0;
   }
 
-  PRUint32 length = he->mAtom->GetLength();
+  uint32_t length = he->mAtom->GetLength();
   if (length != k->mLength) {
     return false;
   }
@@ -254,7 +223,7 @@ static bool
 AtomTableInitEntry(PLDHashTable *table, PLDHashEntryHdr *entry,
                    const void *key)
 {
-  static_cast<AtomTableEntry*>(entry)->mAtom = nsnull;
+  static_cast<AtomTableEntry*>(entry)->mAtom = nullptr;
 
   return true;
 }
@@ -275,14 +244,14 @@ static const PLDHashTableOps AtomTableOps = {
 #ifdef DEBUG
 static PLDHashOperator
 DumpAtomLeaks(PLDHashTable *table, PLDHashEntryHdr *he,
-              PRUint32 index, void *arg)
+              uint32_t index, void *arg)
 {
   AtomTableEntry *entry = static_cast<AtomTableEntry*>(he);
   
   AtomImpl* atom = entry->mAtom;
   if (!atom->IsPermanent()) {
-    ++*static_cast<PRUint32*>(arg);
-    nsCAutoString str;
+    ++*static_cast<uint32_t*>(arg);
+    nsAutoCString str;
     atom->ToUTF8String(str);
     fputs(str.get(), stdout);
     fputs("\n", stdout);
@@ -314,7 +283,7 @@ NS_PurgeAtomTable()
 #ifdef DEBUG
     const char *dumpAtomLeaks = PR_GetEnv("MOZ_DUMP_ATOM_LEAKS");
     if (dumpAtomLeaks && *dumpAtomLeaks) {
-      PRUint32 leaked = 0;
+      uint32_t leaked = 0;
       printf("*** %d atoms still exist (including permanent):\n",
              gAtomTable.entryCount);
       PL_DHashTableEnumerate(&gAtomTable, DumpAtomLeaks, &leaked);
@@ -323,19 +292,17 @@ NS_PurgeAtomTable()
 #endif
     PL_DHashTableFinish(&gAtomTable);
     gAtomTable.entryCount = 0;
-    gAtomTable.ops = nsnull;
+    gAtomTable.ops = nullptr;
   }
 }
 
 AtomImpl::AtomImpl(const nsAString& aString, PLDHashNumber aKeyHash)
 {
   mLength = aString.Length();
-  nsStringBuffer* buf = nsStringBuffer::FromString(aString);
+  nsRefPtr<nsStringBuffer> buf = nsStringBuffer::FromString(aString);
   if (buf) {
-    buf->AddRef();
     mString = static_cast<PRUnichar*>(buf->Data());
-  }
-  else {
+  } else {
     buf = nsStringBuffer::Alloc((mLength + 1) * sizeof(PRUnichar));
     mString = static_cast<PRUnichar*>(buf->Data());
     CopyUnicodeTo(aString, 0, mString, mLength);
@@ -350,9 +317,12 @@ AtomImpl::AtomImpl(const nsAString& aString, PLDHashNumber aKeyHash)
   NS_ASSERTION(buf && buf->StorageSize() >= (mLength+1) * sizeof(PRUnichar),
                "enough storage");
   NS_ASSERTION(Equals(aString), "correct data");
+
+  // Take ownership of buffer
+  buf.forget();
 }
 
-AtomImpl::AtomImpl(nsStringBuffer* aStringBuffer, PRUint32 aLength,
+AtomImpl::AtomImpl(nsStringBuffer* aStringBuffer, uint32_t aLength,
                    PLDHashNumber aKeyHash)
 {
   mLength = aLength;
@@ -400,13 +370,13 @@ PermanentAtomImpl::~PermanentAtomImpl()
 
 NS_IMETHODIMP_(nsrefcnt) PermanentAtomImpl::AddRef()
 {
-  NS_ASSERTION(NS_IsMainThread(), "wrong thread");
+  MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
   return 2;
 }
 
 NS_IMETHODIMP_(nsrefcnt) PermanentAtomImpl::Release()
 {
-  NS_ASSERTION(NS_IsMainThread(), "wrong thread");
+  MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
   return 1;
 }
 
@@ -423,8 +393,8 @@ PermanentAtomImpl::IsPermanent()
 }
 
 void* PermanentAtomImpl::operator new ( size_t size, AtomImpl* aAtom ) CPP_THROW_NEW {
-  NS_ASSERTION(!aAtom->IsPermanent(),
-               "converting atom that's already permanent");
+  MOZ_ASSERT(!aAtom->IsPermanent(),
+             "converting atom that's already permanent");
 
   // Just let the constructor overwrite the vtable pointer.
   return aAtom;
@@ -483,54 +453,78 @@ SizeOfAtomTableEntryExcludingThis(PLDHashEntryHdr *aHdr,
   return entry->mAtom->SizeOfIncludingThis(aMallocSizeOf);
 }
 
-size_t NS_SizeOfAtomTableIncludingThis(nsMallocSizeOfFun aMallocSizeOf) {
+static size_t
+SizeOfStaticAtomTableEntryExcludingThis(const nsAString& aKey,
+                                        nsIAtom* const& aData,
+                                        nsMallocSizeOfFun aMallocSizeOf,
+                                        void* aArg)
+{
+  return aKey.SizeOfExcludingThisIfUnshared(aMallocSizeOf);
+}
+
+size_t
+NS_SizeOfAtomTablesIncludingThis(nsMallocSizeOfFun aMallocSizeOf) {
+  size_t n = 0;
   if (gAtomTable.ops) {
-      return PL_DHashTableSizeOfExcludingThis(&gAtomTable,
-                                              SizeOfAtomTableEntryExcludingThis,
-                                              aMallocSizeOf);
+      n += PL_DHashTableSizeOfExcludingThis(&gAtomTable,
+                                            SizeOfAtomTableEntryExcludingThis,
+                                            aMallocSizeOf);
   }
-  return 0;
+  if (gStaticAtomTable) {
+    n += gStaticAtomTable->SizeOfIncludingThis(SizeOfStaticAtomTableEntryExcludingThis,
+                                               aMallocSizeOf);
+  }
+  return n;
 }
 
 #define ATOM_HASHTABLE_INITIAL_SIZE  4096
 
-static inline bool
+static void HandleOOM()
+{
+  fputs("Out of memory allocating atom hashtable.\n", stderr);
+  MOZ_CRASH();
+  MOZ_NOT_REACHED();
+}
+
+static inline void
 EnsureTableExists()
 {
-  if (gAtomTable.ops) {
-    return true;
+  if (!gAtomTable.ops &&
+      !PL_DHashTableInit(&gAtomTable, &AtomTableOps, 0,
+                         sizeof(AtomTableEntry), ATOM_HASHTABLE_INITIAL_SIZE)) {
+    // Initialization failed.
+    HandleOOM();
   }
-  if (PL_DHashTableInit(&gAtomTable, &AtomTableOps, 0,
-                        sizeof(AtomTableEntry), ATOM_HASHTABLE_INITIAL_SIZE)) {
-    return true;
-  }
-  // Initialization failed.
-  gAtomTable.ops = nsnull;
-  return false;
 }
 
 static inline AtomTableEntry*
-GetAtomHashEntry(const char* aString, PRUint32 aLength)
+GetAtomHashEntry(const char* aString, uint32_t aLength)
 {
-  NS_ASSERTION(NS_IsMainThread(), "wrong thread");
-  if (!EnsureTableExists()) {
-    return nsnull;
-  }
+  MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
+  EnsureTableExists();
   AtomTableKey key(aString, aLength);
-  return static_cast<AtomTableEntry*>
-                    (PL_DHashTableOperate(&gAtomTable, &key, PL_DHASH_ADD));
+  AtomTableEntry* e =
+    static_cast<AtomTableEntry*>
+               (PL_DHashTableOperate(&gAtomTable, &key, PL_DHASH_ADD));
+  if (!e) {
+    HandleOOM();
+  }
+  return e;
 }
 
 static inline AtomTableEntry*
-GetAtomHashEntry(const PRUnichar* aString, PRUint32 aLength)
+GetAtomHashEntry(const PRUnichar* aString, uint32_t aLength)
 {
-  NS_ASSERTION(NS_IsMainThread(), "wrong thread");
-  if (!EnsureTableExists()) {
-    return nsnull;
-  }
+  MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
+  EnsureTableExists();
   AtomTableKey key(aString, aLength);
-  return static_cast<AtomTableEntry*>
-                    (PL_DHashTableOperate(&gAtomTable, &key, PL_DHASH_ADD));
+  AtomTableEntry* e =
+    static_cast<AtomTableEntry*>
+               (PL_DHashTableOperate(&gAtomTable, &key, PL_DHASH_ADD));
+  if (!e) {
+    HandleOOM();
+  }
+  return e;
 }
 
 class CheckStaticAtomSizes
@@ -551,7 +545,7 @@ class CheckStaticAtomSizes
 };
 
 nsresult
-RegisterStaticAtoms(const nsStaticAtom* aAtoms, PRUint32 aAtomCount)
+RegisterStaticAtoms(const nsStaticAtom* aAtoms, uint32_t aAtomCount)
 {
   // this does three things:
   // 1) wraps each static atom in a wrapper, if necessary
@@ -560,19 +554,20 @@ RegisterStaticAtoms(const nsStaticAtom* aAtoms, PRUint32 aAtomCount)
   
   if (!gStaticAtomTable && !gStaticAtomTableSealed) {
     gStaticAtomTable = new nsDataHashtable<nsStringHashKey, nsIAtom*>();
-    if (!gStaticAtomTable || !gStaticAtomTable->Init()) {
+    if (!gStaticAtomTable) {
       delete gStaticAtomTable;
-      gStaticAtomTable = nsnull;
+      gStaticAtomTable = nullptr;
       return NS_ERROR_OUT_OF_MEMORY;
     }
+    gStaticAtomTable->Init();
   }
   
-  for (PRUint32 i=0; i<aAtomCount; i++) {
+  for (uint32_t i=0; i<aAtomCount; i++) {
 #ifdef NS_STATIC_ATOM_USE_WIDE_STRINGS
     NS_ASSERTION(nsCRT::IsAscii((PRUnichar*)aAtoms[i].mStringBuffer->Data()),
                  "Static atoms must be ASCII!");
 
-    PRUint32 stringLen =
+    uint32_t stringLen =
       aAtoms[i].mStringBuffer->StorageSize() / sizeof(PRUnichar) - 1;
 
     AtomTableEntry *he =
@@ -606,7 +601,7 @@ RegisterStaticAtoms(const nsStaticAtom* aAtoms, PRUint32 aAtomCount)
     NS_ASSERTION(nsCRT::IsAscii((char*)aAtoms[i].mStringBuffer->Data()),
                  "Static atoms must be ASCII!");
 
-    PRUint32 stringLen = aAtoms[i].mStringBuffer->StorageSize() - 1;
+    uint32_t stringLen = aAtoms[i].mStringBuffer->StorageSize() - 1;
 
     NS_ConvertASCIItoUTF16 str((char*)aAtoms[i].mStringBuffer->Data(),
                                stringLen);
@@ -622,23 +617,22 @@ RegisterStaticAtoms(const nsStaticAtom* aAtoms, PRUint32 aAtomCount)
   return NS_OK;
 }
 
-nsIAtom*
+already_AddRefed<nsIAtom>
 NS_NewAtom(const char* aUTF8String)
 {
   return NS_NewAtom(nsDependentCString(aUTF8String));
 }
 
-nsIAtom*
+already_AddRefed<nsIAtom>
 NS_NewAtom(const nsACString& aUTF8String)
 {
   AtomTableEntry *he = GetAtomHashEntry(aUTF8String.Data(),
                                         aUTF8String.Length());
 
   if (he->mAtom) {
-    nsIAtom* atom;
-    NS_ADDREF(atom = he->mAtom);
+    nsCOMPtr<nsIAtom> atom = he->mAtom;
 
-    return atom;
+    return atom.forget();
   }
 
   // This results in an extra addref/release of the nsStringBuffer.
@@ -646,38 +640,35 @@ NS_NewAtom(const nsACString& aUTF8String)
   // Actually, now there is, sort of: ForgetSharedBuffer.
   nsString str;
   CopyUTF8toUTF16(aUTF8String, str);
-  AtomImpl* atom = new AtomImpl(str, he->keyHash);
+  nsRefPtr<AtomImpl> atom = new AtomImpl(str, he->keyHash);
 
   he->mAtom = atom;
-  NS_ADDREF(atom);
 
-  return atom;
+  return atom.forget();
 }
 
-nsIAtom*
+already_AddRefed<nsIAtom>
 NS_NewAtom(const PRUnichar* aUTF16String)
 {
   return NS_NewAtom(nsDependentString(aUTF16String));
 }
 
-nsIAtom*
+already_AddRefed<nsIAtom>
 NS_NewAtom(const nsAString& aUTF16String)
 {
   AtomTableEntry *he = GetAtomHashEntry(aUTF16String.Data(),
                                         aUTF16String.Length());
 
   if (he->mAtom) {
-    nsIAtom* atom;
-    NS_ADDREF(atom = he->mAtom);
+    nsCOMPtr<nsIAtom> atom = he->mAtom;
 
-    return atom;
+    return atom.forget();
   }
 
-  AtomImpl* atom = new AtomImpl(aUTF16String, he->keyHash);
+  nsRefPtr<AtomImpl> atom = new AtomImpl(aUTF16String, he->keyHash);
   he->mAtom = atom;
-  NS_ADDREF(atom);
 
-  return atom;
+  return atom.forget();
 }
 
 nsIAtom*
@@ -714,7 +705,7 @@ NS_GetStaticAtom(const nsAString& aUTF16String)
   NS_PRECONDITION(gStaticAtomTableSealed, "Static atom table not sealed yet.");
   nsIAtom* atom;
   if (!gStaticAtomTable->Get(aUTF16String, &atom)) {
-    atom = nsnull;
+    atom = nullptr;
   }
   return atom;
 }
