@@ -66,7 +66,7 @@ nsAString::char_type*
 nsAString::EndWriting()
 {
   char_type *data;
-  uint32_t len = NS_StringGetMutableData(*this, PR_UINT32_MAX, &data);
+  uint32_t len = NS_StringGetMutableData(*this, UINT32_MAX, &data);
   return data + len;
 }
 
@@ -515,6 +515,36 @@ nsAString::ToInteger(nsresult *aErrorCode, uint32_t aRadix) const
 
   return result;
 }
+
+int64_t
+nsAString::ToInteger64(nsresult *aErrorCode, uint32_t aRadix) const
+{
+  NS_ConvertUTF16toUTF8 narrow(*this);
+
+  const char *fmt;
+  switch (aRadix) {
+  case 10:
+    fmt = "%lli";
+    break;
+
+  case 16:
+    fmt = "%llx";
+    break;
+
+  default:
+    NS_ERROR("Unrecognized radix!");
+    *aErrorCode = NS_ERROR_INVALID_ARG;
+    return 0;
+  }
+
+  int64_t result = 0;
+  if (PR_sscanf(narrow.get(), fmt, &result) == 1)
+    *aErrorCode = NS_OK;
+  else
+    *aErrorCode = NS_ERROR_FAILURE;
+
+  return result;
+}
 #endif // XPCOM_GLUE_AVOID_NSPR
 
 // nsACString
@@ -567,7 +597,7 @@ nsACString::char_type*
 nsACString::EndWriting()
 {
   char_type *data;
-  uint32_t len = NS_CStringGetMutableData(*this, PR_UINT32_MAX, &data);
+  uint32_t len = NS_CStringGetMutableData(*this, UINT32_MAX, &data);
   return data + len;
 }
 
@@ -921,6 +951,34 @@ nsACString::ToInteger(nsresult *aErrorCode, uint32_t aRadix) const
 
   return result;
 }
+
+int64_t
+nsACString::ToInteger64(nsresult *aErrorCode, uint32_t aRadix) const
+{
+  const char *fmt;
+  switch (aRadix) {
+  case 10:
+    fmt = "%lli";
+    break;
+
+  case 16:
+    fmt = "%llx";
+    break;
+
+  default:
+    NS_ERROR("Unrecognized radix!");
+    *aErrorCode = NS_ERROR_INVALID_ARG;
+    return 0;
+  }
+
+  int64_t result = 0;
+  if (PR_sscanf(nsCString(*this).get(), fmt, &result) == 1)
+    *aErrorCode = NS_OK;
+  else
+    *aErrorCode = NS_ERROR_FAILURE;
+
+  return result;
+}
 #endif // XPCOM_GLUE_AVOID_NSPR
 
 // Substrings
@@ -1003,7 +1061,7 @@ void
 CompressWhitespace(nsAString& aString)
 {
   PRUnichar *start;
-  uint32_t len = NS_StringGetMutableData(aString, PR_UINT32_MAX, &start);
+  uint32_t len = NS_StringGetMutableData(aString, UINT32_MAX, &start);
   PRUnichar *end = start + len;
   PRUnichar *from = start, *to = start;
 
