@@ -40,7 +40,7 @@ public:
   bool SetCount(int32_t aNewCount);
   // returns the max number that can be held without allocating
   inline int32_t GetArraySize() const {
-    return mImpl ? (int32_t(mImpl->mBits) & kArraySizeMask) : 0;
+    return mImpl ? mImpl->mSize : 0;
   }
 
   void* FastElementAt(int32_t aIndex) const
@@ -90,8 +90,8 @@ public:
   }
 
   bool RemoveElement(void* aElement);
-  bool RemoveElementsAt(int32_t aIndex, int32_t aCount);
-  bool RemoveElementAt(int32_t aIndex) { return RemoveElementsAt(aIndex,1); }
+  void RemoveElementsAt(int32_t aIndex, int32_t aCount);
+  void RemoveElementAt(int32_t aIndex) { return RemoveElementsAt(aIndex,1); }
 
   void   Clear();
 
@@ -118,12 +118,9 @@ protected:
 
   struct Impl {
     /**
-     * Packed bits. The low 30 bits are the array's size.
-     * The two highest bits indicate whether or not we "own" mImpl and
-     * must free() it when destroyed, and whether we have a preallocated
-     * nsAutoVoidArray buffer.
+     * The actual array size.
      */
-    uint32_t mBits;
+    int32_t mSize;
 
     /**
      * The number of elements in the array
@@ -143,53 +140,14 @@ protected:
   bool    mIsAuto;
 #endif
 
-  enum {
-    kArrayOwnerMask = 1 << 31,
-    kArrayHasAutoBufferMask = 1 << 30,
-    kArraySizeMask = ~(kArrayOwnerMask | kArrayHasAutoBufferMask)
-  };
-  enum { kAutoBufSize = 8 };
-
-
   // bit twiddlers
-  void SetArray(Impl *newImpl, int32_t aSize, int32_t aCount, bool aOwner,
-                bool aHasAuto);
-  inline bool IsArrayOwner() const {
-    return mImpl && (mImpl->mBits & kArrayOwnerMask);
-  }
-  inline bool HasAutoBuffer() const {
-    return mImpl && (mImpl->mBits & kArrayHasAutoBufferMask);
-  }
+  void SetArray(Impl *newImpl, int32_t aSize, int32_t aCount);
 
 private:
   /// Copy constructors are not allowed
   nsVoidArray(const nsVoidArray& other);
 };
 
-
-// A zero-based array with a bit of automatic internal storage
-class NS_COM_GLUE nsAutoVoidArray : public nsVoidArray {
-public:
-  nsAutoVoidArray();
-
-  void ResetToAutoBuffer()
-  {
-    SetArray(reinterpret_cast<Impl*>(mAutoBuf), kAutoBufSize, 0, false,
-             true);
-  }
-
-  nsAutoVoidArray& operator=(const nsVoidArray& other)
-  {
-    nsVoidArray::operator=(other);
-    return *this;
-  }
-  
-protected:
-  // The internal storage
-  char mAutoBuf[sizeof(Impl) + (kAutoBufSize - 1) * sizeof(void*)];
-};
-
-
 //===================================================================
 //  nsSmallVoidArray is not a general-purpose replacement for
 //  ns(Auto)VoidArray because there is (some) extra CPU overhead for arrays
@@ -252,8 +210,8 @@ public:
     return InsertElementsAt(aElements, Count());
   }
   bool RemoveElement(void* aElement);
-  bool RemoveElementsAt(int32_t aIndex, int32_t aCount);
-  bool RemoveElementAt(int32_t aIndex);
+  void RemoveElementsAt(int32_t aIndex, int32_t aCount);
+  void RemoveElementAt(int32_t aIndex);
 
   void Clear();
   bool SizeTo(int32_t aMin);
