@@ -1,47 +1,10 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla JavaScript code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1999-2001
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Brendan Eich <brendan@mozilla.org> (Original Author)
- *   Chris Waterson <waterson@netscape.com>
- *   L. David Baron <dbaron@dbaron.org>, Mozilla Corporation
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
  * Double hashing implementation.
- *
- * Try to keep this file in sync with js/src/jsdhash.cpp.
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -50,6 +13,8 @@
 #include "pldhash.h"
 #include "mozilla/HashFunctions.h"
 #include "nsDebug.h"     /* for PR_ASSERT */
+#include "nsAlgorithm.h"
+#include "mozilla/Likely.h"
 
 #ifdef PL_DHASHMETER
 # if defined MOZILLA_CLIENT && defined DEBUG_XXXbrendan
@@ -70,7 +35,7 @@
  */
 #ifdef DEBUG
 
-#define RECURSION_LEVEL(table_) (*(PRUint32*)(table_->entryStore + \
+#define RECURSION_LEVEL(table_) (*(uint32_t*)(table_->entryStore + \
                                             PL_DHASH_TABLE_SIZE(table_) * \
                                             table_->entrySize))
 /*
@@ -81,13 +46,13 @@
  *
  * Only PL_DHashTableFinish needs to allow this special value.
  */
-#define IMMUTABLE_RECURSION_LEVEL ((PRUint32)-1)
+#define IMMUTABLE_RECURSION_LEVEL ((uint32_t)-1)
 
 #define RECURSION_LEVEL_SAFE_TO_FINISH(table_)                                \
     (RECURSION_LEVEL(table_) == 0 ||                                          \
      RECURSION_LEVEL(table_) == IMMUTABLE_RECURSION_LEVEL)
 
-#define ENTRY_STORE_EXTRA                   sizeof(PRUint32)
+#define ENTRY_STORE_EXTRA                   sizeof(uint32_t)
 #define INCREMENT_RECURSION_LEVEL(table_)                                     \
     PR_BEGIN_MACRO                                                            \
         if (RECURSION_LEVEL(table_) != IMMUTABLE_RECURSION_LEVEL)             \
@@ -96,7 +61,7 @@
 #define DECREMENT_RECURSION_LEVEL(table_)                                     \
     PR_BEGIN_MACRO                                                            \
         if (RECURSION_LEVEL(table_) != IMMUTABLE_RECURSION_LEVEL) {           \
-            NS_ASSERTION(RECURSION_LEVEL(table_) > 0, "RECURSION_LEVEL(table_) > 0");              \
+            MOZ_ASSERT(RECURSION_LEVEL(table_) > 0);                          \
             --RECURSION_LEVEL(table_);                                        \
         }                                                                     \
     PR_END_MACRO
@@ -112,7 +77,7 @@
 using namespace mozilla;
 
 void *
-PL_DHashAllocTable(PLDHashTable *table, PRUint32 nbytes)
+PL_DHashAllocTable(PLDHashTable *table, uint32_t nbytes)
 {
     return malloc(nbytes);
 }
@@ -132,7 +97,7 @@ PL_DHashStringKey(PLDHashTable *table, const void *key)
 PLDHashNumber
 PL_DHashVoidPtrKeyStub(PLDHashTable *table, const void *key)
 {
-    return (PLDHashNumber)(PRPtrdiff)key >> 2;
+    return (PLDHashNumber)(ptrdiff_t)key >> 2;
 }
 
 bool
@@ -204,8 +169,8 @@ PL_DHashGetStubOps(void)
 }
 
 PLDHashTable *
-PL_NewDHashTable(const PLDHashTableOps *ops, void *data, PRUint32 entrySize,
-                 PRUint32 capacity)
+PL_NewDHashTable(const PLDHashTableOps *ops, void *data, uint32_t entrySize,
+                 uint32_t capacity)
 {
     PLDHashTable *table;
 
@@ -228,10 +193,10 @@ PL_DHashTableDestroy(PLDHashTable *table)
 
 bool
 PL_DHashTableInit(PLDHashTable *table, const PLDHashTableOps *ops, void *data,
-                  PRUint32 entrySize, PRUint32 capacity)
+                  uint32_t entrySize, uint32_t capacity)
 {
     int log2;
-    PRUint32 nbytes;
+    uint32_t nbytes;
 
 #ifdef DEBUG
     if (entrySize > 16 * sizeof(void *)) {
@@ -250,12 +215,12 @@ PL_DHashTableInit(PLDHashTable *table, const PLDHashTableOps *ops, void *data,
 
     PR_CEILING_LOG2(log2, capacity);
 
-    capacity = PR_BIT(log2);
+    capacity = 1u << log2;
     if (capacity >= PL_DHASH_SIZE_LIMIT)
         return false;
     table->hashShift = PL_DHASH_BITS - log2;
-    table->maxAlphaFrac = (PRUint8)(0x100 * PL_DHASH_DEFAULT_MAX_ALPHA);
-    table->minAlphaFrac = (PRUint8)(0x100 * PL_DHASH_DEFAULT_MIN_ALPHA);
+    table->maxAlphaFrac = (uint8_t)(0x100 * PL_DHASH_DEFAULT_MAX_ALPHA);
+    table->minAlphaFrac = (uint8_t)(0x100 * PL_DHASH_DEFAULT_MIN_ALPHA);
     table->entrySize = entrySize;
     table->entryCount = table->removedCount = 0;
     table->generation = 0;
@@ -286,7 +251,7 @@ PL_DHashTableSetAlphaBounds(PLDHashTable *table,
                             float maxAlpha,
                             float minAlpha)
 {
-    PRUint32 size;
+    uint32_t size;
 
     /*
      * Reject obviously insane bounds, rather than trying to guess what the
@@ -306,7 +271,7 @@ PL_DHashTableSetAlphaBounds(PLDHashTable *table,
                  "PL_DHASH_MIN_SIZE - (maxAlpha * PL_DHASH_MIN_SIZE) >= 1");
     if (PL_DHASH_MIN_SIZE - (maxAlpha * PL_DHASH_MIN_SIZE) < 1) {
         maxAlpha = (float)
-                   (PL_DHASH_MIN_SIZE - PR_MAX(PL_DHASH_MIN_SIZE / 256, 1))
+                   (PL_DHASH_MIN_SIZE - XPCOM_MAX(PL_DHASH_MIN_SIZE / 256, 1))
                    / PL_DHASH_MIN_SIZE;
     }
 
@@ -319,11 +284,11 @@ PL_DHashTableSetAlphaBounds(PLDHashTable *table,
                  "minAlpha < maxAlpha / 2");
     if (minAlpha >= maxAlpha / 2) {
         size = PL_DHASH_TABLE_SIZE(table);
-        minAlpha = (size * maxAlpha - PR_MAX(size / 256, 1)) / (2 * size);
+        minAlpha = (size * maxAlpha - XPCOM_MAX(size / 256, 1u)) / (2 * size);
     }
 
-    table->maxAlphaFrac = (PRUint8)(maxAlpha * 256);
-    table->minAlphaFrac = (PRUint8)(minAlpha * 256);
+    table->maxAlphaFrac = (uint8_t)(maxAlpha * 256);
+    table->minAlphaFrac = (uint8_t)(minAlpha * 256);
 }
 
 /*
@@ -364,7 +329,7 @@ void
 PL_DHashTableFinish(PLDHashTable *table)
 {
     char *entryAddr, *entryLimit;
-    PRUint32 entrySize;
+    uint32_t entrySize;
     PLDHashEntryHdr *entry;
 
 #ifdef DEBUG_XXXbrendan
@@ -398,8 +363,7 @@ PL_DHashTableFinish(PLDHashTable *table)
     }
 
     DECREMENT_RECURSION_LEVEL(table);
-    NS_ASSERTION(RECURSION_LEVEL_SAFE_TO_FINISH(table),
-                 "RECURSION_LEVEL_SAFE_TO_FINISH(table)");
+    MOZ_ASSERT(RECURSION_LEVEL_SAFE_TO_FINISH(table));
 
     /* Free entry storage last. */
     table->ops->freeTable(table, table->entryStore);
@@ -413,7 +377,7 @@ SearchTable(PLDHashTable *table, const void *key, PLDHashNumber keyHash,
     int hashShift, sizeLog2;
     PLDHashEntryHdr *entry, *firstRemoved;
     PLDHashMatchEntry matchEntry;
-    PRUint32 sizeMask;
+    uint32_t sizeMask;
 
     METER(table->stats.searches++);
     NS_ASSERTION(!(keyHash & COLLISION_FLAG),
@@ -440,13 +404,13 @@ SearchTable(PLDHashTable *table, const void *key, PLDHashNumber keyHash,
     /* Collision: double hash. */
     sizeLog2 = PL_DHASH_BITS - table->hashShift;
     hash2 = HASH2(keyHash, sizeLog2, hashShift);
-    sizeMask = PR_BITMASK(sizeLog2);
+    sizeMask = (1u << sizeLog2) - 1;
 
     /* Save the first removed entry pointer so PL_DHASH_ADD can recycle it. */
     firstRemoved = NULL;
 
     for (;;) {
-        if (NS_UNLIKELY(ENTRY_IS_REMOVED(entry))) {
+        if (MOZ_UNLIKELY(ENTRY_IS_REMOVED(entry))) {
             if (!firstRemoved)
                 firstRemoved = entry;
         } else {
@@ -491,7 +455,7 @@ FindFreeEntry(PLDHashTable *table, PLDHashNumber keyHash)
     PLDHashNumber hash1, hash2;
     int hashShift, sizeLog2;
     PLDHashEntryHdr *entry;
-    PRUint32 sizeMask;
+    uint32_t sizeMask;
 
     METER(table->stats.searches++);
     NS_ASSERTION(!(keyHash & COLLISION_FLAG),
@@ -511,7 +475,7 @@ FindFreeEntry(PLDHashTable *table, PLDHashNumber keyHash)
     /* Collision: double hash. */
     sizeLog2 = PL_DHASH_BITS - table->hashShift;
     hash2 = HASH2(keyHash, sizeLog2, hashShift);
-    sizeMask = PR_BITMASK(sizeLog2);
+    sizeMask = (1u << sizeLog2) - 1;
 
     for (;;) {
         NS_ASSERTION(!ENTRY_IS_REMOVED(entry),
@@ -537,20 +501,20 @@ static bool
 ChangeTable(PLDHashTable *table, int deltaLog2)
 {
     int oldLog2, newLog2;
-    PRUint32 oldCapacity, newCapacity;
+    uint32_t oldCapacity, newCapacity;
     char *newEntryStore, *oldEntryStore, *oldEntryAddr;
-    PRUint32 entrySize, i, nbytes;
+    uint32_t entrySize, i, nbytes;
     PLDHashEntryHdr *oldEntry, *newEntry;
     PLDHashMoveEntry moveEntry;
 #ifdef DEBUG
-    PRUint32 recursionLevel;
+    uint32_t recursionLevel;
 #endif
 
     /* Look, but don't touch, until we succeed in getting new entry store. */
     oldLog2 = PL_DHASH_BITS - table->hashShift;
     newLog2 = oldLog2 + deltaLog2;
-    oldCapacity = PR_BIT(oldLog2);
-    newCapacity = PR_BIT(newLog2);
+    oldCapacity = 1u << oldLog2;
+    newCapacity = 1u << newLog2;
     if (newCapacity >= PL_DHASH_SIZE_LIMIT)
         return false;
     entrySize = table->entrySize;
@@ -601,11 +565,10 @@ PL_DHashTableOperate(PLDHashTable *table, const void *key, PLDHashOperator op)
 {
     PLDHashNumber keyHash;
     PLDHashEntryHdr *entry;
-    PRUint32 size;
+    uint32_t size;
     int deltaLog2;
 
-    NS_ASSERTION(op == PL_DHASH_LOOKUP || RECURSION_LEVEL(table) == 0,
-                 "op == PL_DHASH_LOOKUP || RECURSION_LEVEL(table) == 0");
+    MOZ_ASSERT(op == PL_DHASH_LOOKUP || RECURSION_LEVEL(table) == 0);
     INCREMENT_RECURSION_LEVEL(table);
 
     keyHash = table->ops->hashKey(table, key);
@@ -710,8 +673,7 @@ PL_DHashTableRawRemove(PLDHashTable *table, PLDHashEntryHdr *entry)
 {
     PLDHashNumber keyHash;      /* load first in case clearEntry goofs it */
 
-    NS_ASSERTION(RECURSION_LEVEL(table) != IMMUTABLE_RECURSION_LEVEL,
-                 "RECURSION_LEVEL(table) != IMMUTABLE_RECURSION_LEVEL");
+    MOZ_ASSERT(RECURSION_LEVEL(table) != IMMUTABLE_RECURSION_LEVEL);
 
     NS_ASSERTION(PL_DHASH_ENTRY_IS_LIVE(entry),
                  "PL_DHASH_ENTRY_IS_LIVE(entry)");
@@ -727,11 +689,11 @@ PL_DHashTableRawRemove(PLDHashTable *table, PLDHashEntryHdr *entry)
     table->entryCount--;
 }
 
-PRUint32
+uint32_t
 PL_DHashTableEnumerate(PLDHashTable *table, PLDHashEnumerator etor, void *arg)
 {
     char *entryAddr, *entryLimit;
-    PRUint32 i, capacity, entrySize, ceiling;
+    uint32_t i, capacity, entrySize, ceiling;
     bool didRemove;
     PLDHashEntryHdr *entry;
     PLDHashOperator op;
@@ -759,8 +721,7 @@ PL_DHashTableEnumerate(PLDHashTable *table, PLDHashEnumerator etor, void *arg)
         entryAddr += entrySize;
     }
 
-    NS_ASSERTION(!didRemove || RECURSION_LEVEL(table) == 1,
-                 "!didRemove || RECURSION_LEVEL(table) == 1");
+    MOZ_ASSERT(!didRemove || RECURSION_LEVEL(table) == 1);
 
     /*
      * Shrink or compress if a quarter or more of all entries are removed, or
@@ -800,7 +761,7 @@ struct SizeOfEntryExcludingThisArg
 
 static PLDHashOperator
 SizeOfEntryExcludingThisEnumerator(PLDHashTable *table, PLDHashEntryHdr *hdr,
-                                   PRUint32 number, void *arg)
+                                   uint32_t number, void *arg)
 {
     SizeOfEntryExcludingThisArg *e = (SizeOfEntryExcludingThisArg *)arg;
     e->total += e->sizeOfEntryExcludingThis(hdr, e->mallocSizeOf, e->arg);
@@ -850,9 +811,9 @@ void
 PL_DHashTableDumpMeter(PLDHashTable *table, PLDHashEnumerator dump, FILE *fp)
 {
     char *entryAddr;
-    PRUint32 entrySize, entryCount;
+    uint32_t entrySize, entryCount;
     int hashShift, sizeLog2;
-    PRUint32 i, tableSize, sizeMask, chainLen, maxChainLen, chainCount;
+    uint32_t i, tableSize, sizeMask, chainLen, maxChainLen, chainCount;
     PLDHashNumber hash1, hash2, saveHash1, maxChainHash1, maxChainHash2;
     double sqsum, mean, variance, sigma;
     PLDHashEntryHdr *entry, *probe;
@@ -862,7 +823,7 @@ PL_DHashTableDumpMeter(PLDHashTable *table, PLDHashEnumerator dump, FILE *fp)
     hashShift = table->hashShift;
     sizeLog2 = PL_DHASH_BITS - hashShift;
     tableSize = PL_DHASH_TABLE_SIZE(table);
-    sizeMask = PR_BITMASK(sizeLog2);
+    sizeMask = (1u << sizeLog2) - 1;
     chainCount = maxChainLen = 0;
     hash2 = 0;
     sqsum = 0;
