@@ -1,42 +1,9 @@
 #!/usr/bin/env python
 # xpidl.py - A parser for cross-platform IDL (XPIDL) files.
 #
-# ***** BEGIN LICENSE BLOCK *****
-# Version: MPL 1.1/GPL 2.0/LGPL 2.1
-#
-# The contents of this file are subject to the Mozilla Public License Version
-# 1.1 (the "License"); you may not use this file except in compliance with
-# the License. You may obtain a copy of the License at
-# http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS IS" basis,
-# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
-# for the specific language governing rights and limitations under the
-# License.
-#
-# The Original Code is mozilla.org code.
-#
-# The Initial Developer of the Original Code is
-#   Mozilla Foundation.
-# Portions created by the Initial Developer are Copyright (C) 2008
-# the Initial Developer. All Rights Reserved.
-#
-# Contributor(s):
-#   Benjamin Smedberg <benjamin@smedbergs.us>
-#
-# Alternatively, the contents of this file may be used under the terms of
-# either of the GNU General Public License Version 2 or later (the "GPL"),
-# or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
-# in which case the provisions of the GPL or the LGPL are applicable instead
-# of those above. If you wish to allow use of your version of this file only
-# under the terms of either the GPL or the LGPL, and not to allow others to
-# use your version of this file under the terms of the MPL, indicate your
-# decision by deleting the provisions above and replace them with the notice
-# and other provisions required by the GPL or the LGPL. If you do not delete
-# the provisions above, a recipient may use your version of this file under
-# the terms of any one of the MPL, the GPL or the LGPL.
-#
-# ***** END LICENSE BLOCK *****
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 """A parser for cross-platform IDL (XPIDL) files."""
 
@@ -153,13 +120,13 @@ class Builtin(object):
 builtinNames = [
     Builtin('boolean', 'bool'),
     Builtin('void', 'void'),
-    Builtin('octet', 'PRUint8'),
-    Builtin('short', 'PRInt16', True, True),
-    Builtin('long', 'PRInt32', True, True),
-    Builtin('long long', 'PRInt64', True, False),
-    Builtin('unsigned short', 'PRUint16', False, True),
-    Builtin('unsigned long', 'PRUint32', False, True),
-    Builtin('unsigned long long', 'PRUint64', False, False),
+    Builtin('octet', 'uint8_t'),
+    Builtin('short', 'int16_t', True, True),
+    Builtin('long', 'int32_t', True, True),
+    Builtin('long long', 'int64_t', True, False),
+    Builtin('unsigned short', 'uint16_t', False, True),
+    Builtin('unsigned long', 'uint32_t', False, True),
+    Builtin('unsigned long long', 'uint64_t', False, False),
     Builtin('float', 'float', True, False),
     Builtin('double', 'double', True, False),
     Builtin('char', 'char', True, False),
@@ -231,6 +198,8 @@ class NameMap(object):
     def set(self, object):
         if object.name in builtinMap:
             raise IDLError("name '%s' is a builtin and cannot be redeclared" % (object.name), object.location)
+        if object.name.startswith("_"):
+            object.name = object.name[1:]
         if object.name in self._d:
             old = self._d[object.name]
             if old == object: return
@@ -502,20 +471,6 @@ class BaseInterface(object):
             if not isinstance(m, CDATA):
                 self.namemap.set(m)
 
-        self.ops = {
-            'index':
-                {
-                    'getter': None,
-                    'setter': None
-                },
-            'name':
-                {
-                    'getter': None,
-                    'setter': None
-                },
-            'stringifier': None
-            }
-
     def __eq__(self, other):
         return self.name == other.name and self.location == other.location
 
@@ -549,16 +504,11 @@ class BaseInterface(object):
             if self.attributes.scriptable and not realbase.attributes.scriptable:
                 print >>sys.stderr, IDLError("interface '%s' is scriptable but derives from non-scriptable '%s'" % (self.name, self.base), self.location, warning=True)
 
-        forwardedMembers = set()
+            if self.attributes.scriptable and realbase.attributes.builtinclass and not self.attributes.builtinclass:
+                raise IDLError("interface '%s' is not builtinclass but derives from builtinclass '%s'" % (self.name, self.base), self.location)
+
         for member in self.members:
             member.resolve(self)
-            if member.kind is 'method' and member.forward:
-                forwardedMembers.add(member.forward)
-        for member in self.members:
-            if member.kind is 'method' and member.name in forwardedMembers:
-                forwardedMembers.remove(member.name)
-        for member in forwardedMembers:
-            raise IDLError("member '%s' on interface '%s' forwards to '%s' which is not on the interface itself" % (member.name, self.name, member.forward), self.location)
 
         # The number 250 is NOT arbitrary; this number is the maximum number of
         # stub entries defined in xpcom/reflect/xptcall/public/genstubs.pl
@@ -734,7 +684,6 @@ class ConstMember(object):
 class Attribute(object):
     kind = 'attribute'
     noscript = False
-    notxpcom = False
     readonly = False
     implicit_jscontext = False
     nostdcall = False
@@ -743,6 +692,7 @@ class Attribute(object):
     undefined = None
     deprecated = False
     nullable = False
+    infallible = False
     defvalue = None
 
     def __init__(self, type, name, attlist, readonly, nullable, defvalue, location, doccomments):
@@ -790,14 +740,14 @@ class Attribute(object):
 
                 if name == 'noscript':
                     self.noscript = True
-                elif name == 'notxpcom':
-                    self.notxpcom = True
                 elif name == 'implicit_jscontext':
                     self.implicit_jscontext = True
                 elif name == 'deprecated':
                     self.deprecated = True
                 elif name == 'nostdcall':
                     self.nostdcall = True
+                elif name == 'infallible':
+                    self.infallible = True
                 else:
                     raise IDLError("Unexpected attribute '%s'" % name, aloc)
 
@@ -816,15 +766,24 @@ class Attribute(object):
             getBuiltinOrNativeTypeName(self.realtype) != '[domstring]'):
             raise IDLError("Nullable types (T?) is supported only for DOMString",
                            self.location)
+        if self.infallible and not self.realtype.kind == 'builtin':
+            raise IDLError('[infallible] only works on builtin types '
+                           '(numbers, booleans, and raw char types)',
+                           self.location)
+        if self.infallible and not iface.attributes.builtinclass:
+            raise IDLError('[infallible] attributes are only allowed on '
+                           '[builtinclass] interfaces',
+                           self.location)
+
 
     def toIDL(self):
         attribs = attlistToIDL(self.attlist)
         readonly = self.readonly and 'readonly ' or ''
         return "%s%sattribute %s %s;" % (attribs, readonly, self.type, self.name)
-        
+
     def isScriptable(self):
         if not self.iface.attributes.scriptable: return False
-        return not (self.noscript or self.notxpcom)
+        return not self.noscript
 
     def __str__(self):
         return "\t%sattribute %s %s\n" % (self.readonly and 'readonly ' or '',
@@ -842,10 +801,6 @@ class Method(object):
     nostdcall = False
     optional_argc = False
     deprecated = False
-    getter = False
-    setter = False
-    stringifier = False
-    forward = None
 
     def __init__(self, type, name, attlist, paramlist, location, doccomments, raises):
         self.type = type
@@ -864,13 +819,6 @@ class Method(object):
 
                 self.binaryname = value
                 continue
-            if name == 'forward':
-                if value is None:
-                    raise IDLError("forward attribute requires a value",
-                                   aloc)
-
-                self.forward = value
-                continue
 
             if value is not None:
                 raise IDLError("Unexpected attribute value", aloc)
@@ -887,18 +835,6 @@ class Method(object):
                 self.deprecated = True
             elif name == 'nostdcall':
                 self.nostdcall = True
-            elif name == 'getter':
-                if (len(self.params) != 1):
-                    raise IDLError("Methods marked as getter must take 1 argument", aloc)
-                self.getter = True
-            elif name == 'setter':
-                if (len(self.params) != 2):
-                    raise IDLError("Methods marked as setter must take 2 arguments", aloc)
-                self.setter = True
-            elif name == 'stringifier':
-                if (len(self.params) != 0):
-                    raise IDLError("Methods marked as stringifier must take 0 arguments", aloc)
-                self.stringifier = True
             else:
                 raise IDLError("Unexpected attribute '%s'" % name, aloc)
 
@@ -911,34 +847,6 @@ class Method(object):
         self.realtype = self.iface.idl.getName(self.type, self.location)
         for p in self.params:
             p.resolve(self)
-        if self.getter:
-            if getBuiltinOrNativeTypeName(self.params[0].realtype) == 'unsigned long':
-                ops = 'index'
-            else:
-                if getBuiltinOrNativeTypeName(self.params[0].realtype) != '[domstring]':
-                    raise IDLError("a getter must take a single unsigned long or DOMString argument" % self.iface.name, self.location)
-                ops = 'name'
-            if self.iface.ops[ops]['getter']:
-                raise IDLError("a %s getter was already defined on interface '%s'" % (ops, self.iface.name), self.location)
-            self.iface.ops[ops]['getter'] = self
-        if self.setter:
-            if getBuiltinOrNativeTypeName(self.params[0].realtype) == 'unsigned long':
-                ops = 'index'
-            else:
-                if getBuiltinOrNativeTypeName(self.params[0].realtype) != '[domstring]':
-                    print getBuiltinOrNativeTypeName(self.params[0].realtype)
-                    raise IDLError("a setter must take a unsigned long or DOMString argument" % self.iface.name, self.location)
-                ops = 'name'
-            if self.iface.ops[ops]['setter']:
-                raise IDLError("a %s setter was already defined on interface '%s'" % (ops, self.iface.name), self.location)
-            self.iface.ops[ops]['setter'] = self
-        if self.stringifier:
-            if self.iface.ops['stringifier']:
-                raise IDLError("a stringifier was already defined on interface '%s'" % self.iface.name, self.location)
-            if getBuiltinOrNativeTypeName(self.realtype) != '[domstring]':
-                raise IDLError("'stringifier' attribute can only be used on methods returning DOMString",
-                               self.location)
-            self.iface.ops['stringifier'] = self
         for p in self.params:
             if p.retval and p != self.params[-1]:
                 raise IDLError("'retval' parameter '%s' is not the last parameter" % p.name, self.location)
@@ -1101,7 +1009,8 @@ class IDLParser(object):
         'raises': 'RAISES',
         'readonly': 'READONLY',
         'native': 'NATIVE',
-        'typedef': 'TYPEDEF'
+        'typedef': 'TYPEDEF',
+        'Infinity': 'INFINITY'
         }
 
     tokens = [
@@ -1148,7 +1057,7 @@ class IDLParser(object):
     t_IID.__doc__ = r'%(c)s{8}-%(c)s{4}-%(c)s{4}-%(c)s{4}-%(c)s{12}' % {'c': hexchar}
 
     def t_IDENTIFIER(self, t):
-        r'unsigned\ long\ long|unsigned\ short|unsigned\ long|long\ long|[A-Za-z][A-Za-z_0-9]*'
+        r'(unsigned\ long\ long|unsigned\ short|unsigned\ long|long\ long)(?!_?[A-Za-z][A-Za-z_0-9])|_?[A-Za-z][A-Za-z_0-9]*'
         t.type = self.keywords.get(t.value, 'IDENTIFIER')
         return t
 
@@ -1531,9 +1440,11 @@ class IDLParser(object):
 
     def p_optdefvalue(self, p):
         """optdefvalue : '=' STRING
+                       | '=' INFINITY
+                       | '=' '-' INFINITY
                        | """
         if len(p) > 1:
-            p[0] = p[2]
+            p[0] = "".join(p[2:])
         else:
             p[0] = None
 
