@@ -1,44 +1,13 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2000
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Scott Collins <scc@mozilla.org> (original author)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsReadableUtils.h"
+
 #include "nsMemory.h"
 #include "nsString.h"
+#include "nsTArray.h"
 #include "nsUTF8Utils.h"
 
 void
@@ -101,33 +70,11 @@ CopyUTF8toUTF16( const char* aSource, nsAString& aDest )
     AppendUTF8toUTF16(aSource, aDest);
   }
 
-// Like GetMutableData, but returns false if it can't
-// allocate enough memory (e.g. due to OOM) rather than
-// returning zero (which could have other meanings) and
-// throws away the out-param pointer.
-bool
-SetLengthForWriting(nsAString& aDest, PRUint32 aDesiredLength)
-  {
-    PRUnichar* dummy;
-    PRUint32 len = aDest.GetMutableData(&dummy, aDesiredLength);
-    return (len >= aDesiredLength);
-  }
-
-bool
-SetLengthForWritingC(nsACString& aDest, PRUint32 aDesiredLength)
-  {
-    char* dummy;
-    PRUint32 len = aDest.GetMutableData(&dummy, aDesiredLength);
-    return (len >= aDesiredLength);
-  }
-
-
 void
 LossyAppendUTF16toASCII( const nsAString& aSource, nsACString& aDest )
   {
-    PRUint32 old_dest_length = aDest.Length();
-    if (!SetLengthForWritingC(aDest, old_dest_length + aSource.Length()))
-        return;
+    uint32_t old_dest_length = aDest.Length();
+    aDest.SetLength(old_dest_length + aSource.Length());
 
     nsAString::const_iterator fromBegin, fromEnd;
 
@@ -145,9 +92,8 @@ LossyAppendUTF16toASCII( const nsAString& aSource, nsACString& aDest )
 void
 AppendASCIItoUTF16( const nsACString& aSource, nsAString& aDest )
   {
-    PRUint32 old_dest_length = aDest.Length();
-    if (!SetLengthForWriting(aDest, old_dest_length + aSource.Length()))
-        return;
+    uint32_t old_dest_length = aDest.Length();
+    aDest.SetLength(old_dest_length + aSource.Length());
 
     nsACString::const_iterator fromBegin, fromEnd;
 
@@ -186,15 +132,14 @@ AppendUTF16toUTF8( const nsAString& aSource, nsACString& aDest )
     copy_string(aSource.BeginReading(source_start),
                 aSource.EndReading(source_end), calculator);
 
-    PRUint32 count = calculator.Size();
+    uint32_t count = calculator.Size();
 
     if (count)
       {
-        PRUint32 old_dest_length = aDest.Length();
+        uint32_t old_dest_length = aDest.Length();
 
         // Grow the buffer if we need to.
-        if(!SetLengthForWritingC(aDest, old_dest_length + count))
-            return;
+        aDest.SetLength(old_dest_length + count);
 
         // All ready? Time to convert
 
@@ -210,22 +155,32 @@ AppendUTF16toUTF8( const nsAString& aSource, nsACString& aDest )
 
 void
 AppendUTF8toUTF16( const nsACString& aSource, nsAString& aDest )
+{
+  if (!AppendUTF8toUTF16(aSource, aDest, mozilla::fallible_t())) {
+    NS_RUNTIMEABORT("OOM");
+  }
+}
+
+bool
+AppendUTF8toUTF16( const nsACString& aSource, nsAString& aDest,
+                   const mozilla::fallible_t& )
   {
     nsACString::const_iterator source_start, source_end;
     CalculateUTF8Length calculator;
     copy_string(aSource.BeginReading(source_start),
                 aSource.EndReading(source_end), calculator);
 
-    PRUint32 count = calculator.Length();
+    uint32_t count = calculator.Length();
 
     // Avoid making the string mutable if we're appending an empty string
     if (count)
       {
-        PRUint32 old_dest_length = aDest.Length();
+        uint32_t old_dest_length = aDest.Length();
 
         // Grow the buffer if we need to.
-        if(!SetLengthForWriting(aDest, old_dest_length + count))
-          return;
+        if (!aDest.SetLength(old_dest_length + count, mozilla::fallible_t())) {
+          return false;
+        }
 
         // All ready? Time to convert
 
@@ -243,6 +198,8 @@ AppendUTF8toUTF16( const nsACString& aSource, nsAString& aDest )
             aDest.SetLength(old_dest_length);
           }
       }
+
+    return true;
   }
 
 void
@@ -283,7 +240,7 @@ ToNewCString( const nsAString& aSource )
   {
     char* result = AllocateStringCopy(aSource, (char*)0);
     if (!result)
-      return nsnull;
+      return nullptr;
 
     nsAString::const_iterator fromBegin, fromEnd;
     LossyConvertEncoding16to8 converter(result);
@@ -292,7 +249,7 @@ ToNewCString( const nsAString& aSource )
   }
 
 char*
-ToNewUTF8String( const nsAString& aSource, PRUint32 *aUTF8Count )
+ToNewUTF8String( const nsAString& aSource, uint32_t *aUTF8Count )
   {
     nsAString::const_iterator start, end;
     CalculateUTF8Size calculator;
@@ -305,7 +262,7 @@ ToNewUTF8String( const nsAString& aSource, PRUint32 *aUTF8Count )
     char *result = static_cast<char*>
                               (nsMemory::Alloc(calculator.Size() + 1));
     if (!result)
-      return nsnull;
+      return nullptr;
 
     ConvertUTF16toUTF8 converter(result);
     copy_string(aSource.BeginReading(start), aSource.EndReading(end),
@@ -322,7 +279,7 @@ ToNewCString( const nsACString& aSource )
 
     char* result = AllocateStringCopy(aSource, (char*)0);
     if (!result)
-      return nsnull;
+      return nullptr;
 
     nsACString::const_iterator fromBegin, fromEnd;
     char* toBegin = result;
@@ -337,7 +294,7 @@ ToNewUnicode( const nsAString& aSource )
 
     PRUnichar* result = AllocateStringCopy(aSource, (PRUnichar*)0);
     if (!result)
-      return nsnull;
+      return nullptr;
 
     nsAString::const_iterator fromBegin, fromEnd;
     PRUnichar* toBegin = result;
@@ -350,7 +307,7 @@ ToNewUnicode( const nsACString& aSource )
   {
     PRUnichar* result = AllocateStringCopy(aSource, (PRUnichar*)0);
     if (!result)
-      return nsnull;
+      return nullptr;
 
     nsACString::const_iterator fromBegin, fromEnd;
     LossyConvertEncoding8to16 converter(result);
@@ -358,36 +315,53 @@ ToNewUnicode( const nsACString& aSource )
     return result;
   }
 
-PRUnichar*
-UTF8ToNewUnicode( const nsACString& aSource, PRUint32 *aUTF16Count )
+uint32_t
+CalcUTF8ToUnicodeLength( const nsACString& aSource)
   {
     nsACString::const_iterator start, end;
     CalculateUTF8Length calculator;
     copy_string(aSource.BeginReading(start), aSource.EndReading(end),
                 calculator);
-
-    if (aUTF16Count)
-      *aUTF16Count = calculator.Length();
-
-    PRUnichar *result = static_cast<PRUnichar*>
-                                   (nsMemory::Alloc(sizeof(PRUnichar) * (calculator.Length() + 1)));
-    if (!result)
-      return nsnull;
-
-    ConvertUTF8toUTF16 converter(result);
-    copy_string(aSource.BeginReading(start), aSource.EndReading(end),
-                converter).write_terminator();
-    NS_ASSERTION(calculator.Length() == converter.Length(), "length mismatch");
-
-    return result;
+    return calculator.Length();
   }
 
 PRUnichar*
-CopyUnicodeTo( const nsAString& aSource, PRUint32 aSrcOffset, PRUnichar* aDest, PRUint32 aLength )
+UTF8ToUnicodeBuffer( const nsACString& aSource, PRUnichar* aBuffer, uint32_t *aUTF16Count )
+  {
+    nsACString::const_iterator start, end;
+    ConvertUTF8toUTF16 converter(aBuffer);
+    copy_string(aSource.BeginReading(start),
+                aSource.EndReading(end),
+                converter).write_terminator();
+    if (aUTF16Count)
+      *aUTF16Count = converter.Length();
+    return aBuffer;
+  }
+
+PRUnichar*
+UTF8ToNewUnicode( const nsACString& aSource, uint32_t *aUTF16Count )
+  {
+    const uint32_t length = CalcUTF8ToUnicodeLength(aSource);
+    const size_t buffer_size = (length + 1) * sizeof(PRUnichar);
+    PRUnichar *buffer = static_cast<PRUnichar*>(nsMemory::Alloc(buffer_size));
+    if (!buffer)
+      return nullptr;
+
+    uint32_t copied;
+    UTF8ToUnicodeBuffer(aSource, buffer, &copied);
+    NS_ASSERTION(length == copied, "length mismatch");
+
+    if (aUTF16Count)
+      *aUTF16Count = copied;
+    return buffer;
+  }
+
+PRUnichar*
+CopyUnicodeTo( const nsAString& aSource, uint32_t aSrcOffset, PRUnichar* aDest, uint32_t aLength )
   {
     nsAString::const_iterator fromBegin, fromEnd;
     PRUnichar* toBegin = aDest;    
-    copy_string(aSource.BeginReading(fromBegin).advance( PRInt32(aSrcOffset) ), aSource.BeginReading(fromEnd).advance( PRInt32(aSrcOffset+aLength) ), toBegin);
+    copy_string(aSource.BeginReading(fromBegin).advance( int32_t(aSrcOffset) ), aSource.BeginReading(fromEnd).advance( int32_t(aSrcOffset+aLength) ), toBegin);
     return aDest;
   }
 
@@ -397,8 +371,7 @@ CopyUnicodeTo( const nsAString::const_iterator& aSrcStart,
                nsAString& aDest )
   {
     nsAString::iterator writer;
-    if (!SetLengthForWriting(aDest, Distance(aSrcStart, aSrcEnd)))
-        return;
+    aDest.SetLength(Distance(aSrcStart, aSrcEnd));
 
     aDest.BeginWriting(writer);
     nsAString::const_iterator fromBegin(aSrcStart);
@@ -412,9 +385,8 @@ AppendUnicodeTo( const nsAString::const_iterator& aSrcStart,
                  nsAString& aDest )
   {
     nsAString::iterator writer;
-    PRUint32 oldLength = aDest.Length();
-    if(!SetLengthForWriting(aDest, oldLength + Distance(aSrcStart, aSrcEnd)))
-        return;
+    uint32_t oldLength = aDest.Length();
+    aDest.SetLength(oldLength + Distance(aSrcStart, aSrcEnd));
 
     aDest.BeginWriting(writer).advance(oldLength);
     nsAString::const_iterator fromBegin(aSrcStart);
@@ -476,12 +448,12 @@ IsUTF8( const nsACString& aString, bool aRejectNonChar )
     nsReadingIterator<char> done_reading;
     aString.EndReading(done_reading);
 
-    PRInt32 state = 0;
+    int32_t state = 0;
     bool overlong = false;
     bool surrogate = false;
     bool nonchar = false;
-    PRUint16 olupper = 0; // overlong byte upper bound.
-    PRUint16 slower = 0;  // surrogate byte lower bound.
+    uint16_t olupper = 0; // overlong byte upper bound.
+    uint16_t slower = 0;  // surrogate byte lower bound.
 
     nsReadingIterator<char> iter;
     aString.BeginReading(iter);
@@ -490,7 +462,7 @@ IsUTF8( const nsACString& aString, bool aRejectNonChar )
     const char* end = done_reading.get();
     while ( ptr < end )
       {
-        PRUint8 c;
+        uint8_t c;
         
         if (0 == state)
           {
@@ -572,8 +544,8 @@ class ConvertToUpperCase
     public:
       typedef char value_type;
 
-      PRUint32
-      write( const char* aSource, PRUint32 aSourceLength )
+      uint32_t
+      write( const char* aSource, uint32_t aSourceLength )
         {
           char* cp = const_cast<char*>(aSource);
           const char* end = aSource + aSourceLength;
@@ -608,10 +580,10 @@ class CopyToUpperCase
         {
         }
 
-      PRUint32
-      write( const char* aSource, PRUint32 aSourceLength )
+      uint32_t
+      write( const char* aSource, uint32_t aSourceLength )
         {
-          PRUint32 len = NS_MIN(PRUint32(mIter.size_forward()), aSourceLength);
+          uint32_t len = XPCOM_MIN(uint32_t(mIter.size_forward()), aSourceLength);
           char* cp = mIter.get();
           const char* end = aSource + len;
           while (aSource != end) {
@@ -636,8 +608,7 @@ ToUpperCase( const nsACString& aSource, nsACString& aDest )
   {
     nsACString::const_iterator fromBegin, fromEnd;
     nsACString::iterator toBegin;
-    if (!SetLengthForWritingC(aDest, aSource.Length()))
-        return;
+    aDest.SetLength(aSource.Length());
 
     CopyToUpperCase converter(aDest.BeginWriting(toBegin));
     copy_string(aSource.BeginReading(fromBegin), aSource.EndReading(fromEnd), converter);
@@ -651,8 +622,8 @@ class ConvertToLowerCase
     public:
       typedef char value_type;
 
-      PRUint32
-      write( const char* aSource, PRUint32 aSourceLength )
+      uint32_t
+      write( const char* aSource, uint32_t aSourceLength )
         {
           char* cp = const_cast<char*>(aSource);
           const char* end = aSource + aSourceLength;
@@ -687,10 +658,10 @@ class CopyToLowerCase
         {
         }
 
-      PRUint32
-      write( const char* aSource, PRUint32 aSourceLength )
+      uint32_t
+      write( const char* aSource, uint32_t aSourceLength )
         {
-          PRUint32 len = NS_MIN(PRUint32(mIter.size_forward()), aSourceLength);
+          uint32_t len = XPCOM_MIN(uint32_t(mIter.size_forward()), aSourceLength);
           char* cp = mIter.get();
           const char* end = aSource + len;
           while (aSource != end) {
@@ -715,8 +686,7 @@ ToLowerCase( const nsACString& aSource, nsACString& aDest )
   {
     nsACString::const_iterator fromBegin, fromEnd;
     nsACString::iterator toBegin;
-    if (!SetLengthForWritingC(aDest, aSource.Length()))
-        return;
+    aDest.SetLength(aSource.Length());
 
     CopyToLowerCase converter(aDest.BeginWriting(toBegin));
     copy_string(aSource.BeginReading(fromBegin), aSource.EndReading(fromEnd), converter);
@@ -730,7 +700,7 @@ ParseString(const nsACString& aSource, char aDelimiter,
     aSource.BeginReading(start);
     aSource.EndReading(end);
 
-    PRUint32 oldLength = aArray.Length();
+    uint32_t oldLength = aArray.Length();
 
     for (;;)
       {
@@ -913,7 +883,7 @@ RFindInReadable( const nsACString& aPattern, nsACString::const_iterator& aSearch
 bool
 FindCharInReadable( PRUnichar aChar, nsAString::const_iterator& aSearchStart, const nsAString::const_iterator& aSearchEnd )
   {
-    PRInt32 fragmentLength = aSearchEnd.get() - aSearchStart.get();
+    int32_t fragmentLength = aSearchEnd.get() - aSearchStart.get();
 
     const PRUnichar* charFoundAt = nsCharTraits<PRUnichar>::find(aSearchStart.get(), fragmentLength, aChar);
     if ( charFoundAt ) {
@@ -928,7 +898,7 @@ FindCharInReadable( PRUnichar aChar, nsAString::const_iterator& aSearchStart, co
 bool
 FindCharInReadable( char aChar, nsACString::const_iterator& aSearchStart, const nsACString::const_iterator& aSearchEnd )
   {
-    PRInt32 fragmentLength = aSearchEnd.get() - aSearchStart.get();
+    int32_t fragmentLength = aSearchEnd.get() - aSearchStart.get();
 
     const char* charFoundAt = nsCharTraits<char>::find(aSearchStart.get(), fragmentLength, aChar);
     if ( charFoundAt ) {
@@ -940,11 +910,11 @@ FindCharInReadable( char aChar, nsACString::const_iterator& aSearchStart, const
     return false;
   }
 
-PRUint32
+uint32_t
 CountCharInReadable( const nsAString& aStr,
                      PRUnichar aChar )
 {
-  PRUint32 count = 0;
+  uint32_t count = 0;
   nsAString::const_iterator begin, end;
   
   aStr.BeginReading(begin);
@@ -960,11 +930,11 @@ CountCharInReadable( const nsAString& aStr,
   return count;
 }
 
-PRUint32
+uint32_t
 CountCharInReadable( const nsACString& aStr,
                      char aChar )
 {
-  PRUint32 count = 0;
+  uint32_t count = 0;
   nsACString::const_iterator begin, end;
   
   aStr.BeginReading(begin);
@@ -1062,11 +1032,11 @@ NullCString()
     return sNull;
   }
 
-PRInt32
+int32_t
 CompareUTF8toUTF16(const nsASingleFragmentCString& aUTF8String,
                    const nsASingleFragmentString& aUTF16String)
   {
-    static const PRUint32 NOT_ASCII = PRUint32(~0x7F);
+    static const uint32_t NOT_ASCII = uint32_t(~0x7F);
 
     const char *u8, *u8end;
     aUTF8String.BeginReading(u8);
@@ -1079,17 +1049,17 @@ CompareUTF8toUTF16(const nsASingleFragmentCString& aUTF8String,
     while (u8 != u8end && u16 != u16end)
       {
         // Cast away the signedness of *u8 to prevent signextension when
-        // converting to PRUint32
-        PRUint32 c8_32 = (PRUint8)*u8;
+        // converting to uint32_t
+        uint32_t c8_32 = (uint8_t)*u8;
 
         if (c8_32 & NOT_ASCII)
           {
             bool err;
             c8_32 = UTF8CharEnumerator::NextChar(&u8, u8end, &err);
             if (err)
-              return PR_INT32_MIN;
+              return INT32_MIN;
 
-            PRUint32 c16_32 = UTF16CharEnumerator::NextChar(&u16, u16end);
+            uint32_t c16_32 = UTF16CharEnumerator::NextChar(&u16, u16end);
             // The above UTF16CharEnumerator::NextChar() calls can
             // fail, but if it does for anything other than no data to
             // look at (which can't happen here), it returns the
@@ -1140,7 +1110,7 @@ CompareUTF8toUTF16(const nsASingleFragmentCString& aUTF8String,
   }
 
 void
-AppendUCS4ToUTF16(const PRUint32 aSource, nsAString& aDest)
+AppendUCS4ToUTF16(const uint32_t aSource, nsAString& aDest)
   {
     NS_ASSERTION(IS_VALID_CHAR(aSource), "Invalid UCS4 char");
     if (IS_IN_BMP(aSource))
